/**
 *  \file       acmds/ajoin_context.c
 *  \brief      \$Id: ajoin_context.c 1182 2009-05-15 13:59:20Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1182 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/** \addtogroup ajoin
 * \{
 */

#include "ajoin_context.h"
#include <stdlib.h>
#include <string.h>
#include "debug.h"

/// link to the MqErrorS object
#define MQ_ERROR_S context->msgque->error

/*****************************************************************************/
/*                                                                           */
/*                               Request Handler                             */
/*                                                                           */
/*****************************************************************************/

static void JoinHelp ( const char * ) __attribute__ ((noreturn));

/// \brief display help using \b -h or \b --help command-line option
/// \param base the executable usually: <tt>basename(argv[0])</tt>
static void
JoinHelp ( const char * base  )
{
    fprintf(stderr, "usage: %s [OPTION]... [ARGUMENT]...\n", base);
    fputs("\n", stderr);
    fprintf(stderr, "  The %s tool expect data from a msgque client as input and create for\n", base);
    fputs("  every input package an stdout output string by joining the objects of the\n", stderr);
    fputs("  input package together using the delimiter '-d'.\n", stderr);
    fputs("\n", stderr);
    fprintf(stderr, "  %s [ARGUMENT]... syntax:\n", base);
    fprintf(stderr, "    aclient [OPTIONS]... @ %s [OPTIONS]... | ...\n", base);
    fputs("\n", stderr);
    fputs(MqHelp (NULL), stderr);
    fputs("\n", stderr);
    fprintf(stderr,"  %s [OPTION]:\n", base);
    fputs("    -d CHAR          the delimiter         (default: ' ')\n", stderr);
    fputs("    -NUM FORTMAT     the column number(s)  (default: 'empty'\n", stderr);
    fputs("      NUM            a single column with '0' < NUM < '255'\n", stderr);
    fputs("      FORMAT         printf like format    (default: '%s'\n", stderr);
    fputs("    -h, --help       print this help\n", stderr);
    fputs("\n", stderr);

    exit(EXIT_SUCCESS);
}

/// \tool_FTR
static enum MqErrorE
JoinFilter (
  struct MqS * msgque,
  void *data
)
{
  MQ_STR readStr;
  MQ_INT readInt;
  MQ_WID readWid;
  MQ_DBL readDbl;
  MQ_STR delimiter = ((struct JoinCtxS *)msgque->config.myCtx)->delimiter;
  MQ_STR *format = ((struct JoinCtxS *)msgque->config.myCtx)->format;
  enum MqTypeE *type = ((struct JoinCtxS *)msgque->config.myCtx)->type;
  struct MqReadS * const read = msgque->read;
  MQ_SIZE const end = MqReadGetNumItems(read);
  MQ_INT i;
  if (!end)
    return MQ_OK;
  for (i=0; i<end ; i++, format++, type++) {
    if (*format) {
	switch (*type) {
	  case MQ_STRT:
	    MqErrorCheck (MqReadC (read, &readStr));
	    printf (*format, readStr);
	    break;
	  case MQ_INTT:
	    MqErrorCheck (MqReadI (read, &readInt));
	    printf (*format, readInt);
	    break;
	  case MQ_WIDT:
	    MqErrorCheck (MqReadW (read, &readWid));
	    printf (*format, readWid);
	    break;
	  case MQ_DBLT:
	    MqErrorCheck (MqReadD (read, &readDbl));
	    printf (*format, readDbl);
	    break;
	  default:
	    break;
	}
    } else {
	MqErrorCheck (MqReadC (read, &readStr));
	fputs (readStr,stdout);
    }
    fputs ((i < (end-1) ? delimiter : "\n"),stdout);
    
  }
  fflush(stdout);
  return MQ_OK;

error:
  return MqErrorStack (msgque->error);
}

/*****************************************************************************/
/*                                                                           */
/*                                context_init                               */
/*                                                                           */
/*****************************************************************************/

/// \tool_Delete
static void
JoinDelete (
  struct MqS ** msgqueP
)
{
  struct JoinCtxS * context;
  MQ_STR *freeP;
  
  if (!(*msgqueP))
    return;

  context = (struct JoinCtxS*) (*msgqueP)->config.myCtx;
  freeP = context->free;

  while (*freeP) {
    free(*freeP++);
  }

  OptLDelete (&context->formatlist);
  MqSysFree (context);
  MqDelete (msgqueP);
}

/// \tool_Create
enum MqErrorE
JoinCreate (
  struct MqConfigS * config,
  struct MqBufferLS * argv,
  struct MqBufferLS * alfa,
  struct MqS ** msgqueP
)
{
  struct JoinCtxS * context = MqSysCalloc (MQ_ERROR_PANIC, 1, sizeof (struct JoinCtxS));
  struct MqErrorS * error;
  OptLSP format;
  
  // initial values
  context->delimiter = " ";

  // add my config data
  config->myCtx		= (struct MqContextS*) context;
  config->fHelp		= JoinHelp;
  config->fFTR		= JoinFilter;
  config->isServer	= MQ_YES;
  config->fParentCreate	= JoinCreate;
  config->fParentDelete = JoinDelete;

  // create the Message-Queue
  MqErrorCheck (MqCreate (config, argv, alfa, msgqueP));
  error = (*msgqueP)->error;

  // get the delimiter
  MqErrorCheck (MqBufferLCheckOptionC (error, argv, "-d", &context->delimiter));

  // check for the format key
  MqErrorCheck (OptLCreate (error, argv, &format));

  // format found
  if (format->cursize) {

    MQ_SIZE i,idx;
    MQ_STR cur,fmt;
#if defined(MQ_IS_WIN32)
    MQ_STR modfmt = NULL;
    MQ_INT freeIndex = 0;
#endif
    int found, myshort, mylonglong;
    enum MqTypeE type;
    char id[128];
    memset(id,0,128);
    
    // init the search array
    id['h'] = 1;	    // short
    id['l'] = 2;	    // long or longlong
    id['q'] = 3;	    // old: longlong
    id['s'] = 4;	    // string
    id['e'] = id['E'] = id['f'] = id['F'] = id['a'] = id['A'] = id['g'] = id['G'] = 5;	// double
    id['d'] = id['i'] = id['o'] = id['u'] = id['x'] = id['X'] = 6;    // integer

    for (i=0; i<format->cursize; i++) {
      idx = format->data[i].idx;
      context->format[idx] = cur = fmt = (MQ_STR) format->data[i].val->data;;
      found=myshort=mylonglong=0;
      type=-1;
      while (*cur != '\0') {
	if (*cur != '%') {	// skip != %
	  cur++; continue;
	}
	if (*(cur+1) == '%') {	// skip %%
	  cur+=2; continue;
	}
	if (found) {
	  MqErrorV(context->msgque->error,__func__,-1,
	  "column '" MQ_FORMAT_Z "' has multiple '%%' formats '%s' but only one is allowed", 
	      idx, fmt);
	  goto error;
	}
	found=1;
	cur++;
	while (*cur != '\0' && *cur != '%') {
	  if (*cur > 0 && id[(unsigned char)*cur]) {
	    switch (id[(unsigned char)*cur]) {
	      case 1:
		myshort=1; break;
	      case 2:
		if (*(cur+1) == 'l') {
#if defined(MQ_IS_WIN32)
		  modfmt=cur;
#endif
		  mylonglong=1; cur+=2; continue;
		}
		break;
	      case 3:
		mylonglong=1; break;
	      case 4:
		type = MQ_STRT; goto end;
	      case 5:
		type = MQ_DBLT; goto end;
	      case 6:
		if (mylonglong) {
		  type = MQ_WIDT; goto end;
		} else {
		  type = MQ_INTT; goto end;
		}
	    }
	  }
	  cur++;
	}
end:
	cur++;
      }

#  if defined(MQ_IS_WIN32)
    // if the system is WIN32 we have to transform the %ll(x) format to %I64(n) format -> one extra char
      if (type == MQ_WIDT && modfmt != NULL) {
	  MQ_STR tmp, tfmt=fmt;
	  tmp = cur = MqSysMalloc(context->msgque->error, strlen(context->format[idx])+1+1);
	  // 1. copy everything up to modfmt
	  while (tfmt < modfmt) {
	      *cur++ = *tfmt++;
	  }
	  // 2. add I64
	  *cur++ = 'I';
	  *cur++ = '6';
	  *cur++ = '4';
	  modfmt+=2;
	  // 3. add everything after  the %ll format
	  while (*modfmt != '\0') *cur++ = *modfmt++;
	  // 4. add final '\0'
	  *cur++ = '\0';
	  // 5. activate the new format
	  context->free[freeIndex] = tmp;
	  context->format[idx] = tmp;
	  freeIndex++;
      }
#  endif

  //printLog(context->msgque, "idx<%i>, type<%s>, format<%s>\n", idx, MqLogTypeName(type), fmt);
      if (type == -1) {
	MqErrorV(context->msgque->error,__func__,-1,
	  "column '" MQ_FORMAT_Z "' has no valid format '%s'", idx, fmt);
	goto error;
      }
      context->type[idx] = type;
    }
    context->formatlist = format;
  }

  // check for wrong arguments
  MqErrorCheck (MqCheckForLeftOverArguments(*msgqueP, &argv));

  // everything is OK
  return MQ_OK;

error:
  MqErrorStack ((*msgqueP)->error);
  return MqErrorSAppendC((*msgqueP)->error, "use '-h' or '--help' for usage");
}

/** \} server */

