/**
 *  \file       acmds/atool.c
 *  \brief      \$Id: atool.c 1192 2009-05-19 12:53:58Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1192 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/** \ingroup tools
 *  \defgroup atool atool
 *  \{
 *  \brief atool \atool_desc
 *
 *  \verbinclude atool.help
 */

#include "msgque.h"
#include "config.h"
#include "debug.h"

#include <stdlib.h>
#include <string.h>

#define MSGQUE NULL

#ifndef DOXYGEN_SHOULD_SKIP_THIS
enum MqErrorE SplitCreate (struct MqConfigS *, struct MqBufferLS * , struct MqBufferLS *, struct MqS **);
enum MqErrorE CutCreate   (struct MqConfigS *, struct MqBufferLS * , struct MqBufferLS *, struct MqS **);
enum MqErrorE SortCreate  (struct MqConfigS *, struct MqBufferLS * , struct MqBufferLS *, struct MqS **);
enum MqErrorE JoinCreate  (struct MqConfigS *, struct MqBufferLS * , struct MqBufferLS *, struct MqS **);
#endif

/*****************************************************************************/
/*                                                                           */
/*                                     main                                  */
/*                                                                           */
/*****************************************************************************/

/// \brief display help using \b -h or \b --help command-line option
/// \param base the executable usually: <tt>basename(argv[0])</tt>
static void
ToolHelp ( const char * base  )
{
    fprintf(stderr, "\n");
    fprintf(stderr, "usage: %s [OPTION]... [ARGUMENT]...\n", base);
    fprintf(stderr, "\n");
    fprintf(stderr, "  %s was designed to act like a filter for command-line\n", base);
    fprintf(stderr, "  tasks with the basic sub-commands:\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "  split - a Filter to split the stdin into libmsgque objects\n");
    fprintf(stderr, "  cut   - a Filter to cut libmsgque objects into columns\n");
    fprintf(stderr, "  sort  - a Filter to sort columns\n");
    fprintf(stderr, "  join  - a Filter to join columns into a string and send it to stdout\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "  please use \"sub-command -h\" or \"sub-command --help\" for help\n");
    fprintf(stderr, "\n");

    exit(EXIT_SUCCESS);
}

/// \brief convert an application \e name into a \e main function-pointer
/// \param name the name uf the sub-command
/// \return the \e main function-pointer or NULL
MqCreateF ToolMainSelector (
    const char * name
) {
    if (strncmp("split",name,5) == 0) {
	return SplitCreate;
    } else if (strncmp("cut",name,3) == 0) {
	return CutCreate;
    } else if (strncmp("sort",name,4) == 0) {
	return SortCreate;
    } else if (strncmp("join",name,4) == 0) {
	return JoinCreate;
    } else {
	return NULL;
    }
}

/// \brief main entry-point for the tool
/// \param argc the number of command-line arguments
/// \param argv the command-line arguments as an array of strings
/// \return the exit number
int
main (
  int const argc,
  char ** const argv
)
{
  // the parent-context
  struct MqS * msgque;
  struct MqBufferLS *largv, *lalfa, *initB;
  MqCreateF MainF;

  // check the command-line argument
  if (argc < 2) ToolHelp(MqSysBasename(argv[0], MQ_NO));

  // init libmsgque global data
  initB = MqInitCreate();
  MqBufferLAppendC(initB, argv[0]);
  MqMainSelector = ToolMainSelector;

  // find entry-point
  MainF = ToolMainSelector (argv[1]);

  // entry not found
  if (!MainF)
    ToolHelp(MqSysBasename(argv[0], MQ_NO));

  // split command line
  MqBufferLCreateArgv ((argc-1), (argv+1), &largv, &lalfa);

  // call entry point
  MqErrorCheck((*MainF)(MqConfigInit(), largv, lalfa, &msgque));

  // start event-loop and wait forever
  if (msgque->config.isServer == MQ_YES)
    MqProcessEvent (msgque, MQ_TIMEOUT, MQ_WAIT_FOREVER);

  // finish
error:
  MqExit(msgque);
}

/** \} atool */
