/**
 *  \file       acmds/optionL.c
 *  \brief      \$Id: optionL.c 1182 2009-05-15 13:59:20Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1182 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "optionL.h"
#include "debug.h"
#include <errno.h>
#include <stdlib.h>
#include <string.h>

/*****************************************************************************/
/*                                                                           */
/*                    reading options from a MqBufferLS                      */
/*                                                                           */
/*****************************************************************************/

/** \defgroup optionL_api struct OptLS - parsing an MqBufferLS object for options
 *  \ingroup support
 *  \{
 *  \brief parsing options with arguments from an MqBufferLS object
 */

/// \brief parse the \e argv list and find options of syntay \b -NUM \b ARG with 0<=NUM<=255
/// \error
/// \argv
/// \retval out an array of OptLSP as search result
/// \retMqErrorE
enum MqErrorE
OptLCreate (
  struct MqErrorS * const error,
  struct MqBufferLS * const argv,
  struct OptLS ** const out
)
{
  MQ_SIZE  size    = 10;
  MQ_SIZE  cursize = 0;
  OptLSP option  = MqSysCalloc (error, 1, sizeof(OptLS));
  OptLASP data   = MqSysCalloc (error, size, sizeof(OptLAS));

  // check for the sort key
  MQ_STR key, cur, end;
  long idx;

  while (argv->cursize) {
    key = cur = (MQ_STR) (*argv->data)->data;
    if (*cur++ != '-') 
	return MqErrorC (error, __func__, 1, 
		"expect column option of form '-NUM' with 0 <= NUM <= 255");
    errno=idx=0;
    idx = strtol (cur, &end, 10);
    if (idx < 0 || idx > 255 || (idx == 0 && errno != 0) || (end-cur) != strlen (cur))
	return MqErrorV (error, __func__, errno, 
		"number out of range or invalid, unable to convert option '%s' into a column number", key);
    if (argv->cursize==1)
	return MqErrorV (error, __func__, 1, "expect argument for option '%s'", key);
    MqBufferLDeleteItem (error, argv, 0, 1);

    // check if array has enough space
    if (cursize >= size) {
	size = (size+size/3);
	data = MqSysRealloc (error, data, size * sizeof(OptLAS));
    }
  
    // save the data in the array  
    data[cursize].idx = idx;
    data[cursize].val = *argv->data;
    cursize++;

    // cleanup
    MqBufferLDeleteItem (error, argv, 0, 0);
  }

  // the data back
  option->data = data;
  option->cursize = cursize;
  option->size = size;
  *out = option;

  // everything is OK
  return MQ_OK;
}

/// \brief delete an OptLSP array created by #OptLCreate
/// \param out a pointer to an OptLSP object
/// \attDelete
void
OptLDelete (
  OptLSP *out
)
{
    OptLSP opt = *out;
    OptLASP start;
    OptLASP end;
    if (!opt) return;
    start = opt->data;
    end = opt->data + opt->cursize ;
    for (; start < end; start++) {
	MqBufferDelete (&(start->val));
    }
    free (opt->data);
    MqSysFree (*out);
}

/** \} optionL_api */
