/**
 *  \file       src/sys_com.c
 *  \brief      \$Id: sys_com.c 1182 2009-05-15 13:59:20Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1182 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "main.h"
#include "sys_com.h"
#include "error.h"
#include "mq_io.h"

#if defined(MQ_IS_POSIX)
#   define SHUTDOWN_BOTH SHUT_RDWR
#   define SOCKET_ERROR -1
#   define INVALID_SOCKET -1
#   define WIN32_WSA(x)   x
#   define WIN32_socket(x)   x
//#   if defined(SOLARIS_I386)
//#      define BSD_COMP
//#   endif /* SOLARIS_I386 */
#   include <sys/ioctl.h>
#   if HAVE_SYS_FILIO_H
#     include <sys/filio.h>
#   endif
#   include <errno.h>
#   include <signal.h>
static void sysCreate (
) __attribute__ ((constructor));

static void sysCreate (
) {
    signal(SIGPIPE, SIG_IGN);
}

#elif defined(MQ_IS_WIN32)
#   define SHUTDOWN_BOTH SD_BOTH
#   define WIN32_WSA(x)   WSA##x
#   define WIN32_socket(x)   x##socket
#   define WSAENOENT 9999
#else /* MQ_IS_WIN32 */
#   error not supported
#endif /* MQ_IS_WIN32 */

#define MQ_ERROR_S error
#define MSGQUE MqErrorGetMsgque(error)

/*****************************************************************************/
/*                                                                           */
/*                                sys_private                                */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE static enum MqErrorE
sSysMqErrorMsg (
  struct MqErrorS * const error,
  const char *prefix,
  const char * text
)
{
  char buffer[256];
  int num;
  char *str;

  if (error == MQ_ERROR_IGNORE) return MQ_ERROR;

  mq_snprintf(buffer,256,MqMessageText[MQ_ERROR_CAN_NOT],text);
#if defined(MQ_IS_POSIX)
  num = errno;
  str = strerror (num);
#elif defined(MQ_IS_WIN32)
  num = WSAGetLastError ();
  str = strerror (num);
#else
#   error not supported
#endif
  return MqErrorV (error, prefix, num, "%s -> ERRNO<%i> ERR<%s>", buffer, num, str);
}

#if defined(MQ_IS_POSIX)
#   define sSysGetErrorNum errno
#   define sSysSetErrorNum(i) errno=i
#elif defined(MQ_IS_WIN32)
#   define sSysGetErrorNum WSAGetLastError()
#   define sSysSetErrorNum(i) WSASetLastError(i)
#endif

/*****************************************************************************/
/*                                                                           */
/*                                sys_socket                                 */
/*                                                                           */
/*****************************************************************************/

#ifdef MQ_IS_WIN32

enum MqErrorE
SysWSAStartup (
  struct MqErrorS * const error
)
{
  WORD wVersionRequested;
  WSADATA wsaData;
  int err, max = 2, min = 2;

  wVersionRequested = MAKEWORD (max, min);

  err = WSAStartup (wVersionRequested, &wsaData);

  if (err != 0) {
    return MqErrorDbV (MQ_ERROR_WINSOCK, "WSAStartup", err);
  }

  /* Confirm that the WinSock DLL supports 2.2. */
  /* Note that if the DLL supports versions greater    */
  /* than 2.2 in addition to 2.2, it will still return */
  /* 2.2 in wVersion since that is the version we      */
  /* requested.                                        */

  if (LOBYTE (wsaData.wVersion) != max || HIBYTE (wsaData.wVersion) != min) {
    WSACleanup ();
    return MqErrorDbV (MQ_ERROR_WINSOCK, "WSAStartup", WSAGetLastError());
  }

  /* The WinSock DLL is acceptable. Proceed. */
  return MQ_OK;
}

enum MqErrorE
SysWSACleanup (
  struct MqErrorS * const error
)
{
  int err;
  if ((err = WSACleanup ()) != 0) {
    return MqErrorDbV (MQ_ERROR_WINSOCK, "WSACleanup", WSAGetLastError());
  }
  return MQ_OK;
}

#endif /* MQ_IS_WIN32 */


MQ_INLINE enum MqErrorE
SysAccept (
     struct MqErrorS * const error,     /* handle error */
     MQ_SOCK server,            /* the server handle */
     struct sockaddr *addr,     /* addr */
     socklen_t *addrlen,       /* addrlen */
     MQ_SOCK *client            /* OUT the client handle */
) {
next:
  if (unlikely ((*client = accept (server, addr, addrlen)) == INVALID_SOCKET)) {
    if (sSysGetErrorNum == WIN32_WSA (EWOULDBLOCK))
      goto next;
    return sSysMqErrorMsg (error, __func__, "accept");
  }

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysGetSockOpt (
  struct MqErrorS * const error,
  MQ_SOCK const socket,
  int const level,
  int const optname,
  MQ_sockopt_optval_T optval,
  socklen_t * const optlen
)
{
  if (unlikely (getsockopt (socket, level, optname, optval, optlen) == SOCKET_ERROR)) {
    return sSysMqErrorMsg (error, __func__, "getsockopt");
  }

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysSetSockOpt (
  struct MqErrorS * const error,
  MQ_SOCK const socket,
  int const level,
  int const optname,
  const MQ_sockopt_optval_T const optval,
  socklen_t const optlen
) {
  if (unlikely (setsockopt (socket, level, optname, optval, optlen) == SOCKET_ERROR)) {
    return sSysMqErrorMsg (error, __func__, "setsockopt");
  }

  return MQ_OK;
}

/*
MQ_INLINE enum MqErrorE
SysSetSockDebug(error, socket)
    struct MqErrorS *  const   error; 
    MQ_SOCK                 socket;
{
    int i=1;
    if (unlikely(setsockopt(socket, SOL_SOCKET, SO_DEBUG, (char*) &i, sizeof(int)) == SOCKET_ERROR)) {
	return sSysMqErrorMsg(error, __func__, "setsockopt");
    }

    return MQ_OK;
}  
*/

enum MqErrorE
SysConnect (
  struct MqErrorS * const error,
  MQ_SOCK const socket,
  struct sockaddr * const serv_addr,
  socklen_t const addrlen,
  MQ_TIME_T const timeout
)
{
  register struct MqS const * const msgque = MqErrorGetMsgque(error);
  MQ_TIME_T startT = time (NULL);
  do {
    sSysSetErrorNum(0);
    if (unlikely (connect (socket, serv_addr, addrlen) == SOCKET_ERROR)) {
      switch (sSysGetErrorNum) {
	// this is for -> performance
        case WIN32_WSA(EINPROGRESS):
	  break;
        case WIN32_WSA(EISCONN):
	  return MQ_OK;
	case 0:	// this seems to be special in SOLARIS
        case WIN32_WSA (EWOULDBLOCK):
          // case WIN32_WSA(EALREADY):
        case WIN32_WSA (EALREADY):
        case WIN32_WSA (ECONNABORTED):
        case WIN32_WSA (ECONNREFUSED):
        case WIN32_WSA (EINVAL):
        case WIN32_WSA (ENOENT):
	  // if the server is not available try as often as 'timeout' allows
          pUSleep (msgque, 200000L);
          break;
        default:
	  return sSysMqErrorMsg (error, __func__, "connect");
          break;
      }
    } else {
      // connection found -> continue with OK
      return MQ_OK;
    }
  }
  while ((time (NULL) - startT) <= timeout);

  return MqErrorDb (MQ_ERROR_TIMEOUT);
}

MQ_INLINE enum MqErrorE
SysSocket (
     struct MqErrorS * const error,     /* handle error */
     int domain,
     int type,
     int protocol,
     MQ_SOCK *sock              /* OUT */
) {
  if (unlikely ((*sock = socket (domain, type, protocol)) == INVALID_SOCKET)) {
    return sSysMqErrorMsg (error, __func__, "socket");
  }

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysSetAsync (
  struct MqErrorS * const error,
  MQ_SOCK socket
)
{
  u_long curMqState = 1;

  if (unlikely (WIN32_socket (ioctl) (socket, FIONBIO, &curMqState) == -1)) {
    return sSysMqErrorMsg (error, __func__, "ioctl");
  }

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysBind (
     struct MqErrorS * const error,     /* handle error */
     MQ_SOCK socket,
     const struct sockaddr *my_addr,
     const socklen_t addrlen
) {
  if (unlikely (bind (socket, my_addr, addrlen) == SOCKET_ERROR)) {
    return sSysMqErrorMsg (error, __func__, "bind");
  }

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysSelect (
  struct MqErrorS * const error,
  int n,
  fd_set * readfds,
  fd_set * writefds,
  struct timeval const * const timeout
)
{
  struct timeval tv = *timeout;
  sSysSetErrorNum(0);
  switch (select (n, readfds, writefds, NULL, &tv)) {
    case 0:
      return MQ_CONTINUE;
    case SOCKET_ERROR:
      switch (sSysGetErrorNum) {
	case WIN32_WSA (EINTR):
          return MQ_CONTINUE;
	case WIN32_WSA (EBADF): {
	  struct MqS * msgque = MqErrorGetMsgque (error);
          pIoShutdown (msgque->io);
	  pMqGetFirstParent(msgque)->exit_msgque = msgque;
	  return MQ_EXIT;
	}
	default:
          return sSysMqErrorMsg (error, __func__, "select");
      }
  }
  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysListen (
     struct MqErrorS * const error,     /* handle error */
     MQ_SOCK socket,
     int backlog
) {
  if (unlikely (listen (socket, backlog) == SOCKET_ERROR)) {
    return sSysMqErrorMsg (error, __func__, "listen");
  }

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysGetSockName (
  struct MqErrorS * const error,     /* handle error */
  MQ_SOCK socket,            /* the socket */
  struct sockaddr *sockaddr, /* the address package */
  socklen_t *sockaddrlen    /* the length of the address */
) {

  if (unlikely (getsockname (socket, sockaddr, sockaddrlen) == SOCKET_ERROR)) {
    return sSysMqErrorMsg (error, __func__, "getsockname");
  }

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysGetPeerName (
  struct MqErrorS * const error,     /* handle error */
  MQ_SOCK socket,            /* the socket */
  struct sockaddr *sockaddr, /* the address package */
  socklen_t *sockaddrlen    /* the length of the address */
) {
  if (unlikely (getpeername (socket, sockaddr, sockaddrlen) == SOCKET_ERROR)) {
    return sSysMqErrorMsg (error, __func__, "getpeername");
  }

  return MQ_OK;
}

enum MqErrorE
SysCloseSocket (
  struct MqErrorS * const error,
  MQ_BOL doShutdown,
  MQ_SOCK * hdlP
)
{
  struct MqS const * const msgque = MqErrorGetMsgque(error);
  const char * msg="";
  
  if (*hdlP < 0) return MQ_OK;
  if (doShutdown) {
    if (unlikely (shutdown (*hdlP, SHUTDOWN_BOTH) == SOCKET_ERROR)) {
      switch (sSysGetErrorNum) {
	case WIN32_WSA (ENOTCONN):
	case WIN32_WSA (EBADF):
	case WIN32_WSA (EINVAL):
	  break;
	default:
	  msg="shutdown (socket)";
	  goto error;
      }
    }
  }

  if (unlikely (WIN32_socket (close) (*hdlP) == SOCKET_ERROR)) {
    msg="close (socket)";
    goto error;
  }

  MqDLog(msgque,5,"close socket<%i>, shutdown<%i>\n", *hdlP, doShutdown);
  *hdlP *= -1;
  return MQ_OK;

error:
  MqDLog(msgque,5,"close socket<%i>, shutdown<%i>\n", *hdlP, doShutdown);
  *hdlP *= -1;
  return sSysMqErrorMsg (error, __func__, msg);
}

enum MqErrorE
SysSend (
  struct MqErrorS * const error,
  MQ_SOCK hdl,
  MQ_BIN buf,
  MQ_SIZE numBytes,
  int flags,
  MQ_TIME_T timeout
)
{
  MQ_SIZE ldata = 0;

  // send data from buf
  do {
    ldata = send (hdl, (const MQ_buf_T) buf, numBytes, flags);

    // check for errors
    if (unlikely (ldata == -1)) {
//MqDLogV(MqErrorGetMsgque(error),__func__,0,"ERROR sock<%i>, numBytes<%i>, str<%s>\n", 
//	    hdl, numBytes, strerror(errno));
      switch (sSysGetErrorNum) {
	case WIN32_WSA (EWOULDBLOCK): {
	  struct timeval tv = {(long) timeout, 0L};
	  fd_set fds;
	  // now wait until the socket become send-able
	  FD_ZERO(&fds);
	  FD_SET(hdl, &fds);
	  switch (SysSelect (error, (hdl+1), NULL, &fds, &tv)) {
	    case MQ_OK:
	      break;
	    case MQ_CONTINUE:
	      return MqErrorDb (MQ_ERROR_TIMEOUT);
	    case MQ_ERROR:
	      pIoShutdown (MqErrorGetMsgque(error)->io);
	      return MqErrorStack (error);
	    case MQ_EXIT:
	      return MQ_EXIT;
	  }
	  ldata = 0;
	  break;
	}
	case WIN32_WSA (EBADF): {
	  struct MqS * msgque = MqErrorGetMsgque (error);
	  pIoShutdown (msgque->io);
	  pMqGetFirstParent(msgque)->exit_msgque = msgque;
	  return MQ_EXIT;
	}
        default:
          pIoShutdown (MqErrorGetMsgque(error)->io);
          return sSysMqErrorMsg (error, __func__, "send");
      }
    }
    buf += ldata;
    numBytes -= ldata;
  }
  while (numBytes > 0);
  return MQ_OK;
}

enum MqErrorE
SysRecv (
  struct MqErrorS * const error,
  MQ_SOCK const hdl,
  MQ_BIN buf,
  MQ_SIZE numBytes,
  MQ_SIZE * const newSize,
  MQ_TIME_T timeout
)
{
  int const flags = 0;
  register MQ_SIZE ldata = 0;
  *newSize = 0;

  // recv data in buf
  do {
    ldata = recv (hdl, (MQ_buf_T) buf, numBytes, flags);

    // check for errors
    if (unlikely (ldata <= 0)) {
	if (ldata == -1) {
//MqDLogV(MqErrorGetMsgque(error),__func__,0,"ERROR sock<%i>, numBytes<%i>, str<%s>\n", 
//	  hdl, numBytes, strerror(errno));
	  switch (sSysGetErrorNum) {
	    case WIN32_WSA (EWOULDBLOCK): {
	      struct timeval tv = {(long)timeout, 0L};
	      fd_set fds;
	      FD_ZERO(&fds);
	      FD_SET(hdl, &fds);
	      // now wait until the socket become recv-able
	      switch (SysSelect (error, (hdl+1), &fds, NULL, &tv)) {
		case MQ_OK:
		  break;
		case MQ_CONTINUE:
		  return MqErrorDb (MQ_ERROR_TIMEOUT);
		case MQ_ERROR:
		  pIoShutdown (MqErrorGetMsgque(error)->io);
		  return MqErrorStack(error);
		case MQ_EXIT:
		  return MQ_EXIT;
	      }
	      ldata = 0;
	      break;
	    }
	    default:
	      pIoShutdown (MqErrorGetMsgque(error)->io);
	      return sSysMqErrorMsg (error, __func__, "recv");
	  }
	} else if (ldata == 0) {
	  struct MqS * msgque = MqErrorGetMsgque (error);
	  pIoShutdown (msgque->io);
	  pMqGetFirstParent(msgque)->exit_msgque = msgque;
	  return MQ_EXIT;
	}
    }

    buf += ldata;
    numBytes -= ldata;
    *newSize += ldata;
  }
  while (numBytes > 0);

  return MQ_OK;
}

/*****************************************************************************/
/*                                                                           */
/*                                sys_tcp                                    */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE void
SysFreeAddrInfo(
    struct addrinfo **res		///< [in,out] the linked list of addrinfo data freed and set pointer to NULL
) {
  if (!*res) return;
  freeaddrinfo(*res);
  *res = NULL;
}

MQ_INLINE enum MqErrorE
SysGetAddrInfo(
    struct MqErrorS * const error,	///< [in] handle error
    const char *node,			///< [in] name of the host or \c NULL
    const char *service,		///< [in] name of the port or \c NULL
    const struct addrinfo *hints,	///< [in] helper to restrict the amount of return values
    struct addrinfo **res		///< [out] the linked list of the results
) {
  int ret;
  if ((ret = getaddrinfo(node,service,hints,res)) != 0) {
    return MqErrorV (error, __func__, ret, "%s -> ERRNO<%i> ERR<%s>", "getaddrinfo", ret, gai_strerror(ret));
  }
  return MQ_OK;
}

enum MqErrorE
SysGetTcpInfo(
  struct MqErrorS * const error,	    ///< [in] handle error
  struct addrinfo const * const addrinfo,   ///< [in] address information returned by \c getaddrinfo
  const char **host,			    ///< [out] the hostname from addrinfo
  MQ_INT * port				    ///< [out] the port number from addrinfo
) {
  // 1. get sockaddr_in
  struct sockaddr_in * addr = (struct sockaddr_in *) addrinfo->ai_addr;
  // 2. check
  if (addrinfo->ai_family != AF_INET)
    return MqErrorDbV (MQ_ERROR_CAN_NOT, "AF_INET");
  // 3. get host name
  if (unlikely ((*host = inet_ntoa (addr->sin_addr)) == NULL)) {
    return MqErrorDbV (MQ_ERROR_CAN_NOT, "inet_ntoa");
  }
  // 4. get port number
  *port = ntohs (addr->sin_port);

  return MQ_OK;
}

MQ_INLINE enum MqErrorE
SysInetAton (
     struct MqErrorS * const error,	    ///< handle error
     const MQ_STR host,			    ///< the hostname in numbers-and-dots notation
     struct in_addr *in_addr		    ///< OUT: the adress entry struct
) {
#if defined(MQ_IS_POSIX)
  if (unlikely (inet_aton (host, in_addr) == 0))
#elif defined(MQ_IS_WIN32)
  if (unlikely ((in_addr->s_addr = inet_addr (host)) == INADDR_NONE))
#else
#   error not supported
#endif
  {
    return MqErrorV (error, __func__, -1, "unknown host-address '%s'", host);
  }

  return MQ_OK;
}

enum MqErrorE
SysSocketPair (
  struct MqErrorS * const error,	    ///< [in] handle error
  int socks[]				    ///< [out] the result from socketpair
) {
#if defined(HAVE_SOCKETPAIR)
  int oldflags;
  sSysSetErrorNum(0);
  if (unlikely (socketpair (AF_UNIX, SOCK_STREAM, 0, socks) == -1)) {
    return sSysMqErrorMsg (error, __func__, "socketpair");
  }

  if (unlikely ((oldflags = fcntl (socks[0], F_GETFD, 0)) == -1)) {
    return sSysMqErrorMsg (error, __func__, "fcntl F_GETFD");
  }
  oldflags |= FD_CLOEXEC;
  if (unlikely (fcntl (socks[0], F_SETFD, oldflags) == -1)) {
    return sSysMqErrorMsg (error, __func__, "fcntl F_SETFD");
  }

  return MQ_OK;
#else
/* socketpair.c
 * Copyright 2007 by Nathan C. Myers <ncm@cantrip.org>; all rights reserved.
 * This code is Free Software.  It may be copied freely, in original or 
 * modified form, subject only to the restrictions that (1) the author is
 * relieved from all responsibilities for any use for any purpose, and (2)
 * this copyright notice must be retained, unchanged, in its entirety.  If
 * for any reason the author might be held responsible for any consequences
 * of copying or use, license is withheld.  
 */

/* dumb_socketpair:
 *   If make_overlapped is nonzero, both sockets created will be usable for
 *   "overlapped" operations via WSASend etc.  If make_overlapped is zero,
 *   socks[0] (only) will be usable with regular ReadFile etc., and thus 
 *   suitable for use as stdin or stdout of a child process.  Note that the
 *   sockets must be closed with closesocket() regardless.
 */
    struct sockaddr_in addr;
    MQ_SOCK listener;
    socklen_t addrlen = sizeof(addr);

    socks[0] = socks[1] = INVALID_SOCKET;
    MqErrorCheck (SysSocket(error,AF_INET,SOCK_STREAM,0,&listener));

    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(0x7f000001);
    addr.sin_port = 0;

    MqErrorCheck (SysBind(error,listener, (const struct sockaddr*) &addr, sizeof(addr)));
    MqErrorCheck (SysGetSockName(error, listener, (struct sockaddr*) &addr, &addrlen));

    do {
	MqErrorCheck (SysListen(error,listener,1));
	MqErrorCheck (SysSocket(error,AF_INET,SOCK_STREAM,0,&socks[0]));
	MqErrorCheck (SysConnect(error,socks[0],(struct sockaddr*const) &addr, sizeof(addr),1));
	MqErrorCheck (SysAccept(error,listener,NULL,NULL,&socks[1]));
	MqErrorCheck (SysCloseSocket(error,MQ_NO,&listener));
        return MQ_OK;
    } while (0);
    return sSysMqErrorMsg (error, __func__, "socketpair");

error:
  MqErrorCheck (SysCloseSocket(MQ_ERROR_IGNORE,MQ_NO,&listener));
  return MqErrorStack (error);
#endif
}

