/**
 *  \file       csmsgque/context.cs
 *  \brief      \$Id: context.cs 106 2009-06-25 16:33:14Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 106 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/*****************************************************************************/
/*                                                                           */
/*                                context                                    */
/*                                                                           */
/*****************************************************************************/

using System;
using System.Runtime.InteropServices;

namespace csmsgque {

  public abstract partial class MqS
  {

    private static McErrorE ChildCreate (IntPtr configP, IntPtr argv, IntPtr alfa, out IntPtr msgqueP)
    {
      MqS context;
      McErrorE ret;

      // this is the default for:
      //  1. MQ_STATUS_IS_INITIAL
      //  2. MQ_STATUS_IS_DUP & MQ_STATUS_IS_FORK
      IntPtr contextP = McConfigGetMyCtx(configP);
      IntPtr parentP  = McConfigGetParent(configP);

      // 1. setup the SERVER+CHILD context using PARENT data
      if (parentP != IntPtr.Zero && contextP == IntPtr.Zero) {

	// create an object using the default constructor
	MqS parent = (MqS)GCHandle.FromIntPtr(McConfigGetSelfFromManagedContextM(parentP)).Target;
	context = (MqS) Activator.CreateInstance(parent.GetType());

	// create "managed" context using the new created object as input
	McConfigInitManagedContext(configP, (IntPtr) GCHandle.Alloc(context));
      }

      // 2. create the Message-Queue
      ret = McCreate (configP, argv, alfa, out msgqueP);

//McLogData(msgqueP, "ChildCreate");

      // 3. an existing msgque object
      context = (MqS)GCHandle.FromIntPtr(McConfigGetSelfFromManagedContextM(msgqueP)).Target;
      context.msgqueP = msgqueP;

      // on error jump to function-exit
      if (ret == McErrorE.MQ_ERROR) goto error;

      // check for wrong arguments
      if (McCheckForLeftOverArguments(msgqueP, ref argv) == McErrorE.MQ_ERROR) goto error;

      // 5. init the new object
      if (context.ConfigGetIsServer()) {
	switch (ProcCall(msgqueP, ((IServer)context).Config, null)) {
	  case McErrorE.MQ_OK:           break;
	  case McErrorE.MQ_ERROR:        goto error;
	  case McErrorE.MQ_CONTINUE:     return McErrorE.MQ_CONTINUE;
	  case McErrorE.MQ_EXIT:         return McErrorE.MQ_EXIT;
	}
      }

  /*
      if (msgque.config.debug >= 2 && msgque.config.parent != null)
	McDLogV(msgqueP,"ChildCreate",2,"CREATE context<%p>, msgque<%p>\n",context,msgque);
  */

      // everything is OK
      return McErrorE.MQ_OK;

error:
      return McErrorStack (msgqueP);
    }

    private static void ChildDelete (ref IntPtr msgqueP)
    {
      // return if the object was allread deleted
      if (msgqueP == IntPtr.Zero) return;

      // get global reference from msgqueP
      GCHandle GcH = GCHandle.FromIntPtr(McConfigGetSelfFromManagedContextM(msgqueP));
      MqS context = (MqS) GcH.Target;

      // unset the 'hdl'
      context.msgqueP = null;

      // free the msgque object
      McDelete(ref msgqueP);
      
      // unlock my global references
      GcH.Free();
    }

    private static McErrorE ParentCreate (IntPtr configP, IntPtr argv, IntPtr alfa, out IntPtr msgqueP)
    {
      // this is the default for:
      //  1. MQ_STATUS_IS_INITIAL
      //  2. MQ_STATUS_IS_DUP & MQ_STATUS_IS_FORK
      //struct JavaContextS * context = (struct JavaContextS *) config->myCtx;

      // The thread get a new context
      if (McConfigGetIsDupAndThread(configP)) {
        // create "managed" context using the new created object as input
        McConfigInitManagedContext(configP, 
	  (IntPtr) GCHandle.Alloc (
	    (MqS) Activator.CreateInstance(
	      GCHandle.FromIntPtr(
		McConfigGetSelfFromManagedContext(configP)
	      ).Target.GetType()
	    )
	  )
	);
      }

      // create the server-context
      if (ChildCreate(configP, argv, alfa, out msgqueP) == McErrorE.MQ_ERROR) goto error;

      //McDLog (msgque, 2, "CREATE context<%p>, msgque<%p>\n", context, msgque);

      // everything is OK
      return McErrorE.MQ_OK;

error:
      return McErrorStack(msgqueP);
    }

    private static void ParentDelete (ref IntPtr msgqueP)
    {
      // return if the object was allread deleted
      if (msgqueP == IntPtr.Zero) return;
      
      // finally use child-delete to clean the context
      ChildDelete(ref msgqueP);
    }

  } // END - class "MqS"
} // END - namespace "csmsgque"


