/**
 *  \file       csmsgque/csmsgque.cs
 *  \brief      \$Id: csmsgque.cs 108 2009-06-26 11:13:24Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 108 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

using System;
using System.Text;
using System.Collections;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Security.Permissions;

[assembly:AssemblyTitleAttribute("csmsgque")]
[assembly:AssemblyDescriptionAttribute("LibMsgque language binding for C#")]
[assembly:AssemblyConfigurationAttribute("")]
[assembly:AssemblyCompanyAttribute("IPN")]
[assembly:AssemblyProductAttribute("csmsgque")]
[assembly:AssemblyCopyrightAttribute("Copyright (c) IPN 2009")]
[assembly:AssemblyTrademarkAttribute("")]
[assembly:AssemblyCultureAttribute("")]

[assembly:AssemblyVersionAttribute("3.4.*")]
[assembly:ComVisible(false)]
[assembly:CLSCompliantAttribute(true)]
[assembly:SecurityPermission(SecurityAction.RequestMinimum, UnmanagedCode = true)]


/// \brief \libmsgque interface for C#
namespace csmsgque {

#if _DEBUG
  public class DEBUG {
    private static void SF(string id) {
      System.Diagnostics.Process thisProc = System.Diagnostics.Process.GetCurrentProcess();
      System.Diagnostics.StackFrame sf = new System.Diagnostics.StackFrame(2);
      Console.WriteLine(thisProc.Id + ":" + id + " -> " + sf.GetMethod() + " - " + System.Threading.Thread.CurrentThread.ManagedThreadId);
    }
    public static void P(string name, IntPtr id) {
      System.Diagnostics.Process thisProc = System.Diagnostics.Process.GetCurrentProcess();
      System.Diagnostics.StackFrame sf = new System.Diagnostics.StackFrame(1);
      Console.WriteLine(thisProc.Id + ":" + name + " = 0x{0:x}" + 
	  " -> " + sf.GetMethod().Name + " - " + System.Threading.Thread.CurrentThread.ManagedThreadId, (int) id);
    }
    public static void M0() { SF("M0000000000000"); }
    public static void M1() { SF("M1111111111111"); }
    public static void M2() { SF("M2222222222222"); }
    public static void M3() { SF("M3333333333333"); }
    public static void M4() { SF("M4444444444444"); }
    public static void M5() { SF("M5555555555555"); }
    public static void M6() { SF("M6666666666666"); }
  }
#endif // _DEBUG

  /// \brief Interface required to setup a \e server
  ///
  /// A \csmsgque server application have to implement the #csmsgque.IServer interface.
  public interface IServer {

    /// This function is used to configure a new context and act like
    /// constructor. A context is created for every new incomming connection
    /// request and is used to provide context specific services
    /// (#csmsgque.MqS.ServiceCreate) or to init context specific data.
    void Config();
  }

  /// \brief Interface required to setup a \e filter
  ///
  /// A \csmsgque filter application have to implement the #csmsgque.IFilter interface.
  public interface IFilter : IServer {

    /// \api #MqConfigS::fFTR
    void FTR();

    /// \api #MqConfigS::fEOF
    void EOF();
  }

  /// \brief Interface required to setup a service or a callback
  public interface IService {

    /// \brief function to handle the service or callback request
    void Service(MqS ctx);
  }

  /// Main \libmsgque class
  public abstract partial class MqS
  {

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqInitCreate")]
    private static extern IntPtr McInitCreate();

    /// \api #MqInitCreate
    protected static void Init(params string[] argv) {
      IntPtr initB = McInitCreate();
      foreach (string s in argv) {
	McBufferLAppendC(initB, s);
      }
    }

    static MqS() {
      IntPtr initB = McInitCreate();
      if(Type.GetType ("Mono.Runtime") != null) McBufferLAppendC(initB, "mono");
      McBufferLAppendC(initB, Assembly.GetEntryAssembly().Location);
    }

    /// \api #MqConfigS::isSilent
    protected static bool isSilent  = false;
    /// \api #MqConfigS::isString
    protected static bool isString  = false;
    /// \api #MqConfigS::name
    protected static string name    = "csharp";
    /// \api #MqConfigS::debug
    protected static int debug      = 0;
    /// \api #MqConfigS::buffersize
    protected static int buffersize = 4094;

    private const CallingConvention MSGQUE_CC = CallingConvention.Cdecl;
    private const CharSet MSGQUE_CS = CharSet.Ansi;
    private const string MSGQUE_DLL = "msgque";
    //private const string MSGQUE_DLL = "ccmsgque";
    //private const string MSGQUE_DLL = "../ccmsgque/.libs/ccmsgque.dll";
    //private const string MSGQUE_DLL = "../src/.libs/msgque.dll";

    private IntPtr? msgqueP = null;

    /// used to detect nested calles in #ProcCall
    public int callLevel = 0;

    /// save delegate to protect again deleting
    private McCreateF		fChildCreate  = ChildCreate;
    private McDeleteF		fChildDelete  =	ChildDelete;
    private McCreateF		fParentCreate = ParentCreate;
    private McDeleteF		fParentDelete = ParentDelete;
    private McTokenF		fFTR	      = ftrcallC;
    private McTokenDataFreeF	FTRfree	      = srvfree;
    private McTokenF		fEOF	      = ftrcallC;
    private McTokenDataFreeF	EOFfree	      = srvfree;

  /*****************************************************************************/
  /*                                                                           */
  /*                         default enums and types                           */
  /*                                                                           */
  /*****************************************************************************/

    private enum McErrorE :int {
      MQ_OK,
      MQ_ERROR,
      MQ_CONTINUE,
      MQ_EXIT
    };

    private enum MQ_BOL :byte {
      MQ_NO     = 0,
      MQ_YES    = 1
    };

    /// \api #MqWaitOnEventE flags for #MqProcessEvent
    public enum WAIT {
      NO      = 0,
      ONCE    = 1,
      FOREVER = 2,
    };

    [StructLayout(LayoutKind.Sequential, CharSet=MSGQUE_CS)]
    private struct McConfigS {
      public IntPtr		myCtx;
      public MQ_BOL		isSilent;
      public MQ_BOL		isString;
      public MQ_BOL		isServer;
      public int		statusIs;
      public int		startAs;
      public IntPtr		name;
      public int		debug;
      public long		timeout;
      public int		buffersize;
      public IntPtr		parent;
      public McCreateF		fChildCreate;
      public McDeleteF		fChildDelete;
      public McCreateF		fParentCreate;
      public McDeleteF		fParentDelete;
      public McExitF		fParentExit;
      public MQ_BOL		isFilter;
      public McTokenF		fFTR;
      public IntPtr		FTRdata;
      public McTokenDataFreeF	FTRfree;
      public McTokenF		fEOF;
      public IntPtr		EOFdata;
      public McTokenDataFreeF	EOFfree;
      public McEventF		fEvent;
      public IntPtr		fHelp;
      public MQ_BOL		ignoreThread;
      public MQ_BOL		ignoreSpawn;
      public MQ_BOL		ignoreFork;
      public IntPtr		master;
      public int		master_id;
    }

  /*****************************************************************************/
  /*                                                                           */
  /*                            context management                             */
  /*                                                                           */
  /*****************************************************************************/

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqCreate")]
    private static extern McErrorE McCreate([In]IntPtr config, [In]IntPtr argv, [In]IntPtr alfa, [Out] out IntPtr msgqueP);

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqDelete")]
    private static extern void McDelete(ref IntPtr msgqueP);

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqExit")]
    private static extern void McExit([In] IntPtr msgqueP);

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqProcessEvent")]
    private static extern McErrorE McProcessEvent([In]IntPtr msgque, [In]long timeout, [In]int flag);

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqCheckForLeftOverArguments")]
    private static extern McErrorE McCheckForLeftOverArguments([In]IntPtr msgque, ref IntPtr argv);

/*
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqLogData")]
    private static extern void McLogData([In]IntPtr msgqueP, [In]string prefix);
*/

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqGetConfigS")]
    private static extern IntPtr McGetConfigS([In]IntPtr msgqueP);

    /// \api #MqProcessEvent, wait for \e timeout seconds and process event or raise an error
    public void ProcessEvent ( long timeout, WAIT wait) {
      ErrorMqToCsWithCheck(McProcessEvent(msgqueP.Value, timeout, (int)wait));
    }
    /// \api #MqProcessEvent, wait application default time
    public void ProcessEvent (WAIT wait) {
      ErrorMqToCsWithCheck(McProcessEvent(msgqueP.Value, -1, (int)wait));
    }
    /// \api #MqProcessEvent, don't wait just check for an event
    public void ProcessEvent () {
      ErrorMqToCsWithCheck(McProcessEvent(msgqueP.Value, -1, (int)WAIT.NO));
    }

    private static McErrorE ftrcallC (IntPtr msgqueP, IntPtr data)
    {
      // call the service
      if (ProcCall(msgqueP, (Callback) GCHandle.FromIntPtr(data).Target, null) == McErrorE.MQ_OK) 
	return McErrorE.MQ_OK;

      return McErrorStack(msgqueP);
    }

    // used to handle constructor error
    private void MqS_ERROR (ref IntPtr msgqueP) {
      if (ConfigGetIsServer()) {
	McExit(msgqueP);
      } else {
	// set error-message
	ApplicationException ex = new System.ApplicationException(
	  Marshal.PtrToStringAnsi(McErrorGetText(msgqueP)));
	McErrorReset(msgqueP);
	// allways delete the msgque object because this is the last chance to do it
	// !!attention!! never use "?ParentDelete" ot "?ChildeDelete" directly,
	// "McDelete" will call the functions from above
	McDelete(ref msgqueP);
	throw ex;
      }
    }

    /// \brief constructor used to create a \e parent object
    protected MqS(params string[] argv) {
//DEBUG.M0();
      IntPtr msgqueP;
      IntPtr configP = McConfigInit();
      IntPtr largv = McBufferLCreate(10);
      IntPtr lalfa = McBufferLCreate(10);
      IntPtr tgt = largv;

      // Fill config
      McConfigS config = (McConfigS) Marshal.PtrToStructure(configP, typeof(McConfigS));

      config.isSilent = (isSilent == true ? MQ_BOL.MQ_YES : MQ_BOL.MQ_NO);
      config.isString = (isString == true ? MQ_BOL.MQ_YES : MQ_BOL.MQ_NO);
      config.isServer = (this is IServer ? MQ_BOL.MQ_YES : MQ_BOL.MQ_NO);
      config.isFilter = (this is IFilter ? MQ_BOL.MQ_YES : MQ_BOL.MQ_NO);
      config.ignoreFork = MQ_BOL.MQ_YES;
      config.fChildCreate = fChildCreate;
      config.fChildDelete = fChildDelete;
      config.fParentCreate = fParentCreate;
      config.fParentDelete = fParentDelete;

      if (this is IFilter) {
	IFilter f = (IFilter) this;
	config.fFTR        = fFTR;
	config.FTRdata     = (IntPtr) GCHandle.Alloc((Callback)f.FTR);
	config.FTRfree     = FTRfree;
	config.fEOF        = fEOF;
	config.EOFdata     = (IntPtr) GCHandle.Alloc((Callback)f.EOF);
	config.EOFfree     = EOFfree;
      }

      //config.fParentExit = ParentExit;
      Marshal.StructureToPtr(config, configP, true);
      McConfigInitName(configP, name);
      McConfigInitManagedContext(configP, (IntPtr) GCHandle.Alloc(this));

      // fill the argv/alfa
      McBufferLAppendC(tgt, Assembly.GetExecutingAssembly().Location);
      foreach (string a in argv) {
	if (a[0] == '@') {
	  tgt = lalfa;
	  continue;
	}
	McBufferLAppendC(tgt, a);
      }

      // 7. create Context
      if (ParentCreate(configP, largv, lalfa, out msgqueP) == McErrorE.MQ_ERROR)
	MqS_ERROR(ref msgqueP);
    }

    /// \brief constructor used to create a \e child object
    protected MqS(MqS parent, params string[] argv) {
//DEBUG.M0();
      IntPtr msgqueP;
      IntPtr configP = McConfigDup(McGetConfigS(parent.msgqueP.Value));

      // Fill config
      McConfigInitParent(configP, parent.msgqueP.Value);
      McConfigInitManagedContext(configP, (IntPtr) GCHandle.Alloc(this));

      // fill the argv/alfa
      IntPtr largv = McBufferLCreate(10);
      McBufferLAppendC(largv, Assembly.GetExecutingAssembly().Location);
      foreach (string a in argv) {
	McBufferLAppendC(largv, a);
      }

      // 7. create Context
      if (ChildCreate(configP, largv, IntPtr.Zero, out msgqueP) == McErrorE.MQ_ERROR)
	MqS_ERROR(ref msgqueP);
    }

    /// \brief constructor used to create a \e worker object
    protected MqS(MqS master, int master_id, params string[] argv) {
//DEBUG.M0();
      IntPtr msgqueP;
      IntPtr configP = McConfigDup(McGetConfigS(master.msgqueP.Value));

      // Fill config
      McConfigInitMaster(configP, master.msgqueP.Value, master_id);
      McConfigInitManagedContext(configP, (IntPtr) GCHandle.Alloc(this));

      // fill the argv/alfa
      IntPtr largv = McBufferLCreate(10);
      IntPtr lalfa = McBufferLCreate(10);
      IntPtr tgt = largv;
      McBufferLAppendC(largv, Assembly.GetExecutingAssembly().Location);
      foreach (string a in argv) {
	if (a[0] == '@') {
	  tgt = lalfa;
	  continue;
	}
	McBufferLAppendC(tgt, a);
      }

      // 7. create Context
      if (ParentCreate(configP, largv, lalfa, out msgqueP) == McErrorE.MQ_ERROR) {
	MqS_ERROR(ref msgqueP);
      }
    }

    /// \brief default constructor used for the object factory
    protected MqS() {
      msgqueP = null;
    }

    /// \api #MqDelete
    public void Delete() {
      if(msgqueP == null) return;
      IntPtr msgque = msgqueP.Value;
      McDelete (ref msgque);
    }

  } // END - class "MqS"
} // END - namespace "csmsgque"

