/**
 *  \file       csmsgque/misc.cs
 *  \brief      \$Id: misc.cs 50 2009-06-07 12:58:15Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 50 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/*****************************************************************************/
/*                                                                           */
/*                                   misc                                    */
/*                                                                           */
/*****************************************************************************/

using System;
using System.Runtime.InteropServices;

namespace csmsgque {

  public abstract partial class MqS
  {
    private static McErrorE ProcCall (IntPtr msgqueP, Callback callC, IService callI)
    {
      MqS context = (MqS)GCHandle.FromIntPtr(McConfigGetSelfFromManagedContextM(msgqueP)).Target;

      // clean libmsgque error
      McErrorReset(msgqueP);

      // call the function
      try {

	context.callLevel++;
	if (callC != null) {
	  callC();
	} else {
	  callI.Service(context);
	}

      } catch (Exception ex) {

	// is the EROOR from C# or MQ
	if (McErrorGetCode (msgqueP) == McErrorE.MQ_ERROR) {
	  // error is from MQ let upper code decide to handle this
	  return McErrorE.MQ_CONTINUE;
	}

	// ok error is from C#
	// 1. get detail Message
	string message = ex.ToString();
	if (message.Length == 0) {
	// 2. get StackTrace
	  message = ex.StackTrace;
	}
	if (message.Length == 0) {
	// 3. unknown error
	  message = "unknown Exception";
	}
	// 4. Add C#-error-message to MQ error
	McErrorC(msgqueP, "ProcCall", -1, message);

	return McErrorE.MQ_ERROR;

      } finally {

	context.callLevel--;
      }

      // everything is OK
      return McErrorE.MQ_OK;
    }

  } // END - class "MqS"
} // END - namespace "csmsgque"

