/**
 *  \file       csmsgque/read.cs
 *  \brief      \$Id: read.cs 46 2009-06-04 14:01:00Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 46 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/*****************************************************************************/
/*                                                                           */
/*                                   read                                    */
/*                                                                           */
/*****************************************************************************/

using System;
using System.Runtime.InteropServices;

namespace csmsgque {

  public abstract partial class MqS
  {

    // PRIVAT

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadYM")]
    private static extern McErrorE McReadY(IntPtr read, out byte outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadOM")]
    private static extern McErrorE McReadO(IntPtr read, out byte outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadSM")]
    private static extern McErrorE McReadS(IntPtr read, out short outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadIM")]
    private static extern McErrorE McReadI(IntPtr read, out int outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadFM")]
    private static extern McErrorE McReadF(IntPtr read, out float outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadWM")]
    private static extern McErrorE McReadW(IntPtr read, out long outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadDM")]
    private static extern McErrorE McReadD(IntPtr read, out double outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadBM")]
    private static extern McErrorE McReadB(IntPtr read, out IntPtr outV, out int sizeV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadUM")]
    private static extern McErrorE McReadU(IntPtr read, out IntPtr outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadCM")]
    private static extern McErrorE McReadC(IntPtr read, out IntPtr outV);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadL_STARTM")]
    private static extern McErrorE McReadL_START(IntPtr read, IntPtr buf);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadL_ENDM")]
    private static extern void McReadL_END(IntPtr read);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadGetNumItemsM")]
    private static extern int McReadGetNumItems(IntPtr read);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadItemExistsM")]
    private static extern byte McReadItemExists(IntPtr read);
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqReadUndoM")]
    private static extern McErrorE McReadUndo(IntPtr read);

    // PUBLIC

    /// \api #MqReadY
    public byte ReadY() {
      byte outV;
      ErrorMqToCsWithCheck(McReadY(msgqueP.Value, out outV));
      return outV;
    }

    /// \api #MqReadO
    public bool ReadO() {
      byte outV;
      ErrorMqToCsWithCheck(McReadO(msgqueP.Value, out outV));
      return (outV == (byte) MQ_BOL.MQ_YES ? true : false);
    }

    /// \api #MqReadS
    public short ReadS() {
      short outV;
      ErrorMqToCsWithCheck(McReadS(msgqueP.Value, out outV));
      return outV;
    }

    /// \api #MqReadI
    public int ReadI() {
      int outV;
      ErrorMqToCsWithCheck(McReadI(msgqueP.Value, out outV));
      return outV;
    }

    /// \api #MqReadF
    public float ReadF() {
      float outV;
      ErrorMqToCsWithCheck(McReadF(msgqueP.Value, out outV));
      return outV;
    }

    /// \api #MqReadW
    public long ReadW() {
      long outV;
      ErrorMqToCsWithCheck(McReadW(msgqueP.Value, out outV));
      return outV;
    }

    /// \api #MqReadD
    public double ReadD() {
      double outV;
      ErrorMqToCsWithCheck(McReadD(msgqueP.Value, out outV));
      return outV;
    }

    /// \api #MqReadB
    public byte[] ReadB() {
      IntPtr outV;
      int size;
      ErrorMqToCsWithCheck(McReadB(msgqueP.Value, out outV, out size));
      byte[] outB = new byte[size];
      Marshal.Copy(outV,outB,0,size);
      return outB;
    }

    /// \api #MqReadU
    public IntPtr ReadU() {
      IntPtr outV;
      ErrorMqToCsWithCheck(McReadU(msgqueP.Value, out outV));
      return outV;
    }

    /// \api #MqReadC
    public string ReadC() {
      IntPtr outV;
      ErrorMqToCsWithCheck(McReadC(msgqueP.Value, out outV));
      return Marshal.PtrToStringAnsi(outV);;
    }

    /// \api #MqReadL_START
    public void ReadL_START() {
      McReadL_START(msgqueP.Value, IntPtr.Zero);
    }

    /// \api #MqReadL_END
    public void ReadL_END() {
      McReadL_END(msgqueP.Value);
    }

    /// \api #MqReadGetNumItems
    public int ReadGetNumItems() {
      return McReadGetNumItems(msgqueP.Value);
    }

    /// \api #MqReadItemExists
    public bool ReadItemExists() {
      return (McReadItemExists(msgqueP.Value) == (byte) MQ_BOL.MQ_YES ? true : false);
    }

    /// \api #MqReadUndo
    public void ReadUndo() {
      ErrorMqToCsWithCheck(McReadUndo(msgqueP.Value));
    }

  } // END - class "MqS"
} // END - namespace "csmsgque"










