/**
 *  \file       csmsgque/send.cs
 *  \brief      \$Id: send.cs 73 2009-06-12 11:35:53Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 73 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/*****************************************************************************/
/*                                                                           */
/*                                   send                                    */
/*                                                                           */
/*****************************************************************************/

using System;
using System.Runtime.InteropServices;

namespace csmsgque {

public abstract partial class MqS
{

  // PRIVAT

  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendSTARTM")]
  private static extern void McSendSTART(IntPtr msgqueP);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendENDM")]
  private static extern McErrorE McSendEND(IntPtr msgqueP, string token);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendEND_AND_WAITM")]
  private static extern McErrorE McSendEND_AND_WAIT(IntPtr msgqueP, string token, long timeout);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendEND_AND_CALLBACKM")]
  private static extern McErrorE McSendEND_AND_CALLBACK(IntPtr msgqueP, string token, 
	    McTokenF callback, IntPtr data, McTokenDataFreeF datafreeF);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendRETURNM")]
  private static extern McErrorE McSendRETURN(IntPtr msgqueP);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendERRORM")]
  private static extern McErrorE McSendERROR(IntPtr msgqueP);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendFTRM")]
  private static extern McErrorE McSendFTR(IntPtr msgqueP, long timeout);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendEOFM")]
  private static extern McErrorE McSendEOF(IntPtr msgqueP, long timeout);

  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendYM")]
  private static extern void McSendY(IntPtr msgque, byte val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendOM")]
  private static extern void McSendO(IntPtr msgque, byte val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendSM")]
  private static extern void McSendS(IntPtr msgque, short val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendIM")]
  private static extern void McSendI(IntPtr msgque, int val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendFM")]
  private static extern void McSendF(IntPtr msgque, float val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendWM")]
  private static extern void McSendW(IntPtr msgque, long val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendDM")]
  private static extern void McSendD(IntPtr msgque, double val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendBM")]
  private static extern void McSendB(IntPtr msgque, byte[] val, int size);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendUM")]
  private static extern void McSendU(IntPtr msgque, IntPtr val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendCM")]
  private static extern void McSendC(IntPtr msgque, string val);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendL_STARTM")]
  private static extern void McSendL_START(IntPtr msgque);
  [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqSendL_ENDM")]
  private static extern void McSendL_END(IntPtr msgque);
  
  // protect function-pointer to ignore garbage collection
  private McTokenF fsendcallC = sendcallC;
  private McTokenF fsendcallI = sendcallI;

  // PUBLIC

  /// \api #MqSendSTART
  public void SendSTART() {
    McSendSTART(msgqueP.Value);
  }

  /// \api #MqSendEND
  public void SendEND(string token) {
    ErrorMqToCsWithCheck(McSendEND(msgqueP.Value, token));
  }

  /// \api #MqSendEND_AND_WAIT
  public void SendEND_AND_WAIT(string token, long timeout) {
    ErrorMqToCsWithCheck(McSendEND_AND_WAIT(msgqueP.Value, token, timeout));
  }

  /// \api #MqSendEND_AND_WAIT
  public void SendEND_AND_WAIT(string token) {
    ErrorMqToCsWithCheck(McSendEND_AND_WAIT(msgqueP.Value, token, -1));
  }

  private static McErrorE sendcallC (IntPtr msgqueP, IntPtr data)
  {
    if (ProcCall(msgqueP, (Callback) GCHandle.FromIntPtr(data).Target, null) == McErrorE.MQ_OK) 
      return McErrorE.MQ_OK;

    return McErrorStack(msgqueP);
  }

  private static McErrorE sendcallI (IntPtr msgqueP, IntPtr data)
  {
    if (ProcCall(msgqueP, null, (IService) GCHandle.FromIntPtr(data).Target) == McErrorE.MQ_OK) 
      return McErrorE.MQ_OK;

    return McErrorStack(msgqueP);
  }

  /// \api #MqSendEND_AND_CALLBACK
  public void SendEND_AND_CALLBACK(string token, Callback call) {
    ErrorMqToCsWithCheck(McSendEND_AND_CALLBACK(msgqueP.Value, token, fsendcallC,
    (IntPtr) GCHandle.Alloc(call), fsrvfree));
  }

  /// \api #MqSendEND_AND_CALLBACK
  public void SendEND_AND_CALLBACK(string token, IService call) {
    ErrorMqToCsWithCheck(McSendEND_AND_CALLBACK(msgqueP.Value, token, fsendcallI,
    (IntPtr) GCHandle.Alloc(call), fsrvfree));
  }

  /// \api #MqSendRETURN
  public void SendRETURN() {
    ErrorMqToCsWithCheck(McSendRETURN(msgqueP.Value));
  }

  /// \api #MqSendFTR
  public void SendFTR(long timeout) {
    ErrorMqToCsWithCheck(McSendFTR(msgqueP.Value, timeout));
  }

  /// \api #MqSendEOF
  public void SendEOF(long timeout) {
    ErrorMqToCsWithCheck(McSendEOF(msgqueP.Value, timeout));
  }

  /// \api #MqSendFTR
  public void SendFTR() {
    ErrorMqToCsWithCheck(McSendFTR(msgqueP.Value, -1));
  }

  /// \api #MqSendEOF
  public void SendEOF() {
    ErrorMqToCsWithCheck(McSendEOF(msgqueP.Value, -1));
  }

  /// \api #MqSendY
  public void SendY(byte val) {
    McSendY(msgqueP.Value, val);
  }

  /// \api #MqSendO
  public void SendO(bool val) {
    McSendO(msgqueP.Value, (byte) (val ? MQ_BOL.MQ_YES : MQ_BOL.MQ_NO));
  }

  /// \api #MqSendS
  public void SendS(short val) {
    McSendS(msgqueP.Value, val);
  }

  /// \api #MqSendI
  public void SendI(int val) {
    McSendI(msgqueP.Value, val);
  }

  /// \api #MqSendF
  public void SendF(float val) {
    McSendF(msgqueP.Value, val);
  }

  /// \api #MqSendW
  public void SendW(long val) {
    McSendW(msgqueP.Value, val);
  }

  /// \api #MqSendD
  public void SendD(double val) {
    McSendD(msgqueP.Value, val);
  }

  /// \api #MqSendB
  public void SendB(byte[] val) {
    McSendB(msgqueP.Value, val, val.Length);
  }

  /// \api #MqSendU
  public void SendU(IntPtr val) {
    McSendU(msgqueP.Value, val);
  }

  /// \api #MqSendC
  public void SendC(string val) {
    McSendC(msgqueP.Value, val);
  }

  /// \api #MqSendL_START
  public void SendL_START() {
    McSendL_START(msgqueP.Value);
  }

  /// \api #MqSendL_END
  public void SendL_END() {
    McSendL_END(msgqueP.Value);
  }
}

// END - NameSpace csmsgque
}










