/**
 *  \file       csmsgque/service.cs
 *  \brief      \$Id: service.cs 73 2009-06-12 11:35:53Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 73 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/*****************************************************************************/
/*                                                                           */
/*                                service                                    */
/*                                                                           */
/*****************************************************************************/

using System;
using System.Runtime.InteropServices;

namespace csmsgque {

  public abstract partial class MqS
  {
    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqServiceCreate")]
    private static extern McErrorE McServiceCreate([In]IntPtr msgqueP, [In]string token, 
	[In]McTokenF srvcall, [In]IntPtr data, [In]McTokenDataFreeF srvfree);

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqServiceDelete")]
    private static extern McErrorE McServiceDelete([In]IntPtr msgqueP, [In]string token);

    [DllImport(MSGQUE_DLL, CallingConvention=MSGQUE_CC, CharSet=MSGQUE_CS, EntryPoint = "MqIsTransaction")]
    private static extern MQ_BOL McIsTransaction([In]IntPtr msgqueP);

    private static McErrorE srvError (IntPtr msgqueP) {
      // we have an MQ error
      // check to be in a transaction or not
      if (McIsTransaction(msgqueP) == MQ_BOL.MQ_NO) {
	// no transaction
	// return server error from a MqSendEND call asyncrone
	return McSendERROR (msgqueP);
      } else {
	// in a transaction
	// return a "server" error from a MqSendEND_AND_WAIT or MqSendEND_AND_CALLBACK call
	McSendSTART(msgqueP);
	return McSendRETURN (msgqueP);
      }
    }

    private static McErrorE srvcallC (IntPtr msgqueP, IntPtr data)
    {

      // call the service
      switch (ProcCall(msgqueP, (Callback) GCHandle.FromIntPtr(data).Target, null)) {
	case McErrorE.MQ_OK:         return McErrorE.MQ_OK;
	case McErrorE.MQ_ERROR:      return McErrorE.MQ_ERROR;
	case McErrorE.MQ_EXIT:       return McErrorE.MQ_EXIT;
	case McErrorE.MQ_CONTINUE:   break;
      }

      // report the error
      return srvError(msgqueP);
    }

    private static McErrorE srvcallI (IntPtr msgqueP, IntPtr data)
    {

      // call the service
      switch (ProcCall(msgqueP, null, (IService) GCHandle.FromIntPtr(data).Target)) {
	case McErrorE.MQ_OK:         return McErrorE.MQ_OK;
	case McErrorE.MQ_ERROR:      return McErrorE.MQ_ERROR;
	case McErrorE.MQ_EXIT:       return McErrorE.MQ_EXIT;
	case McErrorE.MQ_CONTINUE:   break;
      }

      // report the error
      return srvError(msgqueP);
    }

    private static void srvfree (IntPtr msgqueP, IntPtr data) {
      GCHandle.FromIntPtr(data).Free();
    }

    // protect function-pointer to ignore garbage collection
    private McTokenF fsrvcallC = srvcallC;
    private McTokenF fsrvcallI = srvcallI;
    private McTokenDataFreeF fsrvfree = srvfree;

    // PUBLIC  #########################################################################

    /// \api #MqServiceCreate
    public void ServiceCreate(string token, Callback call) {
        McServiceCreate(msgqueP.Value, token, fsrvcallC, (IntPtr) GCHandle.Alloc(call), fsrvfree);
    }

    /// \api #MqServiceCreate
    public void ServiceCreate(string token, IService call) {
        McServiceCreate(msgqueP.Value, token, fsrvcallI, (IntPtr) GCHandle.Alloc(call), fsrvfree);
    }

    /// \api #MqServiceDelete
    public void ServiceDelete(string token) {
      McServiceDelete(msgqueP.Value, token);
    }


  } // END - class "MqS"
} // END - namespace "csmsgque"

