/**
 *  \file       javamsgque/MqS.java
 *  \brief      \$Id: MqS.java 1153 2009-05-09 18:05:50Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1153 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

///\brief The \javamsgque MqS Package
package javamsgque;

/** \defgroup javamain_api MqS Class
\{
\brief Java-Msgque Super-Class

The #javamsgque.MqS class ist used as super-class for a \javamsgque based application.
The class is \e abstract because the developper have to implement one of the
following interfaces ts create a valid \javamsgque application
 - #javamsgque.IServer
 - #javamsgque.IClient
 .

This is an example for a valid application class:
\verbatim
class MyApplication extends javamsgque.MqS implements javamsgque.IServer {
....
}
\endverbatim
*/


///\brief Java-Msgque Super-Class
public abstract class MqS {

  static {
    System.loadLibrary("javamsgque");
  }

  private long hdl;

//
// ========================================================================
/// \ingroup javamain_api
/// \defgroup javamisc_api Misc Functions
/// \{
/// \brief A collection of Misc Functions


  /// \brief constructor used to create a \e parent object
  protected MqS(String... argv) {
    pCreate(null, null, 0, argv);
  }

  /// \brief constructor used to create a \e child object
  protected MqS(MqS parent, String... argv) {
    pCreate(parent, null, 0, argv);
  }

  /// \brief constructor used to create a \e worker object
  protected MqS(MqS master, int master_id, String... argv) {
    pCreate(null, master, master_id, argv);
  }

  /// \brief default constructor used for the object factory
  protected MqS() {
  }

  /// \api #MqDelete
  public void Delete () {
    pDelete();
  }

  protected void finalize() {
/*
    if (hdl != 0) {
      pDelete();
    }
*/
  } 

  private native void	pCreate		    (MqS parent, MqS master, int master_id, String[] argv);
  private native void	pDelete		    ();
  protected static native void	Init	    (String... argv);

  public enum WAIT {
    NO(0), ONCE(1), FOREVER(2);

    private int flag;

    WAIT(int setFlag)
    {
      flag = setFlag;
    }

    public int GetFlag()
    {
      return(flag);
    }
  }

  private native void	pProcessEvent	    (int timeout, int flags);

  /// \api #MqProcessEvent, wait for \e timeout seconds and process event or raise an error
  public void ProcessEvent (int timeout, WAIT flags) {
    pProcessEvent(timeout,flags.GetFlag());
  }
  /// \api #MqProcessEvent, wait application default time
  public void ProcessEvent (WAIT flags) {
    pProcessEvent(-1,flags.GetFlag());
  }
  /// \api #MqProcessEvent, don't wait just check for an event
  public void ProcessEvent () {
    pProcessEvent(-1,WAIT.NO.GetFlag());
  }

  /// start event-loop and wait forever
  public native void	ServerStart	    ();

  /// \api #MqErrorC
  public native void    ErrorC              (String func, int num, String message);

/// \}

//
// ========================================================================
/// \ingroup javamain_api
/// \defgroup javaservice_api Service Management
/// \{
/// \brief Setup and delete services
///
/// A service is usually defined on a server (interface #javamsgque.IServer) in the
/// #javamsgque.IServer.ServerConfig function.
///
  /// \api #MqServiceCreate
  public native void	ServiceCreate	    (String token, IService service);
  /// \api #MqServiceDelete
  public native void	ServiceDelete	    (String token);

/// \}
//
// ========================================================================
/// \ingroup javamain_api
/// \defgroup javasend_api Send Data
/// \{
/// \brief Collect and send data using a LibMsgque package

  /// \api #MqSendSTART
  public native void	SendSTART		();
  /// \api #MqSendEND
  public native void	SendEND			(String token);
  /// \api #MqSendEND_AND_WAIT
  public native void	SendEND_AND_WAIT	(String token, int timeout);
  /// \api #MqSendEND_AND_CALLBACK
  public native void	SendEND_AND_CALLBACK	(String token, ICallback callback);
  /// \api #MqSendRETURN
  public native void	SendRETURN		();
  /// \api #MqSendFTR
  /// \attention only use this function in a #javamsgque.IFilter.FTR service
  public native void	SendFTR			(int timeout);
  /// \attention only use this function in a #javamsgque.IFilter.EOF service
  /// \api #MqSendEOF
  public native void	SendEOF			(int timeout);
  /// \api #MqSendY
  public native void	SendY			(byte y);
  /// \api #MqSendO
  public native void	SendO			(boolean o);
  /// \api #MqSendS
  public native void	SendS			(short s);
  /// \api #MqSendI
  public native void	SendI			(int i);
  /// \api #MqSendF
  public native void	SendF			(float f);
  /// \api #MqSendW
  public native void	SendW			(long w);
  /// \api #MqSendD
  public native void	SendD			(double l);
  /// \api #MqSendC
  public native void	SendC			(String s);
  /// \api #MqSendB
  public native void	SendB			(byte[] b);
  /// \api #MqSendU
  public native void	SendU			(Buffer buf);
  /// \api #MqSendL_START
  public native void	SendL_START		();
  /// \api #MqSendL_END
  public native void	SendL_END		();

/// \}
//
// ========================================================================
/// \ingroup javamain_api
/// \defgroup javaread_api Read Data
/// \{
/// \brief Read and split a \libmsgque package into several items
//

  /// \api #MqReadI
  public native byte	ReadY		    ();
  /// \api #MqReadI
  public native boolean	ReadO		    ();
  /// \api #MqReadI
  public native short	ReadS		    ();
  /// \api #MqReadI
  public native int	ReadI		    ();
  /// \api #MqReadF
  public native float	ReadF		    ();
  /// \api #MqReadW
  public native long	ReadW		    ();
  /// \api #MqReadD
  public native double	ReadD		    ();
  /// \api #MqReadC
  public native String	ReadC		    ();
  /// \api #MqReadB
  public native byte[]	ReadB		    ();
  /// \api #MqReadU
  public native Buffer	ReadU		    ();
  /// \api #MqReadL_START
  public native void	ReadL_START	    ();
  /// \api #MqReadL_END
  public native void	ReadL_END	    ();
  /// \api #MqReadGetNumItems
  public native int	ReadGetNumItems	    ();
  /// \api #MqReadItemExists
  public native boolean	ReadItemExists	    ();
  /// \api #MqReadUndo
  public native void	ReadUndo	    ();

/// \}

//
// ========================================================================
/// \ingroup javamain_api
/// \defgroup javabuffer_get_api Get Buffer Data
/// \{
/// \brief Get Data from a bufferHdl
//

  /// \api #MqBufferGetType
  ///
  /// Use the following sequence to check and extract typeless data
  /// \code
  /// int bufferHdl = ctx.ReadU();
  /// if (ctx.GetType(bufferHdl) == 'I') {
  ///   int i = ctx.GetI(bufferHdl);
  ///   ...
  /// }
  /// \endcode
  public native char	GetType		    (Buffer buf);
  /// \api #MqBufferGetY
  public native byte	GetY		    (Buffer buf);
  /// \api #MqBufferGetO
  public native boolean	GetO		    (Buffer buf);
  /// \api #MqBufferGetS
  public native short	GetS		    (Buffer buf);
  /// \api #MqBufferGetI
  public native int	GetI		    (Buffer buf);
  /// \api #MqBufferGetF
  public native float	GetF		    (Buffer buf);
  /// \api #MqBufferGetW
  public native long	GetW		    (Buffer buf);
  /// \api #MqBufferGetD
  public native double	GetD		    (Buffer buf);
  /// \api #MqBufferGetC
  public native String	GetC		    (Buffer buf);
  /// \api "free"
  public native byte[]	GetB		    (Buffer buf);
  /// \api #MqReadL_START
  /// \attention use the Read? sytle function after the buffer-list START
  public native void	GetL_START	    (Buffer buf);
  /// \api #MqReadL_END
  public native void	GetL_END	    ();

/// \}
//
// ========================================================================
/// \ingroup javamain_api
/// \defgroup javaconfig_api Access To Configuration Data
/// \{
/// \brief Configuration data of the \javamsgque object
//

  /// \api #MqConfigGetIsServer
  public native boolean	ConfigGetIsServer   ();
  /// \api #MqConfigGetIsParent
  public native boolean	ConfigGetIsParent   ();
  /// \api #MqConfigGetIsSlave
  public native boolean	ConfigGetIsSlave   ();
  /// \api #MqConfigGetIsString
  public native boolean	ConfigGetIsString   ();
  /// \api #MqConfigGetIsSilent
  public native boolean	ConfigGetIsSilent   ();
  /// \api #MqConfigGetName
  public native String	ConfigGetName	    ();
  /// \api #MqConfigGetDebug
  public native int	ConfigGetDebug	    ();
  /// \api #MqConfigGetParent
  public native MqS	ConfigGetParent	    ();
  /// \api #MqConfigGetMaster
  public native MqS	ConfigGetMaster	    ();
  /// \api #MqGetCtxId
  public native int	ConfigGetCtxId	    ();
  /// \api #MqGetToken
  public native String	ConfigGetToken	    ();

/// \}
//
// ========================================================================
/// \ingroup javamain_api
/// \defgroup javaslave_api Manage Slave Objects
/// \{
/// \brief Configuration data of the \javamsgque object
//

  /// \api #MqSlaveCreate
  public native void	SlaveCreate	    (int id, MqS slave);
  /// \api #MqSlaveDelete
  public native void	SlaveDelete	    (int id);
  /// \api #MqSlaveGet
  public native MqS	SlaveGet	    (int id);

/// \}
}

/// \}   end MqS




