/**
 *  \file       javamsgque/context_java.c
 *  \brief      \$Id: context_java.c 1186 2009-05-19 06:41:03Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1186 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#define MSGQUE msgque
#include "context_java.h"

/// global data
JavaVM	    *cached_jvm;

jclass	    NS(Class_MqS), NS(Class_IClient), NS(Class_IServer), NS(Class_RuntimeException),
	    NS(Class_Throwable), NS(Class_Class), NS(Class_Object), NS(Class_IService),
	    NS(Class_NullPointerException), NS(Class_System),
	    NS(Class_ICallback), NS(Class_StackTraceElement), NS(Class_StringWriter),
	    NS(Class_PrintWriter), NS(Class_IFilter), NS(Class_Buffer) ;

jfieldID    NS(FID_MqS_hdl), NS(FID_Buffer_hdl);

jmethodID   NS(MID_Throwable_getMessage), NS(MID_Class_getName), NS(MID_IService_Service),
	    NS(MID_IServer_ServerConfig), NS(MID_System_exit),
	    NS(MID_ICallback_Callback), NS(MID_Throwable_printStackTrace),
	    NS(MID_StringWriter_init), NS(MID_PrintWriter_init), NS(MID_StringWriter_toString),
	    NS(MID_IFilter_FTR), NS(MID_IFilter_EOF);

JNIEXPORT jint JNICALL
JNI_OnLoad(JavaVM *jvm, void *reserved)
{
  JNIEnv *env;
  jclass cls;
  cached_jvm = jvm;  /* cache the JavaVM pointer */

  if ((*jvm)->GetEnv(jvm, (void*) &env, JNI_VERSION_1_6)) {
    return JNI_ERR; /* JNI version not supported */
  }

#define check(v,c) if ((v=(c))==NULL) return JNI_ERR
#define checkC(v,n) check(cls, (*env)->FindClass(env, n));\
    check(v, (*env)->NewGlobalRef(env, cls));
#define checkF(v,c,n,s) check(v, (*env)->GetFieldID(env,c,n,s));
#define checkSF(v,c,n,s) check(v, (*env)->GetStaticFieldID(env,c,n,s));
#define checkM(v,c,n,s) check(v, (*env)->GetMethodID(env,c,n,s));
#define checkSM(v,c,n,s) check(v, (*env)->GetStaticMethodID(env,c,n,s));

  // setup "jclass"
  checkC(NS(Class_MqS),			    "javamsgque/MqS");
  checkC(NS(Class_Buffer),		    "javamsgque/Buffer");
  checkC(NS(Class_IService),		    "javamsgque/IService");
  checkC(NS(Class_IClient),		    "javamsgque/IClient");
  checkC(NS(Class_IServer),		    "javamsgque/IServer");
  checkC(NS(Class_ICallback),		    "javamsgque/ICallback");
  checkC(NS(Class_IFilter),		    "javamsgque/IFilter");
  checkC(NS(Class_RuntimeException),	    "java/lang/RuntimeException");
  checkC(NS(Class_NullPointerException),    "java/lang/NullPointerException");
  checkC(NS(Class_Throwable),		    "java/lang/Throwable");
  checkC(NS(Class_Class),		    "java/lang/Class");
  checkC(NS(Class_Object),		    "java/lang/Object");
  checkC(NS(Class_System),		    "java/lang/System");
  checkC(NS(Class_StringWriter),	    "java/io/StringWriter");
  checkC(NS(Class_PrintWriter),		    "java/io/PrintWriter");

  // setup "jfieldID"
  checkF(NS(FID_MqS_hdl),		    NS(Class_MqS),	    "hdl",		"J");
  checkF(NS(FID_Buffer_hdl),                NS(Class_Buffer),       "hdl",              "J");

  // setup "jmethodID"
  checkM(NS(MID_Throwable_getMessage),	    NS(Class_Throwable),    "getMessage",	"()Ljava/lang/String;");
  checkM(NS(MID_Throwable_printStackTrace), NS(Class_Throwable),    "printStackTrace",  "(Ljava/io/PrintWriter;)V");
  checkM(NS(MID_Class_getName),		    NS(Class_Class),	    "getName",		"()Ljava/lang/String;");
  checkM(NS(MID_IService_Service),	    NS(Class_IService),	    "Service",		"(Ljavamsgque/MqS;)V");
  checkM(NS(MID_IServer_ServerConfig),	    NS(Class_IServer),	    "ServerConfig",	"()V");
  checkM(NS(MID_StringWriter_init),	    NS(Class_StringWriter), "<init>",		"()V");
  checkM(NS(MID_StringWriter_toString),	    NS(Class_StringWriter), "toString",		"()Ljava/lang/String;");
  checkM(NS(MID_PrintWriter_init),	    NS(Class_PrintWriter),  "<init>",		"(Ljava/io/Writer;)V");
  checkSM(NS(MID_System_exit),		    NS(Class_System),	    "exit",		"(I)V");
  checkM(NS(MID_ICallback_Callback),	    NS(Class_ICallback),    "Callback",		"(Ljavamsgque/MqS;)V");
  checkM(NS(MID_IFilter_FTR),		    NS(Class_IFilter),	    "FTR",		"()V");
  checkM(NS(MID_IFilter_EOF),		    NS(Class_IFilter),	    "EOF",		"()V");

#undef check
#undef checkC
#undef checkF
#undef checkSF
#undef checkM

  return JNI_VERSION_1_6;
}

JNIEXPORT void JNICALL 
JNI_OnUnload(JavaVM *jvm, void *reserved)
{
  JNIEnv *env;
  void *val;
  if ((*jvm)->GetEnv(jvm, &val, JNI_VERSION_1_6)) {
    return;
  }
  env = (JNIEnv*) val;
  (*env)->DeleteGlobalRef(env, NS(Class_MqS));
  (*env)->DeleteGlobalRef(env, NS(Class_Buffer));
  (*env)->DeleteGlobalRef(env, NS(Class_IService));
  (*env)->DeleteGlobalRef(env, NS(Class_IClient));
  (*env)->DeleteGlobalRef(env, NS(Class_IServer));
  (*env)->DeleteGlobalRef(env, NS(Class_ICallback));
  (*env)->DeleteGlobalRef(env, NS(Class_IFilter));
  (*env)->DeleteGlobalRef(env, NS(Class_Throwable));
  (*env)->DeleteGlobalRef(env, NS(Class_Class));
  (*env)->DeleteGlobalRef(env, NS(Class_Object));
  (*env)->DeleteGlobalRef(env, NS(Class_System));
  (*env)->DeleteGlobalRef(env, NS(Class_StringWriter));
  (*env)->DeleteGlobalRef(env, NS(Class_PrintWriter));
  (*env)->DeleteGlobalRef(env, NS(Class_RuntimeException));
  (*env)->DeleteGlobalRef(env, NS(Class_NullPointerException));
  return;
}

/** \brief create a new child-context in server mode
 *
 * \param[in,out] config configuration data
 * \param[in] argv server specific configuration data
 * \param[in] alfa this should allways be NULL
 * \param[out] msgqueP the new created #MqS object
 * \retMqErrorE
 **/
static enum MqErrorE
MQ_DECL NS(ChildCreate) (
  struct MqConfigS  * config,
  struct MqBufferLS * argv,
  struct MqBufferLS * alfa,
  struct MqS ** msgqueP
) {
  struct MqS * msgque;
  JNIEnv *env;
  jclass class;

  // this is the default for:
  //  1. MQ_STATUS_IS_INITIAL
  //  2. MQ_STATUS_IS_DUP & MQ_STATUS_IS_FORK
  struct JavaContextS * context = (struct JavaContextS *) config->myCtx;

  // 1. setup the SERVER+CHILD context using PARENT data
  if (config->parent != NULL && context == NULL) {
    struct JavaContextS * parentContext = (struct JavaContextS *) config->parent->config.myCtx;
    context = MqSysCalloc (MQ_ERROR_PANIC, 1, sizeof (*context));
    config->myCtx = (struct MqContextS *) context;
    memcpy(context,parentContext,sizeof(*context));
    context->class = (*context->env)->NewGlobalRef(context->env,context->class);
    context->self = NULL;    // the object belongs to the PARENT
  }

  // setup variables
  env = context->env;
  class = context->class;

  // 2. create the Message-Queue
  MqErrorCheck (MqCreate (config, argv, alfa, msgqueP));
  msgque = *msgqueP;

  // check for wrong arguments
  MqErrorCheck (MqCheckForLeftOverArguments(msgque, &argv));

  // 3. create the Java Object
  if (context->self == NULL) {
    jobject new;
    if (context->factoryConstr == NULL) {
      char buf[200];
      mq_snprintf(buf, 200, 
	"ERROR, the factory default constructor '%s()' is required", 
	  NS(pGetClassName)(env, context->class));
      return MqErrorC(msgque->error, __func__, -1, buf);
    }
    new = (*env)->NewObject(env, class, context->factoryConstr) ;
    if (new == NULL) {
      return NS(pErrorJavaToMq)(env, msgque);
    }
    context->self = (*env)->NewGlobalRef(env,new);
    if (context->self == NULL) {
      return NS(pErrorJavaToMq)(env, msgque);
    }
  }

  // 4. set the 'hdl'
  (*env)->SetLongField(env, context->self, NS(FID_MqS_hdl), (jlong) msgque);
  
  // 5. init the new object
  if (msgque->config.isServer == MQ_YES) {
    switch (NS(ProcCall)(msgque, context->self, context->class, NS(MID_IServer_ServerConfig), NULL)) {
      case MQ_OK:	    break;
      case MQ_ERROR:	    goto error;
      case MQ_CONTINUE:	    return MQ_CONTINUE;
      case MQ_EXIT:	    return MQ_EXIT;
    }
  }

  if (unlikely(msgque->config.debug >= 2) && unlikely(msgque->config.parent != NULL))
    MqDLogV(msgque,__func__,2,"CREATE context<%p>, msgque<%p>\n",context,msgque);

  // everything is OK
  return MQ_OK;

error:
  return MqErrorStack ((*msgqueP)->error);
}

/// \brief delete a child-context
/// \param[in,out] msgqueP the context to delete
////
static void
MQ_DECL NS(ChildDelete) (
  struct MqS ** msgqueP
)
{
  struct JavaContextS *context;
  struct MqS * msgque;
  JNIEnv *env;

  // return if the object was allread deleted
  if (!msgqueP && !(*msgqueP)) return;
  msgque = *msgqueP;

  context = (struct JavaContextS *) msgque->config.myCtx;

  if (context != NULL) {
    env = context->env;

    // the onDelete is necessary to avoid the double freeing of memory.
    // if a parent is deleted all childs are deleted as well.
    // during child delete the command-object is deleted (Tcl_DeleteCommandFromToken).
    // if the command-object is deleted the "ContextDelete" is called again -> double free
    if (context->onDelete == MQ_YES) return;
    context->onDelete = MQ_YES;

    if (context->self != NULL) {
      // unset the 'hdl'
      (*env)->SetLongField(env, context->self, NS(FID_MqS_hdl), (jlong) 0L);
      // unlock my global references
      (*env)->DeleteGlobalRef(env, context->self);
    }
    if (context->class != NULL) {
      (*env)->DeleteGlobalRef(env, context->class);
    }
  }

  MqDelete (msgqueP);
  MqSysFree (context);
}

/// \brief create a TCL parent context
///
///
static enum MqErrorE
MQ_DECL NS(ParentCreate) (
  struct MqConfigS  * config,
  struct MqBufferLS * argv,
  struct MqBufferLS * alfa,
  struct MqS       ** msgqueP
)
{
  struct MqS * msgque;
  
  // this is the default for:
  //  1. MQ_STATUS_IS_INITIAL
  //  2. MQ_STATUS_IS_DUP & MQ_STATUS_IS_FORK
  struct JavaContextS * context = (struct JavaContextS *) config->myCtx;
  //struct MqErrorS * error = MQ_ERROR_PANIC;

  // The thread get a new context
  if ((config->statusIs & MQ_STATUS_IS_DUP) && (config->statusIs & MQ_STATUS_IS_THREAD)) {
    // ceate a new context
    struct JavaContextS * const new_context = MqSysCalloc (MQ_ERROR_PANIC, 1, sizeof (*new_context));
    JavaVMAttachArgs args = {JNI_VERSION_1_6, "libmsgque", NULL};
    void * vP;
    (*cached_jvm)->AttachCurrentThread(cached_jvm, &vP, &args);
    new_context->env = (JNIEnv*) vP;
    new_context->self = NULL;
    new_context->class = (*new_context->env)->NewGlobalRef(new_context->env, context->class);
    new_context->onDelete = MQ_NO;
    new_context->factoryConstr = context->factoryConstr;
    context = new_context;
    config->myCtx = (struct MqContextS *) context;
  }

  // create the server-context
  MqErrorCheck (NS(ChildCreate)(config, argv, alfa, msgqueP));
  msgque = *msgqueP;

  MqDLog (msgque, 2, "CREATE context<%p>, msgque<%p>\n", context, msgque);

  // everything is OK
  return MQ_OK;

error:
  return MqErrorStack((*msgqueP)->error);
}

/// \brief delete a parent-context in server mode
///
/// \param[in,out] msgqueP the context to delete
///
static void
MQ_DECL NS(ParentDelete) (
  struct MqS ** msgqueP
)
{
  struct MqS * msgque;
  struct JavaContextS * context;
  int state_is_dup_thread;

  // return if the object was allread deleted
  if (!msgqueP && !(*msgqueP)) return;
  msgque = *msgqueP;
  SETUP_context

  // the onDelete is necessary to avoid the double freeing of memory.
  // if a parent is deleted all childs are deleted as well.
  if (context->onDelete == MQ_YES) return;

  // save data !befor! NS(ChildDelete) will delete it
  state_is_dup_thread = (
      ( msgque->config.statusIs & MQ_STATUS_IS_DUP) &&
	  (msgque->config.statusIs & MQ_STATUS_IS_THREAD)
  );

  // finally use child-delete to clean the context
  NS(ChildDelete)(msgqueP);

  // cleanup the env
  if (state_is_dup_thread) (*cached_jvm)->DetachCurrentThread(cached_jvm);
}

/// \brief delete a parent process or thread
///
/// \param[in] isThread bool value set to \c 1 (yes) if it is a thread and \c 0 (no) if not
/// \param[in] num the exit number
///
/// \e libmsgque has the \e thread created -> \e libmsgque
/// have to delete the thread -> no \e thread action in this handler
///
///
static void
MQ_DECL NS(ParentExit) (
  int isThread,
  int num
)
{
  JNIEnv *env;
  if ((*cached_jvm)->GetEnv(cached_jvm, (void*) &env, JNI_VERSION_1_6) == JNI_OK) {
    if (isThread) {
    } else {
      (*env)->CallStaticVoidMethod(env, NS(Class_System), NS(MID_System_exit), (jint) num);
    }
  }
}

static enum MqErrorE 
MQ_DECL NS(sFilter) (
  struct MqS * msgque,
  MQ_PTR dataP
)
{
  struct JavaContextS * context = CONTEXT;
  if (NS(ProcCall)(msgque, context->self, context->class, (jmethodID) dataP, NULL) == MQ_OK)
    return MQ_OK;

  return MqErrorStack(msgque->error);
}

static void
NS(MqS_pCreateError) (
  JNIEnv	*env, 
  struct MqS	**msgqueP
)
{
  struct MqS * msgque = *msgqueP;
  if (msgque->config.isServer == MQ_NO) {
    // set error-message
    ErrorMqToJava();
    // allways delete the msgque object because this is the last chance to do it
    // !!attention!! never use "?ParentDelete" ot "?ChildeDelete" directly,
    // "MqDelete" will call the functions from above
    MqDelete(msgqueP);
  } else {
    MqExit(msgque);
  }
}

JNIEXPORT void JNICALL NS(MqS_pCreate) (
  JNIEnv *	env, 
  jobject	self, 
  jobject	parentO, 
  jobject	masterO, 
  jint		master_id, 
  jobjectArray	argv
)
{
  register struct JavaContextS * context = MqSysCalloc (MQ_ERROR_PANIC, 1, sizeof (*context));
  struct MqConfigS * config;
  struct MqS * msgque = NULL;

  struct MqBufferLS * const largv = MqBufferLCreate (10);
  struct MqBufferLS * const lalfa = MqBufferLCreate (10);
  struct MqBufferLS * tgt = largv;

  jclass c;
  jobject obj;
  const char *str;
  int i;
  jsize argc;

  // fill the context
  context->env  = env;
  context->self = (*env)->NewGlobalRef(env, self);
  if (context->self == NULL) return;
  c = (*env)->GetObjectClass(env, self);
  if (c == NULL) return;
  context->class = (jclass)  (*env)->NewGlobalRef(env, c);
  if (context->class == NULL) return;
  context->factoryConstr = NULL;

  // init config
  if (parentO != NULL)  {
    struct MqS * const parent = ((struct MqS *) (*env)->GetLongField(env,parentO,NS(FID_MqS_hdl)));
    config = MqConfigDup (&parent->config);
    MqConfigInitParent (config, parent);
  } else if (masterO != NULL)   {
    struct MqS * const master = ((struct MqS *) (*env)->GetLongField(env,masterO,NS(FID_MqS_hdl)));
    config = MqConfigDup (&master->config);
    MqConfigInitMaster (config, master, master_id);
  } else {
    // parent config
    config = MqConfigInit();

    config->isSilent	= NS(pGetBoolFromStaticJavaField)	(env, c, "silent",	"Z");
    config->isString	= NS(pGetBoolFromStaticJavaField)	(env, c, "string",	"Z");
    config->isServer	= NS(pGetBoolFromStaticJavaField)	(env, c, "server",	"Z");
    config->isFilter	= NS(pGetBoolFromStaticJavaField)	(env, c, "filter",	"Z");

    config->debug	= NS(pGetIntFromStaticJavaField)	(env, c, "debug",	"I");
    config->buffersize	= NS(pGetIntFromStaticJavaField)	(env, c, "buffersize",	"I");
    config->timeout	= NS(pGetIntFromStaticJavaField)	(env, c, "timeout",	"I");

    MqConfigInitName (config, NS(pGetStringFromStaticJavaField)	(env, c, "name",	"Ljava/lang/String;"));

    config->ignoreFork	  = MQ_YES;

    config->fChildCreate  = NS(ChildCreate);
    config->fChildDelete  = NS(ChildDelete);
    config->fParentCreate = NS(ParentCreate);
    config->fParentDelete = NS(ParentDelete);
    config->fParentExit	  = NS(ParentExit);
  }

  // fill the config
  config->myCtx = (struct MqContextS *) context;

  // check requirements
  if ((*env)->IsInstanceOf(env, self, NS(Class_IClient)) == JNI_FALSE) {
    ErrorStringToJava("interface 'MqS.IClient', 'MqS.IServer' or 'MqS.IFilter' is required");
    return;
  }
  if ((*env)->IsInstanceOf(env, self, NS(Class_IFilter)) == JNI_TRUE) {
    config->fFTR	= NS(sFilter);
    config->FTRdata	= NS(MID_IFilter_FTR);
    config->FTRfree	= NULL;
    config->fEOF	= NS(sFilter);
    config->EOFdata	= NS(MID_IFilter_EOF);
    config->EOFfree	= NULL;
  } 

  // if the client is used as 'slave' a factory-constructor is required
  context->factoryConstr = (*env)->GetMethodID(env,c,"<init>","()V");
  if (context->factoryConstr == NULL) {
    char buf[200];
    mq_snprintf(buf, 200, 
      "ERROR, the factory default constructor '%s()' is required", 
	NS(pGetClassName)(env, c));
    ErrorStringToJava(buf);
    return;
  }

  // fill the argv/alfa
  argc = (*env)->GetArrayLength(env, argv);
  MqBufferLAppendC(tgt,config->name);
  for (i=0; i<argc; i++) {
    obj = (*env)->GetObjectArrayElement(env, argv, i);
    str = (*env)->GetStringUTFChars(env, obj, NULL);
    if (str[0] == MQ_ALFA) {
      tgt = lalfa;
      continue;
    }
    MqBufferLAppendC(tgt,str);
    (*env)->ReleaseStringUTFChars(env,obj,str);
  }

  // 7. create Context
  if (parentO == NULL) {
    MqErrorCheck(NS(ParentCreate)(config, largv, lalfa, &msgque));
  } else {
    MqErrorCheck(NS(ChildCreate)(config, largv, lalfa, &msgque));
  }
  return;

error:
  NS(MqS_pCreateError)(env, &msgque);
}

JNIEXPORT void JNICALL NS(MqS_pDelete) (
  JNIEnv *	env, 
  jobject	self 
)
{
  struct MqS * msgque = ((struct MqS *) (*env)->GetLongField(env,self,NS(FID_MqS_hdl)));
  if (msgque == NULL) return;
  MqDelete(&msgque);
}

JNIEXPORT void JNICALL NS(MqS_pProcessEvent) (
  JNIEnv *	env, 
  jobject	self, 
  jint		timeout,
  jint		flags
)
{
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqProcessEvent(msgque, timeout, (enum MqWaitOnEventE) flags));
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_ErrorC) (
  JNIEnv    *env,
  jobject   self,
  jstring   func,
  jint      num,
  jstring   message
)
{
  const char *funcC;
  const char *mesgC;
  SETUP_msgque;
  funcC = (*env)->GetStringUTFChars(env, func, NULL);
  mesgC = (*env)->GetStringUTFChars(env, message, NULL);
  MqErrorC(msgque->error, funcC, num, mesgC);
  (*env)->ReleaseStringUTFChars(env, func, funcC);
  (*env)->ReleaseStringUTFChars(env, func, mesgC);
  ErrorMqToJava();
error:
  return ;
}

JNIEXPORT void JNICALL NS(MqS_Init) (
  JNIEnv *	env, 
  jclass	cls,
  jobjectArray	argv
)
{
  jobject obj;
  const char *str; 
  jsize argc = (*env)->GetArrayLength(env, argv);
  struct MqBufferLS *initB = MqInitCreate();
  jsize i;
  for (i=0; i<argc; i++) {
    obj = (*env)->GetObjectArrayElement(env, argv, i);
    str = (*env)->GetStringUTFChars(env, obj, NULL);
    MqBufferLAppendC(initB,str);
    (*env)->ReleaseStringUTFChars(env,obj,str);
  }
}
