/**
 *  \file       javamsgque/read_java.c
 *  \brief      \$Id: read_java.c 1182 2009-05-15 13:59:20Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1182 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "context_java.h"

extern jfieldID	    NS(FID_MqS_hdl);
extern jfieldID     NS(FID_Buffer_hdl);
extern jclass	    NS(Class_NullPointerException);
extern jclass       NS(Class_Buffer);

///
///  READ
///

JNIEXPORT void JNICALL NS(MqS_ReadL_1START) (
  JNIEnv *	env, 
  jobject	self
)
{
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadL_START(msgque->read, NULL));
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_ReadL_1END) (
  JNIEnv *	env, 
  jobject	self 
)
{
  SETUP_msgque;
  MqReadL_END(msgque->read);
error:
  return;
}

JNIEXPORT jbyte JNICALL NS(MqS_ReadY) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_BYT i;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadY(msgque->read, &i));
  return i;
error:
  return 0;
}

JNIEXPORT jboolean JNICALL NS(MqS_ReadO) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_BOL i;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadO(msgque->read, &i));
  return (jboolean)i;
error:
  return 0;
}

JNIEXPORT jshort JNICALL NS(MqS_ReadS) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_SRT i;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadS(msgque->read, &i));
  return i;
error:
  return 0;
}

JNIEXPORT jint JNICALL NS(MqS_ReadI) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_INT i;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadI(msgque->read, &i));
  return i;
error:
  return 0;
}

JNIEXPORT jfloat JNICALL NS(MqS_ReadF) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_FLT i;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadF(msgque->read, &i));
  return i;
error:
  return 0;
}

JNIEXPORT jlong JNICALL NS(MqS_ReadW) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_WID w;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadW(msgque->read, &w));
  return w;
error:
  return 0L;
}

JNIEXPORT jdouble JNICALL NS(MqS_ReadD) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_DBL d;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadD(msgque->read, &d));
  return d;
error:
  return 0.0;
}

JNIEXPORT jstring JNICALL NS(MqS_ReadC) (
  JNIEnv *	env, 
  jobject	self 
)
{
  MQ_STR s;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadC(msgque->read, &s));
  return (*env)->NewStringUTF(env,s);
error:
  return NULL;
}

JNIEXPORT jbyteArray JNICALL NS(MqS_ReadB) (
  JNIEnv *	env, 
  jobject	self
)
{
  MQ_BIN b;
  MQ_SIZE len;
  jbyteArray tmp;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadB(msgque->read, &b, &len));
  tmp = (*env)->NewByteArray(env,len);
  (*env)->SetByteArrayRegion(env,tmp,0,len,(jbyte*)b);
  return tmp;
error:
  return NULL;
}

JNIEXPORT jobject JNICALL NS(MqS_ReadU) (
  JNIEnv *      env,
  jobject       self
)
{
  MQ_BUF buffer;
  jobject ret;
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadU(msgque->read, &buffer));
  ret = (*env)->AllocObject(env, NS(Class_Buffer));
  (*env)->SetLongField(env,ret,NS(FID_Buffer_hdl),(jlong) buffer);
  return ret;
error:
  return NULL;
}

JNIEXPORT jint JNICALL NS(MqS_ReadGetNumItems) (
  JNIEnv *	env, 
  jobject	self
)
{
  SETUP_msgque;
  return MqReadGetNumItems(msgque->read);
error:
  return 0;
}

JNIEXPORT jboolean JNICALL NS(MqS_ReadItemExists) (
  JNIEnv *	env, 
  jobject	self
)
{
  SETUP_msgque;
  return (MqReadItemExists(msgque->read) ? JNI_TRUE : JNI_FALSE);
error:
  return 0;
}

JNIEXPORT void JNICALL NS(MqS_ReadUndo) (
  JNIEnv *	env, 
  jobject	self
)
{
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqReadUndo(msgque->read));
error:
  return;
}


