/**
 *  \file       javamsgque/send_java.c
 *  \brief      \$Id: send_java.c 1186 2009-05-19 06:41:03Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1186 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "context_java.h"

extern jfieldID	    NS(FID_MqS_hdl);
extern jfieldID     NS(FID_Buffer_hdl);
extern jclass	    NS(Class_NullPointerException);
extern jclass	    NS(Class_RuntimeException);
extern jmethodID    NS(MID_ICallback_Callback);

///
///   SEND
///

JNIEXPORT void JNICALL NS(MqS_SendSTART) (
  JNIEnv *	env, 
  jobject	self 
)
{
  SETUP_msgque;
  MqSendSTART(msgque->send);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendEND) (
  JNIEnv *	env, 
  jobject	self,
  jobject	token
)
{
  const char *str;
  SETUP_msgque;
  str = (*env)->GetStringUTFChars(env,token,NULL);
  ErrorMqToJavaWithCheck(MqSendEND(msgque->send,str));
  (*env)->ReleaseStringUTFChars(env,token,str);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendEND_1AND_1WAIT) (
  JNIEnv *	env, 
  jobject	self,
  jobject	token,
  jint		timeout
)
{
  const char *str;
  SETUP_msgque;
  str = (*env)->GetStringUTFChars(env,token,NULL);
  ErrorMqToJavaWithCheck(MqSendEND_AND_WAIT(msgque->send,str,timeout));
  (*env)->ReleaseStringUTFChars(env,token,str);
error:
  return;
}

static enum MqErrorE MQ_DECL NS(sCallback) (
  struct MqS * msgque,
  MQ_PTR data
)
{
  if (NS(ProcCall)(msgque, (jobject) data, NULL, NS(MID_ICallback_Callback), CONTEXT->self) == MQ_OK) 
    return MQ_OK;

  return MqErrorStack(msgque->error);
}

static void MQ_DECL NS(sFree) (
  struct MqS const * const msgque,
  MQ_PTR data
)
{
  JNIEnv * env = CONTEXT->env;
  (*env)->DeleteGlobalRef(env, (jobject) data);
}

JNIEXPORT void JNICALL NS(MqS_SendEND_1AND_1CALLBACK) (
  JNIEnv *	env, 
  jobject	self,
  jobject	token,
  jobject	callback
)
{
  const char *str;
  jobject obj;
  SETUP_msgque;
  str = (*env)->GetStringUTFChars(env,token,NULL);
  obj = (*env)->NewGlobalRef(env, callback);
  ErrorMqToJavaWithCheck(MqSendEND_AND_CALLBACK(msgque->send, str, NS(sCallback), obj, NS(sFree)));
  (*env)->ReleaseStringUTFChars(env,token,str);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendRETURN) (
  JNIEnv *	env, 
  jobject	self 
)
{
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqSendRETURN(msgque->send));
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendFTR) (
  JNIEnv *	env, 
  jobject	self,
  jint		timeout
)
{
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqSendFTR(msgque->send, timeout));
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendEOF) (
  JNIEnv *	env, 
  jobject	self,
  jint		timeout
)
{
  SETUP_msgque;
  ErrorMqToJavaWithCheck(MqSendEOF(msgque->send, timeout));
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendL_1START) (
  JNIEnv *	env, 
  jobject	self 
)
{
  SETUP_msgque;
  MqSendL_START(msgque->send);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendL_1END) (
  JNIEnv *	env, 
  jobject	self 
)
{
  SETUP_msgque;
  MqSendL_END(msgque->send);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendY) (
  JNIEnv *	env, 
  jobject	self,
  jbyte		y
)
{
  SETUP_msgque;
  MqSendY(msgque->send,y);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendO) (
  JNIEnv *	env, 
  jobject	self,
  jboolean	o
)
{
  SETUP_msgque;
  MqSendO(msgque->send,(MQ_BOL)o);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendS) (
  JNIEnv *	env, 
  jobject	self,
  jshort	s
)
{
  SETUP_msgque;
  MqSendS(msgque->send,s);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendI) (
  JNIEnv *	env, 
  jobject	self,
  jint		i
)
{
  SETUP_msgque;
  MqSendI(msgque->send,i);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendF) (
  JNIEnv *	env, 
  jobject	self,
  jfloat	f
)
{
  SETUP_msgque;
  MqSendF(msgque->send,f);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendW) (
  JNIEnv *	env, 
  jobject	self,
  jlong		w
)
{
  SETUP_msgque;
  MqSendW(msgque->send,(MQ_WID)w);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendD) (
  JNIEnv *	env, 
  jobject	self,
  jdouble	d
)
{
  SETUP_msgque;
  MqSendD(msgque->send,d);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendC) (
  JNIEnv *	env, 
  jobject	self,
  jstring	s
)
{
  const char * str;
  SETUP_msgque;
  str = (*env)->GetStringUTFChars(env,s,NULL);
  MqSendC(msgque->send,str);
  (*env)->ReleaseStringUTFChars(env,s,str);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendB) (
  JNIEnv *	env, 
  jobject	self,
  jbyteArray	b
)
{
  jbyte * tmp;
  SETUP_msgque;
  tmp = (*env)->GetByteArrayElements(env,b,NULL);
  MqSendB(msgque->send,(MQ_BIN)tmp,(*env)->GetArrayLength(env,b));
  (*env)->ReleaseByteArrayElements(env,b,tmp,0);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_SendU) (
  JNIEnv *	env, 
  jobject	self,
  jobject	buf
)
{
  MQ_BUF buffer;
  SETUP_msgque;
  SETUP_buffer;
  MqSendU(msgque->send,buffer);
error:
  return;
}
