/**
 *  \file       javamsgque/service_java.c
 *  \brief      \$Id: service_java.c 1186 2009-05-19 06:41:03Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1186 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "context_java.h"

extern jfieldID	    NS(FID_MqS_hdl);
extern jmethodID    NS(MID_IService_Service);
extern jclass	    NS(Class_NullPointerException);

struct pServiceDataS {
  jobject   obj;
  jclass    cls;
};

static enum MqErrorE MQ_DECL NS(sService) (
  struct MqS * msgque,
  MQ_PTR dataP
)
{
  struct pServiceDataS *data = (struct pServiceDataS*) dataP;

  // call the service
  switch (NS(ProcCall)(msgque, data->obj, data->cls, NS(MID_IService_Service), CONTEXT->self)) {
    case MQ_OK:		return MQ_OK;
    case MQ_ERROR:	return MQ_ERROR;
    case MQ_EXIT:	return MQ_EXIT;
    case MQ_CONTINUE:	break;
  }

  // we have an MQ error
  // check to be in a transaction or not
  if (MqIsTransaction(msgque) == MQ_NO) {
    // no transaction
    // return server error from a MqSendEND call asyncrone
    return MqSendERROR (msgque->send);
  } else {
    // in a transaction
    // return a "server" error from a MqSendEND_AND_WAIT or MqSendEND_AND_CALLBACK call
    MqSendSTART(msgque->send);
    return MqSendRETURN (msgque->send);
  }
}

static void MQ_DECL NS(sFree) (
  struct MqS const * const msgque,
  MQ_PTR dataP
)
{
  struct JavaContextS * context;
  JNIEnv * env;
  struct pServiceDataS *data = (struct pServiceDataS*) dataP;
  SETUP_context
  env = context->env;
  (*env)->DeleteGlobalRef(env,data->obj);
  (*env)->DeleteGlobalRef(env,data->cls);
  MqSysFree(dataP);
}

JNIEXPORT void JNICALL NS(MqS_ServiceCreate) (
  JNIEnv *  env, 
  jobject   self, 
  jstring   token,
  jobject   service
)
{
  struct pServiceDataS *data = MqSysMalloc(MQ_ERROR_PANIC, sizeof(*data));
  const jclass class = (*env)->GetObjectClass(env, service);
  const char *tokenC = (*env)->GetStringUTFChars(env, token, NULL);
  
  SETUP_msgque;
  
  data->obj = (*env)->NewGlobalRef(env, service);
  data->cls = (*env)->NewGlobalRef(env, class);

  ErrorMqToJavaWithCheck(MqServiceCreate(msgque, tokenC, NS(sService), data, NS(sFree)));
  (*env)->ReleaseStringUTFChars(env, token, tokenC);
error:
  return;
}

JNIEXPORT void JNICALL NS(MqS_ServiceDelete) (
  JNIEnv *  env, 
  jobject   self, 
  jstring   token
)
{
  const char *tokenC = (*env)->GetStringUTFChars(env, token, NULL);
  
  SETUP_msgque;

  ErrorMqToJavaWithCheck(MqServiceDelete(msgque, tokenC));
  (*env)->ReleaseStringUTFChars(env, token, tokenC);
error:
  return;
}
