.TH "pymsgque" n "2 Jul 2009" "Version 3.4" "pymsgque" \" -*- nroff -*-
.ad l
.nh
.SH NAME
pymsgque \- python interface to the libmsgque library
.SH "SYNOPSIS"
.PP
\fBpymsgque.MqS\fP(...)
.SH "INDEX"
.PP
.IP "\(bu" 2
\fBSETUP AND CONFIGURATION\fP
.IP "  \(bu" 4
\fBParent\fP \fBChild\fP \fBWorker\fP \fBInit\fP \fBClient\fP \fBServer\fP \fBFilter\fP
.PP

.IP "\(bu" 2
\fBCOMMAND LINE ARGUMENTS\fP
.IP "  \(bu" 4
\fBMisc\fP \fBTcp\fP \fBUds\fP \fBPipe\fP
.PP

.IP "\(bu" 2
\fBCONTEXT COMMAND\fP
.IP "  \(bu" 4
\fBMisc\fP \fBSend\fP \fBRead\fP \fBService\fP \fBConfig\fP \fBGet\fP \fBSlave\fP
.PP

.PP
.SH "DESCRIPTION"
.PP
The msgque project is an infrastructure to link software together to act like a single software. To link mean distributing work from one software to an other software an wait or not wait for an answer. The linking is done using unix or inet domain sockets and is based on packages send from one software to an other software and back. The msgque project is used to handle all the different aspects for setup and maintain the link and is responsible for:
.IP "\(bu" 2
starting and stopping the server application
.IP "\(bu" 2
starting and stopping the communication interface
.IP "\(bu" 2
sending and receiving package data
.IP "\(bu" 2
reading and writing data from or into a package
.IP "\(bu" 2
setup and maintain the event-handling for an asynchronous transfer
.IP "\(bu" 2
propagate the error messages from the server to the client
.PP
.SH "SETUP AND CONFIGURATION"
.PP
The instance setup is done using:
.PP
.IP "1." 4
constructor keywords: \fIargv, silent, string, server, filter, debug, buffersize, timeout, name, parent, FTR, EOF, config\fP
.IP "2." 4
and command-line arguments available as: \fIsys.argv\fP (s. \fBCOMMAND LINE ARGUMENTS\fP)
.PP
.SS "constructor: MqS(argv, ?argv=list,keyword=arg,...?)"
Create a \fIparent\fP message-queue \fIINSTANCE\fP. A parent instance is responsible to setup the connection and offer the communication layer between \fIclient\fP and \fIserver\fP.
.PP
The pymsgque package add the \fIMqS\fP type to python. This type was designed to act as a base class for a python class like:
.PP
\fBExample:\fP 
.PP
.nf
import sys
import pymsgque
...
class MyServer(pymsgque.MqS):
  def __init__(self, argv):
    pymsgque.MqS.__init__(self,
      argv    = argv,
      server  = True,
      name    = 'MyDefaultName',
      config  = self.config)
  def Config(self):
    ...
...
srv = MyServer(sys.argv)
srv.ProcessEvent(wait='FOREVER')
srv.Delete()

.fi
.PP
.SS "constructor: MqS(parent=INSTANCE,...)"
Create a \fIchild\fP message-queue from a \fIparent\fP pymsgque instance. A child is using the same process or thread as the parent but a different context. With a different context a child is able to act on different services and is using a different namespace on the shared server (default: create a parent and not a child)
.PP
\fBExample:\fP 
.PP
.PP
.nf
PARENT = MqS(...)
CHILD  = MqS(parent=PARENT,..)
.fi
.PP
.SS "constructor: MqS(master=INSTANCE,?master_id=0?,...)"
Create a worker as \fIslave\fP of the \fImaster\fP using the image of the \fImaster\fP self and started as an independent process or thread using \fI--fork\fP, \fI--thread\fP or \fI--spawn\fP. (s. \fBSlave methods\fP) . 
.PP
\fBParameters:\fP
.RS 4
\fImaster\fP a pymsgque instance as PARENT without a CHILD 
.br
\fIid\fP a \fIinteger\fP used as \fIunique\fP identifer for the master/slave link (default: 0)
.RE
.PP
\fBExample:\fP 
.PP
 
.PP
.nf
class Server(pymsgque.MqS):
  def __init__(self, argv, master=None, master_id=0):
    if master == None:
      pymsgque.MqS.__init__(self, 
        argv      = argv,
        server    = True, 
        name      = 'server', 
        config    = self.config)
    else:
      pymsgque.MqS.__init__(self, 
        argv      = argv,
        master    = master,
        master_id = master_id,
        config    = self.config)

.fi
.PP
 ... 
.PP
.nf
Server(argv=['worker-client', '--name', 'cl1', '@', 'WORKER', '--name', 'sv1'], master=self, master_id=id)

.fi
.PP
.SS "static: MqS.Init(list)"
Initialize the process startup argument prefix. The startup-prefix have to be the name of the executable found in the \fCPATH\fP and additional arguments like a script name or required startup options. The startup-prefix is used for two different purpose:
.IP "\(bu" 2
To start a new entity using the \fI--spawn\fP command-line option
.IP "\(bu" 2
To replace the \fISELF\fP token after the \fI@\fP command-line argument
.PP
.PP
The startup-prefix is set during library startup of \fIpymsgque\fP.
.SS "CLIENT configuration"
This section is used for the \fIclient\fP, \fIserver\fP and \fIfilter\fP setup.
.SS "MqS(buffersize=NUM,...)"
Set the send and receive buffersize to \fINUM\fP (default: 4KB)
.SS "MqS(debug=NUM,...)"
Debug a message-queue application. Valid values are 0 <= \fINUM\fP <= 9 using 0 for 'no debugging' and 9 for 'maximum debugging'. (default: 0)
.SS "MqS(timeout=NUM,...)"
At startup the \fIclient\fP is waiting for maximum \fINUM\fP seconds to connect to the server (default: 20 seconds)
.SS "MqS(name=STRING,...)"
Use \fISTRING\fP as basename of the new message-queue object. The name shows up in the debugging output and is used as prefix for the new created command. (default: the name of the executable)
.SS "MqS(silent=BOOLEAN,...)"
Write (NO) or don't write (YES) any output to stdout or stderr. (default: write to stdout or stderr)
.SS "MqS(string=BOOLEAN,...)"
Use (YES), as much as possible, strings in the data-package. Convert any native data-type, like integer or double, into a string (sending) and back to native (receiving). (default: use binary)
.SS "SERVER configuration"
This section is used for the \fIserver\fP and \fIfilter\fP setup.
.SS "MqS(server=True,...)"
Create the new context in \fIserver\fP mode (default: no, be a client)
.br
.SS "MqS(config=CALLABLE,...)"
Use the method to configure a new \fIserver\fP instance. The request for a new instance is done by the client and the server acts on this request. For every new instance the server create the internal data structure and finally call this method. The main usage of the method is to define new services and to initialize instance specific variables.
.PP
.PP
.nf
def Config(self):
  self.ServiceCreate(SERVICE_NAME, CALLABLE)
...
srv = MqS(server=True,config=Config,...)
.fi
.PP
.SS "FILTER configuration"
This section is \fIonly\fP used for the \fIfilter\fP setup.
.SS "MqS(filter=True, ... argv=?@ ...?)"
Create a \fIserver\fP in \fIfilter\fP mode. This option requires an additional server in the \fIargv\fP argument list. (s. \fBFILTER MODE\fP)
.SS "MqS(FTR=CALLABLE,...)"
A service method to act on filter data rows. Every filter input data is a list of filter data rows and every row is a list of filter data columns. Every row is send from one filter-command to the following filter-command as FTR service request.
.SS "MqS(EOF=CALLABLE,...)"
A service function to act on End-Of-Filter data. This service is called after the whole filter data was send. Sometimes the filter data can not be served as filter data rows (example: sorting of the input rows need to read all rows before the data can be send to the next filter command) and the EOF function is used to continue send filter data rows.
.SH "COMMAND LINE ARGUMENTS"
.PP
The command line arguments are added to the configuration using \fBSETUP AND CONFIGURATION\fP using the \fIargv\fP argument and overwrite the vendor default configuration values.
.SS "basic configuration"
Configuration options used to setup the basic behaviour.
.SS "--buffersize NUM"
Same as \fBMqS(buffersize=NUM,...)\fP
.SS "--debug NUM"
Same as \fBMqS(debug=NUM,...)\fP
.SS "--timeout NUM"
Same as \fBMqS(timeout=NUM,...)\fP
.SS "--name STRING"
Same as \fBMqS(name=STRING,...)\fP
.SS "--silent"
Same as \fBMqS(silent=BOOLEAN,...)\fP
.SS "--string"
Same as \fBMqS(string=BOOLEAN,...)\fP
.SS "--srvname STRING"
Use \fISTRING\fP as a client specific prefix in the server debugging output. This is used to link debugging and error messages on the server to a specific client connection. (default: empty)
.SS "--fork, --spawn"
User preferences on HOWTO start a new entity. (default: not set)
.SS "--daemon"
Close all default IO (e.g stdout, stdin, stderr) and fork the server into the background. (default: no daemon) 
.PP
\fBAttention:\fP
.RS 4
this option require the \fBfork\fP system-call and is \fBnot\fP compatible with threads.
.RE
.PP
.SS "--tcp configuration"
Use inet-domain-sockets for the client/server link:
.SS "--host NAME"
Use the hostname \fINAME\fP as the remote target of the tcp connection (default: localhost)
.SS "--port NUMBER"
Use the port \fINUMBER\fP as the remote target of the tcp connection. If the \fINUMBER\fP is a character string the port-name have to be resolved by the local port-name resolution. This options is required. (default: required but not defined)
.SS "--myhost NAME"
Use the hostname \fINAME\fP as the local target of the tcp connection and is used to select a different interface (default: localhost)
.SS "--myport NUMBER"
Use the port \fINUMBER\fP as the local target of the tcp connection. If the \fINUMBER\fP is a character string the port-name have to be resolved by the local port-name resolution (default: random)
.SS "--uds configuration"
Use \fIuds\fP (http://en.wikipedia.org/wiki/Unix_domain_socket) socket communication. The \fIuds\fP communication is usually 50% faster than a local \fItcp\fP communication.
.SS "--file NAME"
The argument \fINAME\fP have to be a valid local file-name with valid privileges on the directory. The \fIuds\fP \fIserver\fP create the file and listen to incoming connection. This option is required. (default: required but not defined)
.SS "--pipe configuration"
Use a \fIsocketpair\fP for the client/server link. This option expect an additional server after the first \fI@\fP command-line argument and is found using the \fCPATH\fP environment variable.
.SS "--socket NUMBER"
The \fI--socket\fP option is special because it is mainly used for internal purpose to submit the socket from the client to the server started as pipe by the client. The only public usage for this option is to serve as interface for an existing tool like (x)inet. The (x)inet tool listen to a tcp/ip port and start for every connection the proper entry from \fI/etc/\fP(x)inetd.conf with the file-descriptor \fI0\fP ready for read and write. This file descriptor have to be submitted to a libmsgque server.
.SH "CONTEXT COMMAND"
.PP
A context object (ctx) is an instance of MqS and is created with: \fBSETUP AND CONFIGURATION\fP and is destroyed with: \fBctx.Delete()\fP. The context is used for sending and receiving data, setup and delete services and to manage the event-queue.
.SS "Misc methods"
Various commands used for misc purpose.
.SS "ctx.ProcessEvent(?timeout=sec?, ?wait=('NO'|'ONCE'|'FOREVER')?)"
Start the Event-Loop and check for the next event. In \fIclient\fP mode this command is usually used together with \fBctx.SendEND_AND_CALLBACK(token, callback)\fP to process the results:
.PP
.PP
.nf
for i in range(1,5):
  ctx.SendSTART()
  ...
  ctx.SendEND_AND_CALLBACK('MYSV', MyCallback)
for i in range(1,5):
  ctx.ProcessEvent(100,wait='ONCE')
.fi
.PP
.PP
\fBParameters:\fP
.RS 4
\fItimeout\fP don't wait more than TIMEOUT seconds to get an answer from the server. If no answer is available an error is raised. (default: -1, use application default) 
.br
\fIwait\fP use flag value to define the action (default: NO)
.IP "\(bu" 2
\fBNO\fP, don't wait for an event do just the check
.IP "\(bu" 2
\fBONCE\fP, use \fItimeout\fP seconds to wait for exact \fIone\fP event
.IP "\(bu" 2
\fBFOREVER\fP, wait forever and only come back on \fIerror\fP or on \fIexit\fP 
.PP
.RE
.PP
.SS "ctx.Delete()"
Shutdown of the server-connection and delete the \fILibMsgque\fP object belonging to the \fIpymsgque.MqS\fP instance.
.PP
A future call to \fCctx.__init__\fP(...) will recreate the \fILibMsgque\fP object and reuse the \fIMqS\fP instance. To delete the entire \fIMqS\fP instance, including the LibMsgque object, use the python \fCdel\fP function.
.SS "ctx.ErrorC(prefix, errnum, message)"
Create a pymsgque runtime error. This method is usually used in a service to report and error back to the client.
.PP
\fBParameters:\fP
.RS 4
\fIprefix\fP a \fIstring\fP as function or location name 
.br
\fIerrnum\fP an \fIinteger\fP as error number 
.br
\fImessage\fP a \fIstring\fP as error message
.RE
.PP
.SS "Send methods"
The 'Send' methods are used to build and send a data package from the \fBclient\fP to the \fBserver\fP an back. The data is using a \fBTYPE IDENTIFIER\fP and the method's can wait or not for an answer. During waiting the the application is still able to work on MqS python events. A typical data-package is build like:
.PP
.PP
.nf
ctx.SendSTART()
ctx.SendI(12345)
ctx.SendC('my first service call')
ctx.SendEND_AND_WAIT('SRV1')
.fi
.PP
.SS "ctx.SendSTART()"
Initialize a LibMsgque package. This method is required.
.SS "ctx.SendEND(token)"
Send the package without expecting an answer. (FireAndForget)
.PP
\fBParameters:\fP
.RS 4
\fItoken\fP a 4 byte string used to identify the remote service 
.RE
.PP
\fBAttention:\fP
.RS 4
if on the server an \fBerror\fP is raised during service-processing the error is send back as \fBasynchronous event\fP.
.RE
.PP
.SS "ctx.SendEND_AND_WAIT(token, ?timeout?)"
Send the package using \fItoken\fP and wait \fItimeout\fP seconds for an answer.
.PP
\fBParameters:\fP
.RS 4
\fItoken\fP a 4 byte string used to identify the remote service 
.br
\fItimeout\fP don't wait more than TIMEOUT seconds to get an answer from the server. If no answer is available an error is raised. (default: -1, use application default)
.RE
.PP
.SS "ctx.SendEND_AND_CALLBACK(token, callback)"
Send the package and don't wait for the answer.
.PP
\fBParameters:\fP
.RS 4
\fItoken\fP a 4 byte string used to identify the remote service 
.br
\fIcallback\fP a function to process the results
.RE
.PP
.SS "ctx.SendRETURN()"
Answer an incoming service-call started with \fBctx.SendEND_AND_WAIT(token, ?timeout?)\fP or \fBctx.SendEND_AND_CALLBACK(token, callback)\fP. The return package is build-up like the previous service call package but with a \fISendRETURN\fP as last method instead.
.SS "ctx.SendFTR(?timeout?)"
Send a data package using \fBFILTER MODE\fP.
.PP
\fBParameters:\fP
.RS 4
\fItimeout\fP don't wait more than TIMEOUT seconds to get an answer from the server. If no answer is available an error is raised. (default: -1, use application default)
.RE
.PP
.SS "ctx.SendEOF(?timeout?)"
Send a EndOfFile package using \fBFILTER MODE\fP.
.PP
\fBParameters:\fP
.RS 4
\fItimeout\fP don't wait more than TIMEOUT seconds to get an answer from the server. If no answer is available an error is raised. (default: -1, use application default)
.RE
.PP
.SS "ctx.Send[T](...)"
Add an item using using \fBT\fP as \fBTYPE IDENTIFIER\fP postfix to the package. (example: \fISendY\fP)
.SS "ctx.SendL_START()"
Start to add an embedded list item to the package. This allows to submit a tree like package structure.
.br
 Example, to send a list as \fC{1, {'a', 2.2}, 'B'}\fP use:
.PP
.PP
.nf
ctx.SendSTART()
ctx.SendI(1)
ctx.SendL_START()
ctx.SendC('a')
ctx.SendD(2.2)
ctx.SendL_END()
ctx.SendC('B')
ctx.SendEND_AND_WAIT('MYTK')
.fi
.PP
.SS "ctx.SendL_END()"
End to add an embedded list item to the package.
.SS "Read methods"
Read data from an incoming LibMsgque data package:
.IP "1." 4
on a server to serve an incoming service-call from the client
.IP "2." 4
on a client to process the return-data from a previous service-call
.PP
Reading data is a passive task in opposite to sending data which is an active task. Passive mean that the read is triggered by an incoming data-package and not by the software-workflow.
.SS "ctx.Read[T]()"
Read an item using \fBT\fP as \fBTYPE IDENTIFIER\fP postfix from the package. (example: \fIReadY\fP) 
.PP
\fBReturns:\fP
.RS 4
item of type [T]
.RE
.PP
.SS "ctx.ReadL_START()"
Start to read an embedded list item from the package.
.SS "ctx.ReadL_END()"
End to read an embedded list item from the package.
.SS "ctx.ReadGetNumItems()"
\fBReturns:\fP
.RS 4
the number of items left in the \fIread\fP package.
.RE
.PP
.SS "ctx.ReadItemExists()"
Check if are still items in the read package.
.PP
\fBReturns:\fP
.RS 4
1(yes) or 0(no)
.RE
.PP
.SS "ctx.ReadUndo()"
Undo the last read. This is useful if an error was raised or the buffer item (returned from \fCReadU\fP) has not the expected type.
.SS "Service methods"
To setup and to maintain a service is the main purpose of the LibMsgque infrastructure.
.SS "ctx.ServiceCreate(token, service)"
Setup a service.
.PP
\fBParameters:\fP
.RS 4
\fItoken\fP a 4 byte string used to identify the remote service 
.br
\fIservice\fP a function to serve the service-request
.RE
.PP
.SS "ctx.ServiceDelete(token)"
Delete a service.
.PP
\fBParameters:\fP
.RS 4
\fItoken\fP a 4 byte string used to identify the remote service
.RE
.PP
.SS "'Config' methods"
The 'Config' methods are used to access final configuration data \fIafter\fP the default values \fIand\fP the command line arguments were applied.
.SS "ctx.ConfigGetIsString()"
\fBReturns:\fP
.RS 4
boolean, true if object is using the \fIstring\fP mode.
.RE
.PP
.SS "ctx.ConfigGetIsSilent()"
\fBReturns:\fP
.RS 4
boolean, true if object is using the \fIsilent\fP mode.
.RE
.PP
.SS "ctx.ConfigGetIsServer()"
\fBReturns:\fP
.RS 4
boolean, true if object is using the \fIserver\fP mode.
.RE
.PP
.SS "ctx.ConfigGetIsParent()"
\fBReturns:\fP
.RS 4
boolean, true if object is a \fIparent\fP object.
.RE
.PP
.SS "ctx.ConfigGetIsSlave()"
\fBReturns:\fP
.RS 4
boolean, true if object is a \fIslave\fP object. (s. \fBSlave methods\fP),
.RE
.PP
.SS "ctx.ConfigGetParent()"
\fBReturns:\fP
.RS 4
if \fIctx\fP is a \fIchild-context\fP than return the \fIparent-context\fP otherwise \fINULL\fP.
.RE
.PP
.SS "ctx.ConfigGetMaster()"
\fBReturns:\fP
.RS 4
if \fIctx\fP is a \fIslave-context\fP than return the \fImaster-context\fP otherwise \fINULL\fP.
.RE
.PP
.SS "ctx.ConfigGetDebug()"
\fBReturns:\fP
.RS 4
the \fB--debug NUM\fP value.
.RE
.PP
.SS "ctx.ConfigGetName()"
\fBReturns:\fP
.RS 4
the \fB--name STRING\fP value.
.RE
.PP
.SS "ctx.ConfigGetCtxId()"
\fBReturns:\fP
.RS 4
an identifier as integer and is \fIunique\fP per parent-context. The \fIparent-context\fP is allways \fC0\fP and every new \fIchild-context\fP get a new identifer by adding 1.
.RE
.PP
.SS "ctx.ConfigGetToken()"
This command is needed on the server to implement a generic service (A \fIServiceCreate\fP with different \fItoken\fP but with the same \fIservice\fP method). In the service this command is used to extract the token of the current transaction. 
.PP
\fBReturns:\fP
.RS 4
the current transaction \fItoken\fP. (s. \fBTOKEN IDENTIFIER\fP),
.RE
.PP
.SS "Get methods"
The 'Get' methods are used to access data in a LibMsgque MqBufferS object returned by \fIReadU\fP.
.SS "ctx.Get[T](buffer)"
Get an item using \fBT\fP as \fBTYPE IDENTIFIER\fP postfix from the buffer. (example: \fIGetY\fP) 
.PP
\fBParameters:\fP
.RS 4
\fIbuffer\fP a LibMsgque buffer object returned by \fCReadU\fP 
.RE
.PP
\fBReturns:\fP
.RS 4
an item of type \fI\fP[T]
.RE
.PP
.SS "ctx.GetType(buffer)"
\fBParameters:\fP
.RS 4
\fIbuffer\fP a LibMsgque buffer object returned by \fCReadU\fP 
.RE
.PP
\fBReturns:\fP
.RS 4
the \fBTYPE IDENTIFIER\fP as a string one character length.
.RE
.PP
.SS "ctx.GetL_START(buffer)"
Start to read an embedded list item from the buffer. 
.PP
\fBParameters:\fP
.RS 4
\fIbuffer\fP a LibMsgque buffer object returned by \fCReadU\fP 
.RE
.PP
.SS "ctx.GetL_END()"
End to read an embedded list item from the buffer.
.SS "Slave methods"
The master/slave link is used to create a mesh of nodes by linking different PARENT context objects together. \fBThe master control the slave.\fP
.br
 The link is used to perform the following tasks:
.IP "\(bu" 2
report error messages from the slave to the master
.IP "\(bu" 2
to create a slave-child-context if a master-child-context is created
.IP "\(bu" 2
to delete a slave-context if a master-context is deleted
.PP
In difference to the client/server link the master/slave link connect two independent msgque-context in the same process or thread (e.g. node). This leads to the restriction that only the master-msgque-context can be a server because only on server per node is possible.
.SS "ctx.SlaveCreate(id,slave)"
Create a master/slave link between the current msgque object and a client msgque object. 
.br
 
.PP
\fBParameters:\fP
.RS 4
\fIctx\fP the \fImaster\fP pymsgque object as PARENT without a CHILD 
.br
\fIid\fP a \fIinteger\fP used as \fIunique\fP identifer for the master/slave link 
.br
\fIslave\fP a pymsgque object as CLIENT-PARENT without a CHILD
.RE
.PP
.SS "ctx.SlaveDelete(id)"
By default the \fIslave-context\fP will be deleted if the \fImaster-context\fP is deleted. In addition the \fIparent-slave-context\fP can be deleted explicit. It is an error to delete a \fIchild-slave-context\fP. If \fIid\fP is invalid than nothing will happen. 
.PP
\fBParameters:\fP
.RS 4
\fIid\fP a \fIinteger\fP used as \fIunique\fP identifer for the master/slave link
.RE
.PP
.SS "ctx.SlaveGet(id)"
\fBParameters:\fP
.RS 4
\fIid\fP a \fIinteger\fP used as \fIunique\fP identifer for the master/slave link 
.RE
.PP
\fBReturns:\fP
.RS 4
the \fIslave\fP pymsgque object or \fINULL\fP if \fIid\fP is not valid or if \fIctx\fP is not a \fImaster\fP 
.RE
.PP
.SH "TOKEN IDENTIFIER"
.PP
The \fItoken\fP is a 4 byte string and identifies the service. The \fItoken\fP is used in:
.IP "\(bu" 2
\fBctx.SendEND(token)\fP
.IP "\(bu" 2
\fBctx.SendEND_AND_WAIT(token, ?timeout?)\fP
.IP "\(bu" 2
\fBctx.SendEND_AND_CALLBACK(token, callback)\fP
.IP "\(bu" 2
\fBctx.ServiceCreate(token, service)\fP
.IP "\(bu" 2
\fBctx.ServiceDelete(token)\fP
.PP
.SH "TYPE IDENTIFIER"
.PP
A LibMsgque data package is type safe, this mean that every item has a type prefix and every \fCRead\fP[TYPE] have to match the previous \fCSend\fP[TYPE] with the same \fITYPE\fP. One exception, the cast from and to string (TYPE=C) is allowed. If the \fCREAD\fP[TYPE] does \fBnot\fP match the package type identifier an error is raised. The following type identifier's are allowed:
.IP "\(bu" 2
\fCY\fP : 1 byte signed character
.IP "\(bu" 2
\fCO\fP : 1 byte boolean character using 0 (no) and 1 (yes)
.IP "\(bu" 2
\fCS\fP : 2 byte signed short
.IP "\(bu" 2
\fCI\fP : 4 byte signed integer
.IP "\(bu" 2
\fCW\fP : 8 byte signed long long integer
.IP "\(bu" 2
\fCF\fP : 4 byte float
.IP "\(bu" 2
\fCD\fP : 8 byte double
.IP "\(bu" 2
\fCB\fP : unsigned char array used for binary data
.IP "\(bu" 2
\fCC\fP : string data using a \fC\\0\fP at the end
.IP "\(bu" 2
\fCL\fP : list type able to hold a list of all items from above
.IP "\(bu" 2
\fCU\fP : typeless buffer able to hold one item of all types
.PP
.SH "FILTER MODE"
.PP
The filter mode is related to a special usage of the pymsgque software called a command pipeline. A command pipeline is well known from the \fBunix\fP shell to link different commands together to act like a single application: 
.PP
.nf

    command1 | command2 | command3

.fi
.PP
 A command pipeline with pymsgque queues is using the special token \fB@\fP to link the commands together: 
.PP
.nf

    msgcmd1 @ msgcmd2 @ msgcmd3

.fi
.PP
 To create a pymsgque filter use:
.IP "\(bu" 2
\fC \fBMqS(filter=True, ... argv=?@ ...?)\fP \fP
.PP
and to send data to the next filter command use:
.IP "\(bu" 2
\fC \fBctx.SendFTR(?timeout?)\fP \fP
.IP "\(bu" 2
\fC \fBctx.SendEOF(?timeout?)\fP \fP
.PP
.SH "PORTABILITY ISSUES"
.PP
The library was tested on Linux, FreeBSD and WindowsXP using a x86_64 or ppc processor
.SH "EXAMPLES"
.PP
.SS "1. create a server to multiplicate 2 double values and return the result"
.PP
.nf
import sys
import pymsgque
def MMUL(ctx):
  ctx.SendSTART()
  ctx.SendD(ctx.ReadD() * ctx.ReadD())
  ctx.SendRETURN()
def ServerConfig(ctx):
  ctx.ServiceCreate("MMUL",MMUL)
srv = pymsgque.MqS(sys.argv, server=True, name="MyMulServer", config=ServerConfig)
srv.ProcessEvent(wait="FOREVER")
srv.Delete()
.fi
.PP
 Start \fImulserver.py\fP using \fITCP\fP port \fI7777\fP and create a \fIfork\fP for every incoming connection 
.PP
.nf

> python mulserver.py --tcp --port 7777 --fork

.fi
.PP
.SS "2. in a client call the service from 1."
.PP
.nf
import sys
import pymsgque
ctx = pymsgque.MqS(sys.argv, name="MyMul")
ctx.SendSTART()
ctx.SendD(3.67)
ctx.SendD(22.3)
ctx.SendEND_AND_WAIT("MMUL")
print(ctx.ReadD())
ctx.Delete()
.fi
.PP
 Use \fImulclient.py\fP to connect to \fImulserver.py\fP using \fITCP\fP port \fI7777\fP: 
.PP
.nf

> python mulclient.py --tcp --port 7777

.fi
.PP
.SS "3. create a filter to wrap every column in a '<>' pair"
.PP
.nf
import sys
import pymsgque
def FTRcmd(ctx):
  ctx.SendSTART()
  while ctx.ReadItemExists():
    ctx.SendC("<" + ctx.ReadC() + ">")
  ctx.SendFTR()
srv = pymsgque.MqS(sys.argv, server=True, filter=True, name="ManFilter", FTR=FTRcmd)
srv.ProcessEvent(wait="FOREVER")
srv.Delete()
.fi
.PP
 Use \fImanfilter.py\fP in a LibMsgque command pipeline: 
.PP
.nf

> echo -e "1:2:3\na:b:c" | atool split -d : @ python manfilter.py @ atool join -d :

.fi
.PP
.SH "SEE ALSO"
.PP
libmsgque, python
.SH "KEYWORDS"
.PP
python, unix, socket, message, msgque 
.SH "Author"
.PP 
Generated automatically by Doxygen for pymsgque from the source code.
