/**
 *  \file       pymsgque/msgque_python.h
 *  \brief      \$Id: msgque_python.h 1203 2009-05-20 08:58:27Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1203 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef MSGQUE_PYTHON_H
#define MSGQUE_PYTHON_H

#include <Python.h>

// public visible names with package prefix
#define NS(n)	Python_pymsgque_ ## n

// HAVE_DECLSPEC_DLL is defined in the "python" header files
// DLL_EXPORTS is needed by "msgque.h"
#if defined(HAVE_DECLSPEC_DLL) && !defined(DLL_EXPORTS)
# define DLL_EXPORTS
#endif

#include "msgque.h"

#define MQ_IS_POSIX
#include "../src/debug.h"

/*****************************************************************************/
/*                                                                           */
/*                              context_basic                                */
/*                                                                           */
/*****************************************************************************/

typedef struct {
    PyObject_HEAD
    /* Type-specific fields go here. */
    struct MqS * msgque;
} PyMqs_Obj;

/// \brief object to link the \libmsgque object with application specific data
struct PythonContextS {
  struct MqS * msgque;          ///< link to the \libmsgque object
  PyObject *class;		///< current PYTHON class
  PyObject *self;		///< current PYTHON object
  PyObject *config;		///< config method/function
  MQ_BOL onDelete;              ///< Is this context allready in deletion?
  MQ_INT callLevel;             ///< code evaluated by "pCallVoidMethod" ?
};

#define SETUP_msgque \
  struct MqS * msgque = ((PyMqs_Obj*)self)->msgque; \
  if (msgque == NULL) { \
    PyErr_SetString(PyExc_ValueError, "pymsgque object already deleted");\
    goto error; \
  }

#define SETUP_buffer \
  if (!PyCObject_Check(buf)) { \
    PyErr_SetString(PyExc_TypeError, "invalid buffer object, expect 'ReadU' first");\
    goto error; \
  } \
  buffer = PyCObject_AsVoidPtr(buf);

#define CONTEXT ((struct PythonContextS *) msgque->config.myCtx)

#define SETUP_context \
  context = CONTEXT;

#define ErrorMqToPython() NS(pErrorFromMq)(__func__, __FILE__, __LINE__, msgque)
#define ErrorMqToPythonWithCheck(PROC) \
  if (unlikely((PROC) == MQ_ERROR)) { \
    ErrorMqToPython(); \
    goto error; \
  }

#undef printO
#define printO(o)   fprintf(stdout,"%s -> ",#o);PyObject_Print((PyObject*)o,stdout,0);fprintf(stdout,"\n");

// from misc_python.c

enum MqErrorE NS(pCallVoidMethod) (struct MqS*, PyObject*, PyObject*);
void NS(pErrorFromMq) (const char *, const char *, const int, struct MqS * const);

// from context_python.c
enum MqErrorE NS(ChildCreate) (struct MqConfigS*, struct MqBufferLS*, struct MqBufferLS*, struct MqS**);
enum MqErrorE NS(ParentCreate) (struct MqConfigS*, struct MqBufferLS*, struct MqBufferLS*, struct MqS**);
void NS(ChildDelete) (struct MqS**);
void NS(ParentDelete) (struct MqS**);
void NS(ParentExit) (int, int);

#endif /* MSGQUE_PYTHON_H */








