/**
 *  \file       pymsgque/read_python.c
 *  \brief      \$Id: read_python.c 1182 2009-05-15 13:59:20Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1182 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "msgque_python.h"

///
///  READ
///

PyObject* NS(ReadL_START) (
  PyObject*	self
)
{
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadL_START(msgque->read, NULL));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(ReadL_END) (
  PyObject*	self 
)
{
  SETUP_msgque;
  MqReadL_END(msgque->read);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(ReadY) (
  PyObject*	self 
)
{
  MQ_BYT i;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadY(msgque->read, &i));
  return PyLong_FromLong(i);
error:
  return NULL;
}

PyObject* NS(ReadO) (
  PyObject*	self 
)
{
  MQ_BOL i;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadO(msgque->read, &i));
  if (i == MQ_YES) {
    Py_RETURN_TRUE;
  } else {
    Py_RETURN_FALSE;
  }
error:
  return NULL;
}

PyObject* NS(ReadS) (
  PyObject*	self 
)
{
  MQ_SRT i;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadS(msgque->read, &i));
  return PyLong_FromLong(i);
error:
  return NULL;
}

PyObject* NS(ReadI) (
  PyObject*	self 
)
{
  MQ_INT i;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadI(msgque->read, &i));
  return PyLong_FromLong(i);
error:
  return NULL;
}

PyObject* NS(ReadF) (
  PyObject*	self 
)
{
  MQ_FLT f;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadF(msgque->read, &f));
  return PyFloat_FromDouble((double) f);
error:
  return NULL;
}

PyObject* NS(ReadW) (
  PyObject*	self 
)
{
  MQ_WID w;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadW(msgque->read, &w));
  return PyLong_FromLongLong((PY_LONG_LONG)w);
error:
  return NULL;
}

PyObject* NS(ReadD) (
  PyObject*	self 
)
{
  MQ_DBL d;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadD(msgque->read, &d));
  return PyFloat_FromDouble((double) d);
error:
  return NULL;
}

PyObject* NS(ReadC) (
  PyObject*	self 
)
{
  MQ_STR s;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadC(msgque->read, &s));
  return PyUnicode_FromString(s);
error:
  return NULL;
}

PyObject* NS(ReadB) (
  PyObject*	self
)
{
  MQ_BIN b;
  MQ_SIZE len;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadB(msgque->read, &b, &len));
  return PyByteArray_FromStringAndSize((const char*)b, len);
error:
  return NULL;
}

PyObject* NS(ReadU) (
  PyObject*	self
)
{
  MQ_BUF buffer;
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqReadU(msgque->read, &buffer));
  return PyCObject_FromVoidPtr(buffer, NULL);
error:
  return NULL;
}

PyObject* NS(ReadGetNumItems) (
  PyObject*	self
)
{
  SETUP_msgque;
  return PyLong_FromLong(MqReadGetNumItems(msgque->read));
error:
  return NULL;
}

PyObject* NS(ReadItemExists) (
  PyObject*	self
)
{
  SETUP_msgque;
  if (MqReadItemExists(msgque->read)) {
    Py_RETURN_TRUE;
  } else {
    Py_RETURN_FALSE;
  }
error:
  return NULL;
}

PyObject* NS(ReadUndo) (
  PyObject*	self
)
{
  SETUP_msgque;
  MqReadUndo(msgque->read);
  Py_RETURN_NONE;
error:
  return NULL;
}
