/**
 *  \file       pymsgque/send_python.c
 *  \brief      \$Id: send_python.c 1182 2009-05-15 13:59:20Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1182 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "msgque_python.h"

///
///   SEND
///

PyObject* NS(SendSTART) (
  PyObject	*self 
)
{
  SETUP_msgque;
  MqSendSTART(msgque->send);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendEND) (
  PyObject	*self,
  PyObject	*args
)
{
  const char *token;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "s:SendEND", &token)) {
    return NULL;
  }
  ErrorMqToPythonWithCheck(MqSendEND(msgque->send,token));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendEND_AND_WAIT) (
  PyObject	*self,
  PyObject	*args
)
{
  const char *token;
  int timeout = -1;
  SETUP_msgque;

  if (!PyArg_ParseTuple(args, "s|i:SendEND_AND_WAIT", &token, &timeout)) {
    return NULL;
  }

  ErrorMqToPythonWithCheck(MqSendEND_AND_WAIT(msgque->send,token,timeout));

  Py_RETURN_NONE;
error:
  return NULL;
}

static enum MqErrorE NS(sCallback) (
  struct MqS * msgque,
  MQ_PTR data
)
{
  if (NS(pCallVoidMethod)(msgque, (PyObject*) data, CONTEXT->self) == MQ_OK) 
    return MQ_OK;

  return MqErrorStack(msgque->error);
}

static void NS(sFree) (
  struct MqS const * const msgque,
  MQ_PTR data
)
{
  PyObject *callback = (PyObject*)data;
  Py_XDECREF(callback);
}

PyObject* NS(SendEND_AND_CALLBACK) (
  PyObject	*self,
  PyObject	*args
)
{
  const char * token;
  PyObject *callback;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "sO:SendEND_AND_CALLBACK", &token, &callback)) {
    return NULL;
  }
  Py_INCREF(callback);
  ErrorMqToPythonWithCheck(MqSendEND_AND_CALLBACK(msgque->send, token, NS(sCallback), callback, NS(sFree)));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendRETURN) (
  PyObject	*self
)
{
  SETUP_msgque;
  ErrorMqToPythonWithCheck(MqSendRETURN(msgque->send))
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendFTR) (
  PyObject	*self,
  PyObject	*args
)
{
  int timeout = -1;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "|i:SendFTR", &timeout)) {
    return NULL;
  }
  ErrorMqToPythonWithCheck(MqSendFTR(msgque->send, timeout));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendEOF) (
  PyObject	*self,
  PyObject	*args
)
{
  int timeout = -1;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "|i:SendEOF", &timeout)) {
    return NULL;
  }
  ErrorMqToPythonWithCheck(MqSendEOF(msgque->send, timeout));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendL_START) (
  PyObject	*self ,
  PyObject	*arg
)
{
  SETUP_msgque;
  MqSendL_START(msgque->send);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendL_END) (
  PyObject	*self,
  PyObject	*arg
)
{
  SETUP_msgque;
  MqSendL_END(msgque->send);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendY) (
  PyObject	*self,
  PyObject	*args
)
{
  short int y;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "h:SendY", &y)) {
    return NULL;
  }
  MqSendY(msgque->send, (MQ_BYT) y);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendO) (
  PyObject	*self,
  PyObject	*args
)
{
  PyObject *O;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "O!:SendO", &PyBool_Type, &O)) {
    return NULL;
  }
  MqSendO(msgque->send, (O == Py_True ? MQ_YES : MQ_NO));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendS) (
  PyObject	*self,
  PyObject	*args
)
{
  short int s;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "h:SendS", &s)) {
    return NULL;
  }
  MqSendS(msgque->send,s);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendI) (
  PyObject	*self,
  PyObject	*args
)
{
  int i;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "i:SendI", &i)) {
    return NULL;
  }
  MqSendI(msgque->send,i);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendF) (
  PyObject	*self,
  PyObject	*args
)
{
  float f;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "f:SendF", &f)) {
    return NULL;
  }
  MqSendF(msgque->send,f);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendW) (
  PyObject	*self,
  PyObject	*args
)
{
  PY_LONG_LONG w;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "L:SendW", &w)) {
    return NULL;
  }
  MqSendW(msgque->send,w);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendD) (
  PyObject	*self,
  PyObject	*args
)
{
  double d;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "d:SendD", &d)) {
    return NULL;
  }
  MqSendD(msgque->send,d);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendC) (
  PyObject	*self,
  PyObject	*args
)
{
  const char *c;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "s:SendC", &c)) {
    return NULL;
  }
  MqSendC(msgque->send,c);
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendB) (
  PyObject	*self,
  PyObject	*args
)
{
  PyObject *o;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "O!:SendB", &PyByteArray_Type, &o)) {
    return NULL;
  }
  MqSendB(msgque->send,(MQ_BIN)PyByteArray_AS_STRING(o),PyByteArray_GET_SIZE(o));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(SendU) (
  PyObject	*self,
  PyObject	*buf
)
{
  MQ_BUF buffer;
  SETUP_msgque;
  SETUP_buffer;
  MqSendU(msgque->send,buffer);
  Py_RETURN_NONE;
error:
  return NULL;
}
