/**
 *  \file       pymsgque/service_python.c
 *  \brief      \$Id: service_python.c 1182 2009-05-15 13:59:20Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1182 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "msgque_python.h"

static enum MqErrorE NS(sService) (
  struct MqS * msgque,
  MQ_PTR dataP
)
{
  PyObject *callable = (PyObject*) dataP;

  // call the service
  switch (NS(pCallVoidMethod)(msgque, callable, (PyObject*)CONTEXT->self)) {
    case MQ_OK:		return MQ_OK;
    case MQ_ERROR:	return MQ_ERROR;
    case MQ_EXIT:	return MQ_EXIT;
    case MQ_CONTINUE:	break;
  }

  // we have an MQ error
  // check to be in a transaction or not
  if (MqIsTransaction(msgque) == MQ_NO) {
    // no transaction
    // return server error from a MqSendEND call asyncrone
    return MqSendERROR (msgque->send);
  } else {
    // in a transaction
    // return a "server" error from a MqSendEND_AND_WAIT or MqSendEND_AND_CALLBACK call
    MqSendSTART(msgque->send);
    return MqSendRETURN (msgque->send);
  }
}

static void NS(sServiceFree) (
  struct MqS const * const msgque,
  MQ_PTR dataP
)
{
  PyObject* callable = (PyObject*) dataP;
  Py_XDECREF(callable);
}

PyObject* NS(ServiceCreate) (
  PyObject  *self, 
  PyObject  *args
)
{
  const char *token;
  PyObject *callable;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "sO:ServiceCreate", &token, &callable)) {
    return NULL;
  }

  // verify the callable object
  if (!PyCallable_Check(callable)) {
      PyErr_SetString(PyExc_TypeError, "second parameter must be callable");
      return NULL;
  }
  Py_INCREF(callable);

  // create the service
  ErrorMqToPythonWithCheck(MqServiceCreate(msgque, token, NS(sService), (void*) callable, NS(sServiceFree)));
  Py_RETURN_NONE;
error:
  return NULL;
}

PyObject* NS(ServiceDelete) (
  PyObject  *self, 
  PyObject  *args
)
{
  const char *token;
  SETUP_msgque;
  if (!PyArg_ParseTuple(args, "s:ServiceDelete", &token)) {
    return NULL;
  }
  ErrorMqToPythonWithCheck(MqServiceDelete(msgque, token));
  Py_RETURN_NONE;
error:
  return NULL;
}
