/**
 *  \file       src/buffer.h
 *  \brief      \$Id: buffer.h 1033 2009-04-10 10:36:50Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1033 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef BUFFER_H
#   define BUFFER_H


//MQ_INLINE MQ_INT pByte2INT (MQ_BINB const * const bin);
MQ_INLINE MQ_INT pByte2INT (MQ_STRB const * const bin);

/// \brief create a reference to an existing \e MqBufferS
struct MqBufferS * pBufferCreateRef (
  struct MqBufferS const * const buf
);

/// \brief expand \e MqBufferS to a new \a newSize
/// \header
/// \buf
/// \param newSize the new requested size of the \e buf object
/// \attention only use #pBufferNewSize or #pBufferAddSize
///
/// the header is only used if debugging is on and an message is written
/// to stderr identified by \e header
void _pBufferNewSize (
  register struct MqBufferS * const buf,
  MQ_STRB const * const header,
  MQ_SIZE const newSize
);

/// \brief the prototype for a pBufferGetA? function
typedef enum MqErrorE (
  *pBufferGetAF
) (
  struct MqErrorS * const,
  struct MqBufferS * const buf,
  union MqBufferAtomU * const ato,
  enum MqTypeE const type
);

/// \brief generic function to read a #MqBufferAtomU union value from an #MQ_BUF object
/// \error
/// \buf
/// \atom_out
/// \atom_type
/// \retMqErrorE
enum MqErrorE
pBufferGetA1 (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  union MqBufferAtomU * const ato,
  enum MqTypeE const type
);

/// \copydoc pBufferGetA1
enum MqErrorE
pBufferGetA2(
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  union MqBufferAtomU * const ato,
  enum MqTypeE const type
);

/// \copydoc pBufferGetA1
enum MqErrorE
pBufferGetA4(
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  union MqBufferAtomU * const ato,
  enum MqTypeE const type
);

/// \copydoc pBufferGetA1
enum MqErrorE
pBufferGetA8(
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  union MqBufferAtomU * const ato,
  enum MqTypeE const type
);

/// \brief set \e MqBufferS to \a newSize
/// \buf
/// \param newSize the new requested size of the \e buf object
#define pBufferNewSize(buf, newSize) \
	if (unlikely(buf->alloc == MQ_STATIC) || unlikely(newSize > buf->size)) \
	    _pBufferNewSize(buf, __func__, newSize);

/// \brief set \b dynamic \e MqBufferS to \a newSize
/// \buf
/// \param newSize the new requested size of the \e buf object
#define pBufferNewSizeDyn(buf, newSize) \
	if (unlikely(newSize > buf->size)) \
	    _pBufferNewSize(buf, __func__, newSize);

/// \brief add \a addSize to a \e MqBufferS
/// \buf
/// \param addSize add size to the current size
#define pBufferAddSize(buf, addSize) \
	if (unlikely(buf->alloc == MQ_STATIC) || unlikely((buf->cursize + addSize) > buf->size)) \
	    _pBufferNewSize(buf, __func__, (buf->cursize + addSize));

/// \brief add \a addSize to a \b dynamic \e MqBufferS
/// \buf
/// \param addSize add size to the current size
#define pBufferAddSizeDyn(buf, addSize) \
	if (unlikely((buf->cursize + addSize) > buf->size)) \
	    _pBufferNewSize(buf, __func__, (buf->cursize + addSize));


#endif // BUFFER_H




