/**
 *  \file       src/generic_io.h
 *  \brief      \$Id: generic_io.h 1176 2009-05-14 13:57:36Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1176 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef GENERIC_IO_H
#define GENERIC_IO_H

#define GENERIC_SAVE_ERROR(generiC) ((generiC)&&(generiC)->error?(generiC)->error:MQ_ERROR_IGNORE)

// this is needed for "sockaddr" and "socklen_t"
#include "sys_com.h"

/*****************************************************************************/
/*                                                                           */
/*                                data                                       */
/*                                                                           */
/*****************************************************************************/

/// \brief everything \e Generic needs for local storage
struct GenericS {
  struct MqIoS * io;		///< link to 'io' object
  struct MqErrorS * error;      ///< my MqErrorS
  MQ_SOCK sock;                 ///< the socket
  char stringbuffer[20];	///< context temporar memory
};

/*****************************************************************************/
/*                                                                           */
/*                                set/get                                    */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/*                             generic_io				     */
/*                                                                           */
/*****************************************************************************/

enum MqErrorE GenericCreate (
  struct MqIoS * const io,
  struct MqBufferLS * const argv,
  struct GenericS ** const out
);

void GenericDelete (
  struct GenericS ** const genericP
) __attribute__((nonnull));

enum MqErrorE GenericCreateSocket (
  struct GenericS * const generiC,
  int const domain,
  int const type,
  int protocol
);

enum MqErrorE GenericServer (
  struct GenericS * const generiC,	///< the current generic handle
  struct sockaddr * const sockaddr,	///< the address package
  socklen_t const sockaddrle		///< the length of the address
);

enum MqErrorE GenericConnect (
  struct GenericS * const generiC,
  struct sockaddr * const sockaddr,
  socklen_t const sockaddrlen,
  MQ_TIME_T timeout
);

enum MqErrorE GenericBind (
  struct GenericS const * const generiC,    ///< the current generic handle
  struct sockaddr *sockaddr,		    ///< the address package
  socklen_t sockaddrlen 		    ///< the length of the address
);

enum MqErrorE GenericGetSockName (
  struct GenericS const * const generiC,    ///< the current generic handle
  struct sockaddr *sockaddr,		    ///< the address package
  socklen_t *sockaddrlen 		    ///< the length of the address
);

enum MqErrorE GenericMqGetPeerName (
  struct GenericS const * const generiC,    ///< the current generic handle
  struct sockaddr *sockaddr,		    ///< the address package
  socklen_t *sockaddrlen		    ///< the length of the address
);

#endif /* GENERIC_IO_H */
