/**
 *  \file       src/log.c
 *  \brief      \$Id: log.c 1186 2009-05-19 06:41:03Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1186 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "main.h"
#include "log.h"
#include "buffer.h"
#include "read.h"
#include "error.h"
#include "sys.h"
#include "token.h"

#if defined(_MSC_VER)
// need for "timeval"
#  include <WinSock2.h>
// need for "getpid"
#  include <process.h>
#endif

#define SEEK(ptr,pos) ptr->cur.B = (ptr->data + pos); *(ptr->cur.C-1) = BUFFER_CHAR;

#define MSGQUE msgque

/*****************************************************************************/
/*                                                                           */
/*                               log_basic                                   */
/*                                                                           */
/*****************************************************************************/

void
pLogHEX (
  struct MqS const * const msgque,
  MQ_STRB const * const func,
  MQ_BINB const * data,
  MQ_SIZE const len
)
{  
# define COL 8
  MQ_BINB const * const dataE = (data + len);
  MQ_BINB const * dataL;
  MQ_INT pos=0;
  MQ_BINB as[COL+1], *asP;

  MqDLogV(msgque,func,0,"START hex log >>>>>>>>>>>>>\n");
  for (;data<dataE;) {
    // init "as"
    dataL = as + COL;
    for (asP=as;asP<dataL;asP++) *asP='.';
    *asP = '\0';
    // print data
    dataL = data + COL;
    if (dataL > dataE) dataL = dataE;
    for (asP=as;data<dataL;data++,pos++,asP++) {
      fprintf(stderr,"[%3d:%2x] ", pos, *data);
      *asP = (isprint(*data) ? *data : '.');
    }
    fprintf(stderr,"= [%s]\n", as);
  }
  MqDLogV(msgque,func,0,"END hex log >>>>>>>>>>>>>\n");
# undef COL
}

#define MQ_USE_DATE_TIME_LOGGING

#if defined(MQ_USE_DATE_TIME_LOGGING)
static const char *
sLogTime (
  char * TIME
)
{
  struct tm *t;
  struct mq_timeval tv;
  time_t time;

  SysGetTimeOfDay (NULL, &tv, NULL);
  time = tv.tv_sec;
  t = localtime (&time);
  sprintf (TIME, "[%4d-%02d-%02d:%02d-%02d-%02d]", t->tm_year + 1900,
           t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec);
  TIME[21] = '\0';

  return TIME;
}
#else
#   define sLogTime(dummy) ""
#endif

void
MqLog (
  FILE * channel,
  MQ_STRB const * const fmt,
  ...
)
{
  va_list ap;
  if (channel == NULL) return;
  va_start (ap, fmt);
  vfprintf (channel, fmt, ap);
  fflush (channel);
  va_end (ap);
}

static void
sLogVL (
  struct MqS const * const msgque,
  MQ_STRB const * const proc,
  MQ_INT level,
  MQ_STRB const * const fmt,
  va_list ap,
  FILE *channel
) {
  char time_buf[50];

	if (channel == NULL) return;
  if (msgque) {
    MQ_STR name = msgque->config.name;
    if (msgque->config.isSilent) return;
    if (MQ_IS_SERVER (msgque))
      fprintf (channel, "%c> (%s:%i) ", (MQ_IS_CHILD (msgque) ? 's' : 'S'), name ,mq_getpid());
    else
      fprintf (channel, "%c> (%s:%i) ", (MQ_IS_CHILD (msgque) ? 'c' : 'C'), name ,mq_getpid());
    if (proc) fprintf (channel, "%s [%i-%i-%p-%s]: ",
             sLogTime (time_buf), level, msgque->ctxId, msgque, proc);
  } else {
    if (proc) {
      fprintf (channel, "X> %s [%i-%i-%p-%s]: ",
		 sLogTime (time_buf), level, 0, NULL, proc);
    }
  }

  vfprintf (channel, fmt, ap);
  fflush (channel);
}

void
MqDLogV (
  struct MqS const * const msgque, 
  MQ_STRB const * const proc,
  MQ_INT level,
  MQ_STRB const * const fmt,
  ...
)
{
  va_list ap;
  va_start (ap, fmt);
  sLogVL (msgque, proc, level, fmt, ap, stderr);
  va_end (ap);
}

void
MqDLogEVL (
  struct MqS const * const msgque,
  MQ_STRB const * const proc,
  MQ_STRB const * const fmt,
  va_list ap
)
{
  sLogVL (msgque, proc, 0, fmt, ap, stderr);
}

void
MqDLogEV (
  struct MqS const * const msgque,
  MQ_STRB const * const proc,
  MQ_STRB const * const fmt,
  ...
)
{
  va_list ap;
  va_start (ap, fmt);
  MqDLogEVL (msgque, proc, fmt, ap);
  va_end (ap);
}

// ##########################################################################

static void
sLogDynItem (
  struct MqS const * const msgque,
  struct MqBufferS * const dyn,
  const char *prefix,
  const MQ_INT level,
  struct MqBufferS * space
);

static void
sLogDynList (
  struct MqS const * const msgque,
  struct MqBufferS * const dyn,
  const char *prefix,
  const MQ_INT level,
  struct MqBufferS * space
) {
  // init
  MqBufferPush (space, "   ");

  // read numItems
  dyn->type = MQ_STRING_TYPE(msgque->config.isString);
  dyn->cur.B = dyn->data;
  dyn->numItems = (dyn->type==MQ_BINT ? MqBufU2INT(dyn->cur) : str2int(dyn->cur.C,NULL,16));
  dyn->cur.B += (HDR_INT_LEN + 1);

  // start-msg
  if (likely (!msgque->config.isSilent))
    MqLog (stderr, "numItems<" MQ_FORMAT_Z ">\n", dyn->numItems);

  // print item
  sLogDynItem (msgque, dyn, prefix, level, space);

  // cleanup readRef
  MqBufferPop (space, "   ");

  return;
}

static void
sLogRET (
  struct MqS const * const msgque,
  struct MqBufferS * const dyn,
  const char *prefix,
  const MQ_INT level,
  struct MqBufferS * space
)
{
  MQ_INT retNum;

	// set binary or string
  dyn->type = MQ_STRING_TYPE(msgque->config.isString);

  // read numItems
  SEEK (dyn, RET_Num_S);
  retNum = (dyn->type==MQ_BINT ? MqBufU2INT(dyn->cur) : str2int(dyn->cur.C,NULL,16));
  SEEK (dyn, RET_NumItems_S);
  dyn->numItems = (dyn->type==MQ_BINT ? MqBufU2INT(dyn->cur) : str2int(dyn->cur.C,NULL,16));
  SEEK (dyn, RET_SIZE + 1);

  // Message
  if (likely (!msgque->config.isSilent))
      MqLog (stderr, "code<%c> retNum<%i> numItems<" MQ_FORMAT_Z ">\n", 
	  pReadGetReturnCode(msgque->read), retNum, dyn->numItems);

  // print item's
  MqBufferPush (space, "   ");
  sLogDynItem (msgque, dyn, prefix, level, space);
  MqBufferPop (space, "   ");

  return;
}

static void
sLogDynItem (
  struct MqS const * const msgque,
  struct MqBufferS * const dyn,
  const char *prefix,
  const MQ_INT level,
  struct MqBufferS * space
)
{
  struct MqBufferS * hd = pBufferCreateRef (dyn);
  register enum MqTypeE type;

  while (dyn->numItems) {
    MqErrorCheck (pReadWord (msgque->error, dyn, hd));
    type = hd->type;
    MqDLogV (msgque, prefix, level, "%s%7i : %s : ", space->data, 
	hd->cursize, MqLogTypeName(hd->type));
    if (msgque->config.isSilent)
      continue;
    switch (type) {
      case MQ_STRT:
        MqLog (stderr, "%s\n", hd->data);
        break;
      case MQ_BYTT:
        MqLog (stderr, MQ_FORMAT_Y "\n", MqBufU2BYT(hd->cur));
        break;
      case MQ_BOLT:
        MqLog (stderr, MQ_FORMAT_O "\n", MqBufU2BOL(hd->cur));
        break;
      case MQ_SRTT:
        MqLog (stderr, MQ_FORMAT_S "\n", MqBufU2SRT(hd->cur));
        break;
      case MQ_INTT:
        MqLog (stderr, MQ_FORMAT_I "\n", MqBufU2INT(hd->cur));
        break;
      case MQ_FLTT:
        MqLog (stderr, MQ_FORMAT_F "\n", MqBufU2FLT(hd->cur));
        break;
      case MQ_WIDT:
        MqLog (stderr, MQ_FORMAT_W "\n", MqBufU2WID(hd->cur));
        break;
      case MQ_DBLT:
        MqLog (stderr, MQ_FORMAT_D "\n", MqBufU2DBL(hd->cur));
        break;
      case MQ_LSTT:
        MqLog (stderr, ">>>> ");
        sLogDynList (msgque, hd, prefix, level, space);
        break;
      case MQ_RETT:
        MqLog (stderr, ">>>> ");
        sLogRET (msgque, hd, prefix, level, space);
        break;
      case MQ_BINT: 
        MqLog (stderr, "?binary?\n");
        break;
    }
  }

  MqBufferDelete (&hd);

  return;

error:
  MqPanicC (msgque->error, __func__, -1, "logging not possible");
}

// ##########################################################################

void
pLogHDR (
  struct MqS * const msgque,
  const char *prefix,
  const MQ_INT level,
  struct MqBufferS * const buf
)
{
  struct MqBufferS * hd = MqBufferDup (buf);
  register MQ_STR t;
  register MQ_STR e;

  if (likely(msgque->config.isString == MQ_NO)) {
    SEEK (hd, HDR_CtxId_S);
    sprintf (hd->cur.C, MQ_FORMAT_XI (HDR_INT_LEN), MqBufU2INT(hd->cur));
    SEEK (hd, HDR_BdySize_S);
    sprintf (hd->cur.C, MQ_FORMAT_XI (HDR_INT_LEN), MqBufU2INT(hd->cur));
    SEEK (hd, HDR_Token_S);
  }

  SEEK (hd, HDR_Trans_S);
  mq_snprintf (hd->cur.C, 4, "%d", hd->cur.A->I);
  hd->data[HDR_Code_S-1] = BUFFER_CHAR;
  hd->data[HDR_SIZE] = '\0';

  // fix \0
  t = (MQ_STR) hd->data;
  e = (MQ_STR) (hd->data + buf->cursize);
  while (t < e) {
    if (!isprint (*t))
      *t = '?';
    t++;
  }

  MqDLogV (msgque, prefix, level, "HEADER <%s> ...\n", hd->data);

  MqBufferDelete (&hd);
}

/*****************************************************************************/
/*                                                                           */
/*                               log_body                                    */
/*                                                                           */
/*****************************************************************************/

void
pLogBDY (
  struct MqS * const msgque,
  const char *prefix,
  const MQ_INT level,
  struct MqBufferS * const buf
)
{
  struct MqBufferS * hd;
  struct MqBufferS * space;

	// no SYSTEM RETR body
  if (!strcmp (pTokenGetCurrent (msgque->srvT), "_SRT"))
    return;

  // init
  hd = MqBufferDup (buf);
  space = MqBufferCreate (100);
  hd->cur.B = hd->data + BDY_SIZE + 1;
  hd->type = MQ_STRING_TYPE(msgque->config.isString);

  // start-msg
  MqDLogV (msgque, prefix, level, ">>>>> BDY size<" MQ_FORMAT_Z "> numItems<" MQ_FORMAT_Z ">\n",
           hd->cursize - BDY_SIZE - 1, hd->numItems);

  // items
  sLogDynItem (msgque, hd, prefix, level, space);

  // end-msg
  MqDLogV (msgque, prefix, level, "<<<<< BDY\n");

  // cleanup readRef
  MqBufferDelete (&hd);
  MqBufferDelete (&space);

  return;
}

/*****************************************************************************/
/*                                                                           */
/*                               log_advanced                                */
/*                                                                           */
/*****************************************************************************/

#if defined _DEBUG
void
pLogMqBuffer (
  struct MqS * const msgque,
  const char *header,
  MQ_INT level,
  struct MqBufferS * buf
)
{
  MqDLogV (msgque, header, level,
      "pointer<%p> type<%c>, alloc<%s>, size<" MQ_FORMAT_Z ">, cursize<" MQ_FORMAT_Z ">, data<%s>\n",
      buf, buf->type,
      (buf->alloc == MQ_DYNAMIC ? "MQ_DYNAMIC" : "STATIC"), buf->size,
      buf->cursize, buf->data);
}
#endif

const char *
MqLogStatus (
  enum MqErrorE status
)
{
  switch (status) {
    case MQ_OK:		return "OK";
    case MQ_ERROR:	return "ERROR";
    case MQ_CONTINUE:	return "CONTINUE";
    case MQ_EXIT:	return "EXIT";
  }
  return "unknown";
}

const char *
MqLogTypeName (
  const enum MqTypeE type
)
{
  switch (type) {
    case MQ_STRT: return "STR";
    case MQ_BINT: return "BIN";
    case MQ_BYTT: return "BYT";
    case MQ_BOLT: return "BOL";
    case MQ_SRTT: return "SRT";
    case MQ_INTT: return "INT";
    case MQ_FLTT: return "FLT";
    case MQ_WIDT: return "WID";
    case MQ_DBLT: return "DBL";
    case MQ_LSTT: return "LST";
    case MQ_RETT: return "RET";
  }
  return "UNKNOWN";
}

MQ_STRB const *
MqLogServerOrClient (
  struct MqS const * const msgque
)
{
  return (MQ_IS_SERVER (msgque) ? "SERVER" : "CLIENT");
}

MQ_STRB const *
MqLogParentOrChild (
  struct MqS const * const msgque
)
{
  return (MQ_IS_PARENT (msgque) ? "PARENT" : "CHILD");
}

MQ_STR
MqLogC (
  MQ_STR buf,
  MQ_STR str,
  MQ_SIZE size
)
{
  register MQ_STR g = str;
  register MQ_STR t = buf;
  register MQ_STR e = buf + size;
  for (;t < e; t++, g++) {
    *t = (*g ? (isprint (*g) ? *g : '?') : 'o');
  }
  *t = '\0';

  return buf;
}




