/**
 *  \file       src/main.h
 *  \brief      \$Id: main.h 1177 2009-05-14 14:42:43Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1177 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef MAIN_H
#   define MAIN_H

#   define MQ_PRIVATE

#if defined(DLL_EXPORT) && !defined(MQ_IGNORE_EXTERN)
#  define MQ_DDL_EXPORT __declspec(dllexport)
#else
#  define MQ_DDL_EXPORT
#endif

#   include "config.h"

#   include "msgque.h"
#   include "msgque_private.h"


#   include <stdlib.h>
#if !defined(_MSC_VER)
#   include <unistd.h>
#endif
#   include <sys/types.h>
#   include <stdio.h>
#   include <ctype.h>
#   include <time.h>
#   include <string.h>

#   include "debug.h"

/*****************************************************************************/
/*                                                                           */
/*                            msgque error codes                             */
/*                                                                           */
/*****************************************************************************/

/// \brief error-codes used by \libmsgque
enum MqMessageNumE {
    MQ_ERROR_TIMEOUT,
    MQ_ERROR_NOT_A_CLIENT,
    MQ_ERROR_CAN_NOT_START_NON_PIPE_SERVER,
    MQ_ERROR_CAN_NOT_START_CLIENT,
    MQ_ERROR_CAN_NOT_START_SERVER,
    MQ_ERROR_OPTION_REQUIRED,
    MQ_ERROR_INVALID_HEADER,
    MQ_ERROR_RETURN_WITHOUT_TRANSACTION,
    MQ_ERROR_TOKEN_LENGTH,
    MQ_ERROR_OPTION_WRONG,
    MQ_ERROR_CAST_TO_NON_ATOM,
    MQ_ERROR_SEND_END_WITHOUT_START,
    MQ_ERROR_TYPE,
    MQ_ERROR_REQUEST_ARGUMENTS,    
    MQ_ERROR_CHECK_CLIENT,
    MQ_ERROR_FREE_1,
    MQ_ERROR_CAN_NOT,
    MQ_ERROR_WINSOCK,
    MQ_ERROR_CREATE_CHILD,
    MQ_ERROR_INVALID_OPTION,
    MQ_ERROR_TOKEN_FTR,
    MQ_ERROR_OPTION_ARG,
    MQ_ERROR_OPTION_FORBIDDEN,
    MQ_ERROR_WAIT_FOR_TOKEN,
    MQ_ERROR_UNDO_FORBIDDEN,
    MQ_ERROR_READ_HDL,
    MQ_ERROR_NO_INIT,
    MQ_ERROR_MASTER_SLAVE,
    MQ_ERROR_ID_IN_USE,
    MQ_ERROR_WORKER_IS_NO_SERVER,
    MQ_ERROR_SLAVE_CHILD_DELETE,
    MQ_ERROR_ID_OUT_OF_RANGE,
    MQ_ERROR_INVALID_ARGUMENT, 
    MQ_ERROR_MASTER_SLAVE_CLIENT,
    MQ_MESSAGE_END,
};

#if defined(MQ_IS_MAIN)

/// \brief error-messages used by \libmsgque
const char* MqMessageText[MQ_MESSAGE_END] = {
    /* MQ_ERROR_TIMEOUT */			
	"TIMEOUT",
    /* MQ_ERROR_NOT_A_CLIENT */			
	"it is not allowed to use a server in a client context" ,
    /* MQ_ERROR_CAN_NOT_START_NON_PIPE_SERVER */		
	"can not start '%s' server using '@ server ...' syntax",
    /* MQ_ERROR_CAN_NOT_START_CLIENT */		
	"can not start PIPE client without '@ server ...' as argument",
    /* MQ_ERROR_CAN_NOT_START_SERVER */		
	"can not start server '%s'" ,
    /* MQ_ERROR_OPTION_REQUIRED */			
	"for a '%s' setup the option '%s' is required",
    /* MQ_ERROR_INVALID_HEADER */		
	"invalid header, expect <HDR+...>" ,
    /* MQ_ERROR_RETURN_WITHOUT_TRANSACTION */	
	"calling a service with return arguments without transaction, use MqSendEND_AND_WAIT instead of MqSendEND" ,
    /* MQ_ERROR_TOKEN_LENGTH */			
	"expect token as string of size 4 but got '%s'" ,
    /* MQ_ERROR_OPTION_WRONG */			
	"for a '%s' connection the option '%s' is wrong",
    /* MQ_ERROR_CAST_TO_NON_ATOM */		
	"it is not allowed to cast to a non atom (MQ_ATO) type" ,
    /* MQ_ERROR_SEND_END_WITHOUT_START */		
	"use first MqSendSTART and than MqSendEND" ,
    /* MQ_ERROR_TYPE */
	"invalid type, expect '%s' but got '%s'",
    /* MQ_ERROR_REQUEST_ARGUMENTS */
	"request more arguments than available",
    /* MQ_ERROR_CHECK_CLIENT */
	"unable to CHECK client connection, shutdown the server",
    /* MQ_ERROR_FREE_1 */
	"",
    /* MQ_ERROR_CAN_NOT */
	"can not '%s'",
    /* MQ_ERROR_WINSOCK */
	"can not '%s'-> ERR<%i>\ncan not find a usable WinSock DLL",
    /* MQ_ERROR_CREATE_CHILD */
	"[OKS] can not create CHILD",
    /* MQ_ERROR_INVALID_OPTION */
	"invalid [OPTION]... found:",
    /* MQ_ERROR_TOKEN_FTR */
	"got an '_FTR' system token but no filter was defined",
    /* MQ_ERROR_OPTION_ARG */
	"expect argument for option '%s'",
    /* MQ_ERROR_OPTION_FORBIDDEN */
	"for a '%s' setup the option '%s' is forbidden",
    /* MQ_ERROR_WAIT_FOR_TOKEN */
	"TIMEOUT: wait '%i' sec for token '%s' but got only token '%s'",
    /* MQ_ERROR_UNDO_FORBIDDEN */
	"maximum undo level reached",
    /* MQ_ERROR_READ_HDL */
	"the buffer handle '%i' is out of range",
    /* MQ_ERROR_NO_INIT */
	"unable to continue, the 'init' object is not available",
    /* MQ_ERROR_MASTER_SLAVE */
	"the '%s' have to be a 'parent' without 'child' objects",
    /* MQ_ERROR_ID_IN_USE */
	"'%s' identifer '%i' is already in use",
    /* MQ_ERROR_WORKER_IS_NO_SERVER */
	"the worker is no 'server' context",
    /* MQ_ERROR_SLAVE_CHILD_DELETE */
	"it is not allowed to delete a slave 'child' context",
    /* MQ_ERROR_ID_OUT_OF_RANGE */
	"'%s' identifer out of range (%i <= %i <= %i)",
    /* MQ_ERROR_INVALID_ARGUMENT */
	"invalid argument, expect '%s' but got '%s'",
    /* MQ_ERROR_MASTER_SLAVE_CLIENT */
	"the '%s' have to be a 'client' and a 'parent' without 'child' objects"
};

#else
extern const char* MqMessageText[MQ_MESSAGE_END];
#endif

/*****************************************************************************/
/*                                                                           */
/*                  msgque string to number convertion                       */
/*                                                                           */
/*****************************************************************************/

#define str2wid		mq_strtoll
#define str2int		strtol
#if defined(HAVE_STRTOD)
#  define str2dbl	strtod
#else
#  error function strtod is required
#endif
#if defined(HAVE_STRTOF)
#  define str2flt	strtof
#endif
#if defined(HAVE_STRTOUL)
#  define str2ptr	strtoul
#elif defined(HAVE_STRTOULL)
#  define str2ptr	strtoull
#endif

#endif // MAIN_H
