/**
 *  \file       src/mq_io.h
 *  \brief      \$Id: mq_io.h 1176 2009-05-14 13:57:36Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1176 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef MQ_IO_H
#   define MQ_IO_H

/*****************************************************************************/
/*                                                                           */
/*                                io_init                                    */
/*                                                                           */
/*****************************************************************************/

enum MqErrorE pIoCreate (
  struct MqS * const msgque,
  struct MqBufferLS * const argv,
  struct MqIoS ** const out
);

void pIoDelete (
  struct MqIoS ** const ioP
) __attribute__((nonnull));

/*****************************************************************************/
/*                                                                           */
/*                                 io_atom                                   */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE enum MqErrorE pIoRead (
  struct MqIoS const * const io,
  register struct MqBufferS * const buf,
  MQ_SIZE const numBytes
);

MQ_INLINE enum MqErrorE pIoSend (
  struct MqIoS const * const io,
  struct MqBufferS const * const buf
);

enum MqErrorE pIoConnect (
  struct MqIoS * const io
);

MQ_INLINE enum MqErrorE pIoSelect (
  struct MqIoS * const io,
  enum MqIoSelectE const typ,
  struct timeval * const timeout
);

MQ_INLINE enum MqErrorE pIoSelectAll (
  struct MqIoS * const io,
  enum MqIoSelectE const typ,
  struct timeval * const timeout
);

MQ_INLINE enum MqErrorE pIoSelectStart (
  struct MqIoS * const io,
  struct timeval const * const timeout,
  EventCreateF const proc
);

MQ_INLINE enum MqErrorE pIoShutdown (
  register struct MqIoS const * const io
);

/*****************************************************************************/
/*                                                                           */
/*                                 io_misc                                   */
/*                                                                           */
/*****************************************************************************/

/// \brief the socket-type, UDS or TCP
enum IoComE { 
#if defined(MQ_IS_POSIX)
	IO_UDS, 
#endif
	IO_TCP, IO_PIPE };

/// \brief howto start a new server
enum IoStartServerE {
  MQ_START_SERVER_AS_PIPE,
#if defined(HAVE_FORK)
  MQ_START_SERVER_AS_FORK,
  MQ_START_SERVER_AS_INLINE_FORK,
#endif
#if defined(MQ_HAS_THREAD)
  MQ_START_SERVER_AS_THREAD,
#endif
  MQ_START_SERVER_AS_SPAWN,
};

/// \brief start the PIPE Server 
///
/// with \e fMain and \e name we have the possibility to select
/// howto create a process
enum MqErrorE pIoStartServer (
  struct MqIoS * const io,
  enum IoStartServerE startType,
  MQ_SOCK * sockP,
  struct MqBufferLS ** alfaP,
  struct MqIdS * idP
);

/// \brief check if communication is possible
MQ_BOL
pIoCheck (
  struct MqIoS * const io
);

/// \brief helper to convert #IoStartServerE into a human readable string
const char* pIoLogStartType (
  enum IoStartServerE startType
);

MQ_STR pIoLogId (
  MQ_STR buf,
  MQ_INT size,
  struct MqIdS id
);

/// \brief helper to covert the #IoComE into a human readable string
const char * pIoLogCom (
  enum IoComE com
);

enum MqErrorE pIoEventAdd (
  struct MqIoS * const io,
  MQ_SOCK * sockP
);

MQ_INLINE MQ_TIME_T pIoGetTimeout(
  struct MqIoS * const io
);

MQ_BOL pIoIsRemote (struct MqIoS * const);

#endif /* MQ_IO_H */


