/**
 *  \file       src/msgque.c
 *  \brief      \$Id: msgque.c 1192 2009-05-19 12:53:58Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1192 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#define MQ_IS_MAIN 1

#include "main.h"
#include "sys.h"

#include "error.h"
#include "buffer.h"
#include "bufferL.h"
#include "log.h"
#include "send.h"
#include "read.h"
#include "mq_io.h"
#include "token.h"
#include "uds_io.h"
#include "swap.h"
#include "trans.h"
#include "cache.h"
#include "slave.h"

#if defined(_MSC_VER)
// need for "timeval"
#	include <WinSock2.h>
#endif

#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif

#define TOKEN_LEN (HDR_TOK_LEN+1)
#define MSGQUE msgque
#define MQ_ERROR_S msgque->error

struct pChildS {
    struct pChildS * left     ;
    struct MqS     * msgque   ;
    struct pChildS * right    ;
};

MQ_DDL_EXPORT struct MqBufferLS * MqInitBuf = NULL;

MQ_DDL_EXPORT MqMainSelectorF MqMainSelector = NULL;

static enum MqErrorE
pWaitForToken (
  struct MqS * const msgque,
  MQ_TIME_T const timeout,
  MQ_STRB const * const str
);

static void pMqCleanup (
) __attribute__ ((destructor));

/*****************************************************************************/
/*                                                                           */
/*                                private                                    */
/*                                                                           */
/*****************************************************************************/

void
pMqCleanup()
{
  if (MqInitBuf != NULL) MqBufferLDelete(&MqInitBuf);
}

MQ_STR
MqHelp ( const char * tool )
{
    // this is a "WKML" - W)ell K)nown M)emory L)eak
    struct MqBufferS * buf = MqBufferCreate (1000);

    if (tool) {
	MqBufferAppendV (buf, "  %s [ARGUMENT]... syntax:\n", tool);
	MqBufferAppendV (buf, "    aclient [OPTION]... %c %s [OPTION]... %c [ARGUMENT]...\n", MQ_ALFA, tool, MQ_ALFA);
	MqBufferAppendV (buf, "\n");
    }
    MqBufferAppendV (buf, "  msgque [OPTION]:\n");
    MqBufferAppendV (buf, "    --help-msgque    print msgque specific help\n");

    return (MQ_STR) buf->data;
}

static MQ_STR
pMqHelp ()
{
    // this is a "WKML" - W)ell K)nown M)emory L)eak
    struct MqBufferS * buf = MqBufferCreate (1000);

    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, "'--help-msgque' summarize:\n");
    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, "These are the options for the 'libmsgque' library passed by\n");
    MqBufferAppendV (buf, "the second argument (argv) of 'MqCreate'\n");
    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, " msgque wrapper [OPTION]:\n");
    MqBufferAppendV (buf, "  --name str         the display name of the msgque      (default: executable)\n");
    MqBufferAppendV (buf, "  --srvname str      the display name of the server      (default: executable)\n");
    MqBufferAppendV (buf, "  --string           use string based protocol           (default: no=binary)\n");
    MqBufferAppendV (buf, "  --silent           use silent mode                     (default: no)\n");
    MqBufferAppendV (buf, "  --debug #          use debug level (0-9)               (default: 0)\n");
    MqBufferAppendV (buf, "  --daemon pidfile   start as daemon and save the pid    (default: no)\n");
    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, " generic tcp/uds io [OPTION]:\n");
    MqBufferAppendV (buf, "  --timeout #        timeout for connect                 (default: %i sec)\n", MQ_TIMEOUT90);
    MqBufferAppendV (buf, "  --buffersize #     buffer size in bytes                (default: 4096)\n");
    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, " create a new entity as [OPTION]:\n");
    MqBufferAppendV (buf, "  --fork             new process using 'fork'            (default: no)\n");
    MqBufferAppendV (buf, "  --thread           new thread                          (default: no)\n");
    MqBufferAppendV (buf, "  --spawn            new process using 'spawn'           (default: no)\n");
    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, " tcp specific [OPTION]:\n");
    MqBufferAppendV (buf, "  --tcp              use a Tcp Sockets                   (default: no)\n");
    MqBufferAppendV (buf, "   --host str        use 'str' as remote host name       (default: empty)\n");
    MqBufferAppendV (buf, "   --port str        use 'str' as remote port name       (default: empty)\n");
    MqBufferAppendV (buf, "   --myhost str      use 'str' as local host name        (default: empty)\n");
    MqBufferAppendV (buf, "   --myport str      use 'str' as local port name        (default: empty)\n");
    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, " uds specific [OPTION]:\n");
    MqBufferAppendV (buf, "  --uds              use Unix Domain Sockets             (default: no)\n");
    MqBufferAppendV (buf, "   --file str        use 'str' as filename               (default: empty)\n");
    MqBufferAppendV (buf, "\n");
    MqBufferAppendV (buf, " pipe specific [OPTION]:\n");
    MqBufferAppendV (buf, "  --pipe             use Socket-Pair Sockets             (default: yes)\n");
    MqBufferAppendV (buf, "\n");

    return (MQ_STR) buf->data;
}

static enum MqErrorE
sMqCheckArg (
  register struct MqS * const msgque,
  register struct MqBufferLS * const argv
)
{
  struct MqErrorS * const error = MQ_ERROR_S;

  // The "--name" is used in debugging/error messages as prefix and
  // should be available as soon as possible.
  // The first argument should be the name of the executable and
  // will be used to identify the socket if no '--name' option is available
  if (msgque->config.name == NULL) {
    msgque->config.name = mq_strdup("unknown");
  }

  // check to force SLAVE requirements
  if (MQ_IS_SLAVE(msgque)) {
    msgque->config.isServer = MQ_NO;
    msgque->config.FTRdata = NULL;
    msgque->config.EOFdata = NULL;
  }

  // check to force CLIENT requirements
  if (MQ_IS_CHILD(msgque)) {
    struct MqS * parent = msgque->config.parent;
    msgque->endian = parent->endian;
    msgque->config.FTRdata = NULL;
    msgque->config.EOFdata = NULL;
  }

  // parse "my" arguments
  if (argv != NULL && argv->cursize > 0) {
    struct MqBufferS *name = NULL, *first = NULL;
		MQ_BOL help, help2, mybool;
    // allways read the first item from argv
    MqBufferLGetU(error, argv,0,&first);
    // if "MqMainToolName" is empty -> fill it with a default value
    if (MqInitBuf == NULL && first != NULL) {
      struct MqBufferLS * initB = MqInitCreate();
      MqBufferLAppendC(initB, first->cur.C);
    }
    // check for the "--name" option
    MqErrorCheck (MqBufferLCheckOptionU (error, argv, "--name", &name));
    if (unlikely(name != NULL)) {
      MqSysFree(msgque->config.name);
      msgque->config.name = mq_strdup(name->cur.C);
      MqBufferDelete(&name);
    } else if (!strcmp(msgque->config.name,"unknown")) {
      if (first != NULL) {
	MqSysFree(msgque->config.name);
	msgque->config.name = MqSysBasename (first->cur.C, MQ_NO);
      } else if (MqInitBuf != NULL) {
	MqSysFree(msgque->config.name);
	msgque->config.name = MqSysBasename (MqInitBuf->data[0]->cur.C, MQ_NO);
      }
    }
    MqBufferDelete(&first);

    // check on my own help
    MqErrorCheck (MqBufferLCheckOptionO (error, argv, "--help-msgque", &help));
    if (help == MQ_YES) {
      fputs(pMqHelp(),stderr);
      SysExit(0,EXIT_SUCCESS);
    }
    if (argv && msgque->config.fHelp) {
      MqErrorCheck (MqBufferLCheckOptionO (error, argv, "-h", &help));
      MqErrorCheck (MqBufferLCheckOptionO (error, argv, "--help", &help2));
      if (help || help2) {
	(*msgque->config.fHelp)(msgque->config.name);
      }
    }

    /* START: this code overwrite the default values for thild "child" */

    // the "MQ_IS_PARENT" is necessary because the "MqBufferLCheckOptionO" will overwrite
    // default to "MQ_NO"
    if (MQ_IS_PARENT(msgque)) {
      // setup the default binary mode 
      mybool = MQ_NO;
      // attention: do not use "msgque->config.isString" as argument to "MqBufferLCheckOptionO"
      // because this will overwrite the default from "config"
      MqErrorCheck (MqBufferLCheckOptionO (error, argv, "--string", &mybool));
      if (mybool) msgque->config.isString = MQ_YES;
      if (MQ_IS_SERVER (msgque)) {
	// the "server" allways default to MQ_YES, because the "client" set the
	// default in the "_IAA" message
	msgque->config.isString = MQ_YES;
      }

      // close everything which makes me depend on the calling process
      MqErrorCheck (MqBufferLCheckOptionC (error, argv, "--daemon", &msgque->daemon_pid_file));
      if (MQ_IS_SERVER(msgque) && msgque->daemon_pid_file) {
	if (msgque->config.ignoreFork == MQ_YES) {
	  msgque->daemon_pid_file = NULL;
	  return MqErrorDbV (MQ_ERROR_OPTION_FORBIDDEN, "local", "--daemon");
	} else {
	  MqErrorCheck(SysDaemonize(MQ_ERROR_S, msgque->daemon_pid_file));
	  msgque->config.isSilent = MQ_YES;
	}
      }

      mybool = MQ_NO;
      MqErrorCheck (MqBufferLCheckOptionO (error, argv, "---duplicate", &mybool));

      if (mybool) {
	msgque->config.statusIs = (enum MqStatusIsE) (msgque->config.statusIs | MQ_STATUS_IS_DUP);
      }

      MqErrorCheck (MqBufferLCheckOptionO (error, argv, "--fork", &mybool));

      if (mybool) {
	if (msgque->config.ignoreFork == MQ_YES) {
	  return MqErrorDbV (MQ_ERROR_OPTION_FORBIDDEN, "local", "--fork");
	} else {
	  msgque->config.startAs = MQ_START_FORK;
	  if (MQ_IS_SERVER(msgque) && msgque->config.statusIs == MQ_STATUS_IS_DUP)
	    msgque->config.statusIs = (enum MqStatusIsE) (msgque->config.statusIs | MQ_STATUS_IS_FORK);
	}
      }

      MqErrorCheck (MqBufferLCheckOptionO (error, argv, "--thread", &mybool));
      if (mybool) {
	if (msgque->config.ignoreThread == MQ_YES) {
	  return MqErrorDbV (MQ_ERROR_OPTION_FORBIDDEN, "local", "--thread");
	} else {
	  msgque->config.startAs = MQ_START_THREAD;
	  if (MQ_IS_SERVER(msgque) && msgque->config.statusIs == MQ_STATUS_IS_DUP)
	    msgque->config.statusIs = (enum MqStatusIsE) (msgque->config.statusIs | MQ_STATUS_IS_THREAD);
	}
      }

      MqErrorCheck (MqBufferLCheckOptionO (error, argv, "--spawn", &mybool));
      if (mybool) {
	if (MqInitBuf == NULL) {
	  return MqErrorDb (MQ_ERROR_NO_INIT);
	} else if (msgque->config.ignoreSpawn == MQ_YES) {
	  return MqErrorDbV (MQ_ERROR_OPTION_FORBIDDEN, "local", "--spawn");
	} else {
	  msgque->config.startAs = MQ_START_SPAWN;
	  if (MQ_IS_SERVER(msgque) && msgque->config.statusIs == MQ_STATUS_IS_DUP)
	    msgque->config.statusIs = (enum MqStatusIsE) (msgque->config.statusIs | MQ_STATUS_IS_SPAWN);
	}
      }
    }

    // for PARENT and CHILD
    MqErrorCheck (MqBufferLCheckOptionO (error, argv, "--silent", &help));
    if (help) msgque->config.isSilent = help;
    MqErrorCheck (MqBufferLCheckOptionI (error, argv, "--debug",  &msgque->config.debug));

    /* END: this code overwrite default's for "child" */
  }

  return MQ_OK;

error:
  return MqErrorStack(MQ_ERROR_S);
}

/*****************************************************************************/
/*                                                                           */
/*                                msgque/init                                */
/*                                                                           */
/*****************************************************************************/

struct MqConfigS *
MqConfigInit (
  void
) {
  struct MqConfigS * config = (struct MqConfigS *) MqSysCalloc(MQ_ERROR_PANIC,1,sizeof(*config));
  // if changed, change MqConfigDupInplace too
  config->buffersize = 4096;
  config->timeout = MQ_TIMEOUT90;
#if !defined(MQ_HAS_THREAD)
  config->ignoreThread = MQ_YES;
#endif
#if !defined(HAVE_FORK)
  config->ignoreFork = MQ_YES;
#endif
  return config;
}

void
MqConfigFree (
  struct MqConfigS ** configP
) {
  MqSysFree(*configP);
}

void
MqConfigInitManagedContext (
  struct MqConfigS * const config,
  void * self
) {
  struct MqManagedContextS * context;
  if (config == NULL) return;
  if (!(config->statusIs & MQ_STATUS_IS_DUP)) {
    MqSysFree(config->myCtx);
  }
  context = (struct MqManagedContextS*) MqSysCalloc(MQ_ERROR_PANIC, 1, sizeof(*context));
  context->self = self;
  config->myCtx = (struct MqContextS *)context;
  // used to free context in #MqDelete
  config->statusIs = (enum MqStatusIsE) (config->statusIs | MQ_STATUS_IS_MANAGED);
}

void*
MqConfigGetSelfFromManagedContext (
  struct MqConfigS const * const config
) {
  return ((struct MqManagedContextS *)config->myCtx)->self;
}

void*
MqConfigGetSelfFromManagedContextM (
  struct MqS const * const msgque
) {
  return ((struct MqManagedContextS *)msgque->config.myCtx)->self;
}

void 
MqConfigInitName (
  struct MqConfigS * const config,
  const char * name
) {
  if (config == NULL || name == NULL) return;
  MqSysFree(config->name);
  config->name = mq_strdup(name);
}

void 
MqConfigInitParent (
  struct MqConfigS * const config,
  struct MqS * const parent
) {
  if (config == NULL) return;
  config->myCtx = NULL;
  config->parent = parent;
}

void 
MqConfigInitMaster (
  struct MqConfigS * const config,
  struct MqS * const master,
  int master_id
) {
  if (config == NULL) return;
  config->myCtx = NULL;
  config->master = master;
  config->master_id = master_id;
}

struct MqConfigS *
MqConfigDup (
  struct MqConfigS const * const config
) {
  struct MqConfigS * myconfig = (struct MqConfigS *) MqSysCalloc(MQ_ERROR_PANIC,1,sizeof(*myconfig));
  memcpy(myconfig,config,sizeof(*myconfig));
  if (config->name != NULL) myconfig->name = mq_strdup(config->name);
  myconfig->statusIs = MQ_STATUS_IS_DUP;
  myconfig->parent = NULL;
  myconfig->master = NULL;
  myconfig->master_id = 0;
  // don't set to NULL because this value is needed (s. javamsgque->ParentCreate)
  // myconfig->myCtx = NULL;
  myconfig->FTRdata = NULL;
  myconfig->EOFdata = NULL;
  return myconfig;
}

// MqConfigGet ...

MQ_INLINE int
MqConfigGetIsServer (
  struct MqS const * const msgque
)
{
  return MQ_IS_SERVER(msgque);
}

MQ_INLINE int
MqConfigGetIsParent (
  struct MqS const * const msgque
)
{
  return MQ_IS_PARENT(msgque);
}

MQ_INLINE int
MqConfigGetIsSlave (
  struct MqS const * const msgque
)
{
  return (msgque->config.master != NULL);
}

MQ_INLINE int
MqConfigGetIsString (
  struct MqS const * const msgque
)
{
  return (msgque->config.isString == MQ_YES);
}

MQ_INLINE int
MqConfigGetIsSilent (
  struct MqS const * const msgque
)
{
  return (msgque->config.isSilent == MQ_YES);
}

MQ_INLINE int
MqConfigGetIsDupAndThread (
  struct MqConfigS const * const config
)
{
  return ((config->statusIs & MQ_STATUS_IS_DUP) && (config->statusIs & MQ_STATUS_IS_THREAD));
}

MQ_INLINE const char *
MqConfigGetName (
  struct MqS const * const msgque
)
{
  return msgque->config.name;
}

MQ_INLINE MQ_INT
MqConfigGetDebug (
  struct MqS const * const msgque
)
{
  return msgque->config.debug;
}

MQ_INLINE struct MqS const *
MqConfigGetParent (
  struct MqConfigS const * const config
)
{
  return config->parent;
}

MQ_INLINE struct MqS const *
MqConfigGetParentM (
  struct MqS const * const msgque
)
{
  return msgque->config.parent;
}

MQ_INLINE struct MqS const *
MqConfigGetMaster (
  struct MqConfigS const * const config
)
{
  return config->master;
}

MQ_INLINE struct MqS const *
MqConfigGetMasterM (
  struct MqS const * const msgque
)
{
  return msgque->config.master;
}

MQ_INLINE struct MqContextS const *
MqConfigGetMyCtx (
  struct MqConfigS const * const config
)
{
  return config->myCtx;
}

/*****************************************************************************/
/*                                                                           */
/*                             create / delete                               */
/*                                                                           */
/*****************************************************************************/

struct MqBufferLS*
MqInitCreate ()
{
  if (MqInitBuf != NULL) MqBufferLDelete(&MqInitBuf);
  MqInitBuf = MqBufferLCreate(2);
  return MqInitBuf;
}

MQ_INLINE const MQ_STRB*
MqInitGetFirst ()
{
  return (MqInitBuf && MqInitBuf->cursize > 0 ? MqInitBuf->data[0]->cur.C : "unknown");
}

enum MqErrorE
MqCreate (
  struct MqConfigS  * config,
  struct MqBufferLS * const argv,
  struct MqBufferLS * alfa,
  struct MqS       ** msgqueP
)
{
  struct MqErrorS * error = MQ_ERROR_PANIC;
  register struct MqS * const msgque = (struct MqS *) MqSysCalloc (error, 1, sizeof (*msgque));	// set memory to zero
  MQ_STR serverexec = NULL;
  MQ_STR srvname = NULL;
  const MQ_STRB *first = MqInitGetFirst();

  *msgqueP = msgque;

  // create error object, for now an error is possible
  pErrorCreate (msgque, &error);
  msgque->error = error;

  // fill the configuration with the default values and free the configuration object
  // the reason for "memcpy" is -> speed. Adding the "config" as pointer would
  // add an additional pointer->dereference to any "config" access.
  memcpy(&msgque->config, config, sizeof(struct MqConfigS));

  MqSysFree(config);

  // save the msgque object in the "context"
  if (
	msgque->config.myCtx &&
	(   
	    MQ_IS_SERVER(msgque) ||
	    (MQ_IS_CLIENT(msgque) && !MQ_IS_FILTER(msgque))
	)
     ) {
      msgque->config.myCtx->msgque = msgque;
  }
    
  // initialize internal data to default
  msgque->temp		= MqBufferCreate (100);

  // handle empty MqBufferLS list alfa
  // after this line "alfa" ponter is NULL (-> no data) or "alfa" ponter != NULL (-> has data)
  if (alfa != NULL) {
    // we have a "master", a "worker" have to be created
    if (alfa->cursize == 0 && msgque->config.master == NULL) {
      MqBufferLDelete (&alfa);
    }
    if (alfa != NULL && alfa->cursize > 0) {
      msgque->isSELF = !strncmp(alfa->data[0]->cur.C, "SELF", 4);
      msgque->isWORKER = !strncmp(alfa->data[0]->cur.C, "WORKER", 6);
      if (!msgque->isSELF && !msgque->isWORKER) first = alfa->data[0]->cur.C;
    }
  }
  serverexec = mq_strdup (first);

  // parse arguments
  MqErrorCheck (sMqCheckArg (msgque, argv));
  MqDLogC(msgque,2,"START-CREATE\n");

  // the child has no server-name, IAA send this to the server (not OKS)
  MqErrorCheck (MqBufferLCheckOptionC (error, argv, "--srvname", &srvname));

  // create my ctxId's
  if (MQ_IS_PARENT (msgque)) {
    msgque->ctxIdA	= (struct MqS **) MqSysCalloc (error, 100, sizeof(msgque));
    msgque->ctxIdZ	= 100;
    msgque->ctxId	= 0;
    msgque->ctxIdA[0]	= msgque;
    msgque->ctxIdP	= msgque;
    msgque->ctxIdR	= 0;
  } else {
    // create and add my contextID to the initial 'parent' contextId storage
    register struct MqS * ctxIdP = msgque->ctxIdP = msgque->config.parent->ctxIdP;
    struct pChildS * link;
    struct pChildS * old;
    ctxIdP->ctxIdR++;
    msgque->ctxId = ctxIdP->ctxIdR;
    if (ctxIdP->ctxIdR >= ctxIdP->ctxIdZ) {
      ctxIdP->ctxIdA = (struct MqS **) MqSysRealloc (error, ctxIdP->ctxIdA, (ctxIdP->ctxIdZ + 100) * sizeof(msgque));
      memset (ctxIdP->ctxIdA + ctxIdP->ctxIdZ, '\0', 100 * sizeof(msgque));
      ctxIdP->ctxIdZ += 100;
    }
    ctxIdP->ctxIdA[msgque->ctxId] = msgque;

    // add myself as 'child' of parent
    link = (struct pChildS *) MqSysMalloc (error, sizeof (struct pChildS));
    old  = msgque->config.parent->childs;
    if (old != NULL) old->left = link;
    link->right  = old;
    link->left   = NULL;
    link->msgque = msgque;
    msgque->config.parent->childs = link;
    msgque->self = link;
  }

  // create private data
  MqErrorCheck (pSlaveCreate (msgque, argv, &alfa, &msgque->slave));
  MqErrorCheck (pSendCreate (msgque, &msgque->send));

  // pReadCreate have to fit into the 'CacheCF' function
  msgque->read = pReadCreate (msgque);
  if (MqErrorGetCode(error) == MQ_ERROR) goto error;

  MqErrorCheck (pIoCreate   (msgque, argv, &msgque->io));
  MqErrorCheck (pTokenCreate (msgque, &msgque->srvT));
  MqErrorCheck (pTransCreate (msgque, &msgque->trans));

  // create the cache data
  pCacheCreate (error, (CacheCF) pReadCreate, (CacheDF) pReadDelete, msgque, &msgque->readCache);

  // context specific initialization
  if (MQ_IS_CLIENT (msgque)) {
  // this is a CLIENT

    if (MQ_IS_CHILD (msgque)) {
    // this is a CHILD

      struct MqSendS * const send = msgque->config.parent->send;

      // PARENT, sending using the DATA of the CHILD as argument
      MqSendSTART (send);
      MqSendI (send, (msgque->config.debug == 0 ? -1 : msgque->config.debug));
      MqSendI (send, (MQ_INT) msgque->config.isSilent);

      // !!Attention wrong error (from the PARENT because the PARENT starts the CHILD on the SERVER)
      MqErrorCheck1 (MqSendEND_AND_WAIT (send, "_OKS", MQ_TIMEOUT_USER));

    } else {
    // this is a PARENT

      struct MqSendS * const send = msgque->send;
      MQ_BOL mystring;
      MQ_BOL myendian;

      // 0. Start the PIPE server (if necessary)
      MqErrorCheck (pIoStartServer(msgque->io, MQ_START_SERVER_AS_PIPE, NULL, &alfa, NULL));

      // 1. connect to the server
      MqErrorCheck (pIoConnect (msgque->io));

      // 2. exchange startup messages
      //      - allways send as string

      // save my original binary-mode
      mystring = msgque->config.isString;

      // for '_IAA' we only use 'string' mode
      msgque->config.isString = MQ_YES;

      MqSendSTART (send);

      // send the binary mode
      MqSendO (send, mystring);

      // send my endian
#     if defined(WORDS_BIGENDIAN)
	MqSendO (send, MQ_YES);
#     else
	MqSendO (send, MQ_NO);
#     endif

      // send the server name
      MqSendC (send, (srvname ? srvname : ""));

      // send package and wait for the answer
      MqDLog(msgque,4,"send token<%s>\n","_IAA");
      if (MqSendEND_AND_WAIT (send, "_IAA", MQ_TIMEOUT_USER) == MQ_ERROR) {
	MqErrorDbV2 (error,MQ_ERROR_CAN_NOT_START_SERVER, 
	  (pIoIsRemote(msgque->io) == MQ_NO ? serverexec : "remote-connect"));
	goto error;
      }

      // read the other endian and set my msgque->endian
      MqReadO(msgque->read, &myendian);

#     if defined(WORDS_BIGENDIAN)
	msgque->endian = (myendian ? MQ_NO : MQ_YES);
#     else
	msgque->endian = (myendian ? MQ_YES : MQ_NO);
#     endif

      // restore the binary mode
      msgque->config.isString = mystring;

      // wait until the server is able to process events (and send _PEO)
      MqErrorSwitch (pWaitForToken (msgque, MQ_TIMEOUT_USER, "_PEO"));

    }; // END PARENT

  } else { // this is a SERVER

    // we have to wait for the "_IAA" message of the "client" to set "msgque->config.debug" proper
    if (MQ_IS_PARENT (msgque)) {
      MqErrorSwitch (pWaitForToken (msgque, MQ_TIMEOUT_USER, "_IAA"));
    }

    // if we are a "filter" -> YES: we have to start the right site (an Server)
    if (MQ_IS_FILTER(msgque)) {

			struct MqBufferS ** arg;
      struct MqBufferS ** end;
      register MQ_STR val;
      struct MqConfigS * filterconfig;
      
      // this is a typical situation for a server in a middel of an alfa syntax:
      //	    acmd1 @ filter @ acmd3
      // filter (this one) is server for acmd1 and client for acmd3

      // DO, convert all '++' option arguments in largv (belonging to the new created filter)
      //     into '--' option arguments

      // step 1, shift the alfa to the new msgque
      struct MqBufferLS * largv = MqBufferLCreate (10);

      // step 2, append to the !beginning! of largv default values from the current server
      MqBufferLAppendC (largv, msgque->config.name);

      // step 3, move all items from argv to largv, convert "++..." to "--..." items
      arg = argv->data;
      end = argv->data + argv->cursize;
      for (;arg < end; arg++) {
	if ((*arg)->type == MQ_STRT && (*arg)->cursize > 2) {
	  val = (*arg)->cur.C;
	  if (val[0] == '+' && val[1] == '+') {
	    val[0] = val[1] = '-';
	  }
	}
	MqBufferLAppend (largv, *arg, -1);
      }

      // step 4, reset argv
      argv->cur = argv->data;
      argv->cursize = 0;

      // step 5, create the new msgque and fill the myFilter
      filterconfig = MqConfigDup(&msgque->config);
      filterconfig->isServer = MQ_NO;
      filterconfig->fChildCreate = NULL;
      filterconfig->fChildDelete = NULL;
      filterconfig->fParentCreate = NULL;
      filterconfig->fParentDelete = NULL;
      filterconfig->fParentExit = NULL;
      filterconfig->fFTR = NULL;
      filterconfig->FTRfree = NULL;
      filterconfig->fEOF = NULL;
      filterconfig->EOFfree = NULL;
      if (MqCreate (filterconfig, largv, alfa, &msgque->myFilter) == MQ_ERROR) {
	MqErrorCopy (error, msgque->myFilter->error);
	return MQ_ERROR;
      }
      // the alfa is now owned by the "filter"
      alfa = NULL;

      // step 6, shift all remaining items from largv back to argv
      arg = largv->data;
      end = largv->data + largv->cursize;
      for (;arg < end; arg++) {
	MqBufferLAppend (argv, *arg, -1);
      }
      largv->cur = largv->data;
      largv->cursize = 0;
      MqBufferLDelete(&largv);

    }; // END filter startup
  }; // END SERVER

  // cleanup
  MqBufferLDelete (&alfa);
  MqSysFree(srvname);
  MqSysFree(serverexec);

  MqDLog(msgque,2,"END-CREATE string<%s>, silent<%s>, debug<%i>, endian<%i>\n",
	(msgque->config.isString?"yes":"no"), (msgque->config.isSilent?"yes":"no"), 
	    msgque->config.debug, msgque->endian);

  return MQ_OK;

error1:
  MqErrorCopy (error, msgque->config.parent->error);

error:
  return MqErrorStack (MQ_ERROR_S);
}

void
MqDelete (
  struct MqS ** const msgqueP
)
{
  struct MqS *msgque;
  if (unlikely(msgqueP == NULL || *msgqueP == NULL)) return;
  msgque = *msgqueP;

  // report error if available
  pErrorReport(msgque->error);

  // Try to invoke the "DeleteHandler" first
  if (msgque->onDelete == MQ_NO) {
    msgque->onDelete = MQ_YES;
    if (MQ_IS_CHILD(msgque) && msgque->config.fChildDelete) {
      (*msgque->config.fChildDelete) (msgqueP);
      return;
    } else if (MQ_IS_PARENT(msgque) && msgque->config.fParentDelete) {
      (*msgque->config.fParentDelete) (msgqueP);
      return;
    }
  }

  // shutdown depending context
  pMqShutdown (msgque);
  
  // delete my ctxId
  if (MQ_IS_PARENT (msgque)) {
    // delete the entire ctxIdA
    MqSysFree (msgque->ctxIdA);
    msgque->ctxIdZ = 0;
    msgque->ctxIdR = 0;
    // only the SERVER + PARENT is owner of the daemon_pid_file
    if (MQ_IS_SERVER(msgque) && msgque->daemon_pid_file) {
      // make a PANIC (1'st argument = NULL) if the daemon_pid_file is gone
      // -> this is need for testing
      SysUnLink(NULL, msgque->daemon_pid_file);
      MqSysFree (msgque->daemon_pid_file);
    }
  } else {
    // delete my entry from the 'parent' msgque->childs entry
    struct pChildS * left  = msgque->self->left;
    struct pChildS * right = msgque->self->right;
    // delete the entry in the ctxIdA array
    msgque->ctxIdP->ctxIdA[msgque->ctxId] = NULL;
    // check the 'left' site of msgque->self
    if (left != NULL)  {
      // msgque->self is !not! the first child of parent
      left->right = right;
    } else {
      // msgque->self is the !most left! child of parent
      msgque->config.parent->childs = right;
    }
    if (right != NULL) right->left = left;
    MqSysFree (msgque->self);
  }

  // delete my cache
  pCacheDelete (&msgque->readCache);

  // delete my helper's
  pSlaveDelete (&msgque->slave);
  pTransDelete (&msgque->trans);
  pTokenDelete (&msgque->srvT);
  pIoDelete    (&msgque->io);
  pReadDelete  (&msgque->read);
  pSendDelete  (&msgque->send);

  // delete my MqBuffer(s)
  MqBufferDelete (&msgque->temp);

  // free my filter/eof function data
  if (msgque->config.FTRdata && msgque->config.FTRfree) {
    (*msgque->config.FTRfree) (msgque, msgque->config.FTRdata);
    msgque->config.FTRdata = NULL;
  }
  if (msgque->config.EOFdata && msgque->config.EOFfree) {
    (*msgque->config.EOFfree) (msgque, msgque->config.EOFdata);
    msgque->config.EOFdata = NULL;
  }

  // last chance to write a log message
  pErrorDelete (&msgque->error);
  MqDLogC(msgque,2,"DELETE\n");
  MqSysFree (msgque->config.name);
  // a "managed" context is owned by libmsgue
  if (msgque->config.statusIs & MQ_STATUS_IS_MANAGED)
    MqSysFree(msgque->config.myCtx);
  MqSysFree (*msgqueP);
  return;
}

void
pMqShutdown (
  struct MqS * const msgque
)
{
  if (msgque == NULL || msgque->onShutdown == MQ_YES) return;
  msgque->onShutdown = MQ_YES;

  // shutdown 'filter'
  if (msgque->myFilter && msgque->myFilter != msgque) {
    MqDelete (&msgque->myFilter);
  }

  // shutdown all childs
  while (msgque->childs) {
    // the next line is necessary because the item "msgque->childs->msgque" and
    // "msgque->childs->msgque->self->msgque" are identical. The last one is 
    // free'd during the following "MqDelete" too and a double-free error
    // will happen
    struct MqS * save = msgque->childs->msgque;
    MqDelete (&save);
  }

  // shutdown all slaves
  pSlaveShutdown (msgque->slave);

  // send message to shutdown the 'server' 
  // -> but only if the server can be achieved -> pIoCheck
  if (MQ_IS_CLIENT (msgque) && pIoCheck (msgque->io)) {
    // ContextDelete should disable the Event-Handler
    // here we disable the external EVENT proc
    msgque->config.fEvent = NULL;
    // shutdown the server ...  !!ATTENTION!! do not use MqSendEND_AND_WAIT
    // because an unexpected "answer" will corrupt the buffer
    MqDLogC(msgque,4,"send token<_SHD>\n");
    MqSendSTART (msgque->send);
    MqSendEND_AND_WAIT (msgque->send, "_SHD", MQ_TIMEOUT_USER);
  }

  // cleanup error
  MqErrorReset(msgque->error);

  return;
}

/*****************************************************************************/
/*                                                                           */
/*                                msgque/event                               */
/*                                                                           */
/*****************************************************************************/

static enum MqErrorE
sMqEventStart (
  struct MqS * const msgque
)
{
  struct MqS * a_msgque = msgque;

  MqDLogC(msgque,6,"START\n");

  // ##################### READ HEADER #####################
  switch (pReadHDR (msgque->read, &a_msgque)) {
    case MQ_OK:
      break;
    case MQ_CONTINUE:
      return MQ_OK;
    case MQ_ERROR:
      MqDLogC(msgque,7,"pReadHDR: report an error\n");
      goto error;
    case MQ_EXIT:
      return MQ_EXIT;
  }

  // ##################### TOKEN Handler #####################
  // The following code will "only" run on the "server" site.
  // An error in a service-handler will !not! shutdown the server
  if (pTokenInvoke (a_msgque->srvT) == MQ_ERROR) {
    if (a_msgque->_trans != 0) {
      // in a transaction, "MqSendRETURN" will convert the msgque error 
      // into an "error" package and send this package back to the client
      MqErrorCheck (MqSendRETURN (a_msgque->send));
    } else if (pIoCheck (a_msgque->io)) {
      // outsite of a transaction we have to send a "real" error,
      // but "only" if the connection "pIoCheck" is still available
      MqErrorCheck (MqSendERROR (a_msgque->send));
    } else {
      // we can not send the error -> shutdown the server
      MqDLogC(msgque,7,"pTokenInvoke: report an error\n");
      goto error;
    }
  }

  MqDLogC(a_msgque,6,"END-OK\n");
  return MQ_OK;

error:
  MqDLogC(a_msgque,6,"END-ERROR\n");
  // the errors are reported into a_msgque->error
  MqErrorStack (a_msgque->error);
  MqErrorCopy (msgque->error, a_msgque->error);
  return MQ_ERROR;
}

enum MqErrorE
pWaitOnEvent (
  struct MqS * const msgque,
  const enum MqIoSelectE what,
  const MQ_TIME_T timeout
)
{
  const MQ_TIME_T startT = time (NULL);
  const MqEventF proc = msgque->config.fEvent;
  register MQ_TIME_T nowT;

  struct MqContextS * const context = msgque->config.myCtx;
  struct timeval tv;

  // set initial timeout
  if (proc) {
    // we have an extern event-handlig procedure (proc) ... give this code the chance 
    // to do somethig usefull
    tv.tv_sec = 0L;
    // why 10000 uSec, to let the select a chance to process the direct response
    // on a previously submitted 'MqSendEND_AND_WAIT' packages (e.g. special _ERR)
    tv.tv_usec = 10000L;
  } else {
    tv.tv_sec = (long)timeout;
    tv.tv_usec = 0L;
  }

  MqDLogC(msgque,6,"START loop\n");
  do {

    // 1. test on 'select'
    switch (pIoSelectAll (msgque->io, what, &tv)) {
      case MQ_CONTINUE:	// nothing found wait for next event
	break;                  
      case MQ_OK:		// found event
	goto end;               
      case MQ_ERROR:		// something wrong happen
	goto error;             
      case MQ_EXIT:		// something unexpected happen
	return MQ_EXIT;
    }

    // 2. call external event-proc
    if (proc != NULL && context != NULL) {
      // this guarding with "____" is important to detect and break-out-of "nested" event calls
      // pWaitOnEvent
      //   -> guard 
      //   -> *proc 
      //     ... -> pWaitOnEvent -> found-an-event
      //   -> check-guard
      pTokenSetCurrent(msgque->srvT,"____");

      //MqDLogC(msgque,7,"call fEvent in<%p>\n", msgque);
      (*proc) (context);
      //MqDLogC(msgque,7,"finish fEvent in<%p>\n", msgque);

      // the fEvent found an event belonging to "msgque" but does not know
      // if the event belongs to the "parent" self or to one of his "childs".
      // -> let upper code decide what to do
      if (!pTokenCheck(msgque->srvT,"____")) {
        //pDLog(msgque,7,"fEvent in<%p>, token<%s>\n", msgque, pTokenGetCurrent(msgque->srvT));
        return MQ_CONTINUE;
      }
    }

    // 3. how long does it takes until now
    nowT = time (NULL) - startT;
  }
  while (nowT < timeout);

  MqDLogC(msgque,6,"END-ERROR-TIMEOUT\n");
  return MqErrorDb(MQ_ERROR_TIMEOUT);

end:
  MqDLogC(msgque,6,"END-OK\n");
  return MQ_OK;

error:
  MqDLogC(msgque,6,"END-ERROR\n");
  return MqErrorStack (msgque->error);
}

/// \attention this code should call all available event-loop functions
enum MqErrorE
pUSleep (
  register struct MqS const * const msgque,
  long const usec
)
{
  struct MqErrorS * const error = MQ_ERROR_S;
  struct MqContextS * const context = msgque->config.myCtx;

  if (msgque->config.fEvent != NULL && context != NULL) {
    const MqEventF proc = msgque->config.fEvent;
    struct mq_timeval start;
    struct mq_timeval now;
    MqErrorCheck (SysGetTimeOfDay (error, &start, NULL));
    do {
      (*proc) (context);
      MqErrorCheck (MqSysUSleep (error, 99999L));
      MqErrorCheck (SysGetTimeOfDay (error, &now, NULL));
    }
    while (((now.tv_sec - start.tv_sec) * 1000000L + (now.tv_usec - start.tv_usec)) < usec);
  } else {
    MqErrorCheck (MqSysUSleep (error, usec));
  }

  return MQ_OK;

error:
  return MqErrorStack (error);
}

enum MqErrorE
MqProcessEvent (
  struct MqS * const msgque,
  MQ_TIME_T timeout,
  enum MqWaitOnEventE const wait
)
{
  enum MqErrorE ret = MQ_OK;
  const int forever = (wait >= MQ_WAIT_FOREVER);
  const int once = (wait >= MQ_WAIT_ONCE);
  struct timeval tv = {0L, 0L};

  // protection code
  if (unlikely(msgque == NULL)) return MQ_OK;

  // first announce that the event-handling is active
  if (forever) {
    MqDLogC(msgque,4,"send token<_PEO>\n");
    MqSendSTART (msgque->send);
    MqErrorCheck (MqSendEND (msgque->send, "_PEO"));
  }

  // set the default for timeout
  if (timeout < 0) {
    timeout = (timeout == -2 ? pIoGetTimeout(msgque->io) : MQ_TIMEOUT);
  }

  MqDLogC(msgque,6,"START\n");
  do {
    // ################ CHECK TO BE READABLE ##################
    if (once) {
      switch (ret = pWaitOnEvent (msgque, MQ_SELECT_RECV, timeout)) {
        case MQ_OK:
          break;
        case MQ_CONTINUE:
	  ret = MQ_OK;
          continue;
        case MQ_ERROR:
          goto error;
        case MQ_EXIT:
          goto end;
      }
    }

    // ##################### Process Events #####################
    MqErrorCheck (ret = pIoSelectStart(msgque->io, &tv, sMqEventStart));
    // !! ATTENTION -> msgque is gone after an server "SHD"
  }
  while (forever && ret == MQ_OK);

end:
  MqDLog(msgque,6,"END-%s\n", MqLogStatus(ret));
  return ret;

error:
  MqDLogC(msgque,6,"END-ERROR\n");
  return MqErrorStack (msgque->error);
}

static enum MqErrorE
pWaitForToken (
  struct MqS * const msgque,
  MQ_TIME_T const timeout,
  MQ_STRB const * const str
)
{
  register MQ_TIME_T const startT = time (NULL);

  // 1. check if the token is allready available
  if (pTokenCheck(msgque->srvT,str)) 
    return MQ_OK;

  // 2. wait for the token
  do {
    MqDLog(msgque,4,"wait for token<%s>\n",str);
    MqErrorSwitch (MqProcessEvent(msgque,timeout,MQ_WAIT_ONCE));
  } while (!pTokenCheck(msgque->srvT,str) && (time(NULL)-startT) < timeout);

  MqDLog(msgque,4,"got token<%s>\n",pTokenGetCurrent(msgque->srvT));

  if (!pTokenCheck(msgque->srvT,str))
    return MqErrorDbV (MQ_ERROR_WAIT_FOR_TOKEN, (time(NULL)-startT), str, pTokenGetCurrent(msgque->srvT));

  return MQ_OK;

error:
  return MqErrorStack (msgque->error);
}

/*****************************************************************************/
/*                                                                           */
/*                                misc                                       */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE MQ_BOL
MqCurrentTokenIs(
  struct MqS const * const msgque,
  char const * const str
)
{
  return pTokenCheck(msgque->srvT,str);
}

MQ_INLINE MQ_BOL
MqIsTransaction (
  struct MqS const * const msgque
)
{
  return ( msgque->_trans == 0 ? MQ_NO : MQ_YES ) ;
}

enum MqErrorE
MqCheckForLeftOverArguments (
  struct MqS const * const msgque,		
  struct MqBufferLS ** argvP
)
{
  MQ_SIZE i;
  MQ_STR str;
  struct MqErrorS * error;
  struct MqBufferLS * argv;

  if (unlikely(argvP == NULL || *argvP == NULL)) {
    return MQ_OK;
  }
  argv = *argvP;
  
  if (argv->cursize == 0) {
    MqBufferLDelete(argvP);
    return MQ_OK;
  }

  error = (msgque != NULL ? msgque->error : MQ_ERROR_PANIC);
  MqErrorDb2 (error,MQ_ERROR_INVALID_OPTION);
  for (i = 0; i < argv->cursize; i++) {
    if ( MqBufferGetC (error, argv->data[i], &str) != MQ_ERROR) 
	MqErrorSAppendV (error, "=> %s", str);
  }
  MqBufferLDelete(argvP);
  return MQ_ERROR;
}

/*****************************************************************************/
/*                                                                           */
/*                              service                                      */
/*                                                                           */
/*****************************************************************************/

enum MqErrorE 
MqServiceCreate(
  struct MqS const * const msgque, 
  MQ_STRB const * const token,
  MqTokenF const proc,
  MQ_PTR data,
  MqTokenDataFreeF datafreeF
)
{
  // protection code
  if (unlikely(msgque == NULL)) return MQ_OK;

  if (strlen(token) != HDR_TOK_LEN) {
    return MqErrorDbV (MQ_ERROR_TOKEN_LENGTH, token);
  }
  return (pTokenAddHdl (msgque->srvT, token, proc, data, datafreeF));
}

enum MqErrorE 
MqServiceDelete(
  struct MqS const * const msgque, 
  MQ_STRB const * const token
)
{
  // protection code
  if (unlikely(msgque == NULL)) return MQ_OK;
  return (pTokenDelHdl (msgque->srvT, token));
}

/*****************************************************************************/
/*                                                                           */
/*                                misc                                       */
/*                                                                           */
/*****************************************************************************/

MQ_SIZE
MqGetCtxId (
  struct MqS const * const msgque
)
{
  return msgque->ctxId;
}

const char *
MqGetToken (
  struct MqS const * const msgque
)
{
  return (msgque->srvT != NULL? pTokenGetCurrent(msgque->srvT) : NULL);
}

struct MqSendS const *
MqGetSendS (
  struct MqS const * const msgque
)
{
  return (msgque != NULL ? msgque->send : NULL);
}

struct MqConfigS const *
MqGetConfigS (
  struct MqS const * const msgque
)
{
  return &msgque->config;
}

struct MqErrorS const *
MqGetErrorS (
  struct MqS const * const msgque
)
{
  return (msgque != NULL ? msgque->error : NULL);
}

MQ_INLINE struct MqS *
pMqGetFirstParent(
  struct MqS * const msgque
)
{
  return msgque->ctxIdP;
}

void
MqExit (
  struct MqS * msgque
)
{
  int num;
  MqExitF parentExitF;
  MQ_BOL isThread;

  // no error object
  if (msgque == NULL) SysExit(0,0);
  if (msgque->onExit == MQ_YES || msgque->config.parent != NULL) 
    MqPanicSYS(msgque->error);
  msgque->onExit = MQ_YES;
  MqDLogC(msgque,3,"EXIT\n");

  // save msgque data because the next section will delete it
  num = MqErrorGetNum(msgque->error);
  parentExitF = msgque->config.fParentExit;
  isThread = (msgque->config.statusIs & MQ_STATUS_IS_THREAD);

  // cleanup all the data
  MqDelete(&msgque);

  // call parent exit function
  if (parentExitF) {
    (*parentExitF)(isThread, num);
  }

  // final exit thread or process
  SysExit(isThread, num);
}

/*****************************************************************************/
/*                                                                           */
/*                                DLL                                        */
/*                                                                           */
/*****************************************************************************/

#if defined(_MSC_VER)

void sGenericCreate ();
void sGenericDelete ();
void sEventCreate ();
void sEventDelete ();

BOOL APIENTRY DllMain( 
    HMODULE hModule,
    DWORD  ul_reason_for_call,
    LPVOID lpReserved
)
{
  switch (ul_reason_for_call)
  {
    case DLL_THREAD_ATTACH:
    case DLL_THREAD_DETACH:
      return FALSE;
    case DLL_PROCESS_ATTACH:
      sEventCreate();
      sGenericCreate();
      break;
    case DLL_PROCESS_DETACH:
      sEventDelete();
      sGenericDelete();
      break;
  }
  return TRUE;
}
#endif

/*****************************************************************************/
/*                                                                           */
/*                                debugging                                  */
/*                                                                           */
/*****************************************************************************/

#ifdef _DEBUG
void
MqLogData (
  struct MqS const * const msgque,
  MQ_STRB const * const prefix
)
{
  MqDLogV (msgque, prefix, 0, ">>>> MqS (%p)\n", msgque);

  MqDLogV (msgque, prefix, 0, "debug     = " MQ_FORMAT_I "\n", msgque->config.debug);
  MqDLogV (msgque, prefix, 0, "silent    = " MQ_FORMAT_C "\n", (msgque->config.isSilent ? "yes" : "no"));
  MqDLogV (msgque, prefix, 0, "string    = " MQ_FORMAT_C "\n", (msgque->config.isString ? "yes" : "no"));
  MqDLogV (msgque, prefix, 0, "ctxId     = " MQ_FORMAT_I "\n", msgque->ctxId);
  MqDLogV (msgque, prefix, 0, "sOc       = " MQ_FORMAT_C "\n", MqLogServerOrClient (msgque));
  MqDLogV (msgque, prefix, 0, "pOc       = " MQ_FORMAT_C "\n", MqLogParentOrChild (msgque));
  MqDLogV (msgque, prefix, 0, "error     = " MQ_FORMAT_P "\n", msgque->error);

  MqDLogV (msgque, prefix, 0, "<<<< MqS\n");
}

void
MqLogChild (
  struct MqS const * const msgque,
  char const * const prefix
)
{
  struct pChildS * childs;
  MqDLogV(msgque,prefix,0,"MqLogMsgque >>>>>\n");
  MqDLogV(msgque,prefix,0,"server=%s, parent=%p\n", (MQ_IS_SERVER(msgque)?"yes":"no"), msgque->config.parent);
  for(childs = msgque->childs; childs != NULL ; childs = childs->right) {
    MqDLogV(msgque,prefix,0,"child: msgque=%p\n", childs->msgque);
  }
  MqDLogV(msgque,prefix,0,"MqLogMsgque <<<<<\n");
}
#endif /* _DEBUG */


