/**
 *  \file       src/msgque.h
 *  \brief      \$Id: msgque.h 1207 2009-05-20 12:53:39Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1207 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef MQ_MSGQUE_H
#define MQ_MSGQUE_H

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                   Visual C++ Express-Edition                    ### */
/* ###                                                                 ### */
/* ####################################################################### */

#if defined(_MSC_VER)
# define __attribute__(dummy)
# define pid_t int
# define mode_t int
# define __func__ __FUNCTION__
# define va_copy(a,b) a = b
# define mq_strtoll _strtoi64
# define mq_strdup _strdup
# define mq_getpid _getpid
# define mq_unlink _unlink
# define mq_snprintf _snprintf
# define MQ_STDCALL __stdcall
# define MQ_CDECL __cdecl
# define MQ_INLINE
#else
# define mq_strtoll strtoll
# define mq_strdup strdup
# define mq_getpid getpid
# define mq_unlink unlink
# define mq_snprintf snprintf
# define MQ_STDCALL
# define MQ_CDECL
# if defined(__cplusplus)
#   define MQ_INLINE
# else
#   define MQ_INLINE inline
# endif
#endif

#if defined(MQ_COMPILE_AS_CC)
# define MQ_DECL MQ_STDCALL
#else
# define MQ_DECL MQ_CDECL
#endif

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                      M S G Q U E - A P I                        ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \defgroup type_API PUBLIC: Type Definition
 *  \ingroup lib
 *  \{
 *  \brief a collection of common used definitions
 */

/* BEGIN_C_DECLS should be used at the beginning of your declarations,
  so that C++ compilers don't mangle their names.  Use END_C_DECLS at
  the end of C declarations. */

#undef BEGIN_C_DECLS
#undef END_C_DECLS
#if defined(__cplusplus) && !defined(MQ_COMPILE_AS_CC)
# define BEGIN_C_DECLS extern "C" {
# define END_C_DECLS }
#else
# define BEGIN_C_DECLS /* empty */
# define END_C_DECLS /* empty */
#endif

/* Somewhere in the middle of the GCC 2.96 development cycle, we implemented
   a mechanism by which the user can annotate likely branch directions and
   expect the blocks to be reordered appropriately.  Define __builtin_expect
   to nothing for earlier compilers.  */

#if defined(_MSC_VER) || (__GNUC__ == 2 && __GNUC_MINOR__ < 96)
#   define __builtin_expect(x, expected_value) (x)
#endif

/// \brief gcc jump optimization
#define likely(x)       __builtin_expect((x),1)

/// \brief gcc jump optimization
#define unlikely(x)     __builtin_expect((x),0)

#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <limits.h>

//start c++ save defibition
BEGIN_C_DECLS

//15 min timeout
#ifndef MQ_TIMEOUT
/// \brief maximum timeout in sec (900 sec)
#define MQ_TIMEOUT	900
/// \brief long timeout in sec (180 sec)
#define MQ_TIMEOUT5	(MQ_TIMEOUT/5)
/// \brief normal timeout in sec (90 sec)
#define MQ_TIMEOUT10	(MQ_TIMEOUT/10  < 1 ? 1 : MQ_TIMEOUT/10)
/// \brief short timeout in sec (20 sec)
#define MQ_TIMEOUT45	(MQ_TIMEOUT/45  < 1 ? 1 : MQ_TIMEOUT/45)
/// \brief shorter timeout in sec (10 sec)
/// This TIMEOUT is used for socket connection with 'connect'
#define MQ_TIMEOUT90	(MQ_TIMEOUT/90  < 1 ? 1 : MQ_TIMEOUT/90)
/// \brief very short timeout in sec (5 sec)
#define MQ_TIMEOUT180	(MQ_TIMEOUT/180 < 1 ? 1 : MQ_TIMEOUT/180)
/// \brief request the user defined timeout value from the argument of the \c --timeout option
#define MQ_TIMEOUT_USER -2
#endif

/*****************************************************************************/
/*                                                                           */
/*                          windows conform extern                           */
/*                                                                           */
/*****************************************************************************/

// if the source of libmsgque is direct used in external library
// (without dynamic linking) and this library should *not* export
// the symbol's of tclmsgque then the flag MQ_IGNORE_EXTERN have
// to be set
#if defined(PIC) && !defined(MQ_IGNORE_EXTERN)
    // does we build a DLL ?
#   ifdef DLL_EXPORT
        // does we build the libmsgque library ?
#	ifdef MQ_BUILD_LIBMSGQUE_DLL
#	    define MQ_EXTERN __declspec(dllexport)
#	else
#	    define MQ_EXTERN __declspec(dllimport)
#	endif
#   else
    // no DLL
    /// \brief architecture specific extern specifier
#   define MQ_EXTERN extern
#   endif
#else
/// define the external binding
#   define MQ_EXTERN
#endif

/*****************************************************************************/
/*                                                                           */
/*                            types/definition                               */
/*                                                                           */
/*****************************************************************************/

/// \brief connecting of expressions, step 1
#define MQ_CPPXSTR(s) MQ_CPPSTR(s)
/// \brief connecting of expressions, step 2
#define MQ_CPPSTR(s) #s

/// \brief MQ_ALFA the command-line separator between server and client
#define MQ_ALFA '@'
#define MQ_ALFA_STR "@"

struct MqSendS;
struct MqReadS;
struct MqIoS;
struct MqTransS;
struct MqErrorS;
struct MqBufferLS;
struct MqBufferS;
union  MqBufferU;
struct MqS;
struct MqConfigS;
struct MqEventS;

/*****************************************************************************/
/*                                                                           */
/*                            types/generic                                  */
/*                                                                           */
/*****************************************************************************/

/// 1 byte \b byte data-type
typedef signed char MQ_BYT;
/// 1 byte \b boolean data-type
typedef signed char MQ_BOL;
/// 2 byte \b short data-type
typedef short MQ_SRT;
/// 4 byte \b integer data-type
typedef int MQ_INT;
/// 4 byte \b float data-type
typedef float MQ_FLT;
/// 8 byte \b wide integer data-type
#if defined(_MSC_VER)
typedef __int64 MQ_WID;
#else
typedef long long int MQ_WID;
#endif
/// 8 byte \b double data-type
typedef double MQ_DBL;
/// 8 byte \b atom data-type
typedef union MqBufferAtomU MQ_ATO;

/// array size data-type
/// \attention -> need signed integer in pReadWord
///            -> need int because of pSwapI in pReadHDR
typedef int MQ_SIZE;

/// \brief OS wide MQ_TIME_T
typedef MQ_WID MQ_TIME_T;

/// handle data-type
typedef int MQ_HDL;

/// pointer basic data-type
typedef void MQ_PTRB;
/// string basic data-type
typedef char MQ_STRB;
/// binary basic data-type
typedef unsigned char MQ_BINB;
/// list basic data-type
typedef unsigned char MQ_LSTB;

/// generic pointer data-type
typedef MQ_PTRB *MQ_PTR;
/// string pointer data-type
typedef MQ_STRB *MQ_STR;
/// binary pointer data-type
typedef MQ_BINB *MQ_BIN;
/// list pointer data-type
typedef MQ_LSTB *MQ_LST;
/// Buffer pointer data-type
typedef struct MqBufferS *MQ_BUF;

/*****************************************************************************/
/*                                                                           */
/*                            enum definitions                               */
/*                                                                           */
/*****************************************************************************/

/// \ingroup msgque_api
/// \brief 'select' type for reading or writing
enum MqIoSelectE {
  MQ_SELECT_RECV = (1 << 0),    ///< 'select' for reading
  MQ_SELECT_SEND = (1 << 1),	///< 'select' for writing
};

/// \ingroup msgque_api
/// \brief boolean NO
#define MQ_NO  0

/// \ingroup msgque_api
/// \brief boolean YES
#define MQ_YES 1

/// \ingroup event_api
/// \brief wait for an event?
enum MqWaitOnEventE {
  MQ_WAIT_NO      = 0,	///< just do the check
  MQ_WAIT_ONCE    = 1,	///< wait for one new event
  MQ_WAIT_FOREVER = 2,	///< wait forever
};

/// \ingroup buffer_api
/// \brief the type is native and has a size of 1 byte
#define MQ_TYPE_IS_1_BYTE   (1<<0)

/// \ingroup buffer_api
/// \brief the type is native and has a size of 2 bytes
#define MQ_TYPE_IS_2_BYTE   (1<<1)

/// \ingroup buffer_api
/// \brief the type is native and has a size of 4 bytes
#define MQ_TYPE_IS_4_BYTE   (1<<2)

/// \ingroup buffer_api
/// \brief the type is native and has a size of 8 bytes
#define MQ_TYPE_IS_8_BYTE   (1<<3)

/// \ingroup buffer_api
/// \brief the type is native
#define MQ_TYPE_IS_NATIVE   (	MQ_TYPE_IS_1_BYTE | MQ_TYPE_IS_2_BYTE |	    \
				MQ_TYPE_IS_4_BYTE | MQ_TYPE_IS_8_BYTE	)

/// \ingroup buffer_api
/// \brief a enum for all data types supported by #MqBufferS
///
/// The type is a one byte character with the following syntax:
/// - bit 1 up to 4 has the size of the native type
/// - bit 5 up to 8 has the type, up to 16 types are allowed
///
/// \attention In the package the space for the type is only \e one char. If additional
/// space is needed the protocol have to be adjusted
enum MqTypeE {
  MQ_BYTT = (1<<4 | MQ_TYPE_IS_1_BYTE),  ///< Y: 1 byte 'byte' type
  MQ_BOLT = (2<<4 | MQ_TYPE_IS_1_BYTE),  ///< O: 1 byte 'boolean' type
  MQ_SRTT = (3<<4 | MQ_TYPE_IS_2_BYTE),  ///< S: 2 byte 'short' type
  MQ_INTT = (4<<4 | MQ_TYPE_IS_4_BYTE),  ///< I: 4 byte 'int' type
  MQ_FLTT = (5<<4 | MQ_TYPE_IS_4_BYTE),  ///< F: 4 byte 'float' type
  MQ_WIDT = (6<<4 | MQ_TYPE_IS_8_BYTE),  ///< W: 8 byte 'long long int' type
  MQ_DBLT = (7<<4 | MQ_TYPE_IS_8_BYTE),  ///< D: 8 byte 'double' type
  MQ_BINT = (8<<4                    ),  ///< B: binary array type
  MQ_STRT = (9<<4                    ),  ///< C: character array type (e.g. with a \\0 at the end)
  MQ_LSTT = (10<<4                   ),  ///< L: list object type
  MQ_RETT = (11<<4                   ),  ///< R: return object type
};

/// \ingroup buffer_api
/// \brief allocation style used for the data-segment in #MqBufferS.
enum MqAllocE {
  MQ_DYNAMIC,                   ///< dynamic allocation (e.g. MqSysMalloc, ...)
  MQ_STATIC                     ///< static allocation (e.g. fixed size string)
};

/// \ingroup error_api
/// \brief collection for the different error-codes
enum MqErrorE {
  MQ_OK,                        ///< everything is OK, no error available (persistent)
  MQ_ERROR,                     ///< exit upper code with an error (persistent)
  MQ_CONTINUE,                  ///< continue with upper code
  MQ_EXIT			///< exit parent context
};

/** \} type_api */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                    M S G Q U E - A P I                          ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \defgroup msgque_api PUBLIC: struct MqS
 *  \ingroup lib
 *  \{
 *  \brief the <EM>MqS API</EM> is used as \libmsgque object.
 *
 *  the \e MqS object is created just after the \e MqContextS object usually
 *  in the function \c ContextCreate. every external \e MqContextS object has
 *  only one \e MqS object and every \e MqS object has only one \e MqContextS
 *  object. the \e MqS object will be created with the \c MqCreate and will be deleted 
 *  with the \c MqDelete function. during creation of a \e MqS object additional
 *  objects will be created too:
<TABLE>
    <TR> <TD> \b object-type </TD>  <TD> \b member </TD>    <TD> \b description </TD> </TR>
    <TR> <TD> MqSendS </TD>	    <TD> send </TD>	    <TD> used for building and sending a Msgque packet </TD> </TR>
    <TR> <TD> MqReadS </TD>	    <TD> read </TD>	    <TD> used for reading and splitting a Msgque packet </TD> </TR>
    <TR> <TD> MqIoS </TD>	    <TD> io </TD>	    <TD> used for doing socket communication </TD> </TR>
    <TR> <TD> MqErrorS </TD>	    <TD> error </TD>	    <TD> used for collection of error data </TD> </TR>
</TABLE>
 */

/*****************************************************************************/
/*                                                                           */
/*                            msgque/definition                              */
/*                                                                           */
/*****************************************************************************/

/// \brief Prototype for a context object
struct MqContextS {
  struct MqS * msgque;		    ///< link between MqContextS and MqS
};

/// \brief Prototype for a managed context object (used for example by C#)
struct MqManagedContextS {
  struct MqS * msgque;		    ///< link between MqContextS and MqS
  void * self;			    ///< link to the managed object
};

/// \brief the prototype for a ContextCreate function
///
/// This function is the MqS::fCreate and MqS::fThreadCreate parameter and is used to :
/// -# create a \e child context
/// -# create a \e parent context as the thread entry point
/// .
typedef enum MqErrorE ( MQ_DECL
  *MqCreateF
) (
  struct MqConfigS  *,		    ///< [in,out] context configuration (NULL is not allowed)
  struct MqBufferLS *,		    ///< [in,out] this is the \e argv object
  struct MqBufferLS *,		    ///< [in,out] this is the \e alfa object
				    ///< (memory will be freed after use)
  struct MqS       **		    ///< [out] the new created #MqS object
) __attribute__((nonnull (1, 4)));

/// \brief the prototype for a ContextDelete function
///
/// This function is the MqS::fDelete and MqS::fThreadDelete parameter and is used to:
/// -# delete a \e child context
/// -# delete a \e parent context as the thread exit point
/// .
typedef void ( MQ_DECL
  *MqDeleteF
) (
  struct MqS ** const msgqueP
);

/// \brief prototype for a generic token processing function
typedef enum MqErrorE ( MQ_DECL
  *MqTokenF
) (
  struct MqS * msgque,
  MQ_PTR data
);

/// \brief prototype for a free additional token data function
typedef void ( MQ_DECL
  *MqTokenDataFreeF
) (
  struct MqS const * const msgque,
  MQ_PTR data
);

/// \brief prototype for a exit-process function (default: \c exit )
typedef void ( MQ_DECL
  *MqExitF
) (
  int isThread,
  int exitcode
);

/// \brief Return a \e main function found by \e name
///
/// This function have to be defined in the target tool (example: \c atool.c)
/// and have to return the \e main function of type #MqCreateF or \e NULL,
/// The result is used as Entry-Point for \e fork and \e thread startup.
typedef MqCreateF ( MQ_DECL
    *MqMainSelectorF
) (
    const char *name     ///< the string name of the function to select
);

/// \brief prototype for a Event-Checking function
/// \ingroup event_api
typedef void ( MQ_DECL
  *MqEventF
) (
  struct MqContextS * const
);

/// \brief application specific help function type
typedef void ( MQ_DECL
  *MqHelpF
) (
  const char *		///< the basename of the tool
) __attribute__ ((noreturn));

/** \defgroup config_api PUBLIC: struct MqConfigS
 *  \ingroup msgque_api
 *  \{
 *  \brief Configuration data of the #MqS object
 *
 *  The configuration object #MqConfigS is created with #MqConfigInit or #MqConfigDup and
 *  it is used as first argument of #MqCreate for creating a new #MqS object.
 */

/// \brief Information about how the \e context was created
enum MqStatusIsE {
  MQ_STATUS_IS_INITIAL = 0,	    ///< context is the \e first context
  MQ_STATUS_IS_DUP     = 1<<0,	    ///< context is created as a duplicate of an other context
  MQ_STATUS_IS_THREAD  = 1<<1,	    ///< context is created as a thread
  MQ_STATUS_IS_FORK    = 1<<2,	    ///< context is created as a fork
  MQ_STATUS_IS_SPAWN   = 1<<3,	    ///< context is created as a spawn
  MQ_STATUS_IS_MANAGED = 1<<4	    ///< context is created as "managed" using #MqConfigInitManagedContext
};

/// \brief User preferences on HOWTO start a new entity
enum MqStartE {
  MQ_START_DEFAULT     = 0,	    ///< use application-context default entity creation
  MQ_START_FORK        = 1,	    ///< create entity as \e fork
  MQ_START_THREAD      = 2,	    ///< create entity as \e thread
  MQ_START_SPAWN       = 3,	    ///< create entity as \e spawn process
};

/// \brief configuration data of the #MqS object
struct MqConfigS {

  // link management
  struct MqContextS * myCtx;		///< link to the #MqContextS object of the user defined configuration data

  // misc configuration
 
  /// \brief Suppress logging output
  /// <TABLE>
  /// <TR>  <TH>type</TH> <TH>default</TH> <TH>option</TH>    <TH>application</TH>   <TH>context</TH>        </TR>
  /// <TR>  <TD>BOOL</TD> <TD>NO</TD>      <TD>--silent</TD>  <TD>server/client</TD> <TD>parent/child</TD>   </TR>
  /// </TABLE>
  MQ_BOL isSilent;

  /// \brief Use \e string mode, send as many data as possible as string
  /// <TABLE>
  /// <TR>  <TH>type</TH> <TH>default</TH> <TH>option</TH>    <TH>application</TH>   <TH>context</TH>        </TR>
  /// <TR>  <TD>BOOL</TD> <TD>NO</TD>      <TD>--string</TD>  <TD>client</TD>        <TD>parent</TD>         </TR>
  /// </TABLE>
  ///
  /// \libmsgque distinguish between a \e string based (option: \c --string) and a
  /// \e binary based (this is the default) data packages. The items is a \libmsgque data
  /// package are of type #MqBufferU including OS specific native data of type #MQ_ATO.
  /// The native data can be transmitted as \e string (slow) or as \e binary (fast).
  /// \e string is used for debugging purpose or if the binary data model is different
  /// between client and server. The difference in endianness (http://en.wikipedia.org/wiki/Endian)
  /// is handled by \libmsgque and does not require using the \c --string option.
  /// The options is only available for the \e client-parent setup.
  MQ_BOL isString;

  /// \brief Create a \e server context
  /// <TABLE>
  /// <TR>  <TH>type</TH> <TH>default</TH> <TH>option</TH> <TH>application</TH>   <TH>context</TH>        </TR>
  /// <TR>  <TD>BOOL</TD> <TD>NO</TD>      <TD>NO</TD>     <TD>server</TD>        <TD>parent</TD>         </TR>
  /// </TABLE>
  ///
  /// If the \e server option is available the #MqCreate will create an \libmsgque object useable
  /// as a server. For detail information read: \main_cspc
  MQ_BOL isServer;

  /// \brief Information about how the \e context was created
  /// \attention This data is \b only set by \e libmsgque and is available for read only
  enum MqStatusIsE statusIs;

  /// \brief User preferences on HOWTO start a new entity
  /// <TABLE>
  /// <TR>  <TH>type</TH> <TH>default</TH>	     <TH>option</TH>   <TH>application</TH>   <TH>context</TH> </TR>
  /// <TR>  <TD>ENUM</TD> <TD>#MQ_START_DEFAULT</TD> <TD>NO</TD>       <TD>client/server</TD> <TD>parent</TD>  </TR>
  /// <TR>  <TD>ENUM</TD> <TD>#MQ_START_FORK</TD>    <TD>--fork</TD>   <TD>client/server</TD> <TD>parent</TD>  </TR>
  /// <TR>  <TD>ENUM</TD> <TD>#MQ_START_THREAD</TD>  <TD>--thread</TD> <TD>client/server</TD> <TD>parent</TD>  </TR>
  /// <TR>  <TD>ENUM</TD> <TD>#MQ_START_SPAWN</TD>   <TD>--spawn</TD>  <TD>client/server</TD> <TD>parent</TD>  </TR>
  /// </TABLE>
  ///
  /// The #MQ_START_DEFAULT depend on the application-context:
  /// - <I>server setup</I>\n Start a \e UDP or \e TCP server 
  ///   capable to serve \b one incoming \e client connection.
  /// - <I>client setup</I>\n Start a \e PIPE server using the following order:
  ///   - \e fork   (if the \c fork system-call is available)
  ///   - \e thread (if \libmsgque was compiled with \c --enable-threads)
  ///   - \e spawn  (this always works)
  ///   .
  /// .
  /// \attention In \e server mode the \c --fork , \c --thread and the \c --spawn options require the 
  ///   #MqConfigS::fParentCreate and #MqConfigS::fParentDelete value to provide an entry-point after 
  ///   the new entity was created. In addition the option #MqConfigS::fParentExit is used to exit the 
  ///   entity.
  /// \attention The \c --fork option require the \b fork system-call and is \b not compatible with
  ///   threads.
  enum MqStartE startAs;

/** \brief The human-readable name of context
 *  <TABLE>
 *  <TR>  <TH>type</TH>   <TH>default</TH> <TH>option</TH>  <TH>application</TH>   <TH>context</TH>   </TR>
 *  <TR>  <TD>STRING</TD> <TD>unknown</TD> <TD>--name</TD>  <TD>server/client</TD> <TD>parent</TD>    </TR>
 *  </TABLE>
 * 
 *  The name is used as prefix for application specific log and error messages:
\verbatim
C> (NAME) [2009-01-12:16-22-27] [4-0-sIoCheckArg]: option:  io->com = PIPE
\endverbatim
 *  and is created using the following steps:
 *  - initial set to \e unknown
 *  - the argument of the \c --name option in \e argv
 *  - the #MqConfigS::name entry from the related configuration object
 *  - the basename of the first option in \e argv
 *  .
 *  \attention 
 *     - the memory of the \e name data-entry is managed by \libmsgque and freed during
 *       deleting of the #MqS object. A Value of \c NULL is allowed. If the #MqConfigS object is 
 *       created with #MqConfigDup the \e name data-entry is a copy of the original \e name data-
 *       entry
 *     - the \e name data-entry will be replaced by the argument of the \c --srvname command-line
 *       option of the client.
 *     - to init this value with a static string use: \code
config->name = mq_strdup("myString");
\endcode
 *     .
 */
  MQ_STR name;

  /// \brief Send additional debugging output to stderr
  /// <TABLE>
  /// <TR>  <TH>type</TH>          <TH>default</TH> <TH>option</TH>  <TH>application</TH>   <TH>context</TH>        </TR>
  /// <TR>  <TD>INTEGER (0-9)</TD> <TD>0</TD>       <TD>--debug</TD> <TD>server/client</TD> <TD>parent/child</TD>   </TR>
  /// </TABLE>
  /// \attention Using pipe (e.g. --pipe) the \e server get the \e debug flag from the \e client.
  MQ_INT debug;

  /// \brief The timeout is used for all kind of low-level socket operations like \c send, \c recv and \c connect
  /// <TABLE>
  /// <TR>  <TH>type</TH>          <TH>default</TH>       <TH>option</TH>    <TH>application</TH>   <TH>context</TH>  </TR>
  /// <TR>  <TD>INTEGER (sec)</TD> <TD>MQ_TIMEOUT180</TD> <TD>--timeout</TD> <TD>server/client</TD> <TD>parent</TD>   </TR>
  /// </TABLE>
  /// \attention This \c timeout is independent of the per transaction timeout of #MqSendEND_AND_WAIT.
  MQ_TIME_T timeout;

  /// \brief OS specific value for socket operation buffer
  /// <TABLE>
  /// <TR>  <TH>type</TH>           <TH>default</TH> <TH>option</TH>       <TH>application</TH>   <TH>context</TH>  </TR>
  /// <TR>  <TD>INTEGER (byte)</TD> <TD>4096</TD>    <TD>--buffersize</TD> <TD>server/client</TD> <TD>parent</TD>   </TR>
  /// </TABLE>
  MQ_INT buffersize;


  /// \brief Create a child-context
  /// <TABLE>
  /// <TR>  <TH>type</TH>    <TH>default</TH> <TH>option</TH> <TH>application</TH>  <TH>context</TH>  </TR>
  /// <TR>  <TD>POINTER</TD> <TD>NULL</TD>    <TD>NO</TD>       <TD>client</TD>       <TD>parent</TD>   </TR>
  /// </TABLE>
  ///
  /// Read more in: \main_cspc
  struct MqS * parent;
 
  // \brief pointer to the Server-Child-Context-Create function
  MqCreateF fChildCreate;

  /// \brief pointer to the Server-Child-Context-Delete function
  /// \attention always call this function using #MqDelete
  MqDeleteF fChildDelete;

  // \brief pointer to the Server-Parent-Context-Create function
  // - A server has, by default, only one Parent-Context. Multiple Parent-Context are available
  // with the #MqCreate -> \c argv option "--thread".
  // - We don't use "#if defined(MQ_HAS_THREAD)" because we don't want to have a thread dependency 
  // in external code -> later if the option "--thread" is used on a *not* thread enabled
  // libmsgque an runtime-error is created.
  MqCreateF fParentCreate;

  /// \brief pointer to the Server-Parent-Context-Delete function
  /// \attention always call this function using #MqDelete
  MqDeleteF fParentDelete;

  /// \brief pointer to the Server-Parent-Exit function (default: \c pthread_exit )
  MqExitF fParentExit;

/** \brief is this a filter (default: MQ_NO => no filter)
 * 
 *  A filter is the tool in a middle of a command pipeline and has a "server" connection to the left
 *  and a "client" connection to the right:
 *  \verbatim
client @ filter @ filter @ filter @ server
\endverbatim
 *  - \c client : start with \e server=MQ_NO
 *  - \c filter : start with \e server=MQ_YES and \e filter=MQ_YES
 *  - \c server : start with \e server=MQ_YES
 */
  MQ_BOL isFilter;

  /// \brief pointer to the Filter function
  MqTokenF fFTR;

  /// \brief pointer to the Filter data
  MQ_PTR FTRdata;

  /// \brief procedure to free FTRdata
  MqTokenDataFreeF FTRfree;

  /// \brief pointer to the EndOfFile function
  MqTokenF fEOF;

  /// \brief pointer to the EndOfFile data
  MQ_PTR EOFdata;

  /// \brief procedure to free EOFdata
  MqTokenDataFreeF EOFfree;

  // misc

  /// \brief Create a link to the calling tool event-handling queue
  /// <TABLE>
  /// <TR>  <TH>type</TH>    <TH>default</TH> <TH>option</TH> <TH>application</TH>   <TH>context</TH>  </TR>
  /// <TR>  <TD>POINTER</TD> <TD>NULL</TD>    <TD>NO</TD>     <TD>server/client</TD> <TD>parent</TD>   </TR>
  /// </TABLE>
  ///
  /// Event-Handling is used to process instructions in the \e background to give the tool-user
  /// the feeling of a non-blocking application. For Example Tcl using the event-handling to
  /// update the Tk user-interface while the application is waiting for data. The event-handling
  /// is usually a loop to check event-sources from time-to-time and act on incoming events.
  MqEventF fEvent;

  /// \brief Create a link to the calling tool \e help function
  /// <TABLE>
  /// <TR>  <TH>type</TH>    <TH>default</TH> <TH>option</TH> <TH>application</TH>   <TH>context</TH>  </TR>
  /// <TR>  <TD>POINTER</TD> <TD>NULL</TD>    <TD>NO</TD>     <TD>server/client</TD> <TD>parent</TD>   </TR>
  /// </TABLE>
  ///
  /// As a service to the programmer the \libmsgque startup code in #MqCreate checks for the
  /// both options \c -h and \c --help to provide a tool-specific help-page and exit.
  MqHelpF fHelp;

  /// \brief do not allow the usage of \e threads
  MQ_BOL ignoreThread;

  /// \brief do not allow the usage of \e spawn
  MQ_BOL ignoreSpawn;

  /// \brief do not allow the usage of \e fork
  MQ_BOL ignoreFork;

  /// \brief SLAVE: a  pointer to the master object or \c NULL
  struct MqS * master;

  /// \brief SLAVE: the ID in the master SLAVES array or \c O
  MQ_SIZE master_id;
};

/**
 * \}
 */

#ifndef MQ_PRIVATE

/// \brief the prefix arguments of the starting application
MQ_EXTERN struct MqBufferLS * MqInitBuf;

/// \brief name of a procedure to return \e main like entry-points 
MQ_EXTERN MqMainSelectorF MqMainSelector;

/** \brief the main \libmsgque object.
 *
 * \copydoc msgque_api
 */

struct MqS {

  const struct MqConfigS config;    ///< public #MqS configuration

  struct MqBufferS * const temp;    ///< misc temporary \e MqBufferS object

  struct MqSendS  * const send;	    ///< object for sending a Msgque packet
  struct MqReadS  * const read;	    ///< object for reading a Msgque packet
  struct MqIoS    * const io;	    ///< object for management of the 'socket' infrastructure
  struct MqErrorS * const error;    ///< object used to store error messages
};

#endif /* !MQ_PRIVATE */

/*****************************************************************************/
/*                                                                           */
/*                               msgque/init                                 */
/*                                                                           */
/*****************************************************************************/

/** \ingroup config_api
 *  \brief Free memory allocated by #MqConfigInit or #MqConfigDup
 *  \attention if the \e config object created by #MqConfigInit or #MqConfigDup is used
 *    as argument to #MqCreate the object is freed by #MqCreate -> do \b not use #MqConfigFree
 *    the free this object again.
 */
MQ_EXTERN void
MQ_DECL MqConfigFree (
  struct MqConfigS **
);

/** \ingroup config_api
 *  \brief create and initialize a new #MqConfigS object
 *  \return the new #MqConfigS object
 */
MQ_EXTERN struct MqConfigS *
MQ_DECL MqConfigInit (
  void
);

/** \ingroup config_api
 *  \brief create and initialize the \c config->myCtx using the \e managed context \e self
 *  \config
 *  \param[in] self a pointer to the managed context usually \e self or \e this
 *  \attention the context is freed at #MqDelete
 */
MQ_EXTERN void
MQ_DECL MqConfigInitManagedContext (
  struct MqConfigS * const config,
  void * self
);

/** \ingroup config_api
 *  \brief get the \e self pointer from #MqManagedContextS
 *  \config
 */
MQ_EXTERN void*
MQ_DECL MqConfigGetSelfFromManagedContext (
  struct MqConfigS const * const config
);

/** \ingroup config_api
 *  \brief get the \e self pointer from #MqManagedContextS
 *  \msgque
 */
MQ_EXTERN void*
MQ_DECL MqConfigGetSelfFromManagedContextM (
  struct MqS const * const msgque
);

/** \ingroup config_api
 *  \brief initialize the \c config->name
 *  \config
 *  \param[in] name a string as name
 */
MQ_EXTERN void
MQ_DECL MqConfigInitName (
  struct MqConfigS * const config,
  const char * name
);

/** \ingroup config_api
 *  \brief initialize the \c config->parent
 *  \config
 *  \param[in] parent the #MqS object of the parent
 */
MQ_EXTERN void
MQ_DECL MqConfigInitParent (
  struct MqConfigS * const config,
  struct MqS * const parent
);


/** \ingroup config_api
 *  \brief set the \c config->master field and initialize the \e static fields
 *
 * The following fields are set to the value of the \e master->config.? field if the
 * \e config->? entry has only the \e default value: 
 * - isSilent, isString, isServer, startAs, debug, timeout, buffersize, ignoreThread, ignoreSpawn, ignoreFork,
 * .
 * The following fields are set to the default value:
 * - isServer, FTRdata, EOFdata
 * .
 *  \config
 *  \param[in] master the #MqS object of the master
 *  \param[in] master_id the #MqS object identifer used to reference this slave object in the master namespace
 */
MQ_EXTERN void 
MQ_DECL MqConfigInitMaster (
  struct MqConfigS * const config,
  struct MqS * const master,
  int master_id
);

/** \ingroup config_api
 *  \brief create a new #MqConfigS object and initialize it with the values from \e templ.
 *  \param[in] templ the template-config-object
 *  \retval the new config object initialized with the data from the \e config object
 */
MQ_EXTERN struct MqConfigS * 
MQ_DECL MqConfigDup (
  struct MqConfigS const * const templ
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief does the \e msgque object is a \e server ?
 *  \msgque
 *  \return the <TT>(msgque->config.isServer == MQ_YES)</TT> value
 */
MQ_EXTERN MQ_INLINE int MQ_DECL MqConfigGetIsServer (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief does the \e msgque object is a \e parent ?
 *  \msgque
 *  \return the <TT>(msgque->config.parent == NULL)</TT> value
 */
MQ_EXTERN MQ_INLINE int MQ_DECL MqConfigGetIsParent (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief does the \e msgque object is a \e slave ?
 *  \msgque
 *  \return the <TT>(msgque->config.master != NULL)</TT> value
 */
MQ_EXTERN MQ_INLINE int MQ_DECL MqConfigGetIsSlave (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief does the \e msgque object is a \e string ?
 *  \msgque
 *  \return the <TT>(msgque->config.isString == MQ_YES)</TT> value
 */
MQ_EXTERN MQ_INLINE int MQ_DECL MqConfigGetIsString (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief does the \e msgque object is \e silent ?
 *  \msgque
 *  \return the <TT>(msgque->config.isSilent == MQ_YES)</TT> value
 */
MQ_EXTERN MQ_INLINE int MQ_DECL MqConfigGetIsSilent (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief does the \e msgque object is \e Duplicate and \e Thread ?
 *  \config
 *  \return the <TT>((config->statusIs & MQ_STATUS_IS_DUP) && (config->statusIs & MQ_STATUS_IS_THREAD))</TT> value
 */
MQ_EXTERN MQ_INLINE int MQ_DECL MqConfigGetIsDupAndThread (
  struct MqConfigS const * const config
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief get the \e name of the \e msgque object
 *  \msgque
 *  \return the \c msgque->config.name value
 *  \attention the \e string is owned by \libmsgque -> do not free !!
 */
MQ_EXTERN MQ_INLINE const char * MQ_DECL MqConfigGetName (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief get the \e debug value of the \e msgque object
 *  \msgque
 *  \return the \c msgque->config.debug value
 */
MQ_EXTERN MQ_INLINE int MQ_DECL MqConfigGetDebug (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief get the \e parent value of the \e config object
 *  \config
 *  \return the #MqConfigS::parent value
 */
MQ_EXTERN MQ_INLINE struct MqS const * MQ_DECL MqConfigGetParent (
  struct MqConfigS const * const config
) __attribute__((nonnull));

/// \sameas{MqConfigGetParent}
MQ_EXTERN MQ_INLINE struct MqS const * MQ_DECL MqConfigGetParentM (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief get the \e master value of the \e config object
 *  \config
 *  \return the #MqConfigS::master value
 */
MQ_EXTERN MQ_INLINE struct MqS const * MQ_DECL MqConfigGetMaster (
  struct MqConfigS const * const config
) __attribute__((nonnull));

/// \sameas{MqConfigGetMaster}
MQ_EXTERN MQ_INLINE struct MqS const * MQ_DECL MqConfigGetMasterM (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \ingroup config_api
 *  \brief get the \e myCtx value of the \e config object
 *  \config
 *  \return the \c config->myCtx value
 */
MQ_EXTERN MQ_INLINE struct MqContextS const * MQ_DECL MqConfigGetMyCtx (
  struct MqConfigS const * const config
) __attribute__((nonnull));

/// \sameas{MqConfigGetMyCtx}
MQ_EXTERN MQ_INLINE struct MqS const * MQ_DECL MqConfigGetMyCtxM (
  struct MqS const * const msgque
) __attribute__((nonnull));

/** \brief setup and return the \e init object
 *  \return a pointer to the initialization buffer
 *  \attention 
 *  - every usage of this function will free the data of the previous \e init object
 *  - the data have to fit to the underlying programming language
 *  .
 *
 *  The data is used as prefix to setup the argument vector for \e spawn.
 *  The following example demonstrate the usage of #MqInitCreate:
\code
struct MqBufferLS * initB = MqInitCreate();
MqBufferLAppendC(initB, "myExec");
MqBufferLAppendC(initB, "myExecArgument_1");
...
\endcode
 */
MQ_EXTERN struct MqBufferLS* MQ_DECL MqInitCreate (void);

/** \brief create a \libmsgque object
 *  \param[in,out] config configuration object
 *  \argv
 *  \alfa
 *  \param[out] msgqueP a pointer to the new created msgque object
 *  \retMqErrorE
 *  \attention 
 *   - The first argument \ref config_api "config" is copied to the #MqS::config element and it's
 *     memory is freed by \libmsgque. Access to the configuration data is available 
 *     by (*msgqueP)->config.
 *   - The \e config->myCtx and (*msgqueP)->config.myCtx point to the same application 
 *     specific #MqContextS object.
 *   - The #MqContextS object is an object template, this mean that the first data-entry
 *     have to be an #MqS object called \e msgque but the second and more data-entries 
 *     are application specific and they are \b not used by libmsgque.
 *   - The following cast is OK:
 *     \code
 *     struct MyContextS * context = (struct MyContextS*) msgque->config.myCtx
 *     \endcode
 *   - #MqContextS::msgque is filled with the new created msgque object
 *   - The \e MqBufferLS argument \e argv is owned by the caller and will be changed
 *     by \libmsgque. All options and option-arguments belonging to \libmsgque 
 *     will be deleted. As a result only the arguments belonging to the calling 
 *     application will be left over. The options and the option-arguments of the \e argv 
 *     array can be read with the \c MqBufferLCheckOption... style of functions. As last
 *     option.handling function:
 *     \code
 *     MqErrorCheck (MqCheckForLeftOverArguments(msgque, &argv));
 *     \endcode
 *     will create an error-message for every option not processed.
 *     The memory of \e &argv is freed and the pointer is set to \c NULL.
 *   - the both arguments \e argv and \e alfa are usually created with:
 *     \code
 *     MqBufferLCreateArgv (MQ_ERROR_PANIC, argc, argv, &largv, &lalfa);
 *     \endcode
 *     during \c main processing. 
 *   - The memory of \e argv is controlled (freed) by the caller.
 *   - The memory of \e alfa is controlled (freed) by the libmsgque.
 *   - If \e MqCreate return an error (#MQ_ERROR) the error object
 *     \c (*msgqueP)->error (#MqS::error) is available.
 *   .
 */
MQ_EXTERN enum MqErrorE MQ_DECL MqCreate (
  struct MqConfigS  * config,
  struct MqBufferLS * const argv,
  struct MqBufferLS * alfa,
  struct MqS       ** msgqueP
) __attribute__((nonnull(1,4)));

/** \brief delete a \libmsgque object
 *  \param msgqueP a pointer to an struct MqS * object
 *  \attDelete
 *
 *  The following steps are performed:
 *   -# if object is a PARENT and #MqConfigS::fParentDelete exist call it and return.
 *   -# if object is a CHILD and #MqConfigS::fChildDelete exist call it and return.
 *   -# if the both functions from above does not exist \b or if the #MqDelete is
 *      called the second time on the same #MqS object, delete the object.
 *   -# If \e error object of the \e msgque parameter has an #MQ_ERROR
 *     state and \e msgque is a parent than the following action is done:
 *     - in \e server mode: send the error to the client (if possible)
 *     - in \e client mode: print the error
 *     .
 *   .
 */

MQ_EXTERN void MQ_DECL MqDelete (
  struct MqS ** const msgqueP
);

/** \brief exit the current process or thread
 *  \msgque
 *  \attention this function will never return
 *
 *  -# Delete the msgque object using: #MqDelete
 *  -# If available call the context specific exit handler \c --fParentExit
 *  -# final call \c SysExit to exit the process
 *  -# It is an Panic error if #MqExit is called for a CHILD msgque object
 *  -# It is an Panic error if #MqExit is called twice for the same object
 *  .
 */

MQ_EXTERN void MQ_DECL MqExit (
  struct MqS * msgque
) __attribute__ ((noreturn));

/** \brief write \libmsgque specific user-help to stderr
 *  \param tool the name of the tool (e.g. argv[0]) or NULL.
 *  \return the help messages as string
 * 
 *  If \e tool != NULL, the function will display a header like:
\code
  tool [ARGUMENT]... syntax:
    aclient [OPTION]... @ tool [OPTION]... @...
\endcode
 *  on the help page.
 */
MQ_EXTERN MQ_STR MQ_DECL MqHelp ( 
  const char * tool 
);

/** check for left over arguments from parameter \e argv of #MqCreate
 *  \msgque
 *  \param[in,out] argvP a pointer to the argv, the memory will be freed and the value will be set to \c NULL
 *  \retMqErrorE
 * 
 *  This code is not part of #MqCreate because sometimes it is necessary 
 *  to check for left over arguments \b after #MqCreate was called and
 *  after the calling function was able to process function specific arguments. 
 *  By Default #MqCreate will delete \b all arguments belonging to \libmsgque
 *  from the \e argv object. If the calling function does \b not expect
 *  additional arguments or arguments were added with spelling errors,
 *  a call to #MqCheckForLeftOverArguments will detect this.
 */
MQ_EXTERN enum MqErrorE MQ_DECL MqCheckForLeftOverArguments (
  struct MqS const * const msgque,		
  struct MqBufferLS ** argvP
);

/*****************************************************************************/
/*                                                                           */
/*                               logging                                     */
/*                                                                           */
/*****************************************************************************/

#ifdef _DEBUG
/// \brief convenience function to log \e MqS configuration data
/// \msgque
/// \param prefix used to identify the data logged to the stderr
MQ_EXTERN void MQ_DECL MqLogData (
  struct MqS const * const msgque,
  MQ_STRB const * const prefix
);

/// \brief convenience function to log \e MqS child dependencies
/// \msgque
/// \param prefix used to identify the data logged to the stderr
MQ_EXTERN void MQ_DECL MqLogChild (
  struct MqS const * const msgque,
  char const * const prefix
);
#endif // _DEBUG

/*****************************************************************************/
/*                                                                           */
/*                                misc                                       */
/*                                                                           */
/*****************************************************************************/

/// \brief create a service handle
/// \msgque
/// \token
/// \param proc an external (non libmsgque) function to provide the service
/// \param data an external (non libmsgque) pointer to a data structure linked to the #MqTokenF function
/// \param datafreeF function to free \e data pointer
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqServiceCreate (
  struct MqS const * const msgque, 
  MQ_STRB const * const token,
  MqTokenF const proc,
  MQ_PTR data,
  MqTokenDataFreeF datafreeF
);

/// \brief delete service handle
/// \msgque
/// \token
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqServiceDelete (
  struct MqS const * const msgque, 
  MQ_STRB const * const token
);

/// \brief return the context-identification
/// \msgque
/// \return the context-identification number
///
/// The context-identification is a number and is used in the protocol to
/// link a data-package to a context-pointer. This is necessary because
/// the communication interface (\c msgque->io) of a parent-context is
/// used by the child-context too.
MQ_EXTERN MQ_SIZE MQ_DECL MqGetCtxId (
  struct MqS const * const msgque
);

/// \brief return the current transaction token
/// \msgque
/// \return the current transaction token as 4 character string
MQ_EXTERN const char *
MQ_DECL MqGetToken (
  struct MqS const * const msgque
);

/// \brief access to #MqS::send
/// \msgque
/// \return the #MqS::send field or \c NULL if not available
MQ_EXTERN struct MqSendS const * MQ_DECL MqGetSendS (
  struct MqS const * const msgque
);

/// \brief access to #MqS::config
/// \msgque
/// \return the #MqS::send field or \c NULL if not available
MQ_EXTERN struct MqConfigS const * MQ_DECL MqGetConfigS (
  struct MqS const * const msgque
);

/// \brief access to #MqS::error
/// \msgque
/// \return the #MqS::error field or \c NULL if not available
MQ_EXTERN struct MqErrorS const * MQ_DECL MqGetErrorS (
  struct MqS const * const msgque
);

/** \} */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                     B U F F E R - A P I                         ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \defgroup buffer_api PUBLIC: struct MqBufferS
 *  \ingroup msgque_api
 *  \{
 *  \brief the <EM>Msgque Buffer API</EM> is used to manage dynamic, generic, mixed typed data.
 *
 *  The MqBufferS struct is used to store and manage #MqTypeE typed data in a
 *  MqBufferU storage. If \libmsgque is working on any kind of data it is working on
 *  MqBufferS objects or on a list of MqBufferS objects called MqBufferLS.
 */

/*****************************************************************************/
/*                                                                           */
/*                              buffer_init                                  */
/*                                                                           */
/*****************************************************************************/

/// \brief union used to set or modify native data from an MqBufferS object
union MqBufferAtomU {
  MQ_BYT    Y;			///< 1 byte byte data
  MQ_BOL    O;			///< 1 byte boolean data
  MQ_SRT    S;			///< 2 byte short data
  MQ_INT    I;			///< 4 byte integer data
  MQ_FLT    F;			///< 4 byte float data
  MQ_WID    W;			///< 8 byte integer data
  MQ_DBL    D;			///< 8 byte double data
  MQ_BINB   B[8];		///< 8 byte ARRAY used for copy
};

/** \brief a union for all data items supported by \e MqBufferS.
 * 
 * A \libmsgque data package is a collection of items of type \e MqBufferU
 * with variable length data of type: #MQ_BIN, #MQ_STR and MQ_LST or a
 * a fixed length data of type #MQ_ATO.
 * The \ref MqBufferS object is using a \ref MqBufferU union 
 * and a \ref MqTypeE enum to define an item.
 **/
union MqBufferU {
  MQ_ATO *A;			///< atom for native data-types
  MQ_BIN  B;			///< arbitrary binary data
  MQ_STR  C;			///< C string like data (e.g. with a \\0 at the end)
  MQ_LST  L;			///< list object type data
  MQ_LST  R;			///< return object type data
};

/// \brief initial size of the #MqBufferS:bls object
#define MQ_BLS_SIZE 50

/** \brief defines an object to store generic (e.g mixed typed) data.
 *
 * an \e MqBufferS object is used for 3 different purposes:
 * - store generic typed single data value
 * - build a Msgque packet by appending items to an existing object
 * - read a single item from an existing Msgque packet 
 * .
 * 'reading from' and 'appending to' an existing Msgque packet is done by incrementing
 * the \a cur.B pointer to a new position.
 * a \ref MqBufferS object is using a \ref MqBufferU union 
 * and a \ref MqTypeE enum to define an item.
 * The object is public to give the developer the opportunity to direct access
 * saved data.
 **/

struct MqBufferS {
  MQ_BIN data;                  ///< always point to the beginning of the data-segment
  MQ_SIZE size;                 ///< the size of the data-segment
  MQ_SIZE cursize;              ///< currently used size (always: \a cursize < \a size)
  MQ_SIZE numItems;             ///< the number of items in the data-segment (only if it is a package)
  union MqBufferU cur;		///< point to current position, used for 'appending to' or 'reading from' 
				///< a package (always: data <= cur.B <= data+cursize)
  enum MqAllocE alloc;		///< allocation style, MQ_DYNAMIC or MQ_STATIC
  enum MqTypeE type;            ///< type of the item stored into the data-segment

  /// \brief B)uffer L)ocal S)torage used in #MqBufferGetC to get a string from a MQ_ATO data
  ///
  /// +1 to allow strings with MQ_BLS_SIZE=strlen(str) fit into the buffer with an additional
  /// '\\0' at the end
  MQ_BINB bls[MQ_BLS_SIZE+1];
};

/// \brief extract a MQ_BYT from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_BYT MQ_DECL MqBufU2BYT ( union MqBufferU bufU );
/// \brief extract a MQ_BOL from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_BOL MQ_DECL MqBufU2BOL ( union MqBufferU bufU );
/// \brief extract a MQ_SRT from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_SRT MQ_DECL MqBufU2SRT ( union MqBufferU bufU );
/// \brief extract a MQ_INT from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_INT MQ_DECL MqBufU2INT ( union MqBufferU bufU );
/// \brief extract a MQ_FLT from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_FLT MQ_DECL MqBufU2FLT ( union MqBufferU bufU );
/// \brief extract a MQ_WID from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_WID MQ_DECL MqBufU2WID ( union MqBufferU bufU );
/// \brief extract a MQ_DBL from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_DBL MQ_DECL MqBufU2DBL ( union MqBufferU bufU );
/// \brief extract a MQ_ATO from a \e MqBufferS object
/// \bufU
MQ_EXTERN MQ_INLINE MQ_ATO MQ_DECL MqBufU2ATO ( union MqBufferU bufU );

/// \brief create a new \e MqBufferS with \a size
/// \param size initial size of the buffer, if \e size = 0 the initial size is set to an default value
MQ_EXTERN struct MqBufferS * MQ_DECL MqBufferCreate (
  MQ_SIZE size
);

/// \brief delete a new \e MqBufferS 
/// \param bufP a pointer to an struct MqBufferS * object
/// \attention \attDelete
MQ_EXTERN void MQ_DECL MqBufferDelete (
  struct MqBufferS ** const bufP
);

/// \brief reset a \e MqBufferS to the length zero
/// \buf
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferReset (
  struct MqBufferS * const buf
);

/// \brief copy the \e MqBufferS from \a srce to \a dest
/// \param dest target of the copy
/// \param srce source of the copy
/// \retval the \e dest object
MQ_EXTERN struct MqBufferS * MQ_DECL MqBufferCopy (
  register struct MqBufferS * const dest,
  register struct MqBufferS const * const srce
);

/// \brief create an new object as duplication of an existing object
/// \param srce source of the duplication
/// \retval the new object
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferDup (
  struct MqBufferS const * const srce
);

/*****************************************************************************/
/*                                                                           */
/*                           buffer_create_type                              */
/*                                                                           */
/*****************************************************************************/

/// \brief create and initialize a new #MqBufferS object using native typed value
/// \param[in] val the value to initialize the buffer to
/// \retMqBufferS
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateY (
  MQ_BYT const val
);

/// \copydoc MqBufferCreateY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateO (
  MQ_BOL const val
);

/// \copydoc MqBufferCreateY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateS (
  MQ_SRT const val
);

/// \copydoc MqBufferCreateY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateI (
  MQ_INT const val
);

/// \copydoc MqBufferCreateY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateF (
  MQ_FLT const val
);

/// \copydoc MqBufferCreateY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateW (
  MQ_WID const val
);

/// \copydoc MqBufferCreateY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateD (
  MQ_DBL const val
);

/// \brief create and initialize an new #MqBufferS object using an #MQ_STR object
/// \param val the MQ_STR object
/// \retMqBufferS
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateC (
  MQ_STRB const * const val
);

/// \brief create and initialize an new #MqBufferS object using an #MQ_BIN object
/// \param val the MQ_BIN object
/// \param len the size of the MQ_BIN object
/// \retMqBufferS
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateB (
  MQ_BINB const * const val,
  MQ_SIZE const len
);

/// \brief create and initialize an new #MqBufferS object using an #MQ_BUF object
/// \param val the MQ_BUF object
/// \retMqBufferS
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferCreateU (
  struct MqBufferS const * const val
);

/*****************************************************************************/
/*                                                                           */
/*                              buffer_get                                   */
/*                                                                           */
/*****************************************************************************/

/// \brief read native typed value from an #MQ_BUF object
/// \error
/// \buf
/// \param[out] valP a pointer of the value to read
/// \retMqErrorE
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetY (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_BYT * const valP
);

/// \copydoc MqBufferGetY
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetO (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_BOL * const valP
);

/// \copydoc MqBufferGetY
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetS (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_SRT * const valP
);

/// \copydoc MqBufferGetY
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetI (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_INT * const valP
);

/// \copydoc MqBufferGetY
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetF (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_FLT * const valP
);

/// \copydoc MqBufferGetY
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetW (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_WID * const valP
);

/// \copydoc MqBufferGetY
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetD (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_DBL * const valP
);

/// \brief function to read an #MQ_BIN from an #MQ_BUF object
/// \error
/// \buf
/// \retval out the pointer to an MQ_BIN array object
/// \retval size the size of the array the array pointer
/// \retMqErrorE
/// \attention the return-pointer (\e out) is owned by the #MQ_BUF object -> never free this pointer
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetB (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_BIN * const out,
  MQ_SIZE * const size
);

/// \brief function to read an #MQ_STR from an #MQ_BUF object
/// \error
/// \buf
/// \retval out the pointer to an MQ_STR object
/// \retMqErrorE
/// \attention the return-pointer (\e out) is owned by the #MQ_BUF object -> never free this pointer
MQ_EXTERN enum MqErrorE
MQ_DECL MqBufferGetC (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  MQ_STR * const out
);

/// \brief return the type from a #MQ_BUF object
/// \buf
/// \return the type from a #MqTypeE object as single character value
MQ_EXTERN char MQ_DECL MqBufferGetType (
  struct MqBufferS * const buf
);

/*****************************************************************************/
/*                                                                           */
/*                              buffer_getM                                  */
/*                                                                           */
/*****************************************************************************/

/// \sameas{MqBufferGetY}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetYM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_BYT * const valP
);

/// \sameas{MqBufferGetO}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetOM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_BOL * const valP
);

/// \sameas{MqBufferGetS}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetSM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_SRT * const valP
);

/// \sameas{MqBufferGetI}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetIM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_INT * const valP
);

/// \sameas{MqBufferGetF}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetFM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_FLT * const valP
);

/// \sameas{MqBufferGetW}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetWM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_WID * const valP
);

/// \sameas{MqBufferGetD}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetDM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_DBL * const valP
);

/// \sameas{MqBufferGetB}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetBM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_BIN * const out,
  MQ_SIZE * const size
);

/// \sameas{MqBufferGetC}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqBufferGetCM (
  struct MqS const * const msgque,
  struct MqBufferS * const buf,
  MQ_STR * const out
);

/*****************************************************************************/
/*                                                                           */
/*                              buffer_set                                   */
/*                                                                           */
/*****************************************************************************/

/// \brief set the #MqBufferS object with a native typed value
/// \buf
/// \param[in] val the value to set the buffer to
/// \retMqBufferS
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferSetY (
  struct MqBufferS * const buf,
  MQ_BYT const val
);

/// \copydoc MqBufferSetY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferSetO (
  struct MqBufferS * const buf,
  MQ_BOL const val
);

/// \copydoc MqBufferSetY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferSetS (
  struct MqBufferS * const buf,
  MQ_SRT const val
);

/// \copydoc MqBufferSetY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferSetI (
  struct MqBufferS * const buf,
  MQ_INT const val
);

/// \copydoc MqBufferSetY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferSetF (
  struct MqBufferS * const buf,
  MQ_FLT const val
);

/// \copydoc MqBufferSetY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferSetW (
  struct MqBufferS * const buf,
  MQ_WID const val
);

/// \copydoc MqBufferSetY
MQ_EXTERN MQ_INLINE struct MqBufferS * MQ_DECL MqBufferSetD (
  struct MqBufferS * const buf,
  MQ_DBL const val
);

/// \brief set the #MqBufferS object with an #MQ_STR object
/// \buf
/// \param in the string to set the buffer
/// \retMqBufferS
MQ_EXTERN struct MqBufferS * MQ_DECL MqBufferSetC (
  struct MqBufferS * const buf,
  MQ_STRB const * const in
);

/// \brief set the #MqBufferS object with an #MQ_BIN object
/// \buf
/// \param in the binary to set the buffer
/// \param len the length of the binary data
/// \retMqBufferS
MQ_EXTERN struct MqBufferS * MQ_DECL MqBufferSetB (
  struct MqBufferS * const buf,
  MQ_BINB const * const in,
  MQ_SIZE const len
);

/// \brief set the #MqBufferS object with an #MQ_BUF object
/// \buf
/// \param in the MQ_BUF to set the buffer
/// \retMqBufferS
MQ_EXTERN struct MqBufferS * MQ_DECL MqBufferSetU (
  struct MqBufferS * const buf,
  struct MqBufferS const * const in
);

/// \brief set the MqBufferS to a vararg \e string with \e format
/// \buf
/// \format
/// \retMqBufferS
MQ_EXTERN struct MqBufferS * MQ_DECL MqBufferSetV (
  struct MqBufferS * const buf,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 2, 3)));

/// \brief change the type of an MqBufferS object to \e type
/// \error
/// \buf
/// \type
/// \retMqErrorE
MQ_EXTERN enum MqErrorE
MQ_DECL MqBufferCastTo (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  enum MqTypeE const type
);

/*****************************************************************************/
/*                                                                           */
/*                              buffer_append                                */
/*                                                                           */
/*****************************************************************************/

/// \brief append a single \a string to a \e MqBufferS
/// \buf
/// \param string the text to append to \e buf
/// \retval the size of the string appended to the MqBufferS object
MQ_EXTERN MQ_SIZE MQ_DECL MqBufferAppendC (
  struct MqBufferS * const buf,
  MQ_STRB const * const string
);

/// \brief append a single string with \a format and \a var_list arguments to a MqBufferS
/// \buf
/// \format
/// \var_list
/// \retval the size of the string appended to the MqBufferS object
MQ_EXTERN MQ_SIZE MQ_DECL MqBufferAppendVL (
  struct MqBufferS * const buf,
  MQ_STRB const * const fmt,
  va_list var_list
);

/// \brief append a single string with \a format and \a ... arguments to a MqBufferS
/// \buf
/// \format
/// \retval the size of the string appended to the MqBufferS object
MQ_EXTERN MQ_SIZE MQ_DECL MqBufferAppendV (
  struct MqBufferS * const buf,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 2, 3)));

/*****************************************************************************/
/*                                                                           */
/*                              buffer_push                                  */
/*                                                                           */
/*****************************************************************************/

/// \brief add \a str to the \e MqBufferS
/// \buf
/// \param string the text to append to \e buf
/// \retval the size of the string appended to the MqBufferS object
MQ_EXTERN MQ_SIZE MQ_DECL MqBufferPush (
  register struct MqBufferS * const buf,
  MQ_STRB const * const string
);

/// \brief delete \a str from the \e MqBufferS
/// \buf
/// \param string the text to remove from \e buf
/// \retval the size of the string removed from the MqBufferS object
MQ_EXTERN MQ_SIZE MQ_DECL MqBufferPop (
  register struct MqBufferS * const buf,
  MQ_STRB const * const string
);

/*****************************************************************************/
/*                                                                           */
/*                              buffer_log                                   */
/*                                                                           */
/*****************************************************************************/

/// \brief log the whole \e MqBufferS object to the stderr device
/// \msgque
/// \buf
/// \param prefix item to identify the output
MQ_EXTERN void MQ_DECL MqBufferLog (
  struct MqS const * const msgque,
  struct MqBufferS const * const buf,
  MQ_STRB const * const prefix
);

/// \brief log the short \e MqBufferS object data to the stderr device
/// \msgque
/// \buf
/// \param prefix item to identify the output
MQ_EXTERN void MQ_DECL MqBufferLogS (
  struct MqS const * const msgque,
  struct MqBufferS const * const buf,
  MQ_STRB const * const prefix
);

/** \} */

/*****************************************************************************/
/*                                                                           */
/*                    transactions (require MqBufferAtomU)                   */
/*                                                                           */
/*****************************************************************************/

/**
 *  \ingroup msgque_api
 *  \{
 */

/// \brief check if the ongoing service-call belongs to an transaction
/// \param[in] msgque the current context
MQ_EXTERN MQ_INLINE MQ_BOL
MQ_DECL MqIsTransaction (
  struct MqS const * const msgque
);


/// \brief check the current token
/// \param[in] msgque the current context
/// \param[in] token the token to compare with
MQ_EXTERN MQ_INLINE MQ_BOL
MQ_DECL MqCurrentTokenIs (
  struct MqS const * const msgque,
  char const * const token
);

/** \} */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                 B U F F E R - L I S T - A P I                   ### */
/* ###                                                                 ### */
/* ####################################################################### */


/** \defgroup buffer_list_api PUBLIC: struct MqBufferLS
 *  \ingroup msgque_api
 *  \{
 *  \brief the <EM>Msgque Buffer List API</EM> is used to manage a list of \e MqBufferS objects.
 *
 *  the memory is allocated and extended dynamically. 
 */

/*****************************************************************************/
/*                                                                           */
/*                          buffer_list_init                                 */
/*                                                                           */
/*****************************************************************************/

/// \brief the object to manage an array of \e MqBufferS items.
///
/// every \e MqBufferLS is able to store a infinite number of \e MqBufferS objects
/// using dynamic allocation. the \e MqBufferS objects are stored in a flat
/// array of \e MqBufferS object pointers. to access an object only the \a data 
/// or the \a cur member is used (e.g. \b \c data[index]). in difference to the
/// \a data member the \a cur member has a floating position to implement an
/// increment/decrement like behaviour
struct MqBufferLS {
  MQ_SIZE size;                 ///< the total allocated size
  MQ_SIZE cursize;              ///< the currently used size (e.g. 0 <= cursize <= size) 
  struct MqBufferS ** data;	///< pointer to the allocated memory (fixed position)
  struct MqBufferS ** cur;	///< pointer to the current position (e.g. data <= cur <= data+cursize, floating position)
} ;

/// \brief create and initialize a MqBufferLS object
/// \param size create the object with a predefined list size
/// \return the new created and initialized MqBufferLS object
MQ_EXTERN struct MqBufferLS * MQ_DECL MqBufferLCreate (
  MQ_SIZE const size
);

/**
 *  \brief create and initialize a MqBufferLS object with argc/argv data
 *  \param argc the argc from the initial \b main function
 *  \param argv the arguments from the initial \b main function
 *  \retval args the new created MqBufferLS object including all arguments from argv \b before the first #MQ_ALFA
 *  \retval alfa the new created MqBufferLS object including all arguments from argv \b after the first #MQ_ALFA
 *
 *  This function is usually called by the client (the initiator) of a msgque request. The \e alfa arguments
 *  will be used to start the server during the creation of the msgque object in #MqCreate.
 *  \code
int
main ( int argc, char **argv)
{
  // the command-line-arguments before (largv) and after (lalfa) the first MQ_ALFA
  struct MqBufferLS * largv, lalfa;
  MqBufferLCreateArgv(NULL, argc, argv, &largv, &lalfa);
....
 * \endcode
 */
MQ_EXTERN void MQ_DECL MqBufferLCreateArgv (
  int const argc,
  char ** const argv,
  struct MqBufferLS ** const args,
  struct MqBufferLS ** const alfa
);

/// \brief create an #MqBufferLS object as copy from an existing \e MqBufferLS object
/// \param in the \e MqBufferLS object to copy from
/// \return out the \e MqBufferLS object to create
MQ_EXTERN struct MqBufferLS * MQ_DECL MqBufferLDup (
  struct MqBufferLS const * const in
);

/// \brief delete a dynamically created \e MqBufferLS object
/// \param bufP the pointer to an struct MqBufferLS * object
/// \attDelete
MQ_EXTERN void MQ_DECL MqBufferLDelete (
  struct MqBufferLS ** bufP
);

/*****************************************************************************/
/*                                                                           */
/*                          buffer_list_append                               */
/*                                                                           */
/*****************************************************************************/

/// \brief append a MqBufferS item into an MqBufferLS object on \e position
/// \bufL
/// \param[in] in the MqBufferS object to append
/// \param[in] position insert \e in at \e position, shift all following arguments one up
/// \attention Set \e position to \b 0 to append to the beginning or set position to \b -1 to append to the end
MQ_EXTERN void MQ_DECL MqBufferLAppend (
  register struct MqBufferLS * const bufL,
  struct MqBufferS * const in,
  MQ_SIZE const position
);

/// \brief coppy a MqBufferLS list into an MqBufferLS object on \e position
/// \bufL
/// \param[in] in the MqBufferLS object to append
/// \param[in] position insert \e in at \e position, shift all following arguments one up
/// \attention Set \e position to \b 0 to append to the beginning or set position to \b -1 to append to the end
MQ_EXTERN void MQ_DECL MqBufferLAppendL (
  register struct MqBufferLS * const bufL,
  struct MqBufferLS * const in,
  MQ_SIZE position
);

/// \brief append a native typed value to the \b end of a MqBufferLS object
/// \bufL
/// \param[in] val the value to append to
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendY (
  struct MqBufferLS * const bufL,
  MQ_BYT const val
);

/// \copydoc MqBufferLAppendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendO (
  struct MqBufferLS * const bufL,
  MQ_BOL const val
);

/// \copydoc MqBufferLAppendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendS (
  struct MqBufferLS * const bufL,
  MQ_SRT const val
);

/// \copydoc MqBufferLAppendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendI (
  struct MqBufferLS * const bufL,
  MQ_INT const val
);

/// \copydoc MqBufferLAppendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendF (
  struct MqBufferLS * const bufL,
  MQ_FLT const val
);

/// \copydoc MqBufferLAppendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendW (
  struct MqBufferLS * const bufL,
  MQ_WID const val
);

/// \copydoc MqBufferLAppendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendD (
  struct MqBufferLS * const bufL,
  MQ_DBL const val
);

/// \brief append an #MQ_STR object to the \b end of an MqBufferLS object
/// \bufL
/// \param val the string for input
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendC (
  struct MqBufferLS * const bufL,
  MQ_STRB const * const val
);

/// \brief append an #MQ_BUF object to the \b end of an MqBufferLS object
/// \bufL
/// \param val the buffer for input
/// \attention after the insert the buffer is owed by the \e buf object -> do \b not free \e val
MQ_EXTERN MQ_INLINE void MQ_DECL MqBufferLAppendU (
  struct MqBufferLS * const bufL,
  MQ_BUF const val
);

/*****************************************************************************/
/*                                                                           */
/*                              bufferL_check                                */
/*                                                                           */
/*****************************************************************************/

/// \brief search for boolean \e option in MqBufferLS list and fill \e var with #MQ_BOL value 
/// for "found" = #MQ_YES and "not found" = #MQ_NO
/// \error
/// \bufL0
/// \optionL
/// \retval var if \e option string is found set \e var with \e def
/// \retMqErrorE
/// \attention if \e option is found the entry is deleted from the MqBufferLS object
MQ_EXTERN enum MqErrorE MQ_DECL MqBufferLCheckOptionO (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_BOL * const var
);

/// \brief search for an \e option with native typed argument in an #MqBufferLS object
/// \error
/// \bufL0
/// \optionL
/// \param[out] valP the native option value to search for
/// \retMqErrorE
/// \attL
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqBufferLCheckOptionY (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_BYT * const valP
);

/// \copydoc MqBufferLCheckOptionY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqBufferLCheckOptionS (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_SRT * const valP
);

/// \copydoc MqBufferLCheckOptionY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqBufferLCheckOptionI (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_INT * const valP
);

/// \copydoc MqBufferLCheckOptionI
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqBufferLCheckOptionF (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_FLT * const valP
);

/// \copydoc MqBufferLCheckOptionI
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqBufferLCheckOptionW (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_WID * const valP
);

/// \copydoc MqBufferLCheckOptionI
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqBufferLCheckOptionD (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_DBL * const valP
);

/// \brief search for an \e option with an #MQ_STR argument in an #MqBufferLS object
/// \error
/// \bufL0
/// \optionL
/// \retval var the MQ_STR argument for return
/// \retMqErrorE
/// \attL \n
/// The return string \e var has dynamic allocated data and have to be freed later
MQ_EXTERN enum MqErrorE MQ_DECL MqBufferLCheckOptionC (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_STR * const var
);

/// \brief search for an \e option with an #MQ_BUF argument in an #MqBufferLS object
/// \error
/// \bufL0
/// \optionL
/// \retval var the MQ_BUF argument for return
/// \retMqErrorE
/// \attL
MQ_EXTERN enum MqErrorE MQ_DECL MqBufferLCheckOptionU (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_STRB const * const opt,
  MQ_BUF * const var
);

/*****************************************************************************/
/*                                                                           */
/*                              bufferL_misc                                 */
/*                                                                           */
/*****************************************************************************/

/// \brief delete the \e index'th list item from the MqBufferLS object
/// \error
/// \bufL
/// \param index the index'th object from the MqBufferLS object starting with 0
/// \param doDelete if \e delete != 0 delete the \e MqBufferS object, associated with the index, too
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqBufferLDeleteItem (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  const MQ_SIZE index,
  const MQ_BOL doDelete
);

/// \brief search #MQ_STR item from a MqBufferLS object starting at \e startindex
/// \bufL
/// \param str the string to search for
/// \param len the length of \e str
/// \param startindex start searching in \e buf from index \e startindex
/// \return The index of the \e str found or -1 if not found. The return value can be used as startindex of following calls to \e MqBufferLSearchC
///
/// a typical usage for this code is parsing an MqBufferLS object for multiple occurrences of a string
/// \code
/// while ((startindex = MqBufferLSearchC (buf, str, startindex)) != -1) {
///   ...
/// }
/// \endcode
/// \attention The size of \e str have to be at least 4 bytes
MQ_EXTERN MQ_SIZE MQ_DECL MqBufferLSearchC (
  struct MqBufferLS const * const bufL,
  MQ_STRB const * const str,
  MQ_SIZE const len,
  MQ_SIZE const startindex
);

/// \brief output a MqBufferLS object for debugging purpose to stderr
/// \msgque
/// \bufL
/// \param func prefix to identify the debugging output
MQ_EXTERN void MQ_DECL MqBufferLLog (
  struct MqS const * const msgque,
  struct MqBufferLS const * const bufL,
  MQ_STRB const * const func
);

/// \brief \e short output a MqBufferLS object for debugging purpose to stderr
/// \msgque
/// \bufL
/// \param func prefix to identify the debugging output
MQ_EXTERN void MQ_DECL MqBufferLLogS (
  struct MqS const * const msgque,
  struct MqBufferLS const * const bufL,
  MQ_STRB const * const func
);

/// \brief get an #MqBufferS object from the buffer list \e buf
/// \error
/// \bufL
/// \param index get the \e index'th element of the \e buf list
/// \retval var the MqBufferS object to return
/// \attention the \e index'th element of the \e buf list \b will be deleted from the buffer list
/// \retMqErrorE
/// \attention if \e bufL is \b not \c NULL the object from \e bufL will be deleted first
MQ_EXTERN enum MqErrorE MQ_DECL MqBufferLGetU (
  struct MqErrorS * const error,
  struct MqBufferLS * const bufL,
  MQ_SIZE const index,
  struct MqBufferS ** const var
);

/** \} */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                     E R R O R - A P I                           ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \defgroup error_api PUBLIC: struct MqErrorS
 *  \ingroup msgque_api
 *  \{
 *  \brief the <EM>MqErrorS API</EM> is used to manage error messages.
 *
 *  the management is done with a \e MqErrorS object created by \c MgCreate.
 *  every \e MqS object has only one \e MqErrorS object and every \e MqErrorS object 
 *  has only one \e MqS object.
 *  the \e MqErrorS object is used to collect all data needed to handle an error.
 */

/// \brief panic on error
///
/// This item is used as special meaning for the \c struct \c MqErrorS 
/// argument of error-functions
#define MQ_ERROR_PANIC ((struct MqErrorS*)NULL)

/// \brief ignore error and do not generate any error-text (don't fill the error object)
///
/// This item is used as special meaning for the \c struct \c MqErrorS 
/// argument of error-functions
#define MQ_ERROR_IGNORE ((struct MqErrorS*)0x1)

/// \brief print error to stderr
///
/// This item is used as special meaning for the \c struct \c MqErrorS 
/// argument of error-functions
#define MQ_ERROR_PRINT ((struct MqErrorS*)0x2)

/// \brief do a \b panic with a vararg argument list
/// \error
/// \prefix
/// \param errnum the error number
/// \format
/// \var_list
/// \attention this function will never return
MQ_EXTERN void MQ_DECL MqPanicVL (
  struct MqErrorS * const error,
  MQ_STRB const * const prefix,
  MQ_INT const errnum,
  MQ_STRB const * const fmt,
  va_list var_list
);

/// \brief do a \b panic with vararg arguments
/// \error
/// \prefix
/// \param errnum the error number
/// \format
/// \attention this function will never return
MQ_EXTERN void MQ_DECL MqPanicV (
  struct MqErrorS * const error,
  MQ_STRB const * const prefix,
  MQ_INT const errnum,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 4, 5)));

/// \brief do a \b panic with \e string as argument
/// \error
/// \prefix
/// \param errnum the error number
/// \param string the string to be displayed
/// \attention this function will never return
#define MqPanicC(error,prefix,errnum,string) MqPanicV(error,prefix,errnum,"%s",string);

/// \brief do a \b panic with a \b system error message
/// \error
/// \attention this function will never return
#define MqPanicSYS(error) MqPanicV(error,__func__,-1,\
	"internal ERROR in function '%s', please contact your local support", __func__);

/// \brief reset a \e MqErrorS object, change error code to #MQ_OK 
/// \error
MQ_EXTERN MQ_INLINE void MQ_DECL MqErrorReset (
  struct MqErrorS * const error
);

/// \sameas{MqErrorReset}
MQ_EXTERN MQ_INLINE void MQ_DECL MqErrorResetM (
  struct MqS const * const msgque
);

/*****************************************************************************/
/*                                                                           */
/*                         create an error message                           */
/*                                                                           */
/*****************************************************************************/

/// \brief set an error-code in \e MqErrorS with vararg list argument
/// \error
/// \prefix
/// \param errorcode identify the code
/// \param errnum the error number
/// \format
/// \var_list
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqErrorSGenVL (
  struct MqErrorS * const error,
  MQ_STRB const * const prefix,
  enum MqErrorE const errorcode,
  MQ_INT const errnum,
  MQ_STRB const * const fmt,
  va_list var_list
);

/// \brief set a error-code in \e MqErrorS with vararg string argument
/// \error
/// \prefix
/// \param errorcode identify the code
/// \param errnum the error number
/// \format
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqErrorSGenV (
  struct MqErrorS * const error,
  MQ_STRB const * const prefix,
  enum MqErrorE const errorcode,
  MQ_INT const errnum,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 5, 6)));

/// \brief customized edition of #MqErrorSGenV
/// \error
/// \prefix
/// \param errnum the error number
/// \param[in] message string message to set
/// \retMqErrorE
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqErrorC (
  struct MqErrorS * const error,
  MQ_STRB const * const prefix,
  MQ_INT const errnum,
  MQ_STRB const * const message
);

/// \sameas{MqErrorC}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqErrorCM (
  struct MqS const * const msgque,
  MQ_STRB const * const prefix,
  MQ_INT const errnum,
  MQ_STRB const * const message
);

/// \brief customized edition of #MqErrorSGenV
#define MqErrorV(error,prefix,errnum,format,...) \
    MqErrorSGenV(error,prefix,MQ_ERROR,errnum,format, __VA_ARGS__)

/*****************************************************************************/
/*                                                                           */
/*                append text to an existing error message                   */
/*                                                                           */
/*****************************************************************************/

/// \brief append a vararg string to a \e MqErrorS
/// \error
/// \format
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqErrorSAppendV (
  struct MqErrorS const * const error,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 2, 3)));

/// \brief append a string to a \e MqErrorS
#define MqErrorSAppendC(error,str)        MqErrorSAppendV(error,"%s",str);

/// \brief append a function and filename to the error-buffer
/// \error
/// \param func the name of the function to display (e.g. __func__)
/// \param file the name of the file to display (e.g. __FILE__)
/// \retMqErrorE
MQ_EXTERN enum MqErrorE
MQ_DECL MqErrorStackP (
  struct MqErrorS const * const error,
  char const * const func,
  char const * const file
);

/// \sameas{MqErrorStackP}
MQ_EXTERN MQ_INLINE enum MqErrorE
MQ_DECL MqErrorStackPM (
  struct MqS const * const msgque,
  char const * const func,
  char const * const file
);

/// \brief convenience wrapper for #MqErrorStackP
# define MqErrorStack(error) \
    MqErrorStackP(error, __func__, __FILE__)

/*****************************************************************************/
/*                                                                           */
/*                              error_append                                 */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/*                              error_set/get                                */
/*                                                                           */
/*****************************************************************************/

/// \brief return the value of \e MqErrorS code member
/// \error
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqErrorGetCode (
  struct MqErrorS const * const error
);

/// \sameas{MqErrorGetCode}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqErrorGetCodeM (
  struct MqS const * const msgque
);

/// \brief set the value of \e MqErrorS code member
/// \error
/// \param code the \e code to set for \e error
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqErrorSetCode (
  struct MqErrorS * const error,
  enum MqErrorE code
);

/// \brief return the value of \e MqErrorS text member
/// \error
MQ_EXTERN MQ_INLINE MQ_STR MQ_DECL MqErrorGetText (
  struct MqErrorS const * const error
);

/// \sameas{MqErrorGetText}
MQ_EXTERN MQ_INLINE MQ_STR MQ_DECL MqErrorGetTextM (
  struct MqS const * const msgque
);

/// \brief return the value of \e MqErrorS num member
/// \error
MQ_EXTERN MQ_INLINE MQ_INT MQ_DECL MqErrorGetNum (
  struct MqErrorS const * const error
);

/// \brief get \e MqErrorS msgque member
/// \error
MQ_EXTERN MQ_INLINE struct MqS * MqErrorGetMsgque (
  struct MqErrorS const * const error
);

/*****************************************************************************/
/*                                                                           */
/*                              error_is/check                               */
/*                                                                           */
/*****************************************************************************/

/// \brief check on error and goto label \e error
#define MqErrorCheck(PROC) if (unlikely((PROC) == MQ_ERROR)) goto error
/// \brief process error message
#define MqErrorSwitch(PROC) switch (PROC) {\
case MQ_OK: break; case MQ_ERROR: goto error; case MQ_EXIT: return MQ_EXIT; case MQ_CONTINUE: return MQ_CONTINUE;\
}
/// \brief process error message
#define MqErrorReturn(PROC) switch (PROC) {\
case MQ_OK: return MQ_OK; case MQ_ERROR: return MqErrorStack (MQ_ERROR_S); case MQ_EXIT: return MQ_EXIT; default: return MQ_CONTINUE;\
}
/// \brief check on error and copy
/// \param[in] PROC a function return an #MqErrorE object
/// \param[in] SRCERR the copy-source of the error
/// \param[out] TGTERR the copy-target of the error
///
/// The following steps are performed:
/// -# check on return value #MQ_ERROR of \e PROC
/// -# if \e yes that copy \e SRCERR to \e TGTERR
/// -# finally jump to the label \e error for additional error handling
#define MqErrorCheckAndCopy(PROC,SRCERR,TGTERR) if (unlikely((PROC) == MQ_ERROR)) {\
  MqErrorCopy(TGTERR, SRCERR); \
  goto error; \
}
/// \brief check on error and goto label \e error1
#define MqErrorCheck1(PROC) if (unlikely((PROC) == MQ_ERROR)) goto error1
/// \brief check on error and goto label \a JUMP
#define MqErrorCheck2(PROC,JUMP) if (unlikely((PROC) == MQ_ERROR)) goto JUMP
/// \brief check return of \e proc to be #MQ_OK and goto label \e ok
#define MqOkCheck(PROC)  if (likely((PROC) == MQ_OK)) goto ok

#define MqErrorRet(PROC) if (unlikely((PROC) == MQ_ERROR)) return MQ_ERROR

/*****************************************************************************/
/*                                                                           */
/*                              error_misc                                   */
/*                                                                           */
/*****************************************************************************/

/// \brief log the \e MqErrorS for debugging
/// \error
/// \prefix
MQ_EXTERN void MQ_DECL MqErrorLog (
  struct MqErrorS const * const error,
  MQ_STRB const * const prefix
);

/// \brief copy a \e MqErrorS from \a in to \a out
/// \param out the #MqS::error object created by #MqCreate
/// \param in  the #MqS::error object created by #MqCreate
MQ_EXTERN enum MqErrorE MQ_DECL MqErrorCopy (
  struct MqErrorS * const out,
  struct MqErrorS * const in
);

/** \} error_api */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                        R E A D - A P I                          ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \ingroup msgque_api
 *  \defgroup read_api PUBLIC: struct MqReadS
 *  \{
 *  \brief Read and split a \libmsgque package into several items.
 *
 *  The reading is done by an \e MqReadS object using an \e MqIoS object for 
 *  doing the socket io. Every \e MqS object has only \b one \e MqReadS object 
 *  and every \e MqReadS object has only \b one \e MqS object. The interface is
 *  used to extract the different #MqTypeE data-types from the package doing
 *  the casting from one type to an other type if necessary.
 */

/*****************************************************************************/
/*                                                                           */
/*                                read_reference                             */
/*                                                                           */
/*****************************************************************************/

/// \brief creating a \e LST reference for reading the list items
/// \read
/// \param[in] buf the buffer object to read the LST object data from or \c NULL to
///   use new data from the \e MqReadS object. The object \e buf is the return value
///   #MqReadU
/// \retMqErrorE
///
/// Example:
/// \code
/// MQ_BUF buf = MqReadU(read);
/// ...
/// MqRead_LST_START(read,buf);
/// MQ_INT myInt = MqReadI(read);
/// ...
/// MqRead_LST_END(read);
/// \endcode
MQ_EXTERN enum MqErrorE MQ_DECL MqReadL_START (
  struct MqReadS * const read,
  MQ_BUF buf
) __attribute__((nonnull(1)));

/// \brief deleting a \e LST reference after reading the list items was finished
/// \param read a pointer to an MqReadS object
MQ_EXTERN void MQ_DECL MqReadL_END (
  struct MqReadS * const read
) __attribute__((nonnull));

/// \sameas{MqReadL_START}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadL_STARTM (
  struct MqS const * const msgque,
  MQ_BUF buf
) __attribute__((nonnull(1)));

/// \sameas{MqReadL_END}
MQ_EXTERN MQ_INLINE void MQ_DECL MqReadL_ENDM (
  struct MqS const * const msgque
) __attribute__((nonnull));

/*****************************************************************************/
/*                                                                           */
/*                              read functions                               */
/*                                                                           */
/*****************************************************************************/

/// \brief read a native typed value from the \e MqReadS buffer
/// \read
/// \param[out] valP a pointer of the value to read
/// \retMqErrorE
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadY (
  struct MqReadS * const read,
  MQ_BYT * const valP
);

/// \copydoc MqReadY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadO (
  struct MqReadS * const read,
  MQ_BOL * const valP
);

/// \copydoc MqReadY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadS (
  struct MqReadS * const read,
  MQ_SRT * const valP
);

/// \copydoc MqReadY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadI (
  struct MqReadS * const read,
  MQ_INT * const valP
);

/// \copydoc MqReadY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadF (
  struct MqReadS * const read,
  MQ_FLT * const valP
);

/// \copydoc MqReadY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadW (
  struct MqReadS * const read,
  MQ_WID * const valP
);

/// \copydoc MqReadY
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadD (
  struct MqReadS * const read,
  MQ_DBL * const valP
);

/// \brief generic function to read an #MQ_STR object from the \e MqReadS object
/// \read
/// \retval out the string to return
/// \retMqErrorE
/// \attention the string returned has \e static allocation and have to be
/// duplicated for long term storage.
MQ_EXTERN enum MqErrorE MQ_DECL MqReadC (
  struct MqReadS * const read,
  MQ_STR * const out
);

/// \brief generic function to read an #MQ_BIN object from the \e MqReadS object
/// \read
/// \retval out the binary data to return
/// \retval len the length of the binary data
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqReadB (
  struct MqReadS * const read,
  MQ_BIN * const out,
  MQ_SIZE * const len
);

/// \brief generic function to read a #MQ_BUF object from the \e MqReadS object
/// \read
/// \retval out the buffer to return
/// \retMqErrorE
/// \attention the object returned \e out is owned by \c MqReadS and is \b only valid
/// up to the next call to any \c MqRead? function. If a long-term object is required
/// use the #MqBufferDup function.
/// \code
/// MQ_BUF buf = MqBufferDup(MqReadU(read);
/// \endcode
MQ_EXTERN enum MqErrorE MQ_DECL MqReadU (
  struct MqReadS * const read,
  MQ_BUF * const out
);

/// \brief read an MqBufferLS object from all items of the \e MqReadS object
/// \read
/// \retval out the buffer list to return
/// \retMqErrorE
MQ_EXTERN enum MqErrorE MQ_DECL MqReadL (
  register struct MqReadS * const read,
  struct MqBufferLS ** const out
);

/*****************************************************************************/
/*                                                                           */
/*                              readM functions                              */
/*                                                                           */
/*****************************************************************************/

/// \sameas{MqReadY}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadYM (
  struct MqS const * const msgque,
  MQ_BYT * const valP
);

/// \sameas{MqReadO}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadOM (
  struct MqS const * const msgque,
  MQ_BOL * const valP
);

/// \sameas{MqReadS}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadSM (
  struct MqS const * const msgque,
  MQ_SRT * const valP
);

/// \sameas{MqReadI}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadIM (
  struct MqS const * const msgque,
  MQ_INT * const valP
);

/// \sameas{MqReadF}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadFM (
  struct MqS const * const msgque,
  MQ_FLT * const valP
);

/// \sameas{MqReadW}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadWM (
  struct MqS const * const msgque,
  MQ_WID * const valP
);

/// \sameas{MqReadD}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadDM (
  struct MqS const * const msgque,
  MQ_DBL * const valP
);

/// \sameas{MqReadC}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadCM (
  struct MqS const * const msgque,
  MQ_STR * const out
);

/// \sameas{MqReadB}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadBM (
  struct MqS const * const msgque,
  MQ_BIN * const out,
  MQ_SIZE * const len
);

/// \sameas{MqReadU}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadUM (
  struct MqS const * const msgque,
  MQ_BUF * const out
);

/*****************************************************************************/
/*                                                                           */
/*                                read_high                                  */
/*                                                                           */
/*****************************************************************************/

/// \brief undo the \b last MqReadX operation
/// \read
/// \retMqErrorE
/// \attention Can \b undo only the \b last MqReadX operation, multiple \b undo
/// will \b corrupt the memory.
MQ_EXTERN enum MqErrorE MQ_DECL MqReadUndo (
  struct MqReadS * const read
);

/// \sameas{MqReadUndo}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqReadUndoM (
  struct MqS const * const msgque
);

/// \brief get the number of items left in the \e MqReadS buffer
/// \read
/// \return the number of items
MQ_EXTERN MQ_INLINE MQ_SIZE MQ_DECL MqReadGetNumItems (
  struct MqReadS const * const read
);

/// \sameas{MqReadGetNumItems}
MQ_EXTERN MQ_INLINE MQ_SIZE MQ_DECL MqReadGetNumItemsM (
  struct MqS const * const msgque
);

/// \brief does an additional item exits in the \e MqReadS buffer
/// \read
/// \return the number of items
MQ_EXTERN MQ_INLINE MQ_BOL MQ_DECL MqReadItemExists (
  struct MqReadS const * const read
);

/// \sameas{MqReadItemExists}
MQ_EXTERN MQ_INLINE MQ_BOL MQ_DECL MqReadItemExistsM (
  struct MqS const * const msgque
);

/// \brief get the MqS object belonging to the current \e MqReadS buffer
/// \read
/// \return a pointer to a MqS object
MQ_EXTERN MQ_INLINE struct MqS * MqReadGetMsgque (
  struct MqReadS const * const read
);

/** \} read_api */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                        S E N D - A P I                          ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \ingroup msgque_api
 *  \defgroup send_api PUBLIC: struct MqSendS
 *  \{
 *  \brief Collect and send data using a LibMsgque package
 *
 *  the management is done by a \e MqSendS object using a \e MqIoS object for 
 *  doing the socket io. every \e MqS object has only \b one \e MqSendS object 
 *  and every \e MqSendS object has only \b one \e MqS object.
 *
 *  example 1: in the client, calling the \e service and wait for an answer
 *  \code
void MyServiceCall(struct MqS * const msgque) { // the libmsgque object
    struct MqSendS * const send = msgque->send; // get the MqSendS object
    ...
    MqSendSTART(send);                          // init the MqSendS buffer object
    MqSendI(send,int_value);                    // 1. argument: a MQ_INT value
    MqSendV(send,"num:%x",num);                 // 2. argument: a vararg string value
    MqSendB(send,mypicture,size);               // 3. argument: a binary picture of size length
    // call service "SRV1" and wait maximum 60 seconds for the results
    MqErrorCheck(MqSendEND_AND_WAIT(send,"SRV1",60);
    // ... get the results
    MqErrorCheck(MqRead?(msgque->read, ???);
    ...
}
 *  \endcode
 *
 *  example 2: in the server, answer the \e service call
 *  \code
static MqErrorE Ot_WAR1(MqSP const msgque, MQ_PTR data) {
    struct MqSendS * const send = msgque->send; // get the MqSendS object
    struct MqReadS * const read = msgque->read; // get the MqReadS object
  // ... do some int work
    MqSendSTART(send);                          // init the MqSendS buffer object
    MqErrorCheck(MqReadI(read,&myInt);         // read a MQ_INT value
    MqErrorCheck(MqReadC(read,&myStr);         // read a MQ_STR value
    MqErrorCheck(MqReadU(read,&myPic);         // read a MqBufferS object to store the picture data
  // ... do some processing
  error:                                        // something is wrong, error back
    return MqSendRETURN(send);			// send the package as an answer of a previous service-call
}
 *  \endcode
 **/

/*****************************************************************************/
/*                                                                           */
/*                              send functions                               */
/*                                                                           */
/*****************************************************************************/

/// \brief append a native typed value to the \e MqSendS buffer
/// \send
/// \param[in] val the value to send
///
/// \attention In \b binary mode the data will be send as binary, in \b string mode as string.
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendY (
  struct MqSendS const * const send,
  const MQ_BYT val
);

/// \copydoc MqSendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendO (
  struct MqSendS const * const send,
  const MQ_BOL val
);

/// \copydoc MqSendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendS (
  struct MqSendS const * const send,
  const MQ_SRT val
);

/// \copydoc MqSendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendI (
  struct MqSendS const * const send,
  const MQ_INT val
);

/// \copydoc MqSendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendF (
  struct MqSendS const * const send,
  const MQ_FLT val
);

/// \copydoc MqSendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendW (
  struct MqSendS const * const send,
  const MQ_WID val
);

/// \copydoc MqSendY
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendD (
  struct MqSendS const * const send,
  const MQ_DBL val
);

/// \brief append a #MQ_STR object to the \e MqSendS buffer
/// \send
/// \param in the string data to send
MQ_EXTERN void MQ_DECL MqSendC (
  struct MqSendS const * const send,
  MQ_STRB const * const in
);

/// \brief append a #MQ_BIN object to the \e MqSendS buffer
/// \send
/// \param in the binary data to send
/// \param len the size of the binary data to send
MQ_EXTERN void MQ_DECL MqSendB (
  struct MqSendS const * const send,
  MQ_BINB const * const in,
  MQ_SIZE const len
);

/// \brief append a #MQ_BUF object to the \e MqSendS buffer
/// \send
/// \param in the pointer to an #MqBufferS object to send
MQ_EXTERN void MQ_DECL MqSendU (
  struct MqSendS * const send,
  struct MqBufferS * const in
);

/// \brief append a \e MqBufferLS object to the \e MqSendS buffer
/// \send
/// \param in the pointer to an #MqBufferLS object to send
/// \attention all items of \e in are appended as single arguments to the call using #MqSendU
MQ_EXTERN void MQ_DECL MqSendL (
  struct MqSendS * const send,
  struct MqBufferLS const * const in
);

/// \brief append a vararg string to the \e MqSendS buffer
/// \send
/// \format
MQ_EXTERN void MQ_DECL MqSendV (
  struct MqSendS const * const send,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 2, 3)));

/// \brief append a vararg string as va_list to the \e MqSendS buffer
/// \send
/// \format
/// \var_list
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendVL (
  struct MqSendS const * const send,
  MQ_STRB const * const fmt,
  va_list var_list
);

/*****************************************************************************/
/*                                                                           */
/*                              sendM functions                              */
/*                                                                           */
/*****************************************************************************/

/// \sameas{MqSendY}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendYM (
  struct MqS const * const msgque,
  const MQ_BYT val
);

/// \sameas{MqSendO}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendOM (
  struct MqS const * const msgque,
  const MQ_BOL val
);

/// \sameas{MqSendS}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendSM (
  struct MqS const * const msgque,
  const MQ_SRT val
);

/// \sameas{MqSendI}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendIM (
  struct MqS const * const msgque,
  const MQ_INT val
);

/// \sameas{MqSendF}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendFM (
  struct MqS const * const msgque,
  const MQ_FLT val
);

/// \sameas{MqSendW}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendWM (
  struct MqS const * const msgque,
  const MQ_WID val
);

/// \sameas{MqSendD}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendDM (
  struct MqS const * const msgque,
  const MQ_DBL val
);

/// \sameas{MqSendC}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendCM (
  struct MqS const * const msgque,
  MQ_STRB const * const in
);

/// \sameas{MqSendB}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendBM (
  struct MqS const * const msgque,
  MQ_BINB const * const in,
  MQ_SIZE const len
);

/// \sameas{MqSendU}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendUM (
  struct MqS const * const msgque,
  struct MqBufferS * const in
);

/*****************************************************************************/
/*                                                                           */
/*                              send_compose                                 */
/*                                                                           */
/*****************************************************************************/

/// \brief initialize the \e MqSendS object and start to create a Msgque package
/// \send
MQ_EXTERN void MQ_DECL MqSendSTART (
  struct MqSendS * const send
);

/** \brief finish and send a \e service Msgque packet without waiting on \e return
 * 
 * This function is used to call a \e remote service or answer a \e service-call.
 * \send
 * \token
 * \retMqErrorE
 */
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendEND (
  struct MqSendS * const send,
  MQ_STRB const * const token
);

/** \brief send a \e service Msgque packet and waiting on \e return
 *
 * This function is used to call a \e remote service and wait for an answer.
 * \send
 * \token
 * \param_timeout
 * \retMqErrorE (without #MQ_CONTINUE)
 */
MQ_EXTERN enum MqErrorE MQ_DECL MqSendEND_AND_WAIT (
  struct MqSendS * const send,
  MQ_STRB const * const token,
  MQ_TIME_T const timeout
);

/** \brief send a \e service Msgque packet with a \e callback function defined
 *
 * This function is used to call a \e remote service and \e don't wait for an answer.
 * The answer will be processed asynchronous in #MqProcessEvent.
 * \send
 * \token
 * \param[in] proc A service procedure called with the results of the transaction
 * \param[in] data A user defined additional data
 * \param[in] datafreeF A function to free \e data after use
 * \retMqErrorE (without #MQ_CONTINUE)
 * \attention This function will \e never block.
 */
MQ_EXTERN enum MqErrorE MQ_DECL MqSendEND_AND_CALLBACK (
  struct MqSendS * const send,
  MQ_STRB const * const token,
  MqTokenF const proc,
  MQ_PTR data,
  MqTokenDataFreeF datafreeF
);

/** \brief send a \e return Msgque packet with data from the current \e MqErrorS object
 * 
 *  Utility function to simplify the \e send task for a \e service handle.
 *
 *  \send
 *  \retMqErrorE
 */
MQ_EXTERN enum MqErrorE MQ_DECL MqSendRETURN (
  struct MqSendS * const send
);

/** \brief send a msgque error from a server to a client
 * 
 *  Utility function to simplify the \e send task for a \e service handle.
 *
 *  \send
 *  \retMqErrorE
 */
MQ_EXTERN enum MqErrorE MQ_DECL MqSendERROR (
  struct MqSendS * const send
);

/** \brief send a \c _FTR \e filter packet
 *  \send
 *  \param[in] timeout timeout used for send data
 *  \retMqErrorE
 *  \attention on a server (#MqConfigS::isServer == #MQ_YES) this function is \b only
 *    allowed if a \e filter was defined too (#MqConfigS::isFilter == #MQ_YES)
 */
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendFTR (
  struct MqSendS * const send,
  MQ_TIME_T const timeout
);

/** \brief send a \c _EOF (EndOfFile) filter package
 *  \send
 *  \param[in] timeout timeout used for send data
 *  \retMqErrorE
 *  \attention on a server (#MqConfigS::isServer == #MQ_YES) this function is \b only
 *    allowed if a \e filter was defined too (#MqConfigS::isFilter == #MQ_YES)
 */
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendEOF (
  struct MqSendS * const send,
  MQ_TIME_T const timeout
);

/*****************************************************************************/
/*                                                                           */
/*                              send_composeM                                */
/*                                                                           */
/*****************************************************************************/

/// \sameas{MqSendSTART}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendSTARTM (
  struct MqS const * const msgque
);

/// \sameas{MqSendEND}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendENDM (
  struct MqS const * const msgque,
  MQ_STRB const * const token
);

/// \sameas{MqSendEND_AND_WAIT}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendEND_AND_WAITM (
  struct MqS const * const msgque,
  MQ_STRB const * const token,
  MQ_TIME_T const timeout
);

/// \sameas{MqSendEND_AND_CALLBACK}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendEND_AND_CALLBACKM (
  struct MqS const * const msgque,
  MQ_STRB const * const token,
  MqTokenF const proc,
  MQ_PTR data,
  MqTokenDataFreeF datafreeF
);

/// \sameas{MqSendRETURN}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendRETURNM (
  struct MqS const * const msgque
);

/// \sameas{MqSendERROR}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendERRORM (
  struct MqS const * const msgque
);

/// \sameas{MqSendFTR}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendFTRM (
  struct MqS const * const msgque,
  MQ_TIME_T const timeout
);

/// \sameas{MqSendEOF}
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSendEOFM (
  struct MqS const * const msgque,
  MQ_TIME_T const timeout
);

/*****************************************************************************/
/*                                                                           */
/*                              send_list                                    */
/*                                                                           */
/*****************************************************************************/

/** \brief start to append an embedded \e list item to the \e MqSendS
 *  \send
 *
 *  example: create a \e list item
 *  \code
MqSendL_START(send);     // start a LST item
MqSendI(send,myInt);        // first LST sub-item
MqSendC(send,"myString");   // second LST sub-item
// ... do additional MqSend?
MqSendL_END(send);       // finish a LST item
 *  \endcode
 */
MQ_EXTERN void MQ_DECL MqSendL_START (
  struct MqSendS * const send
);

/** \brief end to append an embedded \e list item to the \e MqSendS
 *  \send
 */
MQ_EXTERN void MQ_DECL MqSendL_END (
  register struct MqSendS * const send
);

/// \sameas{MqSendL_START}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendL_STARTM (
  struct MqS const * const msgque
);

/// \sameas{MqSendL_END}
MQ_EXTERN MQ_INLINE void MQ_DECL MqSendL_ENDM (
  struct MqS const * const msgque
);

/** \} send_api */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                     E V E N T - A P I                           ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \defgroup event_api PUBLIC: event interface functions
 *  \ingroup lib
 *  \{
 *  \brief handle different aspects of the \e MqS event-queue
 *
 *  the event-queue have to be linked into an existing event-processing infrastructure. 
 *  the linking is done by defining prototypes for two independent functions:
 *  - \b MqEventF is used to call an external event processing function (e.g. \c Tcl_DoOneEvent)
 *  - \b EventCreateF is used to add events, created by the file-handles owned by the \e MqS objects, 
 *       into the external event-queue (e.g. \c Tcl_QueueEvent)
 *  .
 */

/// \brief prototype for a Event-Check function
///
/// This function is called by the \libmsgque event-loop as idle-task in #MqProcessEvent
typedef enum MqErrorE (
  *EventCreateF
) (
  struct MqS * const
);

/** \brief Waiting for an incoming packet on all open msgque objects.
 * 
 *  This function is used to wait for (e.g. #MQ_WAIT_ONCE) or check (e.g. #MQ_WAIT_NO) for an 
 *  incoming event. If an event occurs the header of the Msgque packet is parsed and
 *  the according service-handle is called. if #MQ_WAIT_FOREVER is set the function
 *  will never return, accept for an error or on exit. This function is usually used
 *  on a server to enter the event-loop and wait for incoming service requests.
 * 
 *  \msgque
 *  \param_timeout_with_default
 *  \param wait should the code wait forever (e.g. #MQ_WAIT_FOREVER), just wait for the next event (e.g. #MQ_WAIT_ONCE) or just return after one single test (#MQ_WAIT_NO)"
 *  \retMqErrorE
 *
 *  \attention The \e timeout argument is only used if #MQ_WAIT_ONCE is set.
 *
 *  A typical server looks like:
\code
...
MqErrorCheck(MqCreate(config,argv,alfa,&ctx);
...
MqErrorCheck(MqProcessEvent(ctx, MQ_TIMEOUT, MQ_WAIT_FOREVER);
...
MqDelete(&ctx);
...
\endcode

 **/

MQ_EXTERN enum MqErrorE MQ_DECL MqProcessEvent (
  struct MqS * const msgque,
  MQ_TIME_T timeout,
  enum MqWaitOnEventE const wait
);

/** \} */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                       S L A V E - A P I                         ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \defgroup slave_api PUBLIC: struct MqSlaveS
 *  \ingroup msgque_api
 *  \{
 *  \brief Access to the master / slave feature

The master/slave link is used to create a mesh of nodes and
to link different \e PARENT context objects together.
<B>The master control the slave.</B>

The \e link is used to perform the following tasks:
 - report error messages from the \e slave to the \e master
 - to create a \e slave-child-context if a \e master-child-context is created
 - to delete a \e slave-context if a \e master-context is deleted
 .

In difference to the \e client/server link the \e master/slave link connect
two independent msgque-context in the same process or thread (e.g. node).
This leads to the restriction that only the \e master-msgque-context can be
a \e server because only \e on server per node is possible.

\verbatim
    node-0   |           node-1          |   node-2
===================================================================

| ** client/server link ** | ** client/server link ** |

             | *** master/slave link *** |

                           |- client1-0 -|- server2-0 ...
                           |
             |- server1-0 -|             |- server2-1 ...
             |             |- client1-1 -|- server2-2 ...
  client0-0 -|                           |- server2-3
             |
             |- server1-1 -|- client1-2 -|- server2-4 ...

\endverbatim

 **/

/// \brief the maximun number of slave objects per master
/// \attention this number can be changed but \libmsgque have to be recompiled after
#define MQ_SLAVE_MAX 1024

/// \brief create a \e master/slave link between the current msgque object and a slave 
/// msgque object
///
/// The link has the following requirements:
///  - the \e msgque object have to be a \e PARENT without a \e CHILD
///  - the \e slave object have to be a \e CLIENT-PARENT without a \e CHILD
///  - the \e id have to be unique
///  .
///
/// \msgque
/// \id
/// \param[in] slave a \e slave msgque object created by #MqCreate
/// \retMqErrorE
MQ_EXTERN enum MqErrorE
MQ_DECL MqSlaveCreate (
  struct MqS * const  msgque,
  MQ_SIZE             id,
  struct MqS * const  slave
);

/// \brief delete a \e master/slave
///
/// By default the \e slave context will be deleted if the \e master context is deleted.
/// In addition the \e parent-slave-context can be deleted explicit.
/// It is an \e error to delete a \e child-slave-context.
/// \msgque
/// \id
/// \retMqErrorE
/// \attention it is still possible to delete a \e child-slave-context using #MqDelete but
/// this will break the internal master/slave order
MQ_EXTERN enum MqErrorE
MQ_DECL MqSlaveDelete (
  struct MqS * const  msgque,
  MQ_SIZE   id
);

/// \brief get the slave context
///
/// \msgque
/// \id
/// \return the \e slave context or \c NULL if \e id is not valid
MQ_EXTERN struct MqS *
MQ_DECL MqSlaveGet (
  struct MqS * const  msgque,
  MQ_SIZE   id
);

/** \} slave api */

/* ####################################################################### */
/* ###                                                                 ### */
/* ###                     S Y S T E M - A P I                         ### */
/* ###                                                                 ### */
/* ####################################################################### */

/** \defgroup system_api PUBLIC: system interface functions
 *  \ingroup lib
 *  \{
 *  \brief Access to native system functions including \libmsgque error handling
 **/

/// \brief 'calloc' system call with error handling feature
/// \error
/// \param nmemb the number of members in the memory block
/// \param size the size of the new memory block
/// \return a pointer to the new memory block initialized with '0'
MQ_EXTERN MQ_INLINE MQ_PTR MQ_DECL MqSysCalloc (
  struct MqErrorS * const error,
  MQ_SIZE nmemb,
  MQ_SIZE size
);

/// \brief 'malloc' system call with error handling feature
/// \error
/// \param size the size of the new memory block
/// \return a pointer to the new memory block
MQ_EXTERN MQ_INLINE MQ_PTR MQ_DECL MqSysMalloc (
  struct MqErrorS * const error,
  MQ_SIZE size
);

/// \brief 'realloc' system call with error handling feature
/// \error
/// \param buf the \b old memory block to extend
/// \param size the size of the new memory block
/// \return a pointer to the new memory block
MQ_EXTERN MQ_INLINE MQ_PTR MQ_DECL MqSysRealloc (
  struct MqErrorS * const error,
  MQ_PTR buf,
  MQ_SIZE size
);

/// \brief 'free' system call macro
/// \retval tgt the memory block to delete and set to NULL
#define MqSysFree(tgt) \
    do { \
	if ( likely((tgt) != (NULL)) ) { \
	    free(tgt); \
	    (tgt) = (NULL); \
	} \
    } while (0)

/// \brief 'usleep' system call with error handling feature
/// \error
/// \param usec the micro (10^-6) seconds to sleep
/// \retMqErrorE
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSysUSleep (
  struct MqErrorS * const error,
  unsigned int const usec
);

/// \brief 'sleep' system call with error handling feature
/// \error
/// \param sec the seconds to sleep
/// \retMqErrorE
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL MqSysSleep (
  struct MqErrorS * const error,
  unsigned int const sec
);

/// \brief 'basename' system call with error handling feature
/// \param[in] in name of the string to extract the basename from (value of \e in will not be changed)
/// \param[in] includeExtension add extension like '.exe' to the filename (#MQ_YES or #MQ_NO)
/// \return the basename of \e in (it is save to modify the basename for additional needs)
/// \attention the memory of the basename string returned is owned by the programmen and have to be freed
MQ_EXTERN MQ_STR MQ_DECL MqSysBasename (
  char const * const in, 
  MQ_BOL includeExtension
);

#if defined(_MSC_VER)
/// \brief helper to use the timeval  
struct mq_timeval {
  long tv_sec;              ///< seconds
  long tv_usec;             ///< useconds
};

/// \brief \helper to use timezone
struct mq_timezone {
  int  tz_minuteswest;      ///< minutes W of Greenwich
  int  tz_dsttime;          ///< type of dst correction
};

#else
#define mq_timeval timeval
#define mq_timezone timezone
struct mq_timeval;
struct mq_timezone;
#endif

/// \brief 'gettimeofday' system call with error handling feature
/// \error
/// \retval tv the timeval object
/// \retval tz the timezone object
/// \retMqErrorE
MQ_EXTERN MQ_INLINE enum MqErrorE MQ_DECL SysGetTimeOfDay (
  struct MqErrorS * const error,
  struct mq_timeval * tv,
  struct mq_timezone * tz
);

/** \} system api */

/** \defgroup log_api PUBLIC: logging interface functions
 *  \ingroup lib
 *  \{
 *  \brief logging information on stderr with a common format.
 */

/*****************************************************************************/
/*                                                                           */
/*                               log_basic                                   */
/*                                                                           */
/*****************************************************************************/

/// \brief log vararg string without formatting to \e channel
/// \param channel the file-handle to report the message on
/// \format
MQ_EXTERN void MQ_DECL MqLog (
  FILE * channel,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 2, 3)));

/// \brief log vararg string using formatting
/// \msgque
/// \proc
/// \param level debug level: 0 <= level <= 9
/// \format
/// \attention if the \b -silent option is set nothing will be reported
MQ_EXTERN void MQ_DECL MqDLogV (
  struct MqS const * const msgque,
  MQ_STRB const * const proc,
  MQ_INT level,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 4, 5)));

#if defined(MQ_DISABLE_LOG)
#   define MqDLogC(msgque,level,str)
#   define MqDLog(msgque,level,fmt,MQ_VAR_ARGS)
#else

/// \brief log a plain string 
/// \msgque
/// \param level debug level 0 <= level <= 9
/// \param str string to log
#   define MqDLogC(msgque,level,str) \
    if (unlikely(msgque != NULL && msgque->config.debug >= level)) \
	MqDLogV(msgque,__func__,level,str)

/// \brief log a vararg string using formatting and checking for logging level
/// \msgque
/// \param level debug level 0 <= level <= 9
/// \format
/// \param ... vararg string argument
/// \attention the prefix string is the calling function name
#   define MqDLog(msgque,level,fmt,...) \
    if (unlikely(msgque != NULL && msgque->config.debug >= level)) \
	MqDLogV(msgque,__func__,level,fmt,__VA_ARGS__)
#endif

/// \brief log a vararg error list using formatting
/// \msgque
/// \proc
/// \format
/// \var_list
MQ_EXTERN void MQ_DECL MqDLogEVL (
  struct MqS const * const msgque,
  MQ_STRB const * const proc,
  MQ_STRB const * const fmt,
  va_list var_list
);

/// \brief log a vararg error string using formatting
/// \msgque
/// \proc
/// \format
MQ_EXTERN void MQ_DECL MqDLogEV (
  struct MqS const * const msgque,
  MQ_STRB const * const proc,
  MQ_STRB const * const fmt,
  ...
) __attribute__ ((format (printf, 3, 4)));

/*****************************************************************************/
/*                                                                           */
/*                               log_advanced                                */
/*                                                                           */
/*****************************************************************************/

/// \brief convert a #MqErrorE into a human readable string
/// \param status a value from #MqErrorE
/// \return the string
MQ_EXTERN const char * MQ_DECL MqLogStatus (
  enum MqErrorE status
);

/// \brief get the \e type member from an MqTypeE object in human readable form
/// \type
/// \return the 3 character name of the #MqTypeE without the \c "MQ_" prefix and the \c "T" postfix
MQ_EXTERN const char * MQ_DECL MqLogTypeName (
  const enum MqTypeE type
);

/// \brief convert a MqContextS server member into a human readable string
/// \msgque
/// \return the string \e SERVER or \e CLIENT
MQ_EXTERN MQ_STRB const * MQ_DECL MqLogServerOrClient (
  struct MqS const * const msgque
);

/// \brief convert a MqContextS parent member into a human readable string
/// \msgque
/// \return the string \e PARENT or \e CHILD
MQ_EXTERN MQ_STRB const * MQ_DECL MqLogParentOrChild (
  struct MqS const * const msgque
);

/// \brief convert a raw #MQ_STR into a human readable string.
/// \param buf buffer of size \e size+1 able to store the temporary string representation of \e str
/// \param str the input string
/// \param size the size of \e str
/// \return the string
///
/// replace non ASCII characters with '?' or 'o' (NULL)
MQ_EXTERN MQ_STR MQ_DECL MqLogC (
  MQ_STR buf,
  MQ_STR str,
  MQ_SIZE size
);

/// \}

//end c++ save defibition
END_C_DECLS

#endif /* MQ_MSGQUE_H */



