/**
 *  \file       src/pipe_io.c
 *  \brief      \$Id: pipe_io.c 1176 2009-05-14 13:57:36Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1176 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "main.h"
#include "pipe_io.h"
#include "io_private.h"
#include "error.h"
#include "sys_com.h"

/*###########################################################################*/
/*###                                                                     ###*/
/*###                              posix                                  ###*/
/*###                                                                     ###*/
/*###########################################################################*/

#   include "generic_io.h"
#   include "log.h"
#   include "sys.h"
#   include "buffer.h"
#   include "bufferL.h"

#   define MQ_ERROR_S  pipe->io->msgque->error
#   define MSGQUE pipe->io->msgque

/*****************************************************************************/
/*                                                                           */
/*                                private                                    */
/*                                                                           */
/*****************************************************************************/

/** -brief ...
 */
struct PipeS {
  struct MqIoS * io;		///< link to 'io' object
  struct GenericS * generiC;	///< generic part of the connection

  MQ_SOCK socks[2] ;		///< the result from socketpair
  char stringbuffer[20];	///< context temporar memory
};

/*****************************************************************************/
/*                                                                           */
/*                                public                                     */
/*                                                                           */
/*****************************************************************************/

void
PipeCleanup (
  struct MqErrorS * const error,
  struct MqBufferLS * const argv
) {
  MQ_INT dummy;
  MqBufferLCheckOptionI (error, argv, "--socket", &dummy);
}

static enum MqErrorE
sPipeCheckArg (
  struct PipeS * const pipe,
  struct MqBufferLS * const argv
) {
  MqErrorCheck (MqBufferLCheckOptionI (MQ_ERROR_S, argv, "--socket", &pipe->socks[1]));

  return MQ_OK;

error:
  return MqErrorStack (MQ_ERROR_S);
}

enum MqErrorE
PipeCreate (
  struct MqIoS * const io,
  struct MqBufferLS * const argv,
  struct PipeS ** const pipePtr
)
{
  struct MqS const * const msgque = io->msgque;
  struct MqErrorS * const error = msgque->error;

  // ok we have an PIPE style communication
  register struct PipeS * const pipe = (struct PipeS *const) MqSysCalloc (error, 1, sizeof (*pipe));
  if (likely (pipePtr != NULL))
    *pipePtr = pipe;

  pipe->io = io;
  pipe->socks[0] = -1;
  pipe->socks[1] = -1;

  // check for arguments
  MqErrorCheck (sPipeCheckArg (pipe, argv));

  if (MQ_IS_CLIENT (msgque)) {
    // create the socketpair
    MqErrorCheck (SysSocketPair (error, pipe->socks));
    MqDLog(msgque,4,"create SOCKETPAIR -> sock[0]<%i>, sock[1]<%i>\n", pipe->socks[0], pipe->socks[1]);
  } else {
    if (pipe->socks[1] < 0)
      return MqErrorDb (MQ_ERROR_NOT_A_CLIENT);
    // close the "client" socket of the socketpair -> who set socks[0] ?
    //MqErrorCheck(SysCloseSocket(error,MQ_NO,&pipe->socks[0]));
    MqDLog(msgque,4,"get SOCKET -> %i\n", pipe->socks[1]);
  }

  return MQ_OK;

error:
  return MqErrorStack (error);
}

void PipeDelete (
  struct PipeS ** const pipeP
)
{
  struct PipeS * const pipe = *pipeP;
  if (unlikely(pipe == NULL))	return;		// nothing to do
  SysCloseSocket (MQ_ERROR_S, MQ_YES, (MQ_IS_CLIENT(MSGQUE) ?  &pipe->socks[0] : &pipe->socks[1]));
  MqSysFree (*pipeP);
}

/*****************************************************************************/
/*                                                                           */
/*                                public                                     */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE enum MqErrorE
PipeServer (
  struct PipeS * const pipe
)
{
  // listen for events on the "server" socket of the socketpair
  return pIoEventAdd(pipe->io, &pipe->socks[1]);
}

MQ_INLINE enum MqErrorE
PipeConnect (
  struct PipeS * const pipe
)
{
  // listen for event on the "client" socket of the socketpair
  return pIoEventAdd(pipe->io, &pipe->socks[0]);
}

/*****************************************************************************/
/*                                                                           */
/*                                set/get                                    */
/*                                                                           */
/*****************************************************************************/

MQ_PTR
PipeGetOpt (
  struct PipeS const * const pipe,
  int const opt
) {
  if (unlikely (pipe == NULL))
    return NULL;
  return NULL;
}

MQ_INLINE MQ_SOCK*
PipeGetServerSocket (
  struct PipeS * const pipe
) {
  return &pipe->socks[1];
}



