/**
 *  \file       src/read.h
 *  \brief      \$Id: read.h 1087 2009-04-26 15:03:00Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1087 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef READ_H
#   define READ_H

/*****************************************************************************/
/*                                                                           */
/*                            msgque return codes                            */
/*                                                                           */
/*****************************************************************************/

/// \brief Return-Codes used in #sSend_RET_START function
enum MqReturnE {
    MQ_RETURN_OK	= 'O',		///< return an \b ok package
    MQ_RETURN_ERROR	= 'E',		///< return an \b error package
};

/*****************************************************************************/
/*                                                                           */
/*                                read_init                                  */
/*                                                                           */
/*****************************************************************************/

/** -brief creation of an  \e MqReadS object
 */
struct MqReadS* pReadCreate (
  register struct MqS * const msgque
);

/** -brief deletion of an \e MqReadS object
 */
void pReadDelete (
  register struct MqReadS ** const readP
) __attribute__((nonnull));

/*****************************************************************************/
/*                                                                           */
/*                                read_misc                                  */
/*                                                                           */
/*****************************************************************************/

enum MqErrorE pReadHDR (
  register struct MqReadS * read,
  struct MqS ** a_msgque
);

void pReadSetType(
  struct MqReadS * const read,
  MQ_BOL const type
);

/// \brief get the return-code of the current Msgque packet from the \e MqReadS stack
/// \read
/// \return the return-code
MQ_INLINE enum MqReturnE pReadGetReturnCode (
  struct MqReadS const * const read
);

/// \brief get the return-number of the current Msgque packet from the \e MqReadS stack
/// \read
/// \return the return-number
MQ_INLINE MQ_INT pReadGetReturnNum (
  struct MqReadS const * const read
);

/// \brief reading from a dynamic \e MqBufferS
/// \error
/// \buf
/// \retval out the \e MqBufferS object to be filled
/// \return error-code
/// \attention
/// - the \e out object itself will \b not be created and have to be available 
///   but the \b internal buffer memory is expanded as needed
/// - the numItems of the \e buf object will be decreased by 1
/// - at the end of \e out an \\0 will be attached
/// .
enum MqErrorE pReadWord (
  struct MqErrorS * const error,
  struct MqBufferS * const buf,
  register struct MqBufferS * const out
);

/// \brief creating a \e RET reference for reading the return items
/// \read
/// \retMqErrorE
enum MqErrorE pRead_RET_START (
  register struct MqReadS * const read
) __attribute__((nonnull));

/// \brief deleting a \e RET reference after reading the return items was finished
/// \param read a pointer to an MqReadS object
#define pRead_RET_END(read) MqReadL_END(read)

/*****************************************************************************/
/*                                                                           */
/*                                 read.log                                  */
/*                                                                           */
/*****************************************************************************/

void pReadLog (
  MQ_STRB const * const prefix,
  register struct MqReadS const * const read
);


#endif /* READ_H */





