/**
 *  \file       src/send.c
 *  \brief      \$Id: send.c 1184 2009-05-18 11:54:15Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1184 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "main.h"
#include "send.h"
#include "error.h"
#include "buffer.h"
#include "sys.h"
#include "log.h"
#include "mq_io.h"
#include "token.h"
#include "swap.h"
#include "read.h"
#include "trans.h"
#include "cache.h"
#include "read.h"

// the 'sprintf' function overwrite the first character after the end with a '\0' character,
// we need to put back the BUFFER_CHAR after every write
// #define SEND_PS(cur,data) 
//     sprintf(cur, MQ_FORMAT_Xn(HDR_ATO_LEN), (MQ_PTRZ) data); 
//     *(cur+HDR_ATO_LEN) = BUFFER_CHAR;

//*((MQ_INT*)(ptr)) = data;

#define SEND_I(string,ptr,data) \
    if(unlikely(string)) { \
        sprintf((MQ_STR)ptr, MQ_FORMAT_XI(HDR_INT_LEN), (MQ_INT) data); \
	*((MQ_STR)(ptr+HDR_INT_LEN)) = BUFFER_CHAR; \
	ptr += HDR_INT_LEN; \
    } else { \
	memcpy (ptr,&data,sizeof(MQ_INT)); \
	ptr+=4;*ptr++='\0';*ptr++='\0';*ptr++='\0';*ptr++='\0';\
    }

#define SEND_IS(char,cur,data,len) \
    sprintf(cur, MQ_FORMAT_XI(len), (MQ_INT) data); \
    char = BUFFER_CHAR;

/*****************************************************************************/
/*                                                                           */
/*                               send_data                                   */
/*                                                                           */
/*****************************************************************************/

#define MSGQUE send->msgque
#define MQ_ERROR_S send->msgque->error

/** -brief special data needed for list objects
 */
struct SendSaveS {
  MQ_INT numItems;		///< number of objects in the msgque body
  struct SendSaveS * save;      ///< this is needed for recursion
  MQ_SIZE cursize;              ///< the current size of \e bdy->data
};

/** -brief everything needed for sending msgque packages
 */
struct MqSendS {
  struct MqS * msgque;		///< ...
  struct MqBufferS * buf;       ///< the sending buffer object
  struct SendSaveS * save;      ///< need for List objects
  struct MqCacheS * cache;	///< ...
  MQ_BOL haveStart;		///< #MqSendEND checks if #MqSendSTART was used
};

/*****************************************************************************/
/*                                                                           */
/*                               send_static                                 */
/*                                                                           */
/*****************************************************************************/

static struct SendSaveS *
sSendC (
  struct MqErrorS * const error
)
{
  return (struct SendSaveS *) MqSysMalloc (error, sizeof (struct SendSaveS));
}

static void
sSendD (
  struct SendSaveS ** const ptr
)
{
  MqSysFree (*ptr);
}

/*****************************************************************************/
/*                                                                           */
/*                               send_init                                   */
/*                                                                           */
/*****************************************************************************/

enum MqErrorE
pSendCreate (
  struct MqS * const msgque,
  struct MqSendS ** const out
)
{
  struct MqErrorS * const error = msgque->error;
  register struct MqSendS * const  send = (struct MqSendS *const) MqSysCalloc (error, 1, sizeof (*send));
  register MQ_STR ptr;

  if (likely (out != NULL))
    *out = send;

  send->msgque = msgque;

  send->buf = MqBufferCreate (10000);
  send->haveStart = MQ_NO;

  ptr = (MQ_STR) send->buf->data;

  // Format HDR
  memcpy (ptr, "HDR" BUFFER_CHAR_S, HDR_CtxId_S);
  ptr[HDR_BdySize_S - 1] = BUFFER_CHAR;
  ptr[HDR_Token_S - 1] = BUFFER_CHAR;
  ptr[HDR_Trans_S - 1] = BUFFER_CHAR;
  ptr[HDR_Code_S - 1] = BUFFER_CHAR;
  ptr[HDR_SIZE - 1] = BUFFER_CHAR;

  // Format BDY
  ptr += HDR_SIZE;

  memcpy (ptr, "BDY" BUFFER_CHAR_S, BDY_NumItems_S);

  // create the cache data
  pCacheCreate (error, (CacheCF) sSendC, (CacheDF) sSendD, error, &send->cache);

  return MQ_OK;
}

enum MqErrorE
pSendDelete (
  struct MqSendS ** const sendP
)
{
  struct MqSendS * const send = *sendP;
  if (unlikely (send == NULL))
    return MQ_OK;

  // selete my cache
  pCacheDelete (&send->cache);

  MqBufferDelete (&send->buf);
  MqSysFree (*sendP);
  return MQ_OK;
}

/*****************************************************************************/
/*                                                                           */
/*                               send_atom                                   */
/*                                                                           */
/*****************************************************************************/

static void
sSendLen (
  struct MqSendS const * const send,
  MQ_SIZE const len,
  enum MqTypeE const type,
  MQ_BOL string
)
{

  register struct MqBufferS * buf = send->buf;
// MK1
  register MQ_BIN ist = buf->cur.B - BUFFER_P2_PRENUM;

  *ist++ = BUFFER_CHAR;

 /* set the type */
  *ist++ = (char) type;
  *ist++ = BUFFER_CHAR;

 /* set the size */
  SEND_I (string, ist, len);
  *ist++ = BUFFER_CHAR;

  ist += len;
  buf->numItems++;

  buf->cursize = (ist - buf->data);
  buf->cur.B = ist;
}

static void
sSendA1 (
  struct MqSendS const * const send,
  register union MqBufferAtomU const in,
  enum MqTypeE const type
)
{
  register struct MqBufferS * const buf = send->buf;
  if (unlikely(send->msgque->config.isString)) {
      MQ_SIZE len=0;
      pBufferAddSizeDyn (buf, (BUFFER_P2_PRENUM+50));
      buf->cur.B += BUFFER_P2_PRENUM;
      // MQ_BYTT and MQ_BOLT are using the same basic type "char"
      len = sprintf (buf->cur.C, MQ_FORMAT_Y, in.Y);
      sSendLen(send, len, MQ_STRT, MQ_YES);
  } else {
      register MQ_BIN ist;
      pBufferAddSizeDyn (buf, (BUFFER_P2_NATIVE+4));
      ist = buf->cur.B;
      *ist++ = BUFFER_CHAR;
      *ist++ = type;
      *ist++ = BUFFER_CHAR;
      *ist++ = in.B[0];
      buf->numItems++;
      buf->cursize = (ist - buf->data);
      buf->cur.B = ist;
  }
}

static void
sSendA2 (
  struct MqSendS const * const send,
  register union MqBufferAtomU const in,
  enum MqTypeE const type
)
{
  register struct MqBufferS * const buf = send->buf;
  if (unlikely(send->msgque->config.isString)) {
      MQ_SIZE len=0;
      pBufferAddSizeDyn (buf, (BUFFER_P2_PRENUM+50));
      buf->cur.B += BUFFER_P2_PRENUM;
      len = sprintf (buf->cur.C, MQ_FORMAT_S, in.S);
      sSendLen(send, len, MQ_STRT, MQ_YES);
  } else {
      register MQ_BIN ist;
      pBufferAddSizeDyn (buf, (BUFFER_P2_NATIVE+4));
      ist = buf->cur.B;
      *ist++ = BUFFER_CHAR;
      *ist++ = type;
      *ist++ = BUFFER_CHAR;
      *ist++ = in.B[0];
      *ist++ = in.B[1];
      buf->numItems++;
      buf->cursize = (ist - buf->data);
      buf->cur.B = ist;
  }
}

static void
sSendA4 (
  struct MqSendS const * const send,
  register union MqBufferAtomU const in,
  enum MqTypeE const type
)
{
  register struct MqBufferS * const buf = send->buf;
  if (unlikely(send->msgque->config.isString)) {
      MQ_SIZE len=0;
      pBufferAddSizeDyn (buf, (BUFFER_P2_PRENUM+50));
      buf->cur.B += BUFFER_P2_PRENUM;
      if (type == MQ_INTT) {
	len = sprintf (buf->cur.C, MQ_FORMAT_I, in.I);
      } else {
	len = sprintf (buf->cur.C, MQ_FORMAT_F, in.F);
      }
      sSendLen(send, len, MQ_STRT, MQ_YES);
  } else {
      register MQ_BIN ist;
      pBufferAddSizeDyn (buf, (BUFFER_P2_NATIVE+4));
      ist = buf->cur.B;
      *ist++ = BUFFER_CHAR;
      *ist++ = type;
      *ist++ = BUFFER_CHAR;
      *ist++ = in.B[0];
      *ist++ = in.B[1];
      *ist++ = in.B[2];
      *ist++ = in.B[3];
      buf->numItems++;
      buf->cursize = (ist - buf->data);
      buf->cur.B = ist;
  }
}

static void
sSendA8 (
  struct MqSendS const * const send,
  register union MqBufferAtomU const in,
  enum MqTypeE const type
)
{
  register struct MqBufferS * const buf = send->buf;
  if (unlikely(send->msgque->config.isString)) {
      MQ_SIZE len=0;
      pBufferAddSizeDyn (buf, (BUFFER_P2_PRENUM+50));
      buf->cur.B += BUFFER_P2_PRENUM;
      if (type == MQ_DBLT) {
	len = sprintf (buf->cur.C, MQ_FORMAT_D, in.D);
      } else {
	len = sprintf (buf->cur.C, MQ_FORMAT_W, in.W);
      }
      sSendLen(send, len, MQ_STRT, MQ_YES);
  } else {
      register MQ_BIN ist;
      pBufferAddSizeDyn (buf, (BUFFER_P2_NATIVE+8));
      ist = buf->cur.B;
      *ist++ = BUFFER_CHAR;
      *ist++ = type;
      *ist++ = BUFFER_CHAR;
      *ist++ = in.B[0];
      *ist++ = in.B[1];
      *ist++ = in.B[2];
      *ist++ = in.B[3];
      *ist++ = in.B[4];
      *ist++ = in.B[5];
      *ist++ = in.B[6];
      *ist++ = in.B[7];
      buf->numItems++;
      buf->cursize = (ist - buf->data);
      buf->cur.B = ist;
  }
}

MQ_INLINE void
MqSendY (
  struct MqSendS const * const send,
  const MQ_BYT in
)
{
  union MqBufferAtomU ato;
  ato.Y = in;
  sSendA1(send,ato,MQ_BYTT);
}

MQ_INLINE void
MqSendO (
  struct MqSendS const * const send,
  const MQ_BOL in
)
{
  union MqBufferAtomU ato;
  ato.O = in;
  sSendA1(send,ato,MQ_BOLT);
}

MQ_INLINE void
MqSendS (
  struct MqSendS const * const send,
  const MQ_SRT in
)
{
  union MqBufferAtomU ato;
  ato.S = in;
  sSendA2(send,ato,MQ_SRTT);
}

MQ_INLINE void
MqSendI (
  struct MqSendS const * const send,
  const MQ_INT in
)
{
  union MqBufferAtomU ato;
  ato.I = in;
  sSendA4(send,ato,MQ_INTT);
}

MQ_INLINE void
MqSendF (
  struct MqSendS const * const send,
  const MQ_FLT in
)
{
  union MqBufferAtomU ato;
  ato.F = in;
  sSendA4(send,ato,MQ_FLTT);
}

MQ_INLINE void
MqSendW (
  struct MqSendS const * const send,
  const MQ_WID in
)
{
  union MqBufferAtomU ato;
  ato.W = in;
  sSendA8(send,ato,MQ_WIDT);
}

MQ_INLINE void
MqSendD (
  struct MqSendS const * const send,
  const MQ_DBL in
)
{
  union MqBufferAtomU ato;
  ato.D = in;
  sSendA8(send,ato,MQ_DBLT);
}

void
MqSendC (
  struct MqSendS const * const send,
  MQ_STRB const * const in
)
{
  struct MqBufferS * buf = send->buf;
  MQ_SIZE len = strlen(in);
  pBufferAddSizeDyn (buf, BUFFER_P2_PRENUM+len);
// MK1
  buf->cur.B += BUFFER_P2_PRENUM;
  memcpy (buf->cur.C, in, len);
  sSendLen(send, len, MQ_STRT, MSGQUE->config.isString);
}

void
MqSendB (
  struct MqSendS const * const send,
  MQ_BINB const * const in,
  MQ_SIZE const len
)
{
  register struct MqBufferS * buf = send->buf;
  pBufferAddSize (buf, BUFFER_P2_PRENUM+len);
// MK1
  buf->cur.B += BUFFER_P2_PRENUM;
  memcpy (buf->cur.B, in, len);
  sSendLen (send, len, MQ_BINT, MSGQUE->config.isString);
}

void
MqSendU (
  struct MqSendS * const send,
  struct MqBufferS * const in
)
{
  // reset to the start
  in->cur.B = in->data;
  switch (in->type) {
    case MQ_BYTT: sSendA1 (send, MqBufU2ATO(in->cur), MQ_BYTT); break;
    case MQ_BOLT: sSendA1 (send, MqBufU2ATO(in->cur), MQ_BOLT); break;
    case MQ_SRTT: sSendA2 (send, MqBufU2ATO(in->cur), MQ_SRTT); break;
    case MQ_INTT: sSendA4 (send, MqBufU2ATO(in->cur), MQ_INTT); break;
    case MQ_FLTT: sSendA4 (send, MqBufU2ATO(in->cur), MQ_FLTT); break;
    case MQ_WIDT: sSendA8 (send, MqBufU2ATO(in->cur), MQ_WIDT); break;
    case MQ_DBLT: sSendA8 (send, MqBufU2ATO(in->cur), MQ_DBLT); break;
    case MQ_STRT: MqSendC (send, in->cur.C); break;
    case MQ_BINT: MqSendB (send, in->cur.B, in->cursize); break;
    case MQ_LSTT: {
      struct MqReadS * const read = MSGQUE->read;
      MQ_BUF buf;
      MqReadL_START(read,in);
      MqSendL_START(send);
      while (MqReadItemExists(read)) {
	MqReadU(read,&buf);
	MqSendU(send,buf);
      }
      MqSendL_END(send);
      MqReadL_END(read);
      break;
    }
    case MQ_RETT: MqPanicSYS(MQ_ERROR_S);
  }
}

void
MqSendL (
  struct MqSendS * const send,
  struct MqBufferLS const * const in
)
{
  register struct MqBufferS * const * start = in->data;
  struct MqBufferS * const * const end = in->data + in->cursize;
  for (; start<end; start++) {
    MqSendU (send, *start);
  }
}

MQ_INLINE void
MqSendVL (
  struct MqSendS const * const send,
  MQ_STRB const * const fmt,
  va_list ap
)
{
  register struct MqBufferS * buf = send->buf;
  MQ_SIZE len;
  pBufferAddSize (buf, 100);
// MK1
  buf->cur.B += BUFFER_P2_PRENUM;
  buf->cursize += BUFFER_P2_PRENUM;
  len = MqBufferAppendVL (buf, fmt, ap);
  buf->cur.B -= len;
  sSendLen (send, len, MQ_STRT, MSGQUE->config.isString);
}

void
MqSendV (
  struct MqSendS const * const send,
  MQ_STRB const * const fmt,
  ...
)
{
  va_list ap;
  va_start (ap, fmt);
  MqSendVL (send, fmt, ap);
  va_end (ap);
}

/*****************************************************************************/
/*                                                                           */
/*                              sendM functions                              */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE void
MqSendYM (
  struct MqS const * const msgque,
  const MQ_BYT in
)
{
  union MqBufferAtomU ato;
  ato.Y = in;
  sSendA1(msgque->send,ato,MQ_BYTT);
}

MQ_INLINE void
MqSendOM (
  struct MqS const * const msgque,
  const MQ_BOL in
)
{
  union MqBufferAtomU ato;
  ato.O = in;
  sSendA1(msgque->send,ato,MQ_BOLT);
}

MQ_INLINE void
MqSendSM (
  struct MqS const * const msgque,
  const MQ_SRT in
)
{
  union MqBufferAtomU ato;
  ato.S = in;
  sSendA2(msgque->send,ato,MQ_SRTT);
}

MQ_INLINE void
MqSendIM (
  struct MqS const * const msgque,
  const MQ_INT in
)
{
  union MqBufferAtomU ato;
  ato.I = in;
  sSendA4(msgque->send,ato,MQ_INTT);
}

MQ_INLINE void
MqSendFM (
  struct MqS const * const msgque,
  const MQ_FLT in
)
{
  union MqBufferAtomU ato;
  ato.F = in;
  sSendA4(msgque->send,ato,MQ_FLTT);
}

MQ_INLINE void
MqSendWM (
  struct MqS const * const msgque,
  const MQ_WID in
)
{
  union MqBufferAtomU ato;
  ato.W = in;
  sSendA8(msgque->send,ato,MQ_WIDT);
}

MQ_INLINE void
MqSendDM (
  struct MqS const * const msgque,
  const MQ_DBL in
)
{
  union MqBufferAtomU ato;
  ato.D = in;
  sSendA8(msgque->send,ato,MQ_DBLT);
}

MQ_INLINE void
MqSendCM (
  struct MqS const * const msgque,
  MQ_STRB const * const in
)
{
  MqSendC(msgque->send, in);
}

MQ_INLINE void
MqSendBM (
  struct MqS const * const msgque,
  MQ_BINB const * const in,
  MQ_SIZE const len
)
{
  MqSendB(msgque->send, in, len);
}

MQ_INLINE void
MqSendUM (
  struct MqS const * const msgque,
  struct MqBufferS * const in
)
{
  MqSendU(msgque->send, in);
}

/*****************************************************************************/
/*                                                                           */
/*                              send_compose                                 */
/*                                                                           */
/*****************************************************************************/

void
MqSendSTART (
  struct MqSendS * const send
)
{
  struct MqS const * const msgque = send->msgque;
  register struct MqBufferS * const buf = send->buf;
  register struct HdrS * const cur = (struct HdrS *) buf->data;

  buf->numItems = 0;
  send->haveStart = MQ_YES;
  send->save = NULL;

  // add FLAG data
  if (unlikely(msgque->config.isString)) {
    buf->type = MQ_STRT;
    SEND_IS (cur->charS, cur->ctxId.S, msgque->ctxId, HDR_INT_LEN);

//MqDLogV (msgque, __func__, 0, "msgque->ctxId<%i>, cur->ctxId.B<%s>, string<%i>\n", 
//    msgque->ctxId, cur->ctxId.S, msgque->config.isString);

  } else {
    buf->type = MQ_BINT;
    cur->ctxId.B = msgque->ctxId;

//MqDLogV (msgque, __func__, 0, "msgque->ctxId<%i>, cur->ctxId.B<%i>, string<%i>\n", 
//    msgque->ctxId, cur->ctxId.B, msgque->config.isString);
  }

  // intitialize the return-code
  cur->code = MQ_RETURN_OK;

  buf->cur.B = (((MQ_BIN)cur) + sizeof(struct HdrS) + BDY_SIZE);
  buf->cursize = (sizeof(struct HdrS) + BDY_SIZE);
}

MQ_INLINE enum MqErrorE
MqSendEND (
  struct MqSendS * const send,
  MQ_STRB const * const token
)
{
  return pMqSendEND(send, token, 0);
}

enum MqErrorE
pMqSendEND (
  struct MqSendS * const send,
  MQ_STRB const * const token,
  const MQ_HDL trans
)
{
  register struct MqS * const msgque = send->msgque;
  register struct MqBufferS * const buf = send->buf;

//MqDLog(msgque,0,"trans<%p>\n", trans);

  MQ_SIZE cursize = buf->cursize - HDR_SIZE;
  register struct HdrSendS * const cur = (struct HdrSendS *) buf->data;

  // check if START was used 
  if (send->haveStart == MQ_NO) {
    msgque->_trans = 0;
    return MqErrorDb(MQ_ERROR_SEND_END_WITHOUT_START);
  }
  send->haveStart = MQ_NO;

  // write pToken
  memcpy (cur->hdr.tok, token, HDR_TOK_LEN);

  // the Transaction token will allways be send as binary
  cur->hdr.trans = trans;

  // set some data
  if (unlikely(msgque->config.isString)) {
    if (cursize > BDY_SIZE) {
	SEND_IS (cur->hdr.charO, cur->hdr.size.S, cursize, HDR_INT_LEN);
	SEND_IS (cur->hdr.charE, cur->bdy.num.S, buf->numItems, HDR_INT_LEN);
    } else {
	cursize = 0;
	SEND_IS (cur->hdr.charO, cur->hdr.size.S, cursize, HDR_INT_LEN);
	buf->cursize -= BDY_SIZE;
	buf->cur.B -= BDY_SIZE;
    }
  } else {
    if (cursize > BDY_SIZE) {
	cur->hdr.size.B = cursize;
	cur->bdy.num.B = buf->numItems;
    } else {
	cur->hdr.size.B = 0;
	buf->cursize -= BDY_SIZE;
	buf->cur.B -= BDY_SIZE;
    }
  }

/*
if (MQ_IS_SERVER(msgque)) {
MqDLogV (msgque, __func__, 0, "token<%s>, ctxId<%i>, trans<%p>, string<%i>\n", 
    token, cur->hdr.ctxId.B, cur->hdr.trans, msgque->config.isString);
MqDLogV (msgque, __func__, 0, "body-prefix<%s>\n", buf->data+sizeof(struct HdrS));
MqDLogV (msgque, __func__, 0, "body-item<%s>\n", buf->data+sizeof(struct HdrS)+sizeof(struct BdyS)+1);
}

I1
printLP(msgque)
pLogHEX(msgque, __func__, buf->data, (buf->cursize > 110 ? 110 : buf->cursize));
*/

  MqErrorSwitch (pIoSend (msgque->io, buf));

  return MQ_OK;

error:
  return MqErrorStack (MQ_ERROR_S);
}

enum MqErrorE
MqSendEND_AND_WAIT (
  struct MqSendS * const send,
  MQ_STRB const * const token,
  MQ_TIME_T const timeout
)
{
  struct MqS * const msgque = send->msgque;
  struct MqTransS * const trans = msgque->trans;
  struct MqReadS * read;
  enum MqTransE status;

  // 1. create the transaktion
  MQ_HDL transH =  pTransPop (trans, NULL, NULL, NULL);

  // 2. send the message
  MqErrorSwitch (pMqSendEND (send, token, transH));

  // 3. wait until the transaction has finished
  msgque->_trans = 0;
  do {
    MqErrorSwitch (MqProcessEvent (msgque, timeout, MQ_WAIT_ONCE));
  }
  while (pTransCheckStart (trans, transH));
  msgque->_trans = pTransGetLast (trans, transH);

  // 4. replace 'read'
  read = pTransGetResult (trans, transH);
  if (read) {
    // the next 2 lines are a TRANSACTION -> do not split
    pCachePush (msgque->readCache, msgque->read);
    msgque->read = read;
  }

  // 5. clean transaction
  status = pTransGetStatus (trans, transH);
  pTransPush (trans, transH);
  switch (status) {
    case MQ_TRANS_END: 
      break;
    case MQ_TRANS_CANCEL:
      return MqErrorC (msgque->error, __func__, -1, "transaction CANCEL");
    case MQ_TRANS_START:
      MqPanicC (msgque->error, __func__, -1, "INTERNAL: invalid transaction return-status 'MQ_TRANS_START'");
  }

  // 6. check returnCode
  switch (pReadGetReturnCode (msgque->read)) {
    case MQ_RETURN_OK:
      return MQ_OK;
    case MQ_RETURN_ERROR: {
      struct MqReadS * const read = msgque->read;
      struct MqErrorS * const error = msgque->error;
      MQ_STR msg;
      MqDLog(msgque,5,"%s\n","got ERROR return code");
      MqErrorCheck(pRead_RET_START (read));
    // write my HEADER
      MqErrorV (error, "service-call-error", pReadGetReturnNum (read), 
				"<Tok|%s> <Num|%i>\n", token, pReadGetReturnNum (read));
    // write ERROR-STACK
      while (MqReadItemExists (read)) {
	MqErrorCheck (MqReadC (read, &msg));
	pErrorAppendC (error, msg);
      }
      pRead_RET_END (read);
      return MQ_ERROR;
    }
  }

error:
  pTransPush (trans, transH);
  return MqErrorStack (MQ_ERROR_S);
}

enum MqErrorE
MqSendEND_AND_CALLBACK (
  struct MqSendS * const send,
  MQ_STRB const * const token,
  MqTokenF const proc,
  MQ_PTR data,
  MqTokenDataFreeF datafreeF
)
{
  MQ_HDL transH = pTransPop (send->msgque->trans, proc, data, datafreeF);
  MqErrorReturn (pMqSendEND (send, token, transH)); 
}

enum MqErrorE
pSendSYSTEM_RETR (
  struct MqSendS * const send
)
{
  MqErrorReturn (pMqSendEND (send, "_SRT", send->msgque->_trans));
}

MQ_INLINE enum MqErrorE
MqSendFTR (
  struct MqSendS * const send,
  MQ_TIME_T const timeout
)
{
  MqErrorReturn (MqSendEND_AND_WAIT (send, "_FTR", timeout));
}

MQ_INLINE enum MqErrorE
MqSendEOF (
  struct MqSendS * const send,
  MQ_TIME_T const timeout
)
{
  MqSendSTART (send);
  MqErrorReturn (MqSendEND_AND_WAIT (send, "_EOF", timeout));
}

/*****************************************************************************/
/*                                                                           */
/*                              send_composeM                                */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE void 
MqSendSTARTM (
  struct MqS const * const msgque
)
{
  MqSendSTART (msgque->send);
}

MQ_INLINE enum MqErrorE
MqSendENDM (
  struct MqS const * const msgque,
  MQ_STRB const * const token
)
{
  return pMqSendEND (msgque->send, token, 0);
}

MQ_INLINE enum MqErrorE
MqSendEND_AND_WAITM (
  struct MqS const * const msgque,
  MQ_STRB const * const token,
  MQ_TIME_T const timeout
)
{
  return MqSendEND_AND_WAIT (msgque->send, token, timeout);
}

MQ_INLINE enum MqErrorE
MqSendEND_AND_CALLBACKM (
  struct MqS const * const msgque,
  MQ_STRB const * const token,
  MqTokenF const proc,
  MQ_PTR data,
  MqTokenDataFreeF datafreeF
)
{
  return MqSendEND_AND_CALLBACK (msgque->send, token, proc, data, datafreeF);
}

MQ_INLINE enum MqErrorE
MqSendFTRM (
  struct MqS const * const msgque,
  MQ_TIME_T const timeout
)
{
  return MqSendEND_AND_WAIT (msgque->send, "_FTR", timeout);
}

MQ_INLINE enum MqErrorE
MqSendEOFM (
  struct MqS const * const msgque,
  MQ_TIME_T const timeout
)
{
  struct MqSendS * const send = msgque->send;
  MqSendSTART (send);
  return MqSendEND_AND_WAIT (send, "_EOF", timeout);
}

MQ_INLINE enum MqErrorE
MqSendERRORM (
  struct MqS const * const msgque
)
{
  return MqSendERROR(msgque->send);
}

MQ_INLINE enum MqErrorE
MqSendRETURNM (
  struct MqS const * const msgque
)
{
  return MqSendRETURN(msgque->send);
}

/*****************************************************************************/
/*                                                                           */
/*                              send_LST                                     */
/*                                                                           */
/*****************************************************************************/

void
MqSendL_START (
  struct MqSendS * const send
)
{
  register struct MqBufferS * buf = send->buf;
  register struct SendSaveS * save;

  pBufferAddSize (buf, (BUFFER_P2_PRENUM + HDR_INT_LEN));
// MK1
  buf->cursize += BUFFER_P2_PRENUM;
  buf->cur.B += BUFFER_P2_PRENUM;

  // zustand sichern !
  save = (struct SendSaveS*) pCachePop (send->cache);
  save->save = send->save;
  save->numItems = buf->numItems;
  save->cursize = buf->cursize;

  send->save = save;

  // Init
  buf->numItems = 0;
  buf->cur.B += HDR_INT_LEN;
  buf->cursize += HDR_INT_LEN;
}

void
MqSendL_END (
  register struct MqSendS * const send
)
{
  struct SendSaveS * const save = send->save;
  struct MqS * const msgque = send->msgque;
  struct MqBufferS * const buf = send->buf;
  MQ_INT bodyLen;
  MQ_BIN pos;
  MQ_BOL string = msgque->config.isString;
  
  if (unlikely(save == NULL)) return;

  // set numItems
  bodyLen = (MQ_INT) (buf->cursize - save->cursize);
  pos = buf->data + save->cursize;
  SEND_I (string, pos, buf->numItems);

  // restore "save" 
  send->save = save->save;
  buf->numItems = save->numItems;
  buf->cursize = save->cursize;
  buf->cur.B = buf->data + buf->cursize;

  // master setzen
  sSendLen (send, bodyLen, MQ_LSTT, string);

  // free memory /ATTENTION send->save will be set ABOVE
  pCachePush (send->cache, save);
}

/*****************************************************************************/
/*                                                                           */
/*                              send_LST_M                                   */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE void
MqSendL_STARTM (
  struct MqS const * const msgque
)
{
  MqSendL_START(msgque->send);
}

MQ_INLINE void
MqSendL_ENDM (
  struct MqS const * const msgque
)
{
  MqSendL_END(msgque->send);
}

/*****************************************************************************/
/*                                                                           */
/*                              send_RET                                     */
/*                                                                           */
/*****************************************************************************/

static void
sSend_RET_START (
  struct MqSendS * const send,
  MQ_INT const num,
  enum MqReturnE const code
)
{
  register struct MqBufferS * const buf = send->buf;
  register struct SendSaveS * save;
  register MQ_BIN ist;

  // reset body
  MqSendSTART (send);

  pBufferAddSize (buf, BUFFER_P2_PRENUM + RET_SIZE);
  buf->cursize += BUFFER_P2_PRENUM;
  buf->cur.B += BUFFER_P2_PRENUM;

  // save the state !
  save = (struct SendSaveS*) pCachePop (send->cache);
  save->save = send->save;
  save->numItems = buf->numItems;
  save->cursize = buf->cursize;
  send->save = save;

  // init
  buf->numItems = 0;

  // set the Return-Code
  *(buf->data+HDR_Code_S) = (char) code;

#ifdef MQ_SAVE
  strcpy (ist, "xxxxxxxxxx" BUFFER_CHAR_S);
#endif

  // set the Return-Num
  ist = buf->cur.B;
  SEND_I (send->msgque->config.isString, ist, num);
  *ist++ = BUFFER_CHAR;
  ist += HDR_INT_LEN;

  // the new total length
  buf->cursize = (ist - buf->data);
  buf->cur.B = ist;
}

static void
sSend_RET_END (
  register struct MqSendS * const send
)
{
  register struct MqS * const msgque = send->msgque;
  struct SendSaveS * const save = send->save;
  register struct MqBufferS * const buf = send->buf;
  register MQ_BIN ist = (buf->data + save->cursize + RET_NumItems_S);
  MQ_BOL string = msgque->config.isString;
  MQ_INT bodyLen = (MQ_INT) (buf->cursize - save->cursize);

  // set numItems

#ifdef MQ_SAVE
  strcpy (ist, "xxxxxxxxxx");
#endif

  SEND_I (string, ist, buf->numItems);
  *ist = BUFFER_CHAR;

  // restore "save"
  send->save = save->save;
  buf->numItems = save->numItems;
  buf->cursize = save->cursize;
  buf->cur.B = buf->data + buf->cursize;

  // master setzen
  sSendLen (send, bodyLen, MQ_RETT, string);

  // free memory
  pCachePush (send->cache, save);
}

enum MqErrorE
MqSendERROR (
  struct MqSendS * const send
)
{
  MqSendSTART (send);
  MqSendI (send, MqErrorGetNum (MQ_ERROR_S));
  MqSendC (send, MqErrorGetText (MQ_ERROR_S));
  MqErrorReset (MQ_ERROR_S);
  return MqSendEND (send, "_ERR");
}

enum MqErrorE
MqSendRETURN (
  struct MqSendS * const send
)
{
  register struct MqS * msgque = send->msgque;
  if (msgque->_trans == 0) {
    MqErrorDb (MQ_ERROR_RETURN_WITHOUT_TRANSACTION);
    return MqSendERROR (send);
  }
  switch (MqErrorGetCode (msgque->error)) {
    case MQ_OK: 
      break;
    case MQ_ERROR:
      sSend_RET_START (send, MqErrorGetNum (msgque->error), MQ_RETURN_ERROR);
      MqSendC (send, MqErrorGetText (msgque->error));
      sSend_RET_END (send);
      MqErrorReset (msgque->error);
      break;
    case MQ_EXIT:
      return MQ_EXIT;
    case MQ_CONTINUE:
      MqPanicSYS (msgque->error);
  }
  return pMqSendEND (send, "_RET", msgque->_trans);
}




