/**
 *  \file       src/sys.h
 *  \brief      \$Id: sys.h 1177 2009-05-14 14:42:43Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1177 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef SYS_H
#   define SYS_H

#   include <sys/stat.h>

#if !defined(_MSC_VER)
#   include <sys/time.h>
#endif

/** \defgroup sys_api INTERNAL: Unix/Linux/Windows API
 *  \{
 *  \brief system API layer
 *
 * All functions provided by the underlying OS and system libraries
 * are located in this file. Every external Function has additional
 * code to map OS/system specific error codes/messages into
 * msgque standard error codes/massages
 */

struct stat;

/*****************************************************************************/
/*                                                                           */
/*                                 sys_init                                  */
/*                                                                           */
/*****************************************************************************/

/*
typedef void (
  MQ_DECL *AtExitP
) (
  void
);
*/

/*****************************************************************************/
/*                                                                           */
/*                                sys_env                                    */
/*                                                                           */
/*****************************************************************************/

/** -brief secure reading of an environment variable
 *
 * Environment variables are out of control by the programmer and
 * are sources of error. the following code is used to secure read,
 * write and delete environment variables and integrate the error
 * cases into the msgque error-handling.
 *
 * \param error the \ref MqErrorS object in duty
 * \param name the name of the environment variable (z.B. \c HOME)
 * \param ret point to the resulting buffer space used to store the data of the environment variable
 * \return the error ststus
 */
MQ_INLINE enum MqErrorE SysGetEnv (
  struct MqErrorS * const error,
  const char *name,
  MQ_STR *ret
);

MQ_INLINE enum MqErrorE SysSetEnv (
  struct MqErrorS * const error,
  const char *name,
  const char *str
);

MQ_INLINE enum MqErrorE SysUnSetEnv (
  struct MqErrorS * const error,
  const char *name
);

/*****************************************************************************/
/*                                                                           */
/*                                sys_file                                   */
/*****************************************************************************/

/*
MQ_INLINE enum MqErrorE SysOpen (
  struct MqErrorS * const error,
  MQ_STR pathname,
  int flags,
  mode_t mode,
  MQ_INT *fh
);

MQ_INLINE enum MqErrorE SysStat (
  struct MqErrorS * const error,
  const MQ_STR file,
  struct stat *st
);
*/

/*****************************************************************************/
/*                                                                           */
/*                                sys_io                                     */
/*                                                                           */
/*****************************************************************************/

/*
enum MqErrorE SysRead (
  struct MqErrorS * const error,
  MQ_INT hdl,
  MQ_BIN buf,
  const MQ_SIZE numBytes
);

enum MqErrorE SysWrite (
  struct MqErrorS * const error,
  MQ_INT hdl,
  MQ_BIN buf,
  const MQ_SIZE numBytes
);

enum MqErrorE SysClose (
  struct MqErrorS * const error,
  MQ_INT *hdl
);

*/

#if defined(MQ_IS_POSIX)
MQ_INLINE enum MqErrorE SysUnlink (
  struct MqErrorS * const error,
  const MQ_STR fileName
);
#endif

/*****************************************************************************/
/*                                                                           */
/*                                sys_misc                                   */
/*                                                                           */
/*****************************************************************************/

/*
enum MqErrorE SysAtExit (
  struct MqErrorS * const error,
  AtExitP proc
);
*/

void SysExit (
  int isThread,
  int num
) __attribute__ ((noreturn));

void SysAbort (
) __attribute__ ((noreturn));

#if defined(HAVE_FORK)
enum MqErrorE SysServerFork (
  struct MqErrorS * const error,
  struct MqConfigS * config,
  struct MqBufferLS * const argv,
  struct MqBufferLS * const alfa,
  const MqCreateF fMain,
  const char * name,
  struct MqIdS * idP
);
#endif

#if defined(MQ_HAS_THREAD)
enum MqErrorE SysServerThread (
  struct MqErrorS * const error,
  struct MqConfigS * config,
  struct MqBufferLS * const argv,
  struct MqBufferLS * const alfa,
  const MqCreateF fMain,
  const char * name,
  int state,
  struct MqIdS * idP
);
#endif

enum MqErrorE SysServerSpawn (
  struct MqErrorS * const error,
  char * * argv,
  const char * name,
  struct MqIdS * idP
);

#ifdef HAVE_FORK
enum MqErrorE SysFork (
  struct MqErrorS * const error,
  struct MqIdS * idP
);
#endif

#ifdef MQ_IS_POSIX
enum MqErrorE
SysIgnorSIGCHLD (
  struct MqErrorS * const error
);

enum MqErrorE
SysAllowSIGCHLD (
  struct MqErrorS * const error
);

enum MqErrorE
SysDaemonize (
  struct MqErrorS * const error,
  const MQ_STR pidfile
);
#else
#define SysIgnorSIGCHLD(error) MQ_OK
#define SysAllowSIGCHLD(error) MQ_OK
#define SysDaemonize(error,pidfile) MQ_OK
#endif /* MQ_IS_POSIX */

MQ_INLINE enum MqErrorE
SysUnLink (
  struct MqErrorS * const error,
  const MQ_STR filename
);

MQ_INLINE void SysFreeArgvArray (
  char *** argvP
);

/** \} */

#endif /* SYS_H */




