/**
 *  \file       src/sys_com.h
 *  \brief      \$Id: sys_com.h 1176 2009-05-14 13:57:36Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1176 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef SYS_COM_H
#define SYS_COM_H

#include "error.h"
#include <fcntl.h>

#if defined(MQ_IS_POSIX)
#   include <sys/socket.h>
#   include <netdb.h>
#   include <netinet/in.h>
#   include <arpa/inet.h>
#elif defined(MQ_IS_WIN32)
//  we require WindowsXP or higher -> used for "getaddrinfo"
#		if !defined(_MSC_VER)
#			define WINVER WindowsXP
#			include <w32api.h>
#		endif
#   include <winsock2.h>
#   include <Ws2tcpip.h>
#endif

struct sockaddr;

#ifdef MQ_IS_WIN32
# define MQ_sockopt_optval_T char*
# define MQ_buf_T char*
#else
# define MQ_sockopt_optval_T void*
# define MQ_buf_T void*
#endif


/*****************************************************************************/
/*                                                                           */
/*                                sys_com                                    */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE enum MqErrorE SysAccept (
  struct MqErrorS * const error,        /* handle error */
  MQ_SOCK server,               /* the server handle */
  struct sockaddr *addr,        /* addr */
  socklen_t * addrlen,         /* addrlen */
  MQ_SOCK * client              /* OUT the client handle */
);

MQ_INLINE enum MqErrorE SysGetSockOpt (
  struct MqErrorS * const error,
  MQ_SOCK const socket,
  int const level,
  int const optname,
  MQ_sockopt_optval_T optval,
  socklen_t * const optlen
);

MQ_INLINE enum MqErrorE SysSetSockOpt (
  struct MqErrorS * const error,
  MQ_SOCK const socket,
  int const level,
  int const optname,
  const MQ_sockopt_optval_T const optval,
  socklen_t const optlen
);

// MQ_INLINE enum MqErrorE
// SysSetSockDebug(struct MqErrorS *  const error, MQ_SOCK socket);

enum MqErrorE SysConnect (
  struct MqErrorS * const error,
  MQ_SOCK const socket,
  struct sockaddr * const serv_addr,
  socklen_t const addrlen,
  MQ_TIME_T const timeout
);

MQ_INLINE enum MqErrorE SysSocket (
  struct MqErrorS * const error,
  int domain,
  int type,
  int protocol,
  MQ_SOCK * sock
);

enum MqErrorE SysSocketPair (
  struct MqErrorS * const error,
  int socks[]
);

MQ_INLINE enum MqErrorE SysSetAsync (
  struct MqErrorS * const error,
  MQ_SOCK socket
);

#if defined(MQ_IS_WIN32)

enum MqErrorE SysWSAStartup (
  struct MqErrorS * const error
);

enum MqErrorE SysWSACleanup (
  struct MqErrorS * const error
);

#endif

MQ_INLINE enum MqErrorE SysBind (
  struct MqErrorS * const error,
  MQ_SOCK socket,
  const struct sockaddr *my_addr,
  const socklen_t addrlen
);

MQ_INLINE enum MqErrorE SysSelect (
  struct MqErrorS * const error,
  int n,
  fd_set * readfds,
  fd_set * writefds,
  struct timeval const * const timeout
);

MQ_INLINE enum MqErrorE SysListen (
  struct MqErrorS * const error,
  MQ_SOCK socket,
  int backlog
);

MQ_INLINE enum MqErrorE SysGetSockName (
  struct MqErrorS * const error,        /* handle error */
  MQ_SOCK socket,               /* the socket */
  struct sockaddr *sockaddr,    /* the address package */
  socklen_t * sockaddrlen      /* the length of the address */
);

MQ_INLINE enum MqErrorE SysGetPeerName (
  struct MqErrorS * const error,        /* handle error */
  MQ_SOCK socket,               /* the socket */
  struct sockaddr *sockaddr,    /* the address package */
  socklen_t * sockaddrlen      /* the length of the address */
);

/*
MQ_INLINE enum MqErrorE SysShutdown (
  struct MqErrorS * const error,
  MQ_SOCK hdl
);
*/

MQ_INLINE enum MqErrorE SysCloseSocket (
  struct MqErrorS * const error,
  MQ_BOL doShutdown,
  MQ_SOCK * hdl
);

enum MqErrorE SysSend (
  struct MqErrorS * const error,    
  MQ_SOCK hdl,			    
  MQ_BIN buf,			    
  MQ_SIZE numBytes,		    
  int flags,			    
  MQ_TIME_T timeout		    
);

enum MqErrorE SysRecv (
  struct MqErrorS * const error,    
  MQ_SOCK const hdl,		    
  MQ_BIN buf,			    
  MQ_SIZE numBytes,		    
  MQ_SIZE * const newSize,		    
  MQ_TIME_T timeout		    
);

/*****************************************************************************/
/*                                                                           */
/*                            address information                            */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE void
SysFreeAddrInfo(
    struct addrinfo **res
);

MQ_INLINE enum MqErrorE
SysGetAddrInfo(
    struct MqErrorS * const error,
    const char *node,
    const char *service,
    const struct addrinfo *hints,
    struct addrinfo **res
);

enum MqErrorE
SysGetTcpInfo(
  struct MqErrorS * const error,
  struct addrinfo const * const addrinfo,
  const char **host,
  MQ_INT * port	
);

/*****************************************************************************/
/*                                                                           */
/*                         addational header files                           */
/*                                                                           */
/*****************************************************************************/


#endif /* SYS_H */




