/**
 *  \file       tclmsgque/msgque_tcl.h
 *  \brief      \$Id: msgque_tcl.h 1178 2009-05-15 07:25:57Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1178 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#ifndef MSGQUE_TCL_H
#define MSGQUE_TCL_H

#include "config.h"
#include "msgque.h"

#include "debug.h"

#include <tcl.h>

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>

/*****************************************************************************/
/*                                                                           */
/*                              context_basic                                */
/*                                                                           */
/*****************************************************************************/

#define NS(n)    Tclmsgque_ ## n
#define HD(name) int NS(name) ( Tcl_Interp * interp, struct TclContextS * const context, \
				    int skip, int objc, struct Tcl_Obj *const *objv)

#define SETUP_msgque struct MqS * const msgque = context->msgque;
#define SETUP_context struct TclContextS *context = ((struct TclContextS *)msgque->config.myCtx);
#define SETUP_read struct MqReadS * const read = msgque->read;

#define ErrorMqToTcl() NS(pErrorFromMq)(context->interp, __func__, __FILE__, __LINE__, msgque)
#define ErrorMqToTclWithCheck(PROC) \
  if (unlikely((PROC) == MQ_ERROR)) { \
    ErrorMqToTcl(); \
    goto error; \
  }
#define RETURN_TCL \
  return TCL_OK; \
error: \
  return TCL_ERROR;

#define TclErrorCheck(cmd) if (cmd != TCL_OK) return TCL_ERROR
#define TclErrorCheck2(cmd) if (cmd != TCL_OK) goto error
#define TclErrorCheck3(cmd) if (cmd != TCL_OK) goto error1

/// \brief T)hread L)ocal S)torage identifer (set by "configure")
#if defined(TLS)
#  define MQ_TLS TLS
#else
#  define MQ_TLS
#endif

/*****************************************************************************/
/*                                                                           */
/*                        argument check functions                           */
/*                                                                           */
/*****************************************************************************/

#define CHECK_NOARGS \
  if (skip != objc) {\
    Tcl_WrongNumArgs(interp, skip, objv, "");\
    goto error;\
  }

#define CHECK_OBJ(val) \
  if (skip >= objc) {\
    Tcl_WrongNumArgs(interp, skip, objv, #val " ...");\
    goto error;\
  } else { \
    val = objv[skip]; \
    skip++; \
  }

#define CHECK_OPTIONAL_OBJ(val) \
  if (skip >= objc) {\
    val = NULL; \
  } else { \
    val = objv[skip]; \
    skip++; \
  }

#define CHECK_PROC(val, err) \
  if (skip >= objc) {\
    Tcl_WrongNumArgs(interp, skip, objv, #val " ...");\
    goto error;\
  } else { \
    TclErrorCheck2(NS(ProcCheck)(interp,objv[skip], err)); \
    val = objv[skip]; \
    skip++; \
  }

#define CHECK_MSGQUE(val) \
  if (skip >= objc || !Tcl_GetCommandFromObj(interp, objv[skip])) {\
    Tcl_WrongNumArgs(interp, skip, objv, #val " ...");\
    goto error;\
  } else { \
    MQ_PTR context = NULL; \
    TclErrorCheck2(NS(GetClientData) (interp, objv[skip], &context)); \
    val = ((struct TclContextS *) context)->msgque; \
    skip++; \
  }

#define CHECK_C(val) \
  if (skip >= objc) {\
    Tcl_WrongNumArgs(interp, skip, objv, #val " ...");\
    goto error;\
  } else { \
    val = Tcl_GetString(objv[skip]); \
    skip++; \
  }

#define CHECK_B(val,len) \
  if (skip >= objc) {\
    Tcl_WrongNumArgs(interp, skip, objv, #val " ...");\
    goto error;\
  } else { \
    val = Tcl_GetByteArrayFromObj(objv[skip], &len); \
    skip++; \
  }

#define CHECK_TMPL(val,type,ttype,func) \
  if (skip >= objc) {\
    Tcl_WrongNumArgs(interp, skip, objv, #val " ...");\
    goto error;\
  } else { \
    ttype tmp; \
    TclErrorCheck2(func(interp, objv[skip], &tmp)); \
    val = (type) tmp; \
    skip++; \
  }

#define CHECK_DEFTMPL(val,type,ttype,func) \
  if (skip < objc) {\
    ttype tmp; \
    TclErrorCheck2(func(interp, objv[skip], &tmp)); \
    val = (type) tmp; \
    skip++; \
  }

#define CHECK_Y(val) CHECK_TMPL(val,MQ_BYT,int,Tcl_GetIntFromObj)
#define CHECK_O(val) CHECK_TMPL(val,MQ_BOL,int,Tcl_GetBooleanFromObj)
#define CHECK_S(val) CHECK_TMPL(val,MQ_SRT,int,Tcl_GetIntFromObj)
#define CHECK_I(val) CHECK_TMPL(val,MQ_INT,int,Tcl_GetIntFromObj)
#define CHECK_F(val) CHECK_TMPL(val,MQ_FLT,double,Tcl_GetDoubleFromObj)
#define CHECK_W(val) CHECK_TMPL(val,MQ_WID,Tcl_WideInt,Tcl_GetWideIntFromObj)
#define CHECK_D(val) CHECK_TMPL(val,MQ_DBL,double,Tcl_GetDoubleFromObj)
#define CHECK_U(val) CHECK_TMPL(val,MQ_BUF,long,Tcl_GetLongFromObj)

#define CHECK_DI(val) CHECK_DEFTMPL(val,MQ_INT,int,Tcl_GetIntFromObj)

/*****************************************************************************/
/*                                                                           */
/*                                  types                                    */
/*                                                                           */
/*****************************************************************************/

/** \brief object to link the \libmsgque object with application specific data
 *
 */
struct TclContextS {
  struct MqS * msgque;          ///< link to the \libmsgque object
  Tcl_Obj *self;		///< Tcl command Obj to represent the 'Context'
  Tcl_Command command;          ///< Tcl command to represent the 'Context'
  Tcl_Interp *interp;           ///< Tcl interpreter the command is using
  Tcl_Obj *configProc;		///< Tcl command used to configure a new server-child context
  MQ_BOL onDelete;		///< Is this context allready in deletion?
  MQ_INT callLevel;		///< code evaluated by "ProcCall" ?
};

/*****************************************************************************/
/*                                                                           */
/*                                 functions                                 */
/*                                                                           */
/*****************************************************************************/

// from msgque_tcl.h

MQ_STR NS(printName) ( Tcl_Obj *);
MQ_STR NS(printObj)  ( const char *, Tcl_Obj *);

// from context_tcl.c

int   NS(Context_Init)	    ( Tcl_Interp * interp, int objc, Tcl_Obj * const objv[]);

// from misc_tcl.c
void  NS(pErrorFromMq)	    ( Tcl_Interp*, const char *, const char *, const int, struct MqS * const);
int   NS(GetClientData)	    ( Tcl_Interp*, Tcl_Obj*, MQ_PTR*);
int   NS(ReadNext)	    ( struct MqReadS* const, Tcl_Obj **, Tcl_Obj **);
int   NS(ReadAll)	    ( Tcl_Interp *, struct MqReadS * const, int, Tcl_Obj **);
int   NS(PrepareList)	    ( struct MqS const * const, Tcl_Obj * const, int const);

int   NS(ProcCheck)	    ( Tcl_Interp *, struct Tcl_Obj *, char const * const);
enum MqErrorE 
      NS(ProcCall)	    ( struct MqS * const, MQ_PTR);
void  NS(ProcFree)	    ( struct MqS const * const, MQ_PTR);


#endif // MSGQUE_TCL_H



