/**
 *  \file       tclmsgque/read_tcl.c
 *  \brief      \$Id: read_tcl.c 1178 2009-05-15 07:25:57Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1178 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

#include "msgque_tcl.h"

HD(ReadL_START)
{
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadL_START(read, NULL));
  RETURN_TCL
}

HD(ReadL_END)
{
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  MqReadL_END(read);
  RETURN_TCL
}

HD(ReadY)
{
  MQ_BYT val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadY(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewIntObj(val));
  RETURN_TCL
}

HD(ReadO)
{
  MQ_BOL val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadO(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewBooleanObj(val));
  RETURN_TCL
}

HD(ReadS)
{
  MQ_SRT val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadS(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewIntObj(val));
  RETURN_TCL
}

HD(ReadI)
{
  MQ_INT val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadI(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewIntObj(val));
  RETURN_TCL
}

HD(ReadF)
{
  MQ_FLT val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadF(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewDoubleObj(val));
  RETURN_TCL
}

HD(ReadW)
{
  MQ_WID val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadW(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewWideIntObj(val));
  RETURN_TCL
}

HD(ReadD)
{
  MQ_DBL val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadD(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewDoubleObj(val));
  RETURN_TCL
}

HD(ReadC)
{
  MQ_STR val;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadC(read, &val));
  Tcl_SetObjResult(context->interp, Tcl_NewStringObj(val,-1));
  RETURN_TCL
}

HD(ReadB)
{
  MQ_BIN val;
  MQ_SIZE len;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadB(read, &val, &len));
  Tcl_SetObjResult(context->interp, Tcl_NewByteArrayObj(val,len));
  RETURN_TCL
}

HD(ReadU)
{
  MQ_BUF buffer;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadU(read, &buffer));
  Tcl_SetObjResult(context->interp, Tcl_NewLongObj((long)buffer));
  RETURN_TCL
}

HD(ReadGetNumItems)
{
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  Tcl_SetObjResult(context->interp, Tcl_NewIntObj(MqReadGetNumItems(read)));
  RETURN_TCL
}

HD(ReadItemExists)
{
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  Tcl_SetObjResult(context->interp, Tcl_NewBooleanObj(MqReadItemExists(read)));
  RETURN_TCL
}

HD(ReadUndo)
{
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  ErrorMqToTclWithCheck(MqReadUndo(read));
  RETURN_TCL
}

HD(ReadALL)
{
  Tcl_Obj *RET = Tcl_NewListObj(0,NULL);
  Tcl_Obj *OBJ;
  MQ_BUF buf;
  SETUP_msgque
  SETUP_read
  CHECK_NOARGS
  while (MqReadItemExists(read)) {
    OBJ = NULL;
    MqReadU(read, &buf);
    switch (buf->type) {
      case MQ_BYTT: {
	OBJ = Tcl_NewIntObj(buf->cur.A->Y);
	break;
      }
      case MQ_BOLT: {
	OBJ = Tcl_NewBooleanObj(buf->cur.A->O);
	break;
      }
      case MQ_SRTT: {
	OBJ = Tcl_NewIntObj(buf->cur.A->S);
	break;
      }
      case MQ_INTT: {
	OBJ = Tcl_NewIntObj(buf->cur.A->I);
	break;
      }
      case MQ_FLTT: {
	OBJ = Tcl_NewDoubleObj(buf->cur.A->F);
	break;
      }
      case MQ_WIDT: {
	OBJ = Tcl_NewWideIntObj(buf->cur.A->W);
	break;
      }
      case MQ_DBLT: {
	OBJ = Tcl_NewDoubleObj(buf->cur.A->D);
	break;
      }
      case MQ_BINT: {
	OBJ = Tcl_NewByteArrayObj(buf->cur.B,buf->cursize);
	break;
      }
      case MQ_STRT: {
	OBJ = Tcl_NewStringObj(buf->cur.C,-1);
	break;
      }
      case MQ_LSTT: {
	MqReadL_START(read, buf);
	NS(ReadALL)(interp, context, skip, objc, objv);
	MqReadL_END(read);
	OBJ = Tcl_GetObjResult(interp);
	break;
      }
      case MQ_RETT: {
	break;
      }
    }
    if (OBJ != NULL) Tcl_ListObjAppendElement(context->interp, RET, OBJ);
  }
  Tcl_SetObjResult(interp, RET);
  RETURN_TCL
}

