/**
 *  \file       tests/Server.java
 *  \brief      \$Id: Server.java 1128 2009-05-06 11:37:30Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1128 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

package example;

import javamsgque.*;
import java.util.List;
import java.util.Arrays;

final class Client extends MqS implements IClient, ICallback {
  public static final int	debug	= 0;
  public static final String	name	= "test-client";

  public int i = 0;

  public void Callback (MqS ctx) {
    i = ctx.ReadI();
  }
  
  public Client (int debug) {
    super("--debug", Integer.toString(debug), "@", "SELF", "--name", "test-server");
  }
  
  public Client (Client parent) {
    super(parent);
  }
  
  public Client () {
    super();
  }
}

final class Server extends MqS implements IServer {

  int myInt;
  private Buffer buf;

  public static final String name = "server";

  public Server(String[] argv) { super(argv); }
  public Server(MqS master, int master_id, String... argv) { super(master, master_id, argv); }
  public Server() { super(); }

  public void ServerConfig() {

    if (ConfigGetIsSlave()) {
      // add "slave" services here
    } else {
      // add "master" services here
      ServiceCreate("ECOI", new ECOI());
      ServiceCreate("ECOS", new ECOS());
      ServiceCreate("ECOY", new ECOY());
      ServiceCreate("ECOO", new ECOO());
      ServiceCreate("ECOW", new ECOW());
      ServiceCreate("ECOD", new ECOD());
      ServiceCreate("ECOF", new ECOF());
      ServiceCreate("ECOU", new ECOU());
      ServiceCreate("ECON", new ECON());
      ServiceCreate("ECOC", new ECOC());
      ServiceCreate("ECOL", new ECOL());
      ServiceCreate("ECUL", new ECUL());
      ServiceCreate("RDUL", new RDUL());
      ServiceCreate("ECOB", new ECOB());
      ServiceCreate("ECOM", new ECOM());

      ServiceCreate("SETU", new SETU());
      ServiceCreate("GETU", new GETU());

      ServiceCreate("ECLI", new ECLI());
      ServiceCreate("MSQT", new MSQT());
      ServiceCreate("ERR2", new ERR2());
      ServiceCreate("ERR3", new ERR3());
      ServiceCreate("SLEP", new SLEP());
      ServiceCreate("USLP", new USLP());
      ServiceCreate("CSV1", new CSV1());
      ServiceCreate("SND1", new SND1());
      ServiceCreate("SND2", new SND2());
      ServiceCreate("REDI", new REDI());
      ServiceCreate("JAVA", new JAVA());
      ServiceCreate("GTCX", new GTCX());
      ServiceCreate("GTTO", new GTTO());
      ServiceCreate("BUF1", new BUF1());
      ServiceCreate("BUF2", new BUF2());
      ServiceCreate("BUF3", new BUF3());
      ServiceCreate("LST1", new LST1());
      ServiceCreate("LST2", new LST2());
      ServiceCreate("INIT", new INIT());
    }
  }

// Start SND1
  private Client[] cl = new Client[3];;

  class SND1 implements IService {
    public void Service (MqS ctx) {
      String s = ReadC();
      int id = ReadI();
      SendSTART();
	if (s.equals("START")) {
	  Server parent = (Server) ConfigGetParent();
	  if (parent == null || parent.cl[id] == null) {
	    cl[id] = new Client(ConfigGetDebug());
	  } else {
	    cl[id] = new Client(parent.cl[id]);
	  }
	} else if (s.equals("STOP")) {
	  cl[id].Delete();
	  cl[id] = null;
	} else if (s.equals("SEND")) {
	  cl[id].SendSTART();
	  String TOK = ReadC();
	  cl[id].SendU(ReadU());
	  cl[id].SendEND(TOK);
	} else if (s.equals("WAIT")) {
	  cl[id].SendSTART();
	  cl[id].SendU(ReadU());
	  cl[id].SendEND_AND_WAIT("ECOI", 5);
	  SendI(cl[id].ReadI()+1);
	} else if (s.equals("CALLBACK")) {
	  cl[id].SendSTART();
	  cl[id].SendU(ReadU());
	  cl[id].i = -1;
	  cl[id].SendEND_AND_CALLBACK("ECOI", cl[id]);
	  cl[id].ProcessEvent(10,MqS.WAIT.ONCE);
	  SendI(cl[id].i+1);
	}
      SendRETURN();
    }
  }

// Start SND2

  class SND2 implements IService, ICallback {
    public void Callback (MqS ctx) {
      Server master = (Server) ctx.ConfigGetMaster();
      master.myInt = ctx.ReadI();
    }
    public void Service (MqS ctx) {
      String s = ReadC();
      int id = ReadI();
      MqS cl = SlaveGet(id);
      SendSTART();
	if (s.equals("CREATE")) {
	  List<String> LIST = Arrays.asList("--name", "test-client", "@", "WORKER", "--name", "test-server");
	  while (ReadItemExists()) 
	    LIST.add(ReadC());
	  new Server(ctx, id, LIST.toArray(new String[0]));
	} else if (s.equals("CREATE2")) {
	  cl = new Client(ConfigGetDebug());
	  SlaveCreate(id, cl);
	} else if (s.equals("DELETE")) {
	  SlaveDelete(id);
	  SendC(SlaveGet(id) == null ? "OK" : "ERROR");
	} else if (s.equals("SEND")) {
	  cl.SendSTART();
	  String TOK = ReadC();
	  cl.SendU(ReadU());
	  cl.SendEND(TOK);
	} else if (s.equals("WAIT")) {
	  cl.SendSTART();
	  cl.SendU(ReadU());
	  cl.SendEND_AND_WAIT("ECOI", 5);
	  SendI(cl.ReadI()+1);
	} else if (s.equals("CALLBACK")) {
	  cl.SendSTART();
	  cl.SendU(ReadU());
	  myInt = -1;
	  cl.SendEND_AND_CALLBACK("ECOI", this);
	  cl.ProcessEvent(10,MqS.WAIT.ONCE);
	  SendI(myInt+1);
	} else if (s.equals("MqSendEND_AND_WAIT")) {
	  String TOK = ReadC();
	  cl.SendSTART();
	  while (ReadItemExists())
	    cl.SendU(ReadU());
	  cl.SendEND_AND_WAIT(TOK, 5);
	  while (cl.ReadItemExists())
	    SendU(cl.ReadU());
	} else if (s.equals("MqSendEND")) {
	  String TOK = ReadC();
	  cl.SendSTART();
	  while (ReadItemExists())
	    cl.SendU(ReadU());
	  cl.SendEND(TOK);
	  return;
	} else if (s.equals("ERR-1")) {
	  new Server(ctx, id, "--name", "test-client", "@", "DUMMY");
	} else if (s.equals("isSlave")) {
	  SendO(cl.ConfigGetIsSlave());
	}
      SendRETURN();
    }
  }

  class INIT implements IService {
    public void Service (MqS ctx) {
      int max = ctx.ReadGetNumItems();
      String[] list = new String[max];
      ctx.SendSTART();
      for (int i=0; i<max; i++) {
	list[i] = ctx.ReadC();
      }
      MqS.Init(list);
      ctx.SendRETURN();
    }
  }

  class SETU implements IService {
    public void Service (MqS ctx) {
      buf = ctx.ReadU();
    }
  }

  class GETU implements IService {
    public void Service (MqS ctx) {
      ctx.SendSTART();
      ctx.SendU(buf);
      ctx.SendRETURN();
    }
  }

  public static void main(String[] argv) {
    MqS.Init("java", "example.Server");
    Server srv = new Server(argv);
    srv.ProcessEvent(MqS.WAIT.FOREVER);
    srv.Delete();
  }
}

/////////////////////////////////////////////////////////
///
///  misc other service-classes
///
////////////////////////////////////////////////////////

class GTCX implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendI(ctx.ConfigGetCtxId());
    ctx.SendRETURN();
  }
}

class GTTO implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendC(ctx.ConfigGetToken());
    ctx.SendRETURN();
  }
}

class REDI implements IService {
  int i;
  public void Service (MqS ctx) {
    i = ctx.ReadI();
  }
}

class JAVA implements IService {
  public void Service (MqS ctx) {
    String s = ctx.ReadC();
    ctx.SendSTART();
      if (s.equals("NullPointerException-1")) {
	throw new NullPointerException("myMessage");
      } else if (s.equals("NullPointerException-2")) {
	throw new NullPointerException();
      } else {
	ctx.SendC("nothig to do!!");
      }
    ctx.SendRETURN();
  }
}

class ECOM implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    while (ctx.ReadItemExists()) {
      ctx.SendU(ctx.ReadU());
    }
    ctx.SendRETURN();
  }
}

class CSV1 implements IService {
  public void Service (MqS ctx) {
    int num;

    // call an other service
    ctx.SendSTART ();
    num = ctx.ReadI();
    num++;
    ctx.SendI (num);
    ctx.SendEND_AND_WAIT ("CSV2", 10);

    // read the answer and send the result back
    ctx.SendSTART ();
    num = ctx.ReadI();
    num++;
    ctx.SendI (num);
    ctx.SendRETURN ();
  }
}

class SLEP implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    int i = ctx.ReadI();
    try {
      Thread.sleep(i*1000);
    } catch (Exception ex) {
      // do nothing
    } finally {
      ctx.SendI(i);
      ctx.SendRETURN();
    }
  }
}

class USLP implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    int i = ctx.ReadI();
    try {
      Thread.sleep(i/1000);
    } catch (Exception ex) {
      // do nothing
    } finally {
      ctx.SendI(i);
      ctx.SendRETURN();
    }
  }
}

class ERR2 implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendC("some data");
    ctx.ErrorC("Ot_ERR2", 10, "some error");
    ctx.SendRETURN();
  }
}

class ERR3 implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendRETURN();
    ctx.SendRETURN();
  }
}

// START-JAVA-SERVICE-PROC
class ECOI implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendI(ctx.ReadI());
    ctx.SendRETURN();
  }
}
// END-JAVA-SERVICE-PROC

class ECOS implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendS(ctx.ReadS());
    ctx.SendRETURN();
  }
}

class ECOY implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendY(ctx.ReadY());
    ctx.SendRETURN();
  }
}

class ECOO implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendO(ctx.ReadO());
    ctx.SendRETURN();
  }
}

class ECOW implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendW(ctx.ReadW());
    ctx.SendRETURN();
  }
}

class ECOD implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendD(ctx.ReadD());
    ctx.SendRETURN();
  }
}

class ECOF implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendF(ctx.ReadF());
    ctx.SendRETURN();
  }
}

class ECOB implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendB(ctx.ReadB());
    ctx.SendRETURN();
  }
}

class ECOU implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendU(ctx.ReadU());
    ctx.SendRETURN();
  }
}

class ECON implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendC(ctx.ReadC() + "-" + ctx.ConfigGetName());
    ctx.SendRETURN();
  }
}

class ECLI implements IService {

  private void EchoList(MqS ctx, boolean doincr) {
    Buffer buf;
    while (ctx.ReadGetNumItems() != 0) {
      buf = ctx.ReadU();
      if (ctx.GetType(buf) == 'L') {
	ctx.GetL_START (buf);
	ctx.SendL_START ();
	EchoList (ctx, doincr);
	ctx.SendL_END ();
	ctx.GetL_END ();
      } else if (doincr) {
	ctx.SendI (ctx.GetI(buf)+1);
      } else {
	ctx.SendU (buf);
      }
    }
  }

  public void Service (MqS ctx) {
    Buffer opt = ctx.ReadU();
    boolean doincr = (ctx.GetType(opt) == 'C' && ctx.GetC(opt).equals("--incr"));
    if (!doincr) ctx.ReadUndo();
    ctx.SendSTART();
    EchoList(ctx, doincr);
    ctx.SendRETURN();
  }
}

class ECOL implements IService {

  private void EchoList(MqS ctx) {
    Buffer buf;
    while (ctx.ReadItemExists()) {
      buf = ctx.ReadU();
      if (ctx.GetType(buf) == 'L') {
	ctx.GetL_START (buf);
	ctx.SendL_START ();
	EchoList (ctx);
	ctx.SendL_END ();
	ctx.GetL_END ();
      } else {
	ctx.SendU (buf);
      }
    }
  }

  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.ReadL_START ();
    ctx.SendL_START ();
    EchoList(ctx);
    ctx.SendL_END ();
    ctx.ReadL_END ();
    ctx.SendRETURN();
  }
}

class ECUL implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendI(ctx.ReadI());
    ctx.SendW(ctx.ReadW());
    ctx.SendD(ctx.ReadD());
    ctx.SendU(ctx.ReadU());
    ctx.SendRETURN();
  }
}

class RDUL implements IService {
  public void Service (MqS ctx) {
    ctx.ReadI();
    ctx.ReadW();
    ctx.ReadD();
    ctx.ReadU();
  }
}

class ECOC implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendC(ctx.ReadC());
    ctx.SendRETURN();
  }
}

class MSQT implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    if (ctx.ConfigGetDebug() != 0) {
      ctx.SendC ("debug");
      ctx.SendI (ctx.ConfigGetDebug());
    }
    if (ctx.ConfigGetIsString() == false)
      ctx.SendC ("binary");
    if (ctx.ConfigGetIsSilent() == true)
      ctx.SendC ("silent");
    ctx.SendC ("sOc");
    ctx.SendC (ctx.ConfigGetIsServer() ? "SERVER" : "CLIENT");
    ctx.SendC ("pOc");
    ctx.SendC (ctx.ConfigGetIsParent() ? "PARENT" : "CHILD");
    ctx.SendRETURN();
  }
}

class BUF1 implements IService {
  public void Service (MqS ctx) {
    Buffer buf = ctx.ReadU();
    char typ = ctx.GetType(buf);
    ctx.SendSTART();
    ctx.SendC(Character.toString(typ));
    switch (typ) {
      case 'Y': ctx.SendY(ctx.GetY(buf)); break;
      case 'O': ctx.SendO(ctx.GetO(buf)); break;
      case 'S': ctx.SendS(ctx.GetS(buf)); break;
      case 'I': ctx.SendI(ctx.GetI(buf)); break;
      case 'F': ctx.SendF(ctx.GetF(buf)); break;
      case 'W': ctx.SendW(ctx.GetW(buf)); break;
      case 'D': ctx.SendD(ctx.GetD(buf)); break;
      case 'C': ctx.SendC(ctx.GetC(buf)); break;
      case 'B': ctx.SendB(ctx.GetB(buf)); break;
    }
    ctx.SendRETURN();
  }
}

class BUF2 implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();

    for (int i=0; i<3; i++) {
      Buffer buf = ctx.ReadU();
      ctx.SendC(Character.toString(ctx.GetType(buf)));
      ctx.SendU(buf);
    }

    ctx.SendRETURN();
  }
}

class BUF3 implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    Buffer buf = ctx.ReadU();
    ctx.SendC(Character.toString(ctx.GetType(buf)));
    ctx.SendU(buf);
    ctx.SendI(ctx.ReadI());
    ctx.SendU(buf);
    ctx.SendRETURN();
  }
}

class LST1 implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.SendL_END();
    ctx.SendRETURN();
  }
}

class LST2 implements IService {
  public void Service (MqS ctx) {
    ctx.SendSTART();
    ctx.ReadL_END();
    ctx.SendRETURN();
  }
}

