#+
#§  \file       tests/library.tcl
#§  \brief      \$Id: library.tcl 1205 2009-05-20 10:51:09Z dev1usr $
#§  
#§  (C) 2002 - LibMsgque - Project - Group
#§  
#§  \version    \$Rev: 1205 $
#§  \author     EMail: aotto1968 at users.sourceforge.net
#§  \attention  this software has GPL permissions to copy
#§              please contact AUTHORS for additional information
#§

#set tcl_traceExec 1

##
## -----------------------------------------------------------------------
## test environment setup
## 

if {[file exists [file join .. env.tcl]]} {
  source [file join .. env.tcl]
} else {
  error {unable to find the 'env.tcl' file}
}

## C# using different double format depending on
## system language -> we test only the default language
set env(LANG) C

## we use the "eval" replacement "{*}"
package require Tcl 8.5

## switch to test directory
## test if the start was allready in test directory "make check"
set testdir [file normalize [file dirname [info script]]]
if {[file tail [pwd]] ne "tests"} {
  ## check if I am in the parent of tests directory
    if {[file isdirectory [file join . tests]]} {
	cd [file join . tests]
    } else {
	cd [file dirname [info script]]
    }
}

## helper (for debugging)
proc Print {args} {
  if {[info level] > 1} {
    set STR "[lindex [info level -1] 0] -> "
  } else {
    set STR "GLOBAL -> "
  }
  foreach a $args {
    append STR "$a<[uplevel set $a]>, "
  }
  puts $STR
}

proc Error {args} {
  puts stderr "ERROR: $args"
  exit 1
}

## setup the path to find the !newly! created executable first
lappend PATH [file nativename [file dirname [info nameofexecutable]]]
lappend PATH [file nativename [file normalize [file join .. javamsgque .libs]]]
lappend PATH [file nativename [file normalize [file join .. javamsgque]]]
lappend PATH [file nativename [file normalize [file join .. csmsgque]]]
lappend PATH [file nativename [file normalize [file join .. tests .libs]]]
lappend PATH [file nativename [file normalize [file join .. tests]]]
lappend PATH [file nativename [file normalize [file join .. acmds .libs]]]
lappend PATH [file nativename [file normalize [file join .. acmds]]]
lappend PATH [file nativename [file normalize [file join .. src .libs]]]
lappend PATH [file nativename [file normalize .libs]]
lappend PATH [file nativename [file normalize .]]

switch -exact $tcl_platform(platform) {
    windows { 
	lappend PATH {C:\WINDOWS\system32}
	lappend PATH {C:\cygwin\bin}
	lappend PATH {C:\cygwin\usr\bin}
	lappend PATH {C:\cygwin\usr\X11R6\bin}
	set KILL tskill.exe
	set TCLSH [info nameofexecutable]
    }
    unix {
	lappend PATH /bin
	lappend PATH /usr/bin
	set KILL kill
    }
    default { 
	set KILL kill
    }
}
set env(PATH) "[join $PATH $PATH_SEP]$PATH_SEP$env(PATH)"

## setup TCL path
set TCLLIBPATH [list]
lappend TCLLIBPATH [file normalize [file join .. tclmsgque .libs]]
lappend auto_path [file normalize [file join .. tclmsgque .libs]]
set env(TCLLIBPATH) $TCLLIBPATH

## setup PYTHON path
set PYTHONPATH [list]
lappend PYTHONPATH [file nativename [file normalize [file join .. tests]]]
lappend PYTHONPATH [file nativename [file normalize [file join .. pymsgque]]]
lappend PYTHONPATH [file nativename [file normalize [file join .. pymsgque .libs]]]
set env(PYTHONPATH) [join $PYTHONPATH $PATH_SEP]
#Print env(PYTHONPATH)

## setup JAVA classpath
set CLASSPATH [list]
lappend CLASSPATH [file nativename [file normalize [file join .. tests]]]
lappend CLASSPATH [file nativename [file normalize [file join .. javamsgque]]]
lappend CLASSPATH [file nativename [file normalize [file join .. example java]]]
set env(CLASSPATH) [join $CLASSPATH $PATH_SEP]
#Print env(CLASSPATH)

## setup C# search path
set MONO_PATH [list]
lappend MONO_PATH [file nativename [file normalize [file join .. tests]]]
lappend MONO_PATH [file nativename [file normalize [file join .. csmsgque]]]
lappend MONO_PATH [file nativename [file normalize [file join .. example csharp]]]
set env(MONO_PATH) [join $MONO_PATH $PATH_SEP]
#Print env(MONO_PATH)

## setup LIBRARY search path
set LIBRARY_PATH [list]
lappend LIBRARY_PATH [file nativename [file normalize [file join .. src .libs]]]
lappend LIBRARY_PATH [file nativename [file normalize [file join .. javamsgque .libs]]]
lappend LIBRARY_PATH [file nativename [file normalize [file join .. tclmsgque .libs]]]
lappend LIBRARY_PATH [file nativename [file normalize [file join .. tests .libs]]]
set env(LD_LIBRARY_PATH) "[join $LIBRARY_PATH $PATH_SEP]$PATH_SEP$env(LD_LIBRARY_PATH)"

if {$tcl_platform(os) eq "FreeBSD"} {
    set WAIT 1000
} else {
    set WAIT 0
}

## tcl8.5 introduced an other precision as on 8.4 (reset back)
set tcl_precision 12

## the default io is PIPE
set io  Pipe

##
## -----------------------------------------------------------------------
## option procs
## 

proc optB {_argv opt} {
    upvar $_argv argv
    if {[set IDX [lsearch -exact $argv $opt]] != -1} {
	set argv    [lreplace $argv $IDX $IDX]
	return 1
    } else {
	return 0
    }
}

proc optV {_argv opt {def ""}} {
    upvar $_argv argv
    if {[set IDX [lsearch -exact $argv $opt]] != -1} {
	set IDX2    [expr {$IDX+1}]
	set def	    [lindex $argv $IDX2]
	set argv    [lreplace $argv $IDX $IDX2]
    }
    return $def
}

# delete options
proc optVD {_argv args} {
  upvar $_argv argv
  foreach opt $args {
    while {[set IDX [lsearch -exact $argv $opt]] != -1} {
      set argv    [lreplace $argv $IDX [expr {$IDX+1}]]
    }
  }
}

proc optSet {_argv opt def} {
    upvar $_argv argv
    if {[lsearch -exact $argv $opt] == -1} {
	if {$def ne ""} { lappend argv $opt $def }
	return 0
    } else {
	return 1
    }
}

proc envGet {VAR} {
    global env
    if {[info exists env($VAR)]} {
	return $env($VAR)
    }
    return ""
}

# this code is needed to filter "VAR" list with "args" regexp
proc filterGet {VAR args} {
  set NOT no
  if {$VAR eq "-not"} {
    set NOT yes
    set VAR [lindex $args 0]
    set args [lrange $args 1 end]
  }
  set RET [list]
  foreach e [envGet $VAR] {
    set FLAG yes
    foreach f $args {
      if {![regexp "$f" $e]} {
	set FLAG no
	break
      }
    }
    if {$NOT ? !$FLAG : $FLAG} {
      lappend RET $e
    }
  }
  return $RET
}

proc numGet { num } {
    set ret [list]
    if {$::env(PAR_LST)} {
	set ret $::env(PAR_LST)
    } else {
	for {set i 1} {$i <= $num} {incr i} {
	    lappend ret $i
	}
    }
    return $ret
}

proc getServer {srv} {
    set RET [list]

    # prefix (debugger)
    if {[string match {*valgrind*} $srv]} {
	lappend RET valgrind --trace-children=yes
    }
    if {[string match {*leak*} $srv]} {
	lappend RET --leak-check=full
    }
    if {[string match {*strace*} $srv]} {
	lappend RET strace -f -F
    }
    if {[string match {*gdb*} $srv]} {
	lappend RET gdb --tui --args
    }
    if {[string match {*ddd*} $srv]} {
	lappend RET ddd --gdb --args
    }

    # main executable
    switch -glob $srv {
      *bin*	{ lappend RET server }
      *tcl*	{ lappend RET {*}$::tcl_server }
      *jdb*	{ lappend RET jdb Server}
      *java*	{ lappend RET {*}$::java_server }
      *csharp*	{ lappend RET {*}$::csharp_server }
      *python*	{ lappend RET {*}$::python_server }
    }

    # options
    if {[string match {*fork*} $srv]} {
	lappend RET --fork
    } elseif {[string match {*thread*} $srv]} {
	lappend RET --thread
    } elseif {[string match {*spawn*} $srv]} {
	lappend RET --spawn
    }

    return $RET
}

proc getExample {srv} {
    set RET [list]

    switch -glob $srv {
      *.python	{ lappend RET $::PYTHON [file join $::testdir .. example python [file rootname $srv].py] }
      *.java	{ lappend RET $::JAVA example.[file rootname $srv] }
      *.csharp	{ lappend RET {*}$::CSEXEC [file join .. example csharp [file rootname $srv].exe] }
      *.tcl	{ lappend RET $::TCLSH [file join $::testdir .. example tcl $srv] }
      *.bin	{ lappend RET [file join .. example c [file rootname $srv]$::EXEEXT] }
      *		{ puts stderr "invalid example: $srv"; exit 1 }
    }

    return $RET
}

set portNUM [expr {int(rand()*30000)+10000}]

proc dummy_server {channel clientaddr clientport} {
  close $channel
}

proc FindFreePort {} {
  global env
  ## check if a user set default is available
  if {$env(TS_PORT) ne "PORT"} {
    return $env(TS_PORT)
  }
  ## generate unique port
  global portNUM
  set TRY 10
  while true {
    incr portNUM
    if {[catch {socket -server dummy_server $portNUM} FH ]} {
      Print FH
      incr TRY -1
      if {$TRY < 0} {
	Error "PANIC: unable to get a free port"
      }
      continue
    }
    close $FH
    break
  }
  return $portNUM
}

set CLEANUP_FILES [list]
set fileNUM [expr {int(rand()*30000)+10000}]

proc FindFreeFile {{ext uds}} {
  global env
  ## check if a user set default is available
  if {$ext eq "uds" && $env(TS_FILE) ne "FILE"} {
    return $env(TS_FILE)
  } elseif {$ext eq "pid" && $env(TS_PID) ne "PID"} {
    return $env(TS_PID)
  }
  ## generate unique filename
  global fileNUM CLEANUP_FILES
  while true {
    incr fileNUM
    set FILE    [file join . ${fileNUM}.$ext]
    if {![file exists $FILE]} break
  }
  lappend CLEANUP_FILES $FILE
  return $FILE
}

proc MkUnique {list} {
  set D [list]
  foreach d $list {
    switch -exact -- $d {
      FILE	{lappend D [FindFreeFile uds]}
      PORT	{lappend D [FindFreePort]}
      PID	{lappend D [FindFreeFile pid]}
      default	{lappend D $d}
    }
  }
  return $D
}

proc SetConstraints {args} {
  global env
  set ALL false
  if {[lindex $args 0] eq "all"} {
    set ALL true
  } elseif {[llength $args] != 3} {
    Error "expect 'all' or 3 arguments to SetConstraints: one NUMBER, BIN_LST and SRV_LST item"
  }
  if {$ALL} {
    foreach c {1 2 3} {
      testConstraint $c yes
    }
    foreach c [envGet BIN_LST] {
      testConstraint $c yes
    }
    foreach c {pipe uds tcp bin tcl java csharp python fork spawn thread server} {
      testConstraint $c [expr {[lsearch -glob [envGet SRV_LST] "*$c*"] != -1}]
    }
  } else {
    # 1. cleanup all constraint
    foreach c {1 2 3 string binary uds tcp pipe bin tcl java csharp python fork thread spawn server} {
      testConstraint $c no
    }
    # 2. string/binary
    testConstraint [lindex $args 0] yes
    # 3. pipe/uds/tcp
    testConstraint [lindex $args 1] yes
    # 4. the server parts
    foreach c [split [lindex $args 2] .] {
      testConstraint $c yes
    }
  }
}

##
## -----------------------------------------------------------------------
## init
## 

if {![info exists env(BIN_LST)]} {
  set env(BIN_LST) {string binary}
}
if {![info exists env(COM_LST)]} {
  set env(COM_LST) {uds tcp pipe}
}
if {![info exists env(LNG_LST)]} {
  set env(LNG_LST) {bin tcl python java csharp}
}
if {![info exists env(START_LST)]} {
  set env(START_LST) {fork thread spawn}
}
if {![info exists env(SRV_LST)]} {
  set env(SRV_LST) {
    bin.pipe
    bin.tcp.fork
    bin.tcp.thread
    bin.tcp.spawn
    bin.uds.fork
    bin.uds.thread
    bin.uds.spawn
    tcl.pipe
    tcl.tcp.fork
    tcl.tcp.thread
    tcl.tcp.spawn
    tcl.uds.fork
    tcl.uds.thread
    tcl.uds.spawn
    java.pipe
    java.tcp.thread
    java.uds.thread
    java.tcp.spawn
    java.uds.spawn
    python.pipe
    python.tcp.spawn
    python.tcp.fork
    python.uds.spawn
    python.uds.fork
    csharp.pipe
    csharp.tcp.thread
    csharp.uds.thread
    csharp.tcp.spawn
    csharp.uds.spawn
  }
}

# windows has some restrictions :(
if {$::tcl_platform(platform) eq "windows"} {
  set env(SRV_LST) [filterGet -not SRV_LST uds]
  set env(SRV_LST) [filterGet -not SRV_LST fork]
  set env(COM_LST) [filterGet -not COM_LST uds]
}

# without tcl thread support no tcl thread server
if {[info exists ::tcl_platform(threaded)]} {
  set env(SRV_LST)   [filterGet -not SRV_LST tcl fork]
} else {
  set env(SRV_LST)   [filterGet -not SRV_LST tcl thread]
}

# without --enable-java no java
if {$JAVA eq ""} {
  set env(SRV_LST) [filterGet -not SRV_LST java]
  set env(LNG_LST) [filterGet -not LNG_LST java]
}

# without --enable-python no python
if {$PYTHON eq ""} {
  set env(SRV_LST) [filterGet -not SRV_LST python]
  set env(LNG_LST) [filterGet -not LNG_LST python]
}

# without --enable-csharp no C#
if {$CSEXEC eq ""} {
  set env(SRV_LST) [filterGet -not SRV_LST csharp]
  set env(LNG_LST) [filterGet -not LNG_LST csharp]
}
if {$CSEXEC eq "NATIVE"} {
  set CSEXEC {}
}

# setup server procs
set tcl_server [list $TCLSH [file join $testdir server.tcl]]
set java_server [list $JAVA example.Server]
set csharp_server [list {*}$CSEXEC csserver.exe]
set python_server [list $PYTHON [file join $testdir server.py]]

# try to analyse the "argv" array
if {[info exist argv]} {
    set argv [linsert $argv 0 -verbose {start pass body error}]
    if {[optB argv --testing]} {
	lappend argv --only-pipe --only-binary --only-num 1 
    }
    if {[optB argv --remote-testing]} {
	lappend argv --only-tcp --port 7777 --only-binary \
			--only-thread --use-remote 
    }
    if {![info exist env(USE_REMOTE)]} {
	set env(USE_REMOTE) [optB argv --use-remote]
    }
    if {![info exist env(TS_DEBUG)]} {
	set env(TS_DEBUG)   [optV argv --debug 0]
    }
    if {![info exists env(TS_TIMEOUT)]} {
	set env(TS_TIMEOUT) [optV argv --timeout 10]
    }
    if {![info exist env(TS_SETUP)]} {
	set env(TS_SETUP)	    [optB argv --setup]
    }
    if {![info exist env(TS_THREAD)]} {
	set env(TS_THREAD)  [optB argv --thread]
    }
    if {![info exists env(TS_HOST)]} {
	set env(TS_HOST)   [optV argv --host localhost]
    }
    if {![info exists env(TS_PORT)]} {
	set env(TS_PORT)   [optV argv --port PORT]
    }
    if {![info exists env(TS_FILE)]} {
	set env(TS_FILE)   [optV argv --file FILE]
    }
    if {![info exists env(TS_PID)]} {
	set env(TS_PID)   [optV argv --pid PID]
    }
    if {![info exists env(TS_MAX)]} {
	set env(TS_MAX)   [optV argv --max -1]
    }

    if {[optB argv --only-string]} {
	set env(BIN_LST) string
    } elseif {[optB argv --only-binary]} {
	set env(BIN_LST) binary
    }

    ## reduce the possibilities based on the final "SRV_LST"
    set env(SRV_LST) [optV argv --server $env(SRV_LST)]

    # check for binary or tcl server
    if {[optB argv --only-bin]} {
      set env(LNG_LST) bin
    } elseif {[optB argv --only-tcl]} {
      set env(LNG_LST) tcl
    } elseif {[optB argv --only-java]} {
      set env(LNG_LST) java
    } elseif {[optB argv --only-csharp]} {
      set env(LNG_LST) csharp
    } elseif {[optB argv --only-python]} {
      set env(LNG_LST) python
    }
    set env(SRV_LST) [filterGet SRV_LST "([join $env(LNG_LST) |])"]

    # filter COM_LST (and SRV_LST too)
    if {[optB argv --only-uds]} {
      set env(COM_LST) uds
    } elseif {[optB argv --only-pipe]} {
      set env(COM_LST) pipe
    } elseif {[optB argv --only-tcp]} {
      set env(COM_LST) tcp
    }
    set env(SRV_LST) [filterGet SRV_LST "([join $env(COM_LST) |])"]

    # filter SRV_LST
    if {[optB argv --only-fork]} {
      set env(START_LST) [filterGet START_LST fork]
      set env(SRV_LST)   [filterGet SRV_LST fork]
    } elseif {[optB argv --only-thread]} {
      set env(START_LST) [filterGet START_LST thread]
      set env(SRV_LST)   [filterGet SRV_LST thread]
    } elseif {[optB argv --only-spawn]} {
      set env(START_LST) [filterGet START_LST spawn]
      set env(SRV_LST)   [filterGet SRV_LST spawn]
    }

    # delete everything from COM_LST which doesn't belongs to SRV_LST
    if {![llength [filterGet SRV_LST pipe]]} {
      ## without "pipe" in "SRV_LST" we don't need "pipe" in "COM_LST"
      set env(COM_LST) [filterGet -not COM_LST pipe]
    }

    # set parent/child depth
    if {![info exists env(PAR_LST)]} {
	set env(PAR_LST) [optV argv --only-num 0]
    }

    # print the help
    if {[optB argv -h] || [optB argv --help]} {
	puts "USAGE [file tail $argv0]: OPTIONS ...."
	puts " testing OPTIONS"
	puts "  --only-(string|binary) ....... test only the 'required' feature"
	puts "  --only-(pipe|uds|tcp) ........ test only the 'required' communication"
	puts "  --only-(bin|tcl|java|python|csharp) .. test 'programming-language'"
	puts "  --only-(fork|thread|spawn)  .. test only the 'required' feature"
	puts ""
	puts "  --only-num # ..... test only with parent/child number #"
	puts "  --use-remote ..... use remote server for tcp/udp connection"
	puts "  --max # .......... set the maximun of allowed processes to #"
	puts "  --debug # ........ set debug level between 0 and 9"
	puts "  --host STR ....... use host as TCP/IP connection host"
	puts "  --port # or STR .. use port as TCP/IP connection port"
	puts "  --file STR ....... use file as Unix Domain Socket connection file"
	puts "  --pid STR ........ use file as --daemon pid file"
	puts "  --testing  ....... set environment for testing:"
	puts "              --only-binary, --only-pipe, --only-bin and --only-num 1"
	puts "  --remote-testing   set environment for remote testing:"
	puts "              --only-binary/thread/tcp, --port 7777 and --use-remote"
	puts "  --server STR ..... user STR as server: (default: use all server)"
        puts "          (bin|tcl|java|python|csharp).pipe"
        puts "     or   (bin|tcl|java|python|csharp).(tcp|uds).(fork|thread|spawn)"
	puts "  --setup .......... print additional setup information"
	puts "  --help-tclmsgque . get tclmsgque specific help"
	puts "  --help-msgque .... get libmsgque specific help"
	puts "  --help / -h ...... get help"
	puts ""
	exit 1
    }
    set TS_HELP_TCLMSGQUE [optB argv --help-tclmsgque]
    set TS_HELP_MSGQUE [optB argv --help-msgque]
} ;# no argv

## check for thread
if {$env(TS_THREAD)} {
  if {![info exists tcl_platform(threaded)]} {
    puts "no thread support available, skip ..."
    exit 0
  }
  if {[catch {package require Thread} ERR]} {
    puts "ERROR can not load 'Thread' package: $ERR"
    exit 0
  }
  ## get upper script
  puts "THREAD: start with '$TS_SCRIPT'"
  set ID [thread::create -joinable "
    set argv {}
    unset env(TS_THREAD)
    source $TS_SCRIPT
  "]
  thread::join $ID
  return -code return
}

package require tcltest
namespace import -force ::tcltest::*
configure {*}$::argv

## some basic restrictions
testConstraint local [expr {$env(USE_REMOTE) ? no : yes}]
testConstraint tcl_has_thread [info exists ::tcl_platform(threaded)]
testConstraint use_remote $env(USE_REMOTE)

##
## -----------------------------------------------------------------------
## match
##

proc test_variable {variable result} {
    return [expr {[uplevel set $variable] == $result}]
}
customMatch variable test_variable

proc _expr {x y} {
    return [expr {$x == $y}]
}
customMatch expr _expr

##
##--------------------------------------------------------------------------
## Init TestFbg
##

## autoload the package
package require TclMsgque

if {$TS_HELP_TCLMSGQUE} {
  puts [tclmsgque help]
  exit 0
}

if {$TS_HELP_MSGQUE} {
  puts [tclmsgque MqS --help-msgque]
  exit 0
}

# does libmsgque support thread
if {![tclmsgque support thread]} {
  set env(SRV_LST) [filterGet -not SRV_LST thread]
  set env(START_LST) [filterGet -not START_LST thread]
}

# does libmsgque support fork
if {![tclmsgque support fork]} {
  set env(SRV_LST) [filterGet -not SRV_LST fork]
  set env(START_LST) [filterGet -not START_LST fork]
}

# check if list has still data
foreach l {SRV_LST COM_LST LNG_LST START_LST} {
  if {![llength $env($l)]} {
    Error "$l is empty"
  }
}

##
## -----------------------------------------------------------------------
## TestFbg
##

namespace eval TestFbg {
}

proc optB {_argv opt} {
    upvar $_argv argv
    if {[set IDX [lsearch -exact $argv $opt]] != -1} {
        set argv    [lreplace $argv $IDX $IDX]
        return 1
    } else {
        return 0
    }
}

##
## ---------------------------------------------------------------------------------------------------
## Public High
##

proc SendL { ctx list } {
    foreach {cmd val} $list {
      if {$cmd eq "SendL"} {
	$ctx SendL_START
	SendL $ctx $val
	$ctx SendL_END
      } else {
	$ctx $cmd $val
      }
    }
}
  
proc Call { ctx send token args } {
    $ctx SendSTART
    SendL $ctx $args
    $ctx $send $token
}
  
proc Echo0 { ctx token args } {
    Call $ctx SendEND_AND_WAIT $token {*}$args
}
  
proc Echo1 { ctx token read args} {
    Call $ctx SendEND_AND_WAIT $token {*}$args
    return  [$ctx $read]
}

proc EchoL { ctx token args } {
    Call $ctx SendEND_AND_WAIT $token {*}$args
    return  [$ctx ReadAll]
}
  
##
## -----------------------------------------------------------------------
## procs
##

proc Start {isError id cl} {
  global env FH_LAST
  if {$env(TS_SETUP)} { Print id cl }
  if {$isError} {
    if {[catch {{*}$cl} FH_LAST]} { error $FH_LAST }
  } else {
    set NUM 10
    while {[catch {{*}$cl} FH_LAST]} {
      puts "RETRY: $cl"
      incr NUM
      if {$NUM > 10} { Error $FH_LAST }
    }
  }
  set ::FH($id) $FH_LAST
  if {$env(TS_SETUP)} { puts "start FINISH" }
}

proc Setup {num mode com server args} {
  global env PID PIDFILE FH FH_LAST PORT FILE
  if {[llength $num] == 1} {
    set numParent   1
    set numChild    $num
  } else {
    foreach {numParent numChild} $num break
  }
  if {$env(TS_SETUP)} {
    Print num mode com server args
  }
  set serverSilent [optB args --server-silent]
  set isError [optB args --error]
  set setup_parent [optV args --setup-parent]

  ## 1. setup variables
  lappend args --debug [envGet TS_DEBUG] --$com
  if {$mode eq "string"} { lappend args --string }
  # replase all FILE and PORT placeholder
  set args [MkUnique $args]
  # init client-parent (pargs), client-child (cargs) and server (sargs) arguments
  set cargs	[list]
  set pargs	$args
  set sargs	$args
  # check for PIDFILE
  set idx [lsearch -exact $args --daemon]
  if {$idx != -1} {
    set PIDFILE [lindex $args [expr {$idx+1}]]
  }

  ## 2. for NON-PIPE server start the server as --fork/--thread/--spawn once
  if {$com ne "pipe"} {
    # the name of the server is srvname
    set SRVNAME [optV sargs --srvname]
    if {$SRVNAME ne ""} {lappend sargs --name $SRVNAME}
    # delete all parent options from the child argument list
    optVD args --buffersize --timeout 
    ## ...
    switch -exact -- $com {
      tcp	{
	set PORT    [envGet TS_PORT]
	if {$PORT eq "PORT"} {set PORT [FindFreePort]}
	optSet sargs --port $PORT
	optSet pargs --port $PORT

	# set host for client and server
	optSet sargs --host [envGet TS_HOST]
	optSet pargs --host [envGet TS_HOST]

	# delete all client options from the server argument list
	optVD sargs --myport --myhost

	# delete all parent options from the child argument list
	optVD args --port --host --myhost --myport
      }
      uds	{
	set FILE    [envGet TS_FILE]
	if {$FILE eq "FILE"} {set FILE [FindFreeFile]}
	optSet sargs --file $FILE
	optSet pargs --file $FILE

	# delete all parent options from the child argument list
	optVD args --file
      }
    }
    set sargs [MkUnique $sargs]
    if {$serverSilent} { lappend sargs --silent }
    if {!$env(USE_REMOTE)} {
      set sl [list {*}[getServer $server] {*}$sargs]
      if {$env(TS_SETUP)} {
	Print sl
      }
      if {[catch {exec {*}$sl >&@stdout &} PID]} {
	puts $PID
	exit 1
      }
      after $::WAIT
    }
  }
  
  ## 3. create new tclmsgque object
  set FH_LAST	""

  for {set PNO 0} {$PNO<$numParent} {incr PNO} {
    ## prepare parent arguments
    set cl [list tclmsgque MqS --name client-$PNO --srvname server-$PNO --timeout $env(TS_TIMEOUT) {*}$pargs]
    if {$com eq "pipe"} { 
	lappend cl @ {*}[getServer $server] 
	if {$serverSilent} { lappend cl --silent }
    }
    Start $isError $PNO-0 $cl

    ## start the parent setup code
    if {$setup_parent ne ""} {
      uplevel $setup_parent
    }

    ## start the childs
    for {set CNO 1} {$CNO<$numChild} {incr CNO} {
      ## prepare child arguments
      set cl [list tclmsgque MqS --parent $FH_LAST --name [$FH_LAST ConfigGetName] {*}$cargs]
      ## start the child 
      Start yes $PNO-$CNO $cl
    }
  }
}

proc Cleanup {{wait 0}} {
#Print args
  global PID PIDFILE FH FH_LAST

  ## 2. close all tclmsgque objects
    foreach num [lsort [array names FH {*-0}]] {
	$FH($num) Delete
    }
    array unset FH
    unset -nocomplain FH_LAST
  ## 3. if a separate server was startet (NON-PIPE communication) kill this server
    if {!$::env(USE_REMOTE)} {
      if {[info exists PIDFILE]} {
	if {[file exists $PIDFILE]} {
	  set FP [open $PIDFILE r]
	  set PID [read $FP]
	  close $FP
	}
	unset PIDFILE
      }
      if {[info exists PID]} {
	after $wait
	catch {exec $::KILL $PID}
	unset PID
      }
      if {[llength $::CLEANUP_FILES]} {
	file delete -force {*}$::CLEANUP_FILES
      }
    }
}

proc Example {mode com client server args} {
  global env PID PIDFILE FH FH_LAST
  if {$env(TS_SETUP)} {
    Print mode com client server
  }
  unset -nocomplain PID PIDFILE FH FH_LAST
  set serverSilent [optB args --server-silent]

  ## 1. setup variables
  # replase all FILE and PORT placeholder
  set args [MkUnique $args]

  ## 2. init cl/sl
  set cl [list {*}[getExample $client] --$com --name client {*}$args]
  set sl [list {*}[getExample $server] --name server {*}$args]
  if {$serverSilent} { lappend sl --silent }
  lappend cl --debug [envGet TS_DEBUG]
  if {$mode eq "string"} { lappend cl --string }

  ## 3. for NON-PIPE server start the server as --fork/--thread/--spawn once
  if {$com ne "pipe"} {
    lappend sl --$com
    # the name of the server is srvname
    set SRVNAME [optV sl --srvname]
    if {$SRVNAME ne ""} {lappend sl --name $SRVNAME}
    ## ...
    switch -exact -- $com {
      tcp	{
	set PORT    [envGet TS_PORT]
	if {$PORT eq "PORT"} {set PORT [FindFreePort]}
	optSet cl --port $PORT
	optSet sl --port $PORT

	# set host for client and server
	optSet sl --host [envGet TS_HOST]
	optSet cl --host [envGet TS_HOST]

	# delete all client options from the server argument list
	optVD sl --myport --myhost
      }
      uds	{
	set FILE    [envGet TS_FILE]
	if {$FILE eq "FILE"} {set FILE [FindFreeFile]}
	optSet sl --file $FILE
	optSet cl --file $FILE
      }
    }
    if {!$env(USE_REMOTE)} {
      if {$env(TS_SETUP)} {
	Print sl
      }
      if {[catch {exec {*}$sl >&@stdout &} PID]} {
	puts $PID
	exit 1
      }
      after $::WAIT
    }
  }
  
  ## 3. start client
  if {$com eq "pipe"} { lappend cl @ {*}$sl }
  if {$env(TS_SETUP)} { Print cl }
  set RET [Exec {*}$cl]

  ## 4. cleanup
  Cleanup

  return $RET
}

proc Exec {args} {
    if {[catch {exec {*}$args} ERR]} {
	return [lindex $::errorCode 0]-[lindex $::errorCode 2]-$ERR
    } else {
	return "$ERR"
    }
}

proc ExecLines {start end args} {
    catch {uplevel $args} MSG
    return [join [lrange [split $MSG \n] $start $end ] \n]
}


