#!/bin/bash
#+
#§  \file       tests/performance.sh
#§  \brief      \$Id: performance.sh 1206 2009-05-20 11:48:19Z dev1usr $
#§  
#§  (C) 2009 - LibMsgque - Project - Group
#§  
#§  \version    \$Rev: 1206 $
#§  \author     EMail: aotto1968 at users.sourceforge.net
#§  \attention  this software has GPL permissions to copy
#§              please contact AUTHORS for additional information
#§

typeset -i PORT=7777

## setup the TOTAL perf
FS=" %-20s | %-8s %-8s %-8s %-8s %-8s %-8s %-8s\n"
FI=" %-20s | %8s %8s %8s %8s %8s %8s %8s\n"
exec 3>docs/total.perf
DATE=$(date +"%Y-%m-%d %H:%M:%S")
printf "$FS" "   Tests from:    " "  send  " "  send  " "  send  " " parent " " parent " " child  " " child  " 1>&3
printf "$FS" "$DATE"              "  END   " "CALLBACK" "  WAIT  " " create " " delete " " create " " delete " 1>&3
printf "$FS" -------------------   --------   --------   --------   --------   --------   --------   --------  1>&3

kill_parent=1
sig_spec=-9

function do_kill # <pid>...
{
    kill "$sig_spec" "$@"
}

function kill_children # pid
{
    local target=$1
    local pid=
    local ppid=
    local i
    # Returns alternating ids: first is pid, second is parent
    IFS="
"
    for line in $(ps -o pid,ppid | tail +2); do
	unset IFS
	set $line
	pid=$1
	ppid=$2
	(( ppid == target && pid != $$ )) && {
	    kill_children $pid
	    do_kill $pid
	}
    done
}


Total() {
  F=$1;
  local -a vals
  IFS="
"
  IDX=0
  for LINE in $(<docs/$F.perf) ; do
    unset IFS
    [[ $LINE == *statistics* ]] && {
      ## clean WhiteSpace
      for V in ${LINE:98}; do
	vals[$IDX]=$V
	let IDX++
      done
    }
  done

  printf "$FI" "$F" ${vals[*]} 1>&3
}

ALL="bin_pipe
bin_tcp_fork
bin_tcp_thread
bin_tcp_spawn
bin_uds_fork
bin_uds_thread
bin_uds_spawn
tcl_pipe
tcl_tcp_fork
tcl_tcp_thread
tcl_tcp_spawn
tcl_uds_fork
tcl_uds_thread
tcl_uds_spawn
java_pipe
java_tcp_thread
java_uds_thread
java_tcp_spawn
java_uds_spawn
python_pipe
python_tcp_spawn
python_tcp_fork
python_uds_spawn
python_uds_fork
csharp_pipe
csharp_tcp_thread
csharp_uds_thread
csharp_tcp_spawn
csharp_uds_spawn"

#apply the commandline filter
R=""
for A in $ALL; do
  for C in "$@" ; do
    [[ "$A" == *$C* ]] || continue 2
  done
  R+=" $A"
done

## do the tests
#for SRV in bin tcl java python ; do
for SRV in $R; do

  echo $SRV

  case $SRV in
    *bin*)	
      SERVER=./server 
      ENV=""
    ;;
    *tcl*)	
      SERVER="tclsh ./server.tcl" 
      ENV="./tcl.env"
    ;;
    *java*)	
      SERVER="java example.Server" 
      ENV="./java.env"
    ;;
    *python*)	
      SERVER="python ./server.py"	    
      ENV="./python.env"
    ;;
    *csharp*)	
      SERVER="mono ./csserver.exe"	    
      ENV="./csharp.env"
    ;;
  esac

  [[ $SRV != *thread* ]] && ENV="performance_nothread.env $ENV"
  ENV="performance.env $ENV"

  case $SRV in
    *pipe*)
      CL="./client --all @ $SERVER"
      echo "> $CL" | tee docs/${SRV}.perf
      $ENV $CL 2>&1 | tee -a docs/${SRV}.perf

      continue
    ;;
    *uds*)
      COM_ARGS="--uds --file ./socket.uds"
      COM_RM="./socket.uds"
    ;;
    *tcp*)
      COM_ARGS="--tcp --host localhost --port $PORT"
      COM_RM=""
      let PORT=PORT+1
    ;;
  esac

  case $SRV in
    *thread*)	START=thread;;
    *spawn*)	START=spawn;;
    *fork*)	START=fork;;
  esac

  SV="$SERVER $COM_ARGS --$START"
  CL="./client --all $COM_ARGS"
  echo "> $SV" | tee docs/${SRV}.perf
  echo "> $CL" | tee -a docs/${SRV}.perf
  $ENV $SV 1>/dev/null 2>&1 &
  PID=$! 
  sleep 2
  $CL 2>&1 | tee -a docs/${SRV}.perf
  kill_children $PID

done

# calculate the total
for SRV in $ALL; do
  Total ${SRV}
done

test -f socket.uds && rm socket.uds

