/**
 *  \file       tests/server.cs
 *  \brief      \$Id: server.cs 129 2009-06-30 12:10:31Z dev1usr $
 *  
 *  (C) 2009 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 129 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

using System;
using csmsgque;
using System.Collections.Generic;

namespace example {

  sealed  class Client : MqS {
    public int i;

    static Client() {
      name = "test-client";
    }

    public Client (int debug) : base("--debug", debug.ToString(), "@", "SELF", 
      "--name", "test-server") { }

    public Client (Client parent) : base(parent) { }

    public Client () : base() { }

    public void ECOI_CB () {
      i = ReadI();
    }
  }

  sealed class Server : MqS, IServer {

    IntPtr buf;

    static Server() {
      name = "server";
    }

  // ########################################################################

    public Server(params string[] args) : base(args) {}
    public Server(Server master, int master_id, params string[] args) : base(master,master_id,args) {}
    public Server() : base() {}

  // ########################################################################

    void IServer.Config () {
      if (ConfigGetIsSlave()) {
	// add "slave" services here
      } else {
	// add "master" services here
	ServiceCreate("ECOI", ECOI);
	ServiceCreate("ECOU", ECOU);
	ServiceCreate("SETU", SETU);
	ServiceCreate("GETU", GETU);
	ServiceCreate("ECOW", new ECOW());
	ServiceCreate("ECOS", ECOS);
	ServiceCreate("ECOD", ECOD);
	ServiceCreate("ECOF", ECOF);
	ServiceCreate("ECOC", ECOC);
	ServiceCreate("ECOL", ECOL);
	ServiceCreate("ECLI", ECLI);
	ServiceCreate("LST1", LST1);
	ServiceCreate("LST2", LST2);
	ServiceCreate("ECUL", ECUL);
	ServiceCreate("ECOO", ECOO);
	ServiceCreate("ECOY", ECOY);
	ServiceCreate("ECOB", ECOB);
	ServiceCreate("BUF1", BUF1);
	ServiceCreate("BUF2", BUF2);
	ServiceCreate("BUF3", BUF3);
	ServiceCreate("ERR2", ERR2);
	ServiceCreate("ERR3", ERR3);
	ServiceCreate("CSV1", CSV1);
	ServiceCreate("SLEP", SLEP);
	ServiceCreate("USLP", USLP);
	ServiceCreate("ECON", ECON);
	ServiceCreate("MSQT", MSQT);
	ServiceCreate("SND1", SND1);
	ServiceCreate("SND2", SND2);
	ServiceCreate("REDI", REDI);
	ServiceCreate("RDUL", RDUL);
	ServiceCreate("GTCX", GTCX);
	ServiceCreate("GTTO", GTTO);
	ServiceCreate("LANG", LANG);
	ServiceCreate("INIT", INIT);
      }
    }

  // ########################################################################

    public void INIT () {
      int max = ReadGetNumItems();
      string[] list = new string[max];
      SendSTART();
      for (int i=0; i<max; i++) {
	list[i] = ReadC();
      }
      MqS.Init(list);
      SendRETURN();
    }

    public void LANG () {
      string s = ReadC();
      SendSTART();
	if (s == "NullReferenceException-1") {
	  throw new System.NullReferenceException("myMessage");
	} else if (s == "NullReferenceException-2") {
	  throw new System.NullReferenceException();
	} else {
	  SendC("nothig to do!!");
	}
      SendRETURN();
    }

    public void GTTO () {
      SendSTART();
      SendC(ConfigGetToken());
      SendRETURN();
    }

    public void RDUL () {
      ReadI();
      ReadW();
      ReadD();
      ReadU();
    }

    public void GTCX () {
      SendSTART();
      SendI(ConfigGetCtxId());
      SendRETURN();
    }

    public void REDI() {
      ReadI();
    }

    private Client[] cl = new Client[3];

    public void SND1 () {
      string s = ReadC();
      int id = ReadI();

      SendSTART();
        if (s == "START") {
          Server parent = (Server) ConfigGetParent();
          if (parent == null || parent.cl[id] == null) {
            cl[id] = new Client(ConfigGetDebug());
          } else {
            cl[id] = new Client(parent.cl[id]);
          }
        } else if (s == "STOP") {
          cl[id].Delete();
          cl[id] = null;
        } else if (s == "SEND") {
          cl[id].SendSTART();
          String TOK = ReadC();
          cl[id].SendU(ReadU());
          cl[id].SendEND(TOK);
        } else if (s == "WAIT") {
          cl[id].SendSTART();
          cl[id].SendU(ReadU());
          cl[id].SendEND_AND_WAIT("ECOI", 5);
          SendI(cl[id].ReadI()+1);
        } else if (s == "CALLBACK") {
          cl[id].SendSTART();
          cl[id].SendU(ReadU());
          cl[id].i = -1;
          cl[id].SendEND_AND_CALLBACK("ECOI", cl[id].ECOI_CB);
          cl[id].ProcessEvent(10, MqS.WAIT.ONCE);
          SendI(cl[id].i+1);
        }
      SendRETURN();
    }

    int myInt = -1;
    private void SetMyInt () {
      Server master = (Server) ConfigGetMaster();
      master.myInt = ReadI();
    }

    public void SND2 () {
      string s = ReadC();
      int id = ReadI();
      MqS cl = SlaveGet(id);

      SendSTART();
        if (s == "CREATE") {
          List<string> LIST = new List<string>() {"--name", "test-client", "--srvname", "test-server"};
          while (ReadItemExists())
            LIST.Add(ReadC());
          cl = new Server(this, id, LIST.ToArray());
        } else if (s == "CREATE2") {
	  cl = new Client(ConfigGetDebug());
	  SlaveCreate (id, cl);
        } else if (s == "DELETE") {
          SlaveDelete(id);
          SendC(SlaveGet(id) == null ? "OK" : "ERROR");
        } else if (s == "SEND") {
          cl.SendSTART();
          String TOK = ReadC();
          cl.SendU(ReadU());
          cl.SendEND(TOK);
        } else if (s == "WAIT") {
          cl.SendSTART();
          cl.SendU(ReadU());
          cl.SendEND_AND_WAIT("ECOI", 5);
          SendI(cl.ReadI()+1);
        } else if (s == "CALLBACK") {
          cl.SendSTART();
          cl.SendU(ReadU());
          myInt = -1;
          cl.SendEND_AND_CALLBACK("ECOI", ((Server)cl).SetMyInt);
          cl.ProcessEvent(10, MqS.WAIT.ONCE);
          SendI(myInt+1);
        } else if (s == "MqSendEND_AND_WAIT") {
          string TOK = ReadC();
          cl.SendSTART();
          while (ReadItemExists())
            cl.SendU(ReadU());
          cl.SendEND_AND_WAIT(TOK, 5);
          while (cl.ReadItemExists())
            SendU(cl.ReadU());
        } else if (s == "MqSendEND") {
          string TOK = ReadC();
          cl.SendSTART();
          while (ReadItemExists())
            cl.SendU(ReadU());
          cl.SendEND(TOK);
          return;
        } else if (s == "ERR-1") {
          new Server(this, id, "--name", "test-client", "@", "DUMMY");
        } else if (s == "isSlave") {
          SendO(cl.ConfigGetIsSlave());
        }
      SendRETURN();
    }

    public void MSQT () {
      SendSTART();
      if (ConfigGetDebug() != 0) {
	SendC ("debug");
	SendI (ConfigGetDebug());
      }
      if (ConfigGetIsString() == false)
	SendC ("binary");
      if (ConfigGetIsSilent() == true)
	SendC ("silent");
      SendC ("sOc");
      SendC (ConfigGetIsServer() ? "SERVER" : "CLIENT");
      SendC ("pOc");
      SendC (ConfigGetIsParent() ? "PARENT" : "CHILD");
      SendRETURN();
    }

    public void ECON () {
      SendSTART();
      SendC(ReadC() + "-" + ConfigGetName());
      SendRETURN();
    }

    public void CSV1 () {
      int num;

      // call an other service
      SendSTART ();
      num = ReadI();
      num++;
      SendI (num);
      SendEND_AND_WAIT ("CSV2", 10);

      // read the answer and send the result back
      SendSTART ();
      num = ReadI();
      num++;
      SendI (num);
      SendRETURN ();
    }

    public void SLEP () {
      SendSTART();
      int i = ReadI();
      System.Threading.Thread.Sleep(i*1000);
      SendI(i);
      SendRETURN();
    }

    public void USLP () {
      SendSTART();
      int i = ReadI();
      System.Threading.Thread.Sleep(i/1000);
      SendI(i);
      SendRETURN();
    }

    public void ERR2 () {
      SendSTART();
      SendC("some data");
      ErrorC("Ot_ERR2", 10, "some error");
      SendRETURN();
    }

    public void ERR3 () {
      SendSTART();
      SendRETURN();
      SendRETURN();
    }

    public void BUF1 () {
      IntPtr buf = ReadU();
      char typ = GetType(buf);
      SendSTART();
      SendC(typ.ToString());
      switch (typ) {
	case 'Y': SendY(GetY(buf)); break;
	case 'O': SendO(GetO(buf)); break;
	case 'S': SendS(GetS(buf)); break;
	case 'I': SendI(GetI(buf)); break;
	case 'F': SendF(GetF(buf)); break;
	case 'W': SendW(GetW(buf)); break;
	case 'D': SendD(GetD(buf)); break;
	case 'C': SendC(GetC(buf)); break;
	case 'B': SendB(GetB(buf)); break;
      }
      SendRETURN();
    }

    public void BUF2 () {
      SendSTART();
      for (int i=0; i<3; i++) {
	IntPtr buf = ReadU();
	SendC(GetType(buf).ToString());
	SendU(buf);
      }
      SendRETURN();
    }

    public void BUF3 () {
      SendSTART();
      IntPtr buf = ReadU();
      SendC(GetType(buf).ToString());
      SendU(buf);
      SendI(ReadI());
      SendU(buf);
      SendRETURN();
    }

    public void ECOB () {
      SendSTART();
      SendB(ReadB());
      SendRETURN();
    }

    public void ECOY () {
      SendSTART();
      SendY(ReadY());
      SendRETURN();
    }

    public void ECOO () {
      SendSTART();
      SendO(ReadO());
      SendRETURN();
    }

    public void ECUL () {
      SendSTART();
      SendI(ReadI());
      SendW(ReadW());
      SendD(ReadD());
      SendU(ReadU());
      SendRETURN();
    }

    public void LST1 () {
      SendSTART();
      SendL_END();
      SendRETURN();
    }

    public void LST2 () {
      SendSTART();
      ReadL_END();
      SendRETURN();
    }

    void EchoList(bool doincr) {
      IntPtr buf;
      while (ReadItemExists()) {
	buf = ReadU();
	if (GetType(buf) == 'L') {
	  GetL_START (buf);
	  SendL_START ();
	  EchoList (doincr);
	  SendL_END ();
	  GetL_END ();
	} else if (doincr) {
	  SendI (GetI(buf)+1);
	} else {
	  SendU (buf);
	}
      }
    }

    void ECOL () {
      SendSTART();
      ReadL_START ();
      SendL_START ();
      EchoList(false);
      SendL_END ();
      ReadL_END ();
      SendRETURN();
    }

    void ECLI () {
      IntPtr opt = ReadU();
      bool doincr = (GetType(opt) == 'C' && GetC(opt) == "--incr");
      if (!doincr) ReadUndo();
      SendSTART();
      EchoList(doincr);
      SendRETURN();
    }


    void ECOC () {
      SendSTART();
      SendC(ReadC());
      SendRETURN();
    }

    void ECOF () {
      SendSTART();
      SendF(ReadF());
      SendRETURN();
    }

    void ECOD () {
      SendSTART();
      SendD(ReadD());
      SendRETURN();
    }

    void ECOS () {
      SendSTART();
      SendS(ReadS());
      SendRETURN();
    }

    void ECOI () {
      SendSTART();
      SendI(ReadI());
      SendRETURN();
    }

    void ECOU () {
      SendSTART();
      SendU(ReadU());
      SendRETURN();
    }

    void SETU () {
      buf = ReadU();
    }

    void GETU () {
      SendSTART();
      SendU(buf);
      SendRETURN();
    }

  // ########################################################################

    static void Main(string[] args)
    {
      Server srv = new Server(args);
      srv.ProcessEvent(MqS.WAIT.FOREVER);
      srv.Delete();
    }

  } // finish class "server"

  public class ECOW : IService {
    void IService.Service (MqS ctx) {
      ctx.SendSTART();
      ctx.SendW(ctx.ReadW());
      ctx.SendRETURN();
    }
  }

} // finish namespace "example"
