#+
#§  \file       tests/server.py
#§  \brief      \$Id:$
#§  
#§  (C) 2009 - LibMsgque - Project - Group
#§  
#§  \version    \$Rev:$
#§  \author     EMail: aotto1968 at users.sourceforge.net
#§  \attention  this software has GPL permissions to copy
#§              please contact AUTHORS for additional information
#§

import sys
import time
import pymsgque

class Client(pymsgque.MqS):
  def __init__(self, parent = None, debug = 0):
    if parent == None:
      pymsgque.MqS.__init__(self,
        debug   = debug, 
        name    = "test-client", 
        argv    = ["@", "SELF", "--name", "test-server"])
    else:
      pymsgque.MqS.__init__(self,
        parent  = parent)

class Server(pymsgque.MqS):
  def __init__(self, argv, master=None, master_id=0):
    self.buf = None
    if master == None:
      pymsgque.MqS.__init__(self, 
        argv      = argv,
        server    = True, 
        name      = "server", 
        config    = self.config)
    else:
      pymsgque.MqS.__init__(self, 
        argv      = argv,
        master    = master,
        master_id = master_id,
        config    = self.config)

  def config(self):
    if self.ConfigGetIsSlave():
      # add "slave" services here
      nope
    else:
      # add "master" services here
      self.cl = [None, None, None]
      self.ServiceCreate("ECOS", self.ECOS)
      self.ServiceCreate("ECOI", self.ECOI)
      self.ServiceCreate("ECOW", self.ECOW)
      self.ServiceCreate("ECOF", self.ECOF)
      self.ServiceCreate("ECOD", self.ECOD)
      self.ServiceCreate("ECOC", self.ECOC)
      self.ServiceCreate("ECOU", self.ECOU)
      self.ServiceCreate("ECON", self.ECON)

      self.ServiceCreate("SETU", self.SETU)
      self.ServiceCreate("GETU", self.GETU)
      self.ServiceCreate("MSQT", self.MSQT)
      self.ServiceCreate("SND1", self.SND1)
      self.ServiceCreate("SND2", self.SND2)
      self.ServiceCreate("REDI", self.REDI)
      self.ServiceCreate("GTCX", self.GTCX)
      self.ServiceCreate("GTTO", self.GTTO)
      self.ServiceCreate("ECOL", self.ECOL)
      self.ServiceCreate("ECLI", self.ECLI)
      self.ServiceCreate("ERR2", self.ERR2)
      self.ServiceCreate("ERR3", self.ERR3)
      self.ServiceCreate("ECOY", self.ECOY)
      self.ServiceCreate("ECOO", self.ECOO)
      self.ServiceCreate("ECOB", self.ECOB)
      self.ServiceCreate("SLEP", self.SLEP)
      self.ServiceCreate("USLP", self.USLP)
      self.ServiceCreate("CSV1", self.CSV1)
      self.ServiceCreate("ECUL", self.ECUL)
      self.ServiceCreate("RDUL", self.RDUL)
      self.ServiceCreate("BUF1", self.BUF1)
      self.ServiceCreate("BUF2", self.BUF2)
      self.ServiceCreate("BUF3", self.BUF3)
      self.ServiceCreate("LST1", self.LST1)
      self.ServiceCreate("LST2", self.LST2)
      self.ServiceCreate("INIT", self.INIT)

  def INIT(self):
    self.SendSTART()
    LIST = []
    while self.ReadItemExists() :
      LIST.append(self.ReadC())
    pymsgque.MqS.Init(LIST)
    self.SendRETURN()

  def LST1(self):
    self.SendSTART()
    self.SendL_END()
    self.SendRETURN()

  def LST2(self):
    self.SendSTART()
    self.ReadL_END()
    self.SendRETURN()

  def BUF1(self):
    buf = self.ReadU()
    typ = self.GetType(buf)
    self.SendSTART()
    self.SendC(typ)
    if typ == "Y":
        self.SendY (self.GetY (buf))
    elif typ == "O":
        self.SendO (self.GetO (buf))
    elif typ == "S": 
        self.SendS (self.GetS (buf))
    elif typ == "I": 
        self.SendI (self.GetI (buf))
    elif typ == "F": 
        self.SendF (self.GetF (buf))
    elif typ == "W": 
        self.SendW (self.GetW (buf))
    elif typ == "D": 
        self.SendD (self.GetD (buf))
    elif typ == "C": 
        self.SendC (self.GetC (buf))
    elif typ == "B": 
        self.SendB (self.GetB (buf))
    self.SendRETURN()

  def BUF2(self):
    self.SendSTART()
    for i in range(3):
      buf = self.ReadU();
      self.SendC(self.GetType(buf))
      self.SendU(buf)
    self.SendRETURN()

  def BUF3(self):
    self.SendSTART()
    buf = self.ReadU();
    self.SendC(self.GetType(buf))
    self.SendU(buf)
    self.SendI(self.ReadI())
    self.SendU(buf)
    self.SendRETURN()

  def RDUL(self):
    self.ReadI()
    self.ReadW()
    self.ReadD()
    self.ReadU()

  def ECUL(self):
    self.SendSTART()
    self.SendI(self.ReadI())
    self.SendW(self.ReadW())
    self.SendD(self.ReadD())
    self.SendU(self.ReadU())
    self.SendRETURN()

  def CSV1(self):
    # call an other service
    self.SendSTART ()
    num = self.ReadI()
    num += 1
    self.SendI (num)
    self.SendEND_AND_WAIT ("CSV2", 10)

    # read the answer and send the result back
    self.SendSTART ()
    num = self.ReadI()
    num += 1
    self.SendI (num)
    self.SendRETURN ()

  def SLEP(self):
    i = self.ReadI()
    time.sleep(i)
    self.SendSTART()
    self.SendI(i)
    self.SendRETURN()

  def USLP(self):
    self.SendSTART()
    i = self.ReadI()
    time.sleep(1.0/i)
    self.SendI(i)
    self.SendRETURN()

  def ECOB(self):
    self.SendSTART()
    self.SendB(self.ReadB())
    self.SendRETURN()

  def ECOO(self):
    self.SendSTART()
    self.SendO(self.ReadO())
    self.SendRETURN()

  def ECOY(self):
    self.SendSTART()
    self.SendY(self.ReadY())
    self.SendRETURN()

  def ERR2(self):
    self.SendSTART()
    self.SendC("some data")
    self.ErrorC("Ot_ERR2", 10, "some error");
    self.SendRETURN()

  def ERR3(self):
    self.SendSTART()
    self.SendRETURN()
    self.SendRETURN()

  # my services
  def EchoList(self, doincr):
    while self.ReadItemExists() :
      buf = self.ReadU()
      if self.GetType(buf) == "L" :
        self.GetL_START (buf)
        self.SendL_START ()
        self.EchoList (doincr)
        self.SendL_END ()
        self.GetL_END ()
      elif doincr == True :
        self.SendI (self.GetI(buf)+1)
      else:
        self.SendU (buf)

  def ECLI (self):
    opt = self.ReadU()
    doincr = (self.GetType(opt) == "C" and self.GetC(opt) == "--incr")
    if doincr == False:
      self.ReadUndo()
    self.SendSTART()
    self.EchoList(doincr)
    self.SendRETURN()

  def ECOL (self):
    self.SendSTART()
    self.ReadL_START ()
    self.SendL_START ()
    self.EchoList(None)
    self.SendL_END ()
    self.ReadL_END ()
    self.SendRETURN()

  def GTCX(self):
    self.SendSTART()
    self.SendI(self.ConfigGetCtxId())
    self.SendRETURN()

  def REDI(self):
    i = self.ReadI()

  def GTCX(self):
    self.SendSTART()
    self.SendI(self.ConfigGetCtxId())
    self.SendRETURN()

  def GTTO(self):
    self.SendSTART()
    self.SendC(self.ConfigGetToken())
    self.SendRETURN()

  def ECOS(self):
    self.SendSTART()
    self.SendS(self.ReadS())
    self.SendRETURN()

  def ECOI(self):
    self.SendSTART()
    self.SendI(self.ReadI())
    self.SendRETURN()

  def ECOW(self):
    self.SendSTART()
    self.SendW(self.ReadW())
    self.SendRETURN()

  def ECOF(self):
    self.SendSTART()
    self.SendF(self.ReadF())
    self.SendRETURN()

  def ECOD(self):
    self.SendSTART()
    self.SendD(self.ReadD())
    self.SendRETURN()

  def ECOC(self):
    self.SendSTART()
    self.SendC(self.ReadC())
    self.SendRETURN()

  def ECOU(self):
    self.SendSTART()
    self.SendU(self.ReadU())
    self.SendRETURN()

  def ECON(self):
    self.SendSTART()
    self.SendC(self.ReadC() + "-" + self.ConfigGetName())
    self.SendRETURN()

  def MSQT(self):
    self.SendSTART()
    debug = self.ConfigGetDebug()
    if debug != 0:
      self.SendC ("debug")
      self.SendI (debug)
    if not self.ConfigGetIsString():
      self.SendC ("binary")
    if self.ConfigGetIsSilent():
      self.SendC ("silent")
    self.SendC ("sOc");
    if self.ConfigGetIsServer():
      self.SendC("SERVER")
    else:
      self.SendC("CLIENT")
    self.SendC ("pOc");
    if self.ConfigGetIsParent():
      self.SendC("PARENT")
    else:
      self.SendC("CHILD")
    self.SendRETURN()

  def SETU(self):
    self.buf = self.ReadU()

  def GETU(self):
    self.SendSTART()
    self.SendU(self.buf)
    self.SendRETURN()
    self.buf = None

  def Callback (self, ctx):
    self.i = ctx.ReadI();

  def SND1 (self):
    s = self.ReadC()
    id = self.ReadI()
    self.SendSTART()
    if s == "START" :
      parent = self.ConfigGetParent();
      if parent == None or parent.cl[id] == None :
        self.cl[id] = Client(debug = self.ConfigGetDebug());
      else:
        self.cl[id] = Client(parent = parent.cl[id]);
    elif s == "STOP" :
      self.cl[id].Delete();
      self.cl[id] = None;
    elif s == "SEND" :
      self.cl[id].SendSTART();
      TOK = self.ReadC();
      self.cl[id].SendU(self.ReadU());
      self.cl[id].SendEND(TOK);
    elif s == "WAIT" :
      self.cl[id].SendSTART();
      self.cl[id].SendU(self.ReadU());
      self.cl[id].SendEND_AND_WAIT("ECOI", 5);
      self.SendI(self.cl[id].ReadI()+1);
    elif s == "CALLBACK" :
      self.cl[id].SendSTART();
      self.cl[id].SendU(self.ReadU());
      self.i = -1;
      self.cl[id].SendEND_AND_CALLBACK("ECOI", self.Callback);
      self.cl[id].ProcessEvent(10,wait="ONCE");
      self.SendI(self.i+1);
    self.SendRETURN()

  def Callback2 (self, ctx):
    ctx.ConfigGetMaster().i = ctx.ReadI();

  def SND2 (self):
    s = self.ReadC()
    id = self.ReadI()
    cl = self.SlaveGet(id)
    self.SendSTART()
    if s == "CREATE" :
      LIST = ["worker-client", "--name", "test-client", "--srvname", "test-server"]
      while self.ReadItemExists() :
        LIST.append(self.ReadC())
      Server(argv=LIST, master=self, master_id=id)
    elif s == "CREATE2" :
      cl = Client(debug = self.ConfigGetDebug())
      self.SlaveCreate(id, cl);
    elif s == "DELETE" :
      self.SlaveDelete(id);
      if (self.SlaveGet(id) == None):
        self.SendC("OK")
      else :
        self.SendC("ERROR")
    elif s == "SEND" :
      cl.SendSTART();
      TOK = self.ReadC();
      cl.SendU(self.ReadU());
      cl.SendEND(TOK);
    elif s == "WAIT" :
      cl.SendSTART();
      cl.SendU(self.ReadU());
      cl.SendEND_AND_WAIT("ECOI", 5);
      self.SendI(cl.ReadI()+1);
    elif s == "CALLBACK" :
      cl.SendSTART();
      cl.SendU(self.ReadU());
      self.i = -1;
      cl.SendEND_AND_CALLBACK("ECOI", self.Callback2);
      cl.ProcessEvent(10,wait="ONCE");
      self.SendI(self.i+1);
    elif s == "MqSendEND_AND_WAIT" :
      TOK = self.ReadC()
      cl.SendSTART()
      while self.ReadItemExists() :
        cl.SendU(self.ReadU())
      cl.SendEND_AND_WAIT(TOK, 5);
      while cl.ReadItemExists() :
        self.SendU(cl.ReadU())
    elif s == "MqSendEND" :
      TOK = self.ReadC()
      cl.SendSTART()
      while self.ReadItemExists() :
        cl.SendU(self.ReadU())
      cl.SendEND(TOK);
      return
    elif s == "ERR-1" :
      Server(["worker-client", "--name", "test-client", "@", "DUMMY"], master=self, master_id=id)
    elif s == "isSlave" :
      self.SendO(cl.ConfigGetIsSlave());
      
    self.SendRETURN()

## --------------------------------------------------------
##    Main
##

#try:
#  srv = Server(sys.argv)
#  srv.ProcessEvent(wait="FOREVER")
#except Exception:
#  pass
#finally:
#  srv.Delete()

srv = Server(sys.argv)
srv.ProcessEvent(wait="FOREVER")
srv.Delete()

# vim: softtabstop=0:tabstop=8:shiftwidth=2:expandtab
