#+
#§  \file       tests/server.tcl
#§  \brief      \$Id: server.tcl 1187 2009-05-19 07:56:13Z dev1usr $
#§  
#§  (C) 2007 - LibMsgque - Project - Group
#§  
#§  \version    \$Rev: 1187 $
#§  \author     EMail: aotto1968 at users.sourceforge.net
#§  \attention  this software has GPL permissions to copy
#§              please contact AUTHORS for additional information
#§

package require Tcl 8.5

## load the TclMsgque extension
package require TclMsgque

proc Ot_INIT {ctx} {
  $ctx SendSTART
  set LIST [list]
  while {[$ctx ReadItemExists]} {
    lappend LIST [$ctx ReadC]
  }
  tclmsgque Init {*}$LIST
  $ctx SendRETURN
}

proc Ot_ECOU {ctx} {
  $ctx SendSTART
  $ctx SendU [$ctx ReadU]
  $ctx SendRETURN
}

proc Ot_RDUL {ctx} {
  $ctx ReadI
  $ctx ReadW
  $ctx ReadD
  $ctx ReadB
}

proc Ot_ECUL {ctx} {
  $ctx SendSTART
  $ctx SendI [$ctx ReadI]
  $ctx SendW [$ctx ReadW]
  $ctx SendD [$ctx ReadD]
  $ctx SendB [$ctx ReadB]
  $ctx SendRETURN
}

proc Ot_ECHO {ctx} {
  $ctx SendSTART
  switch -exact [$ctx ConfigGetToken] {
    ECOY  { $ctx SendY [$ctx ReadY] }
    ECOO  { $ctx SendO [$ctx ReadO] }
    ECOS  { $ctx SendS [$ctx ReadS] }
    ECOI  { $ctx SendI [$ctx ReadI] }
    ECOF  { $ctx SendF [$ctx ReadF] }
    ECOW  { $ctx SendW [$ctx ReadW] }
    ECOD  { $ctx SendD [$ctx ReadD] }
    ECOC  { $ctx SendC [$ctx ReadC] }
    ECOU  { $ctx SendU [$ctx ReadU] }
    ECOB  { $ctx SendB [$ctx ReadB] }
    ECON  { $ctx SendC [$ctx ReadC]-[$ctx ConfigGetName] }
  }
  $ctx SendRETURN
}

proc Ot_Error {ctx} {
  $ctx SendSTART
  switch -exact [$ctx ConfigGetToken] {
    ERR2  {
      $ctx ErrorC Ot_ERR2 10 "some error"
    }
    ERR3  {
      # double return error
      $ctx SendRETURN
    }
  }
  $ctx SendRETURN
}

proc Ot_MSQT {ctx} {
  $ctx SendSTART
  if {[$ctx ConfigGetDebug]} {
    $ctx SendC debug
    $ctx SendI [$ctx ConfigGetDebug]
  }
  if {![$ctx ConfigGetIsString]} {
    $ctx SendC binary
  }
  if {[$ctx ConfigGetIsSilent]} {
    $ctx SendC silent
  }
  $ctx SendC "sOc"
  $ctx SendC [expr {[$ctx ConfigGetIsServer] ? "SERVER" : "CLIENT"}]
  $ctx SendC "pOc"
  $ctx SendC [expr {[$ctx ConfigGetIsParent] ? "PARENT" : "CHILD"}]
  $ctx SendRETURN
}

proc Ot_GTCX {ctx} {
  $ctx SendSTART
  $ctx SendI [$ctx ConfigGetCtxId]
  $ctx SendRETURN
}

proc Ot_GTTO {ctx} {
  $ctx SendSTART
  $ctx SendC [$ctx ConfigGetToken]
  $ctx SendRETURN
}

proc Ot_CSV1 {ctx} {
  # call an other service
  $ctx SendSTART
  set num [$ctx ReadI]
  incr num
  $ctx SendI $num
  $ctx SendEND_AND_WAIT CSV2 10

  # read the answer and send the result back
  $ctx SendSTART
  set num [$ctx ReadI]
  incr num
  $ctx SendI $num
  $ctx SendRETURN
}

proc Ot_SLEP {ctx} {
  set i [$ctx ReadI]
  after [expr {$i * 1000}]
  $ctx SendSTART
  $ctx SendRETURN
}

proc Ot_USLP {ctx} {
  set i [$ctx ReadI]
  after [expr {$i / 1000}]
  $ctx SendSTART
  $ctx SendI $i
  $ctx SendRETURN
}

proc EchoList {ctx doincr} {
  while {[$ctx ReadItemExists]} {
    set buf [$ctx ReadU]
    if {[$ctx GetType $buf] eq "L"} {
      $ctx GetL_START $buf
      $ctx SendL_START
      EchoList $ctx $doincr
      $ctx SendL_END
      $ctx GetL_END
    } elseif { $doincr } {
      $ctx SendI [expr {[$ctx GetI $buf]+1}]
    } else {
      $ctx SendU $buf
    }
  }
}

proc Ot_ECLI {ctx} {
  set opt   [$ctx ReadU]
  set doincr  [expr {[$ctx GetType $opt] eq "C" && [$ctx GetC $opt] == "--incr"}]
  if {!$doincr} {
    $ctx ReadUndo
  }
  $ctx SendSTART
  EchoList $ctx $doincr
  $ctx SendRETURN
}

proc Ot_ECOL {ctx} {
  $ctx SendSTART
  $ctx ReadL_START
  $ctx SendL_START
  EchoList $ctx no
  $ctx SendL_END
  $ctx ReadL_END
  $ctx SendRETURN
}

proc Ot_EVAL {ctx} {
  $ctx SendSTART
  eval [$ctx ReadC]
  $ctx SendRETURN
}

proc Ot_SETU {ctx} {
  set ::val($ctx) [$ctx ReadU]
}

proc Ot_GETU {ctx} {
  $ctx SendSTART
  $ctx SendU $::val($ctx)
  $ctx SendRETURN
}

proc Ot_REDI {ctx} {
  ## just read the data ... do nothing else
  $ctx ReadI
}

proc Callback {ctx} {
  set ::i($ctx) [$ctx ReadI] 
}

## init the array
array set ::cl [list]
array set ::i [list]

proc Ot_SND1 {ctx} {
  
  set cmd [$ctx ReadC]
  set id  [$ctx ReadI]

  # link the context specific data into the procedure namespace
  upvar #0 cl($ctx,$id) cl

  $ctx SendSTART
  switch -exact $cmd {
    START {
      set parent  [$ctx ConfigGetParent]
      if {$parent ne "" && [info exists ::cl($parent,$id)]} {
	set cl    [tclmsgque MqS --parent $::cl($parent,$id)]
      } else {
	set	  CMD [list]
	lappend   CMD --name	"test-client"
	lappend   CMD --debug	[$ctx ConfigGetDebug]
	lappend   CMD --silent
	lappend   CMD --spawn
	lappend   CMD @
	lappend   CMD SELF
	lappend   CMD --name	"test-server"
	set cl    [tclmsgque MqS {*}$CMD]
      }
    }
    STOP {
      rename $cl ""
    }
    SEND {
      set TOK [$ctx ReadC]
      $cl SendSTART
      $cl SendU [$ctx ReadU]
      $cl SendEND $TOK
    }
    WAIT {
      $cl SendSTART
      $cl SendU [$ctx ReadU]
      $cl SendEND_AND_WAIT "ECOI" 5
      $ctx SendI [expr {[$cl ReadI] + 1}]
    }
    CALLBACK {
      upvar #0 i($cl) i
      set i -1;
      $cl SendSTART
      $cl SendU [$ctx ReadU]
      $cl SendEND_AND_CALLBACK "ECOI" Callback
      $cl ProcessEvent -timeout 10 -wait ONCE
      $ctx SendI [expr {$i + 1}]
    }
  }
  $ctx SendRETURN
}

proc Callback2 {ctx} {
  set ::i([$ctx ConfigGetMaster]) [$ctx ReadI] 
}

proc Ot_SND2 {ctx} {
  
  set cmd [$ctx ReadC]
  set id  [$ctx ReadI]

  # link the context specific data into the procedure namespace
  set cl [$ctx SlaveGet $id]

  $ctx SendSTART
  switch -exact $cmd {
    CREATE {
      tclmsgque MqS --master $ctx --master-id  $id --name test-client {*}[$ctx ReadAll] @ WORKER --name test-server
    }
    CREATE2 {
      set	CMD [list]
      lappend   CMD --name	"test-client"
      lappend   CMD --debug	[$ctx ConfigGetDebug]
      lappend   CMD --silent
      lappend   CMD --spawn
      lappend   CMD @
      lappend   CMD SELF
      lappend   CMD --name	"test-server"
      set cl    [tclmsgque MqS {*}$CMD]
      $ctx SlaveCreate $id $cl
    }
    DELETE {
      $ctx SlaveDelete $id
      $ctx SendC [expr {[$ctx SlaveGet $id] eq "" ? "OK" : "ERROR"}]
    }
    SEND {
      set TOK [$ctx ReadC]
      $cl SendSTART
      $cl SendU [$ctx ReadU]
      $cl SendEND $TOK
    }
    WAIT {
      $cl SendSTART
      $cl SendU [$ctx ReadU]
      $cl SendEND_AND_WAIT "ECOI" 5
      $ctx SendI [expr {[$cl ReadI] + 1}]
    }
    CALLBACK {
      upvar #0 i($ctx) i
      set i -1;
      $cl SendSTART
      $cl SendU [$ctx ReadU]
      $cl SendEND_AND_CALLBACK "ECOI" Callback2
      $cl ProcessEvent -timeout 10 -wait ONCE
      $ctx SendI [expr {$i + 1}]
    }
    MqSendEND_AND_WAIT {
      set TOK [$ctx ReadC]
      $cl SendSTART
      while {[$ctx ReadItemExists]} {
	$cl SendU [$ctx ReadU]
      }
      $cl SendEND_AND_WAIT $TOK 5
      while {[$cl ReadItemExists]} {
	$ctx SendU [$cl ReadU]
      }
    }
    MqSendEND {
      set TOK [$ctx ReadC]
      $cl SendSTART
      while {[$ctx ReadItemExists]} {
	$cl SendU [$ctx ReadU]
      }
      $cl SendEND $TOK 5
      return
    }
    ERR-1 {
      tclmsgque MqS --master $ctx --master-id  $id --name test-client {*}[$ctx ReadAll] @ DUMMY
    }
    isSlave {
      $ctx SendO [$cl ConfigGetIsSlave]
    }
  }
  $ctx SendRETURN
}

proc Ot_BUF1 {ctx} {
  set buf [$ctx ReadU]
  set typ [$ctx GetType $buf]
  $ctx SendSTART
  $ctx SendC $typ
  switch -exact $typ {
    "Y" { $ctx SendY [$ctx GetY $buf] }
    "O" { $ctx SendO [$ctx GetO $buf] }
    "S" { $ctx SendS [$ctx GetS $buf] }
    "I" { $ctx SendI [$ctx GetI $buf] }
    "F" { $ctx SendF [$ctx GetF $buf] }
    "W" { $ctx SendW [$ctx GetW $buf] }
    "D" { $ctx SendD [$ctx GetD $buf] }
    "C" { $ctx SendC [$ctx GetC $buf] }
    "B" { $ctx SendB [$ctx GetB $buf] }
  }
  $ctx SendRETURN 
}

proc Ot_BUF2 {ctx} {
  $ctx SendSTART

  foreach i {1 2 3} {
    set buf [$ctx ReadU]
    $ctx SendC [$ctx GetType $buf]
    $ctx SendU $buf
  }

  $ctx SendRETURN 
}

proc Ot_BUF3 {ctx} {
  $ctx SendSTART
  set buf [$ctx ReadU]
  $ctx SendC [$ctx GetType $buf]
  $ctx SendU $buf
  $ctx SendI [$ctx ReadI]
  $ctx SendU $buf
  $ctx SendRETURN 
}

proc Ot_LST1 {ctx} {
  $ctx SendSTART
  $ctx SendL_END
  $ctx SendRETURN 
}

proc Ot_LST2 {ctx} {
  $ctx SendSTART
  $ctx ReadL_END
  $ctx SendRETURN 
}

proc ServerConfig {ctx} {

  if {[$ctx ConfigGetIsSlave]} {
    # add "slave" services here
  } else {
    # add "master" services here
    $ctx ServiceCreate RDUL Ot_RDUL
    $ctx ServiceCreate ECUL Ot_ECUL

    $ctx ServiceCreate ECOY Ot_ECHO
    $ctx ServiceCreate ECOO Ot_ECHO
    $ctx ServiceCreate ECOS Ot_ECHO
    # START-TCL-SERVICE-DEFINITION
    $ctx ServiceCreate ECOI Ot_ECHO
    # END-TCL-SERVICE-DEFINITION
    $ctx ServiceCreate ECOF Ot_ECHO
    $ctx ServiceCreate ECOW Ot_ECHO
    $ctx ServiceCreate ECOD Ot_ECHO
    $ctx ServiceCreate ECOC Ot_ECHO
    $ctx ServiceCreate ECOB Ot_ECHO
    $ctx ServiceCreate ECOU Ot_ECHO
    $ctx ServiceCreate ECON Ot_ECHO

    $ctx ServiceCreate ERR1 Ot_Error
    $ctx ServiceCreate ERR2 Ot_Error
    $ctx ServiceCreate ERR3 Ot_Error
    $ctx ServiceCreate MSQT Ot_MSQT
    $ctx ServiceCreate GTCX Ot_GTCX
    $ctx ServiceCreate GTTO Ot_GTTO
    $ctx ServiceCreate SLEP Ot_SLEP
    $ctx ServiceCreate CSV1 Ot_CSV1
    $ctx ServiceCreate USLP Ot_USLP
    $ctx ServiceCreate ECOL Ot_ECOL
    $ctx ServiceCreate ECLI Ot_ECLI
    $ctx ServiceCreate EVAL Ot_EVAL
    $ctx ServiceCreate SETU Ot_SETU
    $ctx ServiceCreate GETU Ot_GETU
    $ctx ServiceCreate SND1 Ot_SND1
    $ctx ServiceCreate SND2 Ot_SND2
    $ctx ServiceCreate REDI Ot_REDI
    $ctx ServiceCreate BUF1 Ot_BUF1
    $ctx ServiceCreate BUF2 Ot_BUF2
    $ctx ServiceCreate BUF3 Ot_BUF3
    $ctx ServiceCreate LST1 Ot_LST1
    $ctx ServiceCreate LST2 Ot_LST2
    $ctx ServiceCreate INIT Ot_INIT
  }
}

# create the initial parent-context and wait forever for events
set srv [tclmsgque MqS --server --config ServerConfig --name server {*}$argv]
$srv ProcessEvent -wait FOREVER
$srv Delete

# the end, do !not! use "exit" because in "thread" mode this will kill the
# entire server and not only the "thread"

