/**
 *  \file       tests/stat.h
 *  \brief      \$Id: stat.h 1175 2009-05-14 13:44:36Z dev1usr $
 *  
 *  (C) 2004 - LibMsgque - Project - Group
 *  
 *  \version    \$Rev: 1175 $
 *  \author     EMail: aotto1968 at users.sourceforge.net
 *  \attention  this software has GPL permissions to copy
 *              please contact AUTHORS for additional information
 */

/** \addtogroup stat_api
 * \{
 */

#ifndef STAT_H
#   define STAT_H

#include "msgque.h"
#include "config.h"

/*****************************************************************************/
/*                                                                           */
/*                        stat context storage                               */
/*                                                                           */
/*****************************************************************************/

/// \brief the statistic context object used to calculate the statistics
///
/// The StatCtxS object is used together with an #StatTimerS object to calculate
/// the statistics in the #StatCtxCalc function using the initial date/time
/// values from the #StatTimerS and the current date/time values updating the
/// #StatCtxS::val object.
///
typedef struct StatCtxS {
  struct MqS * msgque;	    ///< link to the #MqS object
  struct MqBufferS * name;  ///< identify the measurement point
  MQ_INT count;	    ///< the number of times this measurement point was calculated
  MQ_DBL val;	    ///< the average time value
} StatCtxS;

/// \brief pointer to an #StatCtxS object
typedef struct StatCtxS *StatCtxSP;

/// \brief pointer to an #StatTimerS object
typedef struct StatTimerS *StatTimerSP;

/*****************************************************************************/
/*                                                                           */
/*                                stat_init                                  */
/*                                                                           */
/*****************************************************************************/

enum MqErrorE StatInit (
  StatTimerSP timer
);

enum MqErrorE StatInit2 (
  StatTimerSP timer,
  time_t tv_sec,
  MQ_INT tv_usec,
  MQ_INT tz_minuteswest,
  MQ_INT tz_dsttime
);

/*****************************************************************************/
/*                                                                           */
/*                                stat_get                                   */
/*                                                                           */
/*****************************************************************************/

MQ_INLINE struct mq_timeval * StatGetTV (
  StatTimerSP const timer
);

MQ_INLINE struct mq_timezone * StatGetTZ (
  StatTimerSP const timer
);

/*****************************************************************************/
/*                                                                           */
/*                                stat_ctx                                   */
/*                                                                           */
/*****************************************************************************/

StatCtxSP StatCtxCreate (
  struct MqS * const msgque,
  const MQ_STR name,
  MQ_INT count
);

MQ_INLINE void StatCtxDelete (
  StatCtxSP *contextP
);

MQ_INLINE void StatCtxPrint (
  StatCtxSP context
);

void StatCtxCalc (
  StatCtxSP context,
  StatTimerSP const timer
);

StatTimerSP StatCreate (
  struct MqS * const msgque
);

enum MqErrorE StatDelete (
  StatTimerSP *timertP
);

enum MqErrorE StatInit (
  StatTimerSP timer
);

void StatLog (
  const char *prefix,
  StatTimerSP const timer
);

#endif /* STAT_H */

/** \} stat_api */




